@@ -13,6 +13,7 @@ from shrub.v2 import ShrubProject, BuildVariant, ExistingTask
 from evergreen.api import RetryingEvergreenApi, EvergreenApi
 
 # Get relative imports to work when the package is not installed on the PYTHONPATH.
+from buildscripts.patch_builds.task_generation import validate_task_generation_limit
 
 if __name__ == "__main__" and __package__ is None:
     sys.path.append(os.path.dirname(os.path.dirname(os.path.abspath(__file__))))
@@ -84,18 +85,14 @@ def _get_config_options(expansions_file_data, build_variant, run_build_variant):
                          repeat_tests_max, project)
 
 
-def _create_evg_build_variant_map(expansions_file_data, evergreen_conf):
+def _create_evg_build_variant_map(expansions_file_data):
     """
     Generate relationship of base buildvariant to generated buildvariant.
 
     :param expansions_file_data: Config data file to use.
-    :param evergreen_conf: Evergreen configuration.
     :return: Map of base buildvariants to their generated buildvariants.
     """
-    burn_in_tags_gen_variant = expansions_file_data["build_variant"]
-    burn_in_tags_gen_variant_config = evergreen_conf.get_variant(burn_in_tags_gen_variant)
-    burn_in_tag_build_variants = burn_in_tags_gen_variant_config.expansions.get(
-        "burn_in_tag_buildvariants")
+    burn_in_tag_build_variants = expansions_file_data["burn_in_tag_buildvariants"]
 
     if burn_in_tag_build_variants:
         return {
@@ -135,8 +132,7 @@ def _generate_evg_build_variant(
 # pylint: disable=too-many-arguments,too-many-locals
 def _generate_evg_tasks(evergreen_api: EvergreenApi, shrub_project: ShrubProject,
                         task_expansions: Dict[str, Any], build_variant_map: Dict[str, str],
-                        repos: List[Repo], evg_conf: EvergreenProjectConfig,
-                        install_dir: str) -> None:
+                        repos: List[Repo], evg_conf: EvergreenProjectConfig) -> None:
     """
     Generate burn in tests tasks for a given shrub config and group of build variants.
 
@@ -151,7 +147,7 @@ def _generate_evg_tasks(evergreen_api: EvergreenApi, shrub_project: ShrubProject
         task_id = task_expansions[TASK_ID_EXPANSION]
         change_detector = EvergreenFileChangeDetector(task_id, evergreen_api)
         changed_tests = change_detector.find_changed_tests(repos)
-        tests_by_task = create_tests_by_task(build_variant, evg_conf, changed_tests, install_dir)
+        tests_by_task = create_tests_by_task(build_variant, evg_conf, changed_tests)
         if tests_by_task:
             shrub_build_variant = _generate_evg_build_variant(
                 evg_conf.get_variant(build_variant), run_build_variant,
@@ -164,12 +160,12 @@ def _generate_evg_tasks(evergreen_api: EvergreenApi, shrub_project: ShrubProject
                                          repeat_tests_secs=config_options.repeat_tests_secs)
 
             burn_in_generator = GenerateBurnInExecutor(gen_config, repeat_config, evergreen_api)
-            burn_in_generator.add_config_for_build_variant(shrub_build_variant, tests_by_task)
+            burn_in_generator.generate_tasks_for_variant(tests_by_task, shrub_build_variant)
             shrub_project.add_build_variant(shrub_build_variant)
 
 
 def burn_in(task_expansions: Dict[str, Any], evg_conf: EvergreenProjectConfig,
-            evergreen_api: RetryingEvergreenApi, repos: List[Repo], install_dir: str):
+            evergreen_api: RetryingEvergreenApi, repos: List[Repo]):
     """
     Execute main program.
 
@@ -177,13 +173,14 @@ def burn_in(task_expansions: Dict[str, Any], evg_conf: EvergreenProjectConfig,
     :param evg_conf: Evergreen configuration.
     :param evergreen_api: Evergreen.py object.
     :param repos: Git repositories.
-    :param install_dir: path to bin directory of a testable installation
     """
     shrub_project = ShrubProject.empty()
-    build_variant_map = _create_evg_build_variant_map(task_expansions, evg_conf)
+    build_variant_map = _create_evg_build_variant_map(task_expansions)
     _generate_evg_tasks(evergreen_api, shrub_project, task_expansions, build_variant_map, repos,
-                        evg_conf, install_dir)
+                        evg_conf)
 
+    if not validate_task_generation_limit(shrub_project):
+        sys.exit(1)
     write_file_to_dir(CONFIG_DIRECTORY, CONFIG_FILE, shrub_project.json())
 
 
@@ -207,9 +204,7 @@ def _configure_logging(verbose: bool):
 @click.option("--expansion-file", "expansion_file", required=True,
               help="Location of expansions file generated by evergreen.")
 @click.option("--verbose", is_flag=True)
-@click.option("--install-dir", "install_dir", required=True,
-              help="Path to bin directory of a testable installation")
-def main(expansion_file: str, verbose: bool, install_dir: str):
+def main(expansion_file: str, verbose: bool):
     """
     Run new or changed tests in repeated mode to validate their stability.
 
@@ -225,7 +220,7 @@ def main(expansion_file: str, verbose: bool, install_dir: str):
     expansions_file_data = read_config.read_config_file(expansion_file)
     evg_conf = evergreen.parse_evergreen_file(EVERGREEN_FILE)
 
-    burn_in(expansions_file_data, evg_conf, evg_api, repos, install_dir)
+    burn_in(expansions_file_data, evg_conf, evg_api, repos)
 
 
 if __name__ == "__main__":
