@@ -6,6 +6,7 @@ import math
 import sys
 from datetime import timedelta
 from typing import Optional
+
 import yaml
 
 COMMIT_QUEUE_ALIAS = "__commit_queue"
@@ -24,7 +25,7 @@ SPECIFIC_TASK_OVERRIDES = {
         "replica_sets_jscore_passthrough": timedelta(hours=3),
         "replica_sets_update_v1_oplog_jscore_passthrough": timedelta(hours=2, minutes=30),
     },
-    "enterprise-windows-suggested": {
+    "enterprise-windows-required": {
         "replica_sets_jscore_passthrough": timedelta(hours=3),
         "replica_sets_update_v1_oplog_jscore_passthrough": timedelta(hours=2, minutes=30),
     },
@@ -43,14 +44,8 @@ SPECIFIC_TASK_OVERRIDES = {
     "enterprise-rhel-80-64-bit-coverage": {
         "replica_sets_jscore_passthrough": timedelta(hours=2, minutes=30),
     },
-    "macos": {
-        "replica_sets_jscore_passthrough": timedelta(hours=2, minutes=30),
-        "sharded_collections_jscore_passthrough": timedelta(hours=5)
-    },
-    "macos-arm64": {"sharded_collections_jscore_passthrough": timedelta(hours=5)},
+    "macos": {"replica_sets_jscore_passthrough": timedelta(hours=2, minutes=30), },
     "enterprise-macos": {"replica_sets_jscore_passthrough": timedelta(hours=2, minutes=30), },
-    "enterprise-rhel-72-s390x": {"integration_tests_sharded": timedelta(hours=4)},
-    "enterprise-rhel-72-s390x-inmem": {"integration_tests_sharded": timedelta(hours=4)},
 
     # unittests outliers
     # repeated execution runs a suite 10 times
