@@ -9,18 +9,15 @@ import distutils.spawn
 import sys
 import platform
 import random
-import glob
-import textwrap
-import shlex
 
 import pymongo.uri_parser
 
-from buildscripts.idl import gen_all_feature_flag_list
 from buildscripts.idl.lib import ALL_FEATURE_FLAG_FILE
 
 from buildscripts.resmokelib import config as _config
 from buildscripts.resmokelib import utils
 from buildscripts.resmokelib import mongod_fuzzer_configs
+from buildscripts.resmokelib.suitesconfig import SuiteFinder
 
 
 def validate_and_update_config(parser, args):
@@ -51,9 +48,14 @@ def _validate_options(parser, args):
             "Cannot use --replayFile with additional test files listed on the command line invocation."
         )
 
-    if args.additional_feature_flags_file and not os.path.isfile(
-            args.additional_feature_flags_file):
-        parser.error("The specified additional feature flags file does not exist.")
+    if args.run_all_feature_flag_tests or args.run_all_feature_flags_no_tests:
+        if not os.path.isfile(ALL_FEATURE_FLAG_FILE):
+            parser.error(
+                "To run tests with all feature flags, the %s file must exist and be placed in"
+                " your working directory. The file can be downloaded from the artifacts tarball"
+                " in Evergreen. Alternatively, if you know which feature flags you want to enable,"
+                " you can use the --additionalFeatureFlags command line argument" %
+                ALL_FEATURE_FLAG_FILE)
 
     def get_set_param_errors(process_params):
         agg_set_params = collections.defaultdict(list)
@@ -131,19 +133,6 @@ def _validate_config(parser):  # pylint: disable=too-many-branches
             parser.error(f"Found '{resolved_path}', but it is not an executable file")
 
 
-def _find_resmoke_wrappers():
-    # This is technically incorrect. PREFIX_BINDIR defaults to $PREFIX/bin, so
-    # if the user changes it to any some other value, this glob will fail to
-    # detect the resmoke wrapper.
-    # Additionally, the resmoke wrapper will not be found if a user performs
-    # their builds outside of the git repository root, (ex checkout at
-    # /data/mongo, build-dir at /data/build)
-    # We assume that users who fall under either case will explicitly pass the
-    # --installDir argument.
-    candidate_installs = glob.glob("**/bin/resmoke.py", recursive=True)
-    return list(candidate_installs)
-
-
 def _update_config_vars(values):  # pylint: disable=too-many-statements,too-many-locals,too-many-branches
     """Update the variables of the config module."""
 
@@ -158,60 +147,38 @@ def _update_config_vars(values):  # pylint: disable=too-many-statements,too-many
         if cmdline_vars[cmdline_key] is not None:
             config[cmdline_key] = cmdline_vars[cmdline_key]
 
-    if values.command == "run" and os.path.isfile("resmoke.ini"):
-        err = textwrap.dedent("""\
-Support for resmoke.ini has been removed. You must delete
-resmoke.ini and rerun your build to run resmoke. If only one testable
-installation is present, resmoke will automatically locate that installation.
-If you have multiple installations, you must either pass an explicit
---installDir argument to the run subcommand to identify the installation you
-would like to test, or invoke the customized resmoke.py wrapper script staged
-into the bin directory of each installation.""")
+    if os.path.isfile("resmoke.ini"):
         config_parser = configparser.ConfigParser()
         config_parser.read("resmoke.ini")
         if "resmoke" in config_parser.sections():
             user_config = dict(config_parser["resmoke"])
-            err += textwrap.dedent(f"""
-
-Based on the current value of resmoke.ini, after rebuilding, resmoke.py should
-be invoked as either:
-- {shlex.quote(f"{user_config['install_dir']}/resmoke.py")}
-- buildscripts/resmoke.py --installDir {shlex.quote(user_config['install_dir'])}""")
-        raise RuntimeError(err)
-
-    def process_feature_flag_file(path):
-        with open(path) as fd:
-            return fd.read().split()
+            config.update(user_config)
 
     def setup_feature_flags():
         _config.RUN_ALL_FEATURE_FLAG_TESTS = config.pop("run_all_feature_flag_tests")
-        _config.RUN_NO_FEATURE_FLAG_TESTS = config.pop("run_no_feature_flag_tests")
-        _config.ADDITIONAL_FEATURE_FLAGS_FILE = config.pop("additional_feature_flags_file")
+        _config.RUN_ALL_FEATURE_FLAGS = config.pop("run_all_feature_flags_no_tests")
 
+        # Running all feature flag tests implies running the fixtures with feature flags.
         if _config.RUN_ALL_FEATURE_FLAG_TESTS:
-            print("Generating: ", ALL_FEATURE_FLAG_FILE)
-            gen_all_feature_flag_list.gen_all_feature_flags_file()
+            _config.RUN_ALL_FEATURE_FLAGS = True
 
         all_ff = []
         enabled_feature_flags = []
         try:
-            all_ff = process_feature_flag_file(ALL_FEATURE_FLAG_FILE)
+            with open(ALL_FEATURE_FLAG_FILE) as fd:
+                all_ff = fd.read().split()
         except FileNotFoundError:
             # If we ask resmoke to run with all feature flags, the feature flags file
             # needs to exist.
-            if _config.RUN_ALL_FEATURE_FLAG_TESTS or _config.RUN_NO_FEATURE_FLAG_TESTS:
+            if _config.RUN_ALL_FEATURE_FLAGS:
                 raise
 
-        if _config.RUN_ALL_FEATURE_FLAG_TESTS:
+        if _config.RUN_ALL_FEATURE_FLAGS:
             enabled_feature_flags = all_ff[:]
 
-        if _config.ADDITIONAL_FEATURE_FLAGS_FILE:
-            enabled_feature_flags.extend(
-                process_feature_flag_file(_config.ADDITIONAL_FEATURE_FLAGS_FILE))
-
         # Specify additional feature flags from the command line.
         # Set running all feature flag tests to True if this options is specified.
-        additional_feature_flags = config.pop("additional_feature_flags")
+        additional_feature_flags = _tags_from_list(config.pop("additional_feature_flags"))
         if additional_feature_flags is not None:
             enabled_feature_flags.extend(additional_feature_flags)
 
@@ -232,7 +199,7 @@ be invoked as either:
     _config.EXCLUDE_WITH_ANY_TAGS.extend(
         utils.default_if_none(_tags_from_list(config.pop("exclude_with_any_tags")), []))
 
-    if _config.RUN_NO_FEATURE_FLAG_TESTS:
+    if _config.RUN_ALL_FEATURE_FLAGS and not _config.RUN_ALL_FEATURE_FLAG_TESTS:
         # Don't run any feature flag tests.
         _config.EXCLUDE_WITH_ANY_TAGS.extend(all_feature_flags)
     else:
@@ -244,6 +211,7 @@ be invoked as either:
     _config.FLOW_CONTROL_TICKETS = config.pop("flow_control_tickets")
 
     _config.INCLUDE_WITH_ANY_TAGS = _tags_from_list(config.pop("include_with_any_tags"))
+    _config.INCLUDE_TAGS = _tags_from_list(config.pop("include_with_all_tags"))
 
     _config.GENNY_EXECUTABLE = _expand_user(config.pop("genny_executable"))
     _config.JOBS = config.pop("jobs")
@@ -253,19 +221,9 @@ be invoked as either:
     if _config.MIXED_BIN_VERSIONS is not None:
         _config.MIXED_BIN_VERSIONS = _config.MIXED_BIN_VERSIONS.split("-")
 
-    _config.INSTALL_DIR = config.pop("install_dir")
-    if values.command == "run" and _config.INSTALL_DIR is None:
-        resmoke_wrappers = _find_resmoke_wrappers()
-        if len(resmoke_wrappers) == 1:
-            _config.INSTALL_DIR = os.path.dirname(resmoke_wrappers[0])
-        elif len(resmoke_wrappers) > 1:
-            err = textwrap.dedent(f"""\
-Multiple testable installations were found, but installDir was not specified.
-You must either call resmoke via one of the following scripts:
-{os.linesep.join(map(shlex.quote, resmoke_wrappers))}
+    _config.MULTIVERSION_BIN_VERSION = config.pop("old_bin_version")
 
-or explicitly pass --installDir to the run subcommand of buildscripts/resmoke.py.""")
-            raise RuntimeError(err)
+    _config.INSTALL_DIR = config.pop("install_dir")
     if _config.INSTALL_DIR is not None:
         # Normalize the path so that on Windows dist-test/bin
         # translates to .\dist-test\bin then absolutify it since the
@@ -289,9 +247,6 @@ or explicitly pass --installDir to the run subcommand of buildscripts/resmoke.py
     _config.MONGOD_EXECUTABLE = _expand_user(config.pop("mongod_executable"))
 
     mongod_set_parameters = config.pop("mongod_set_parameters")
-    if _config.ENABLED_FEATURE_FLAGS:
-        feature_flag_dict = {ff: "true" for ff in _config.ENABLED_FEATURE_FLAGS}
-        mongod_set_parameters.append(str(feature_flag_dict))
 
     _config.MONGOD_SET_PARAMETERS = _merge_set_params(mongod_set_parameters)
     _config.FUZZ_MONGOD_CONFIGS = config.pop("fuzz_mongod_configs")
@@ -302,16 +257,12 @@ or explicitly pass --installDir to the run subcommand of buildscripts/resmoke.py
             _config.CONFIG_FUZZ_SEED = random.randrange(sys.maxsize)
         else:
             _config.CONFIG_FUZZ_SEED = int(_config.CONFIG_FUZZ_SEED)
-        _config.MONGOD_SET_PARAMETERS, _config.WT_ENGINE_CONFIG = mongod_fuzzer_configs \
-            .fuzz_set_parameters(_config.CONFIG_FUZZ_SEED, _config.MONGOD_SET_PARAMETERS)
+        _config.MONGOD_SET_PARAMETERS, _config.WT_ENGINE_CONFIG, _config.WT_COLL_CONFIG, \
+        _config.WT_INDEX_CONFIG = mongod_fuzzer_configs.fuzz_set_parameters(
+            _config.CONFIG_FUZZ_SEED, _config.MONGOD_SET_PARAMETERS)
 
     _config.MONGOS_EXECUTABLE = _expand_user(config.pop("mongos_executable"))
-
     mongos_set_parameters = config.pop("mongos_set_parameters")
-    if _config.ENABLED_FEATURE_FLAGS:
-        feature_flag_dict = {ff: "true" for ff in _config.ENABLED_FEATURE_FLAGS}
-        mongos_set_parameters.append(str(feature_flag_dict))
-
     _config.MONGOS_SET_PARAMETERS = _merge_set_params(mongos_set_parameters)
 
     _config.MONGOCRYPTD_SET_PARAMETERS = _merge_set_params(config.pop("mongocryptd_set_parameters"))
@@ -331,8 +282,6 @@ or explicitly pass --installDir to the run subcommand of buildscripts/resmoke.py
     _config.REPORT_FAILURE_STATUS = config.pop("report_failure_status")
     _config.REPORT_FILE = config.pop("report_file")
     _config.SERVICE_EXECUTOR = config.pop("service_executor")
-    _config.SHELL_READ_MODE = config.pop("shell_read_mode")
-    _config.SHELL_WRITE_MODE = config.pop("shell_write_mode")
     _config.SPAWN_USING = config.pop("spawn_using")
     _config.EXPORT_MONGOD_CONFIG = config.pop("export_mongod_config")
     _config.STAGGER_JOBS = config.pop("stagger_jobs") == "on"
@@ -341,7 +290,7 @@ or explicitly pass --installDir to the run subcommand of buildscripts/resmoke.py
     _config.SUITE_FILES = config.pop("suite_files")
     if _config.SUITE_FILES is not None:
         _config.SUITE_FILES = _config.SUITE_FILES.split(",")
-    _config.TAG_FILE = config.pop("tag_file")
+    _config.TAG_FILES = config.pop("tag_files")
     _config.TRANSPORT_LAYER = config.pop("transport_layer")
     _config.USER_FRIENDLY_OUTPUT = config.pop("user_friendly_output")
 
@@ -367,23 +316,6 @@ or explicitly pass --installDir to the run subcommand of buildscripts/resmoke.py
     _config.CEDAR_URL = config.pop("cedar_url")
     _config.CEDAR_RPC_PORT = config.pop("cedar_rpc_port")
 
-    def calculate_debug_symbol_url():
-        url = "https://mciuploads.s3.amazonaws.com"
-        project_name = _config.EVERGREEN_PROJECT_NAME
-        variant_name = _config.EVERGREEN_VARIANT_NAME
-        revision = _config.EVERGREEN_REVISION
-        task_id = _config.EVERGREEN_TASK_ID
-        if (variant_name is not None) and (revision is not None) and (task_id is not None):
-            url = "/".join([
-                url, project_name, variant_name, revision,
-                f"debugsymbols/debugsymbols-{_config.EVERGREEN_BUILD_ID}"
-            ])
-            url = url + (".zip" if sys.platform == "win32" else ".tgz")
-            return url
-        return None
-
-    _config.DEBUG_SYMBOLS_URL = calculate_debug_symbol_url()
-
     # Archival options. Archival is enabled only when running on evergreen.
     if not _config.EVERGREEN_TASK_ID:
         _config.ARCHIVE_FILE = None
@@ -395,12 +327,16 @@ or explicitly pass --installDir to the run subcommand of buildscripts/resmoke.py
     _config.ARCHIVE_LIMIT_MB = config.pop("archive_limit_mb")
     _config.ARCHIVE_LIMIT_TESTS = config.pop("archive_limit_tests")
 
-    # Wiredtiger options.
-    _config.WT_COLL_CONFIG = config.pop("wt_coll_config")
+    # Wiredtiger options. Prevent fuzzed wt configs from being overwritten unless user specifies it.
     wt_engine_config = config.pop("wt_engine_config")
-    if wt_engine_config:  # prevents fuzzed wt_engine_config from being overwritten unless user specifies it
+    if wt_engine_config:
         _config.WT_ENGINE_CONFIG = config.pop("wt_engine_config")
-    _config.WT_INDEX_CONFIG = config.pop("wt_index_config")
+    wt_coll_config = config.pop("wt_coll_config")
+    if wt_coll_config:
+        _config.WT_COLL_CONFIG = config.pop("wt_coll_config")
+    wt_index_config = config.pop("wt_index_config")
+    if wt_index_config:
+        _config.WT_INDEX_CONFIG = config.pop("wt_index_config")
 
     # Benchmark/Benchrun options.
     _config.BENCHMARK_FILTER = config.pop("benchmark_filter")
@@ -425,8 +361,9 @@ or explicitly pass --installDir to the run subcommand of buildscripts/resmoke.py
 
     _config.UNDO_RECORDER_PATH = config.pop("undo_recorder_path")
 
-    # Populate the named suites by scanning config_dir/suites
-    named_suites = {}
+    _config.EXCLUDE_TAGS_FILE_PATH = config.pop("exclude_tags_file_path")
+
+    _config.MAX_TEST_QUEUE_SIZE = config.pop("max_test_queue_size")
 
     def configure_tests(test_files, replay_file):
         # `_validate_options` has asserted that at most one of `test_files` and `replay_file` contains input.
@@ -447,17 +384,6 @@ or explicitly pass --installDir to the run subcommand of buildscripts/resmoke.py
 
     configure_tests(config.pop("test_files"), config.pop("replay_file"))
 
-    suites_dir = os.path.join(_config.CONFIG_DIR, "suites")
-    root = os.path.abspath(suites_dir)
-    files = os.listdir(root)
-    for filename in files:
-        (short_name, ext) = os.path.splitext(filename)
-        if ext in (".yml", ".yaml"):
-            pathname = os.path.join(root, filename)
-            named_suites[short_name] = pathname
-
-    _config.NAMED_SUITES = named_suites
-
     _config.LOGGER_DIR = os.path.join(_config.CONFIG_DIR, "loggers")
 
     shuffle = config.pop("shuffle")
