@@ -37,7 +37,6 @@ from buildscripts.burn_in_tests import DEFAULT_REPO_LOCATIONS, create_task_list_
     TaskInfo
 from buildscripts.ciconfig.evergreen import (
     EvergreenProjectConfig,
-    ResmokeArgs,
     Task,
     parse_evergreen_file,
     Variant,
@@ -47,7 +46,6 @@ from buildscripts.patch_builds.selected_tests.selected_tests_service import Sele
 structlog.configure(logger_factory=LoggerFactory())
 LOGGER = structlog.getLogger(__name__)
 
-DEFAULT_TEST_SUITE_DIR = os.path.join("buildscripts", "resmokeconfig", "suites")
 TASK_ID_EXPANSION = "task_id"
 EVERGREEN_FILE = "etc/evergreen.yml"
 EVG_CONFIG_FILE = ".evergreen.yml"
@@ -86,9 +84,7 @@ EXCLUDE_TASK_LIST = [
 ]
 POSSIBLE_RUN_TASK_FUNCS = [
     "generate resmoke tasks",
-    "generate randomized multiversion tasks",
     "run tests",
-    "generate explicit multiversion tasks",
 ]
 
 
@@ -167,17 +163,10 @@ class TaskConfigService:
         for run_task_func in POSSIBLE_RUN_TASK_FUNCS:
             task_def = task.find_func_command(run_task_func)
             if task_def:
-                task_vars = task_def["vars"]
+                task_vars = task_def.get("vars", {})
                 break
 
-        suite_name = ResmokeArgs.get_arg(task_vars["resmoke_args"], "suites")
-        if suite_name:
-            task_vars.update({"suite": suite_name})
-
-        # the suites argument will run all tests in a suite even when individual
-        # tests are specified in resmoke_args, so we remove it
-        resmoke_args_without_suites = ResmokeArgs.remove_arg(task_vars["resmoke_args"], "suites")
-        task_vars["resmoke_args"] = resmoke_args_without_suites
+        task_vars.update({"suite": task.get_suite_name()})
 
         task_name = task.name[:-4] if task.name.endswith("_gen") else task.name
         return {
@@ -401,6 +390,7 @@ class SelectedTestsOrchestrator:
         task_configs = self.get_task_config(build_variant_config, changed_files)
 
         for task_config in task_configs.values():
+            task_def = Task(task_config)
             test_filter = None
             if "selected_tests_to_run" in task_config:
                 test_filter = partial(filter_set, input_set=task_config["selected_tests_to_run"])
@@ -415,9 +405,10 @@ class SelectedTestsOrchestrator:
             gen_params = ResmokeGenTaskParams(
                 use_large_distro=task_config.get("use_large_distro", False),
                 large_distro_name=task_config.get("large_distro_name"),
-                require_multiversion=task_config.get("require_multiversion"),
+                require_multiversion_setup=task_def.require_multiversion_setup(),
+                require_multiversion_version_combo=False,
                 repeat_suites=task_config.get("repeat_suites", 1),
-                resmoke_args=task_config["resmoke_args"],
+                resmoke_args=task_config.get("resmoke_args", ""),
                 resmoke_jobs_max=task_config.get("resmoke_jobs_max"),
                 config_location=self.evg_expansions.get_config_location(),
             )
