@@ -17,7 +17,6 @@ from buildscripts.patch_builds.selected_tests.selected_tests_client import Selec
     TaskMapping
 from buildscripts.selected_tests import EvgExpansions
 from buildscripts.task_generation.gen_config import GenerationConfiguration
-from buildscripts.task_generation.resmoke_proxy import ResmokeProxyConfig
 from buildscripts.task_generation.suite_split import SuiteSplitConfig
 from buildscripts.task_generation.suite_split_strategies import SplitStrategy, greedy_division, \
     FallbackStrategy, round_robin_fallback
@@ -25,7 +24,7 @@ from buildscripts.task_generation.task_types.gentask_options import GenTaskOptio
 from buildscripts.tests.test_burn_in_tests import get_evergreen_config, mock_changed_git_files
 from buildscripts import selected_tests as under_test
 
-# pylint: disable=missing-docstring,invalid-name,unused-argument,protected-access,no-value-for-parameter,too-many-locals
+# pylint: disable=missing-docstring,invalid-name,unused-argument,protected-access,no-value-for-parameter
 
 NS = "buildscripts.selected_tests"
 
@@ -45,8 +44,7 @@ def empty_build_variant(variant_name: str) -> Dict[str, Any]:
     }
 
 
-def configure_dependencies(evg_api, evg_expansions, evg_project_config, selected_test_client,
-                           test_suites_dir=under_test.DEFAULT_TEST_SUITE_DIR):
+def configure_dependencies(evg_api, evg_expansions, evg_project_config, selected_test_client):
     start_date = datetime.utcnow()
     end_date = start_date - timedelta(weeks=2)
 
@@ -59,7 +57,6 @@ def configure_dependencies(evg_api, evg_expansions, evg_project_config, selected
         binder.bind(GenTaskOptions, evg_expansions.build_gen_task_options())
         binder.bind(EvergreenApi, evg_api)
         binder.bind(GenerationConfiguration, GenerationConfiguration.from_yaml_file())
-        binder.bind(ResmokeProxyConfig, ResmokeProxyConfig(resmoke_suite_dir=test_suites_dir))
         binder.bind(SelectedTestsClient, selected_test_client)
 
     inject.clear_and_configure(dependencies)
@@ -104,9 +101,7 @@ class TestAcceptance(unittest.TestCase):
         self.assertEqual(generated_config.file_list[0].file_name, "selected_tests_config.json")
 
     @unittest.skipIf(sys.platform.startswith("win"), "not supported on windows")
-    @patch("buildscripts.util.teststats.HistoricTaskData.get_stats_from_s3")
-    def test_when_test_mappings_are_found_for_changed_files(self, get_stats_from_s3_mock):
-        get_stats_from_s3_mock.return_value = []
+    def test_when_test_mappings_are_found_for_changed_files(self):
         mock_evg_api = self._mock_evg_api()
         mock_evg_config = get_evergreen_config("etc/evergreen.yml")
         mock_evg_expansions = under_test.EvgExpansions(
@@ -154,9 +149,7 @@ class TestAcceptance(unittest.TestCase):
         self.assertEqual(len(rhel_80_with_generated_tasks["tasks"]), 2)
 
     @unittest.skipIf(sys.platform.startswith("win"), "not supported on windows")
-    @patch("buildscripts.util.teststats.HistoricTaskData.get_stats_from_s3")
-    def test_when_task_mappings_are_found_for_changed_files(self, get_stats_from_s3_mock):
-        get_stats_from_s3_mock.return_value = []
+    def test_when_task_mappings_are_found_for_changed_files(self):
         mock_evg_api = self._mock_evg_api()
         mock_evg_config = get_evergreen_config("etc/evergreen.yml")
         mock_evg_expansions = under_test.EvgExpansions(
@@ -242,7 +235,7 @@ class TestGetEvgTaskConfig(unittest.TestCase):
 
         self.assertEqual(evg_task_config["task_name"], "auth")
         self.assertEqual(evg_task_config["build_variant"], "variant")
-        self.assertIsNone(evg_task_config.get("suite"))
+        self.assertEqual(evg_task_config.get("suite"), "auth")
         self.assertEqual(
             evg_task_config["resmoke_args"],
             "--storageEngine=wiredTiger",
@@ -251,15 +244,14 @@ class TestGetEvgTaskConfig(unittest.TestCase):
     def test_task_is_not_a_generate_resmoke_task(self):
         build_variant_conf = MagicMock()
         build_variant_conf.name = "variant"
-        task = build_mock_evg_task("jsCore_auth", "run tests",
-                                   "--suites=core_auth --storageEngine=wiredTiger")
+        task = build_mock_evg_task("jsCore_auth", "run tests", "--storageEngine=wiredTiger")
 
         task_config_service = under_test.TaskConfigService()
         evg_task_config = task_config_service.get_evg_task_config(task, build_variant_conf)
 
         self.assertEqual(evg_task_config["task_name"], "jsCore_auth")
         self.assertEqual(evg_task_config["build_variant"], "variant")
-        self.assertEqual(evg_task_config["suite"], "core_auth")
+        self.assertEqual(evg_task_config["suite"], "jsCore_auth")
         self.assertEqual(
             evg_task_config["resmoke_args"],
             "--storageEngine=wiredTiger",
@@ -285,16 +277,20 @@ class TestGetTaskConfigsForTestMappings(unittest.TestCase):
                         "jstests/core/latch_analyzer.js",
                     ],
                     resmoke_args="",
-                    require_multiversion=None,
+                    require_multiversion_setup=False,
                     distro="",
+                    suite="core_auth",
+                    build_variant="dummy_variant",
                 ),
             "auth_gen":
                 TaskInfo(
                     display_task_name="task 2",
                     tests=["jstests/auth/auth3.js"],
                     resmoke_args="",
-                    require_multiversion=None,
+                    require_multiversion_setup=False,
                     distro="",
+                    suite="auth",
+                    build_variant="dummy_variant",
                 ),
         }
 
@@ -306,7 +302,7 @@ class TestGetTaskConfigsForTestMappings(unittest.TestCase):
         self.assertEqual(
             task_configs["jsCore_auth"]["selected_tests_to_run"],
             {"jstests/core/currentop_waiting_for_latch.js", "jstests/core/latch_analyzer.js"})
-        self.assertEqual(task_configs["auth_gen"]["suite"], "core_auth")
+        self.assertEqual(task_configs["auth_gen"]["suite"], "auth_gen")
         self.assertEqual(task_configs["auth_gen"]["selected_tests_to_run"],
                          {'jstests/auth/auth3.js'})
 
@@ -326,8 +322,10 @@ class TestGetTaskConfigsForTestMappings(unittest.TestCase):
                         "jstests/core/latch_analyzer.js",
                     ],
                     resmoke_args="",
-                    require_multiversion=None,
+                    require_multiversion_setup=False,
                     distro="",
+                    suite="core",
+                    build_variant="dummy_variant",
                 ),
         }
 
@@ -349,8 +347,10 @@ class TestGetTaskConfigsForTestMappings(unittest.TestCase):
                         "jstests/core/latch_analyzer.js",
                     ],
                     resmoke_args="",
-                    require_multiversion=None,
+                    require_multiversion_setup=False,
                     distro="",
+                    suite="core",
+                    build_variant="dummy_variant",
                 ),
         }
 
