@@ -14,6 +14,36 @@ const ChangeStreamWatchMode = Object.freeze({
     kCluster: 3,
 });
 
+/**
+ * Returns true if server version is 5.1 or above. Version 5.1 and above optimizes the change stream
+ * pipeline.
+ *
+ * TODO SERVER-60736: remove this function and update all call-sites.
+ */
+function isChangeStreamsOptimizationEnabled(db) {
+    return MongoRunner.compareBinVersions(db.getSiblingDB("admin").serverStatus().version, "5.1") !=
+        -1;
+}
+
+/**
+ * Returns true if feature flag 'featureFlagChangeStreamPreAndPostImages' is enabled, false
+ * otherwise.
+ */
+function isChangeStreamPreAndPostImagesEnabled(db) {
+    const getParam = db.adminCommand({getParameter: 1, featureFlagChangeStreamPreAndPostImages: 1});
+    return getParam.hasOwnProperty("featureFlagChangeStreamPreAndPostImages") &&
+        getParam.featureFlagChangeStreamPreAndPostImages.value;
+}
+
+/**
+ * Returns true if feature flag 'featureFlagChangeStreamsRewrite' is enabled, false otherwise.
+ */
+function isChangeStreamsRewriteEnabled(db) {
+    const getParam = db.adminCommand({getParameter: 1, featureFlagChangeStreamsRewrite: 1});
+    return getParam.hasOwnProperty("featureFlagChangeStreamsRewrite") &&
+        getParam.featureFlagChangeStreamsRewrite.value;
+}
+
 /**
  * Helper function used internally by ChangeStreamTest. If no passthrough is active, it is exactly
  * the same as calling db.runCommand. If a passthrough is active and has defined a function
@@ -52,6 +82,7 @@ function assertInvalidateOp({cursor, opType}) {
         assert.soon(() => cursor.hasNext());
         const invalidate = cursor.next();
         assert.eq(invalidate.operationType, "invalidate");
+        assert(!cursor.hasNext());
         assert(cursor.isExhausted());
         assert(cursor.isClosed());
         return invalidate;
@@ -59,7 +90,7 @@ function assertInvalidateOp({cursor, opType}) {
     return null;
 }
 
-function pruneOptionalFields(event, expected) {
+function canonicalizeEventForTesting(event, expected) {
     if (!expected.hasOwnProperty("_id"))
         delete event._id;
 
@@ -72,6 +103,22 @@ function pruneOptionalFields(event, expected) {
     if (!expected.hasOwnProperty("lsid"))
         delete event.lsid;
 
+    // TODO SERVER-50301: The 'truncatedArrays' field may not appear in the updateDescription
+    // depending on whether $v:2 update oplog entries are enabled. When the expected event has an
+    // empty 'truncatedFields' we do not require that the actual event contain the field. This
+    // logic can be removed when $v:2 update oplog entries are enabled on all configurations.
+    if (
+        // If the expected event has an empty 'truncatedArrays' field...
+        expected.hasOwnProperty("updateDescription") &&
+        Array.isArray(expected.updateDescription.truncatedArrays) &&
+        expected.updateDescription.truncatedArrays.length == 0 &&
+        // ...And the actual event has no truncated arrays field.
+        event.hasOwnProperty("updateDescription") &&
+        !event.updateDescription.hasOwnProperty("truncatedArrays")) {
+        // Treat the actual event as if it had an empty 'truncatedArrays' field.
+        event.updateDescription.truncatedArrays = [];
+    }
+
     return event;
 }
 /**
@@ -79,7 +126,7 @@ function pruneOptionalFields(event, expected) {
  * resume token and clusterTime unless they are explicitly listed in the expectedEvent.
  */
 function assertChangeStreamEventEq(actualEvent, expectedEvent) {
-    const testEvent = pruneOptionalFields(Object.assign({}, actualEvent), expectedEvent);
+    const testEvent = canonicalizeEventForTesting(Object.assign({}, actualEvent), expectedEvent);
 
     assert.docEq(testEvent,
                  expectedEvent,
@@ -186,6 +233,11 @@ function ChangeStreamTest(_db, name = "ChangeStreamTest") {
 
             assert.soon(
                 () => {
+                    assert.neq(
+                        cursor.id,
+                        NumberLong(0),
+                        "Cursor has been closed unexpectedly. Observed change stream events: " +
+                            tojson(changes));
                     cursor = self.getNextBatch(cursor);
                     changes[i] = getNextDocFromCursor(cursor);
                     return changes[i] !== null;
@@ -435,27 +487,27 @@ function ChangeStreamTest(_db, name = "ChangeStreamTest") {
 }
 
 /**
- * Asserts that the given pipeline will eventually return an error with the provided code,
- * either in the initial aggregate, or a subsequent getMore. Throws an exception if there are
- * any results from running the pipeline, or if it doesn't throw an error within the window of
- * assert.soon().  If 'doNotModifyInPassthroughs' is 'true' and the test is running in a
- * $changeStream upconversion passthrough, then this stream will not be modified and will run as
- * though no passthrough were active.
+ * Asserts that the given pipeline will eventually return an error with the provided code, either in
+ * the initial aggregate, or a subsequent getMore. Throws an exception if there are any results from
+ * running the pipeline, or if it doesn't throw an error within the window of assert.soon(). If
+ * 'doNotModifyInPassthroughs' is 'true' and the test is running in a $changeStream upconversion
+ * passthrough, then this stream will not be modified and will run as though no passthrough were
+ * active.
  */
 ChangeStreamTest.assertChangeStreamThrowsCode = function assertChangeStreamThrowsCode(
     {db, collName, pipeline, expectedCode, doNotModifyInPassthroughs}) {
     try {
+        // Run a passthrough-aware initial 'aggregate' command to open the change stream.
         const res = assert.commandWorked(runCommandChangeStreamPassthroughAware(
             db,
             {aggregate: collName, pipeline: pipeline, cursor: {batchSize: 1}},
             doNotModifyInPassthroughs));
 
-        // Extract the collection name from the cursor since the change stream may be on the whole
-        // database. The 'collName' parameter will be the integer 1 in that case and the getMore
-        // command requires 'collection' to be a string.
-        const getMoreCollName = getCollectionNameFromFullNamespace(res.cursor.ns);
-        assert.commandWorked(
-            db.runCommand({getMore: res.cursor.id, collection: getMoreCollName, batchSize: 1}));
+        // Create a cursor using the command response, and begin issuing getMores. We expect
+        // csCursor.hasNext() to throw the expected code before assert.soon() times out.
+        const csCursor = new DBCommandCursor(db, res, 1);
+        assert.soon(() => csCursor.hasNext());
+        assert(false, `Unexpected result from cursor: ${tojson(csCursor.next())}`);
     } catch (error) {
         assert.eq(error.code, expectedCode, `Caught unexpected error: ${tojson(error)}`);
         return true;
@@ -497,3 +549,51 @@ function assertInvalidChangeStreamNss(dbName, collName = "test", options) {
         (res) => assert.commandFailedWithCode(
             res, [ErrorCodes.InvalidNamespace, ErrorCodes.InvalidOptions]));
 }
+
+/**
+ * Asserts that 'changeStreamPreAndPostImages' collection option is present and is enabled for
+ * collection.
+ */
+function assertChangeStreamPreAndPostImagesCollectionOptionIsEnabled(db, collName) {
+    const collectionInfos = db.getCollectionInfos({name: collName});
+    assert(collectionInfos[0].options["changeStreamPreAndPostImages"]["enabled"] === true);
+}
+
+/**
+ * Asserts that 'changeStreamPreAndPostImages' collection option is absent in the collection.
+ */
+function assertChangeStreamPreAndPostImagesCollectionOptionIsAbsent(db, collName) {
+    const collectionInfos = db.getCollectionInfos({name: collName});
+    assert(!collectionInfos[0].options.hasOwnProperty("changeStreamPreAndPostImages"));
+}
+
+function findPreImagesCollectionDescriptions(db) {
+    return db.getSiblingDB("config").runCommand("listCollections",
+                                                {filter: {name: "system.preimages"}});
+}
+
+/**
+ * Asserts that pre-images collection is absent in configDB.
+ */
+function assertPreImagesCollectionIsAbsent(db) {
+    const result = findPreImagesCollectionDescriptions(db);
+    assert.eq(result.cursor.firstBatch.length, 0);
+}
+
+/**
+ * Asserts that pre-images collection is created in the configDB and has clustered index on _id.
+ */
+function assertPreImagesCollectionExists(db) {
+    const collectionInfos = findPreImagesCollectionDescriptions(db);
+    assert.eq(collectionInfos.cursor.firstBatch.length, 1, collectionInfos);
+    const preImagesCollectionDescription = collectionInfos.cursor.firstBatch[0];
+    assert.eq(preImagesCollectionDescription.name, "system.preimages");
+
+    // Verifies that the pre-images collection is clustered by _id.
+    assert(preImagesCollectionDescription.hasOwnProperty("options"),
+           preImagesCollectionDescription);
+    assert(preImagesCollectionDescription.options.hasOwnProperty("clusteredIndex"),
+           preImagesCollectionDescription);
+    const clusteredIndexDescription = preImagesCollectionDescription.options.clusteredIndex;
+    assert(clusteredIndexDescription, preImagesCollectionDescription);
+}
