@@ -13,6 +13,7 @@ load("jstests/libs/transactions_util.js");
 // This scoping allows the original methods to be called by the overrides below.
 let originalRunCommand = Mongo.prototype.runCommand;
 let originalRunCommandWithMetadata = Mongo.prototype.runCommandWithMetadata;
+let originalMarkNodeAsFailed = Mongo.prototype._markNodeAsFailed;
 
 const denylistedDbNames = ["config", "admin", "local"];
 
@@ -606,6 +607,13 @@ Mongo.prototype.runCommand = function(dbName, cmdObj, options) {
         removeTenantId(resObj);
     }
 
+    Mongo.prototype._markNodeAsFailed = function(hostName, errorCode, errorReason) {
+        if (this.reroutingMongo)
+            originalMarkNodeAsFailed.apply(this.reroutingMongo, [hostName, errorCode, errorReason]);
+        else
+            originalMarkNodeAsFailed.apply(this, [hostName, errorCode, errorReason]);
+    };
+
     return resObj;
 };
 
