@@ -248,7 +248,11 @@ if (typeof _threadInject != "undefined") {
         };
 
         // Transactions are not supported on standalone nodes so we do not run them here.
-        let txnsTestFiles = getFilesRecursive("jstests/core/txns").map(f => ("txns/" + f.baseName));
+        // NOTE: We need to take substring of the full test path to ensure that 'jstests/core/' is
+        // not included.
+        const txnsTestFiles =
+            getFilesRecursive("jstests/core/txns/")
+                .map(fullPathToTest => fullPathToTest.name.substring("jstests/core/".length));
         Object.assign(skipTests, makeKeys(txnsTestFiles));
 
         var parallelFilesDir = "jstests/core";
