@@ -28,13 +28,13 @@ const defaultOptions = {
 
 // This lists all supported releases and needs to be kept up to date as versions are added and
 // dropped.
-// TODO SERVER-26792: In the future, we should have a common place from which both the
-// multiversion setup procedure and this test get information about supported major releases.
 const versions = [
     {binVersion: '4.0', featureCompatibilityVersion: '4.0', testCollection: 'four_zero'},
     {binVersion: '4.2', featureCompatibilityVersion: '4.2', testCollection: 'four_two'},
     {binVersion: '4.4', featureCompatibilityVersion: '4.4', testCollection: 'four_four'},
+    {binVersion: '5.0', featureCompatibilityVersion: '5.0', testCollection: 'five_zero'},
     {binVersion: 'last-lts', testCollection: 'last_lts'},
+    {binVersion: 'last-continuous', testCollection: 'last_continuous'},
     {binVersion: 'latest', featureCompatibilityVersion: latestFCV, testCollection: 'latest'},
 ];
 
@@ -46,7 +46,12 @@ for (let i = 0; i < versions.length; i++) {
     let mongodOptions = Object.extend({binVersion: version.binVersion}, defaultOptions);
 
     // Start a mongod with specified version.
-    let conn = MongoRunner.runMongod(mongodOptions);
+    let conn = null;
+    try {
+        conn = MongoRunner.runMongod(mongodOptions);
+    } catch (e) {
+        print(e);
+    }
 
     assert.neq(null, conn, 'mongod was unable to start up with options: ' + tojson(mongodOptions));
     assert.binVersion(conn, version.binVersion);
