@@ -47,7 +47,8 @@ const allCommands = {
     _configsvrRemoveShard: {skip: isPrimaryOnly},
     _configsvrRemoveShardFromZone: {skip: isPrimaryOnly},
     _configsvrRemoveTags: {skip: isPrimaryOnly},
-    _configsvrRenameCollectionMetadata: {skip: isAnInternalCommand},
+    _configsvrRepairShardedCollectionChunksHistory: {skip: isPrimaryOnly},
+    _configsvrRenameCollectionMetadata: {skip: isPrimaryOnly},
     _configsvrReshardCollection: {skip: isPrimaryOnly},
     _configsvrSetAllowMigrations: {skip: isPrimaryOnly},
     _configsvrUpdateZoneKeyRange: {skip: isPrimaryOnly},
@@ -276,6 +277,7 @@ const allCommands = {
     reIndex: {skip: isNotAUserDataRead},
     renameCollection: {skip: isPrimaryOnly},
     repairDatabase: {skip: isNotAUserDataRead},
+    repairShardedCollectionChunksHistory: {skip: isPrimaryOnly},
     replSetAbortPrimaryCatchUp: {skip: isNotAUserDataRead},
     replSetFreeze: {skip: isNotAUserDataRead},
     replSetGetConfig: {skip: isNotAUserDataRead},
