@@ -3,7 +3,9 @@
 // run a command on a latest version shard that no longer supports that command. To increase test
 // coverage and allow us to run on same- and mixed-version suites, we allow these commands to have a
 // test defined without always existing on the servers being used.
-const commandsRemovedFromMongosSinceLastLTS = [];
+const commandsRemovedFromMongosSinceLastLTS = [
+    "repairShardedCollectionChunksHistory",
+];
 // These commands were added in mongos since the last LTS version, so will not appear in the
 // listCommands output of a last LTS version mongos. We will allow these commands to have a test
 // defined without always existing on the mongos being used.
@@ -19,5 +21,5 @@ const commandsAddedToMongosSinceLastLTS = [
     "testDeprecationInVersion2",
     "testRemoval",
     "testVersions1And2",
-    "testVersion2"
+    "testVersion2",
 ];
