@@ -412,7 +412,7 @@ public:
     long long exactNumberLong() const;
 
     /**
-     * Parses a BSONElement of any numeric type into a positive long long, failing if the value
+     * Parses a BSONElement of any numeric type into a non-negative long long, failing if the value
      * is any of the following:
      *
      * - NaN.
@@ -432,6 +432,17 @@ public:
      */
     StatusWith<long long> parseIntegerElementToLong() const;
 
+    /**
+     * Parses a BSONElement of any numeric type into a non-negative int, failing if the value
+     * is any of the following:
+     *
+     * - NaN
+     * - Negative
+     * - a non-integral number
+     * - too large in the positive or negative direction to fit in an int
+     */
+    StatusWith<int> parseIntegerElementToNonNegativeInt() const;
+
     /**
      * Parses a BSONElement of any numeric type into an integer, failing if the value is:
      *
@@ -1045,7 +1056,11 @@ Status BSONElement::tryCoerce(T* out) const {
             if (!std::isfinite(d)) {
                 return {ErrorCodes::BadValue, "Unable to coerce NaN/Inf to integral type"};
             }
-            if ((d > std::numeric_limits<T>::max()) || (d < std::numeric_limits<T>::lowest())) {
+            constexpr bool sameMax =
+                std::numeric_limits<T>::max() == std::numeric_limits<long long>::max();
+            if ((!sameMax && d > static_cast<double>(std::numeric_limits<T>::max())) ||
+                (sameMax && d >= static_cast<double>(std::numeric_limits<T>::max())) ||
+                (d < std::numeric_limits<T>::lowest())) {
                 return {ErrorCodes::BadValue, "Out of bounds coercing to integral value"};
             }
             val = static_cast<long long>(d);
