@@ -28,12 +28,14 @@
  */
 
 #include "mongo/bson/util/bsoncolumn.h"
+
+#include <algorithm>
+#include <third_party/murmurhash3/MurmurHash3.h>
+
 #include "mongo/bson/bsonobj.h"
 #include "mongo/bson/util/bsoncolumn_util.h"
 #include "mongo/bson/util/simple8b_type_util.h"
 
-#include <algorithm>
-
 namespace mongo {
 using namespace bsoncolumn;
 
@@ -102,6 +104,13 @@ private:
     ElementFunc _elemFunc;
 };
 
+std::size_t hashName(StringData sd) {
+    // Keep in sync with DocumentStorageHasher
+    unsigned out;
+    MurmurHash3_x86_32(sd.rawData(), sd.size(), 0, &out);
+    return out;
+}
+
 }  // namespace
 
 BSONColumn::ElementStorage::Element::Element(char* buffer, int nameSize, int valueSize)
@@ -764,6 +773,7 @@ BSONColumn::BSONColumn(BSONElement bin) {
 
     _binary = bin.binData(_size);
     _name = bin.fieldNameStringData().toString();
+    _nameHash = hashName(_name);
     _init();
 }
 
@@ -772,6 +782,7 @@ BSONColumn::BSONColumn(BSONBinData bin, StringData name) {
     _binary = static_cast<const char*>(bin.data);
     _size = bin.length;
     _name = name.toString();
+    _nameHash = hashName(_name);
     _init();
 }
 
