@@ -29,11 +29,11 @@
 
 #pragma once
 
+#include <list>
 #include <map>
 
 #include "mongo/client/dbclient_connection.h"
 #include "mongo/platform/mutex.h"
-#include "mongo/stdx/list.h"
 #include "mongo/stdx/unordered_map.h"
 #include "mongo/util/net/hostandport.h"
 #include "mongo/util/time_support.h"
