@@ -60,12 +60,12 @@ AuthzManagerExternalStateMongod::makeAuthzSessionExternalState(AuthorizationMana
 Status AuthzManagerExternalStateMongod::query(
     OperationContext* opCtx,
     const NamespaceString& collectionName,
-    const BSONObj& query,
+    const BSONObj& filter,
     const BSONObj& projection,
     const std::function<void(const BSONObj&)>& resultProcessor) {
     try {
         DBDirectClient client(opCtx);
-        client.query(resultProcessor, collectionName, query, &projection);
+        client.query(resultProcessor, collectionName, filter, Query(), &projection);
         return Status::OK();
     } catch (const DBException& e) {
         return e.toStatus();
@@ -76,7 +76,7 @@ Status AuthzManagerExternalStateMongod::findOne(OperationContext* opCtx,
                                                 const NamespaceString& collectionName,
                                                 const BSONObj& query,
                                                 BSONObj* result) {
-    AutoGetCollectionForReadCommand ctx(opCtx, collectionName);
+    AutoGetCollectionForReadCommandMaybeLockFree ctx(opCtx, collectionName);
 
     BSONObj found;
     if (Helpers::findOne(opCtx, ctx.getCollection(), query, found)) {
@@ -88,6 +88,13 @@ Status AuthzManagerExternalStateMongod::findOne(OperationContext* opCtx,
                                 << query);
 }
 
+bool AuthzManagerExternalStateMongod::hasOne(OperationContext* opCtx,
+                                             const NamespaceString& collectionName,
+                                             const BSONObj& query) {
+    AutoGetCollectionForReadCommandMaybeLockFree ctx(opCtx, collectionName);
+    return !Helpers::findOne(opCtx, ctx.getCollection(), query, false).isNull();
+}
+
 namespace {
 
 std::unique_ptr<AuthzManagerExternalState> authzManagerExternalStateCreateImpl() {
