@@ -30,6 +30,8 @@
 #include "mongo/base/status.h"
 #include "mongo/base/status_with.h"
 #include "mongo/db/catalog/collection_options.h"
+#include "mongo/db/coll_mod_gen.h"
+#include "mongo/db/repl/oplog.h"
 
 namespace mongo {
 class BSONObj;
@@ -46,11 +48,20 @@ class OperationContext;
 void addCollectionUUIDs(OperationContext* opCtx);
 
 /**
- * Performs the collection modification described in "cmdObj" on the collection "ns".
+ * Performs the collection modification described in "cmd" on the collection "ns".
  */
-Status collMod(OperationContext* opCtx,
-               const NamespaceString& ns,
-               const BSONObj& cmdObj,
-               BSONObjBuilder* result);
+Status processCollModCommand(OperationContext* opCtx,
+                             const NamespaceStringOrUUID& nsOrUUID,
+                             const CollMod& cmd,
+                             BSONObjBuilder* result);
+
+/**
+ * Performs the collection modification described in "cmd" on the collection "ns". Only checks for
+ * duplicates for the 'applyOps' command.
+ */
+Status processCollModCommandForApplyOps(OperationContext* opCtx,
+                                        const NamespaceStringOrUUID& nsOrUUID,
+                                        const CollMod& cmd,
+                                        repl::OplogApplication::Mode mode);
 
 }  // namespace mongo
