@@ -34,23 +34,17 @@
 #include <boost/optional.hpp>
 
 #include "mongo/base/status.h"
+#include "mongo/db/catalog/clustered_collection_options_gen.h"
+#include "mongo/db/catalog/collection_options_gen.h"
 #include "mongo/db/jsobj.h"
+#include "mongo/db/pipeline/change_stream_pre_and_post_images_options_gen.h"
+#include "mongo/db/timeseries/timeseries_gen.h"
 #include "mongo/util/uuid.h"
 
 namespace mongo {
 
 class CollatorFactoryInterface;
-
-/**
- * A CollectionUUID is a 128-bit unique identifier, per RFC 4122, v4. for a database collection.
- * Newly created collections are assigned a new randomly generated CollectionUUID. In a replica-set
- * or a sharded cluster, all nodes will use the same UUID for a given collection. The UUID stays
- * with the collection until it is dropped, so even across renames. A copied collection must have
- * its own new unique UUID though.
- */
-using CollectionUUID = UUID;
-
-using OptionalCollectionUUID = boost::optional<CollectionUUID>;
+class CreateCommand;
 
 struct CollectionOptions {
     /**
@@ -82,14 +76,20 @@ struct CollectionOptions {
     static StatusWith<CollectionOptions> parse(const BSONObj& options,
                                                ParseKind kind = parseForCommand);
 
-    void appendBSON(BSONObjBuilder* builder) const;
-    BSONObj toBSON() const;
+    /**
+     * Converts a client "create" command invocation.
+     */
+    static CollectionOptions fromCreateCommand(const CreateCommand& cmd);
 
     /**
-     * @param max in and out, will be adjusted
-     * @return if the value is valid at all
+     * Serialize to BSON. The 'includeUUID' parameter is used for the listCollections command to do
+     * special formatting for the uuid. Aside from the UUID, if 'includeFields' is non-empty, only
+     * the specified fields will be included.
      */
-    static bool validMaxCappedDocs(long long* max);
+    void appendBSON(BSONObjBuilder* builder,
+                    bool includeUUID,
+                    const StringDataSet& includeFields) const;
+    BSONObj toBSON(bool includeUUID = true, const StringDataSet& includeFields = {}) const;
 
     /**
      * Returns true if given options matches to this.
@@ -101,8 +101,9 @@ struct CollectionOptions {
     bool matchesStorageOptions(const CollectionOptions& other,
                                CollatorFactoryInterface* collatorFactory) const;
 
-    // Collection UUID. Present for all CollectionOptions parsed for storage.
-    OptionalCollectionUUID uuid;
+    // Collection UUID. If not set, specifies that the storage engine should generate the UUID (for
+    // a new collection). For an existing collection parsed for storage, it will always be present.
+    boost::optional<UUID> uuid;
 
     bool capped = false;
     long long cappedSize = 0;
@@ -121,28 +122,44 @@ struct CollectionOptions {
     bool temp = false;
     bool recordPreImages = false;
 
+    // Change stream options define whether or not to store the pre-images of the documents affected
+    // by update and delete operations in a dedicated collection, that will be used for reading data
+    // via changeStreams. Can not be enabled together with 'recordPreImages' (mutually exclusive).
+    ChangeStreamPreAndPostImagesOptions changeStreamPreAndPostImagesOptions{false};
+
     // Storage engine collection options. Always owned or empty.
     BSONObj storageEngine;
 
-    // Default options for indexes created on the collection. Always owned or empty.
-    BSONObj indexOptionDefaults;
+    // Default options for indexes created on the collection.
+    IndexOptionDefaults indexOptionDefaults;
 
     // Index specs for the _id index.
     BSONObj idIndex;
 
     // Always owned or empty.
     BSONObj validator;
-    std::string validationAction;
-    std::string validationLevel;
+    boost::optional<ValidationActionEnum> validationAction;
+    boost::optional<ValidationLevelEnum> validationLevel;
 
     // The namespace's default collation.
     BSONObj collation;
 
+    // Exists if the collection is clustered.
+    boost::optional<ClusteredCollectionInfo> clusteredIndex;
+
+    // If present, the number of seconds after which old data should be deleted. Only for
+    // collections which are clustered on _id.
+    boost::optional<int64_t> expireAfterSeconds;
+
     // View-related options.
     // The namespace of the view or collection that "backs" this view, or the empty string if this
     // collection is not a view.
     std::string viewOn;
     // The aggregation pipeline that defines this view.
     BSONObj pipeline;
+
+    // The options that define the time-series collection, or boost::none if not a time-series
+    // collection.
+    boost::optional<TimeseriesOptions> timeseries;
 };
 }  // namespace mongo
