@@ -37,7 +37,7 @@
 #include "mongo/db/catalog/collection_catalog.h"
 #include "mongo/db/catalog/collection_impl.h"
 #include "mongo/db/catalog/database_impl.h"
-#include "mongo/db/concurrency/exception_util.h"
+#include "mongo/db/concurrency/write_conflict_exception.h"
 #include "mongo/db/index_builds_coordinator.h"
 #include "mongo/db/op_observer.h"
 #include "mongo/db/operation_context.h"
@@ -83,6 +83,11 @@ Database* DatabaseHolderImpl::getDb(OperationContext* opCtx, StringData ns) cons
     return nullptr;
 }
 
+bool DatabaseHolderImpl::dbExists(OperationContext* opCtx, StringData ns) const {
+    stdx::lock_guard<SimpleMutex> lk(_m);
+    return _dbs.find(_todb(ns)) != _dbs.end();
+}
+
 std::shared_ptr<const ViewCatalog> DatabaseHolderImpl::getViewCatalog(OperationContext* opCtx,
                                                                       StringData dbName) const {
     stdx::lock_guard<SimpleMutex> lk(_m);
@@ -137,7 +142,7 @@ Database* DatabaseHolderImpl::openDb(OperationContext* opCtx, StringData ns, boo
         return db;
 
     // We've inserted a nullptr entry for dbname: make sure to remove it on unsuccessful exit.
-    auto removeDbGuard = makeGuard([this, &lk, dbname] {
+    ScopeGuard removeDbGuard([this, &lk, dbname] {
         if (!lk.owns_lock())
             lk.lock();
         _dbs.erase(dbname);
