@@ -137,6 +137,7 @@ public:
      * If no such index exists, returns NULL.
      */
     const IndexDescriptor* findShardKeyPrefixedIndex(OperationContext* opCtx,
+                                                     const CollectionPtr& collection,
                                                      const BSONObj& shardKey,
                                                      bool requireSingleKey) const override;
 
@@ -160,7 +161,8 @@ public:
      */
     const IndexDescriptor* refreshEntry(OperationContext* opCtx,
                                         Collection* collection,
-                                        const IndexDescriptor* oldDesc) override;
+                                        const IndexDescriptor* oldDesc,
+                                        CreateIndexEntryFlags flags) override;
 
     const IndexCatalogEntry* getEntry(const IndexDescriptor* desc) const override;
 
@@ -169,8 +171,8 @@ public:
     std::vector<std::shared_ptr<const IndexCatalogEntry>> getAllReadyEntriesShared() const override;
 
     using IndexIterator = IndexCatalog::IndexIterator;
-    std::unique_ptr<IndexIterator> getIndexIterator(
-        OperationContext* const opCtx, const bool includeUnfinishedIndexes) const override;
+    std::unique_ptr<IndexIterator> getIndexIterator(OperationContext* opCtx,
+                                                    bool includeUnfinishedIndexes) const override;
 
     // ---- index set modifiers ------
 
@@ -194,13 +196,13 @@ public:
         const BSONObj& original,
         const boost::optional<ResumeIndexInfo>& resumeInfo = boost::none) const override;
 
-    std::vector<BSONObj> removeExistingIndexes(OperationContext* const opCtx,
+    std::vector<BSONObj> removeExistingIndexes(OperationContext* opCtx,
                                                const CollectionPtr& collection,
                                                const std::vector<BSONObj>& indexSpecsToBuild,
-                                               const bool removeIndexBuildsToo) const override;
+                                               bool removeIndexBuildsToo) const override;
 
     std::vector<BSONObj> removeExistingIndexesNoChecks(
-        OperationContext* const opCtx,
+        OperationContext* opCtx,
         const CollectionPtr& collection,
         const std::vector<BSONObj>& indexSpecsToBuild) const override;
 
@@ -221,9 +223,9 @@ public:
     Status dropIndex(OperationContext* opCtx,
                      Collection* collection,
                      const IndexDescriptor* desc) override;
-    Status dropUnfinishedIndex(OperationContext* const opCtx,
+    Status dropUnfinishedIndex(OperationContext* opCtx,
                                Collection* collection,
-                               const IndexDescriptor* const desc) override;
+                               const IndexDescriptor* desc) override;
 
     Status dropIndexEntry(OperationContext* opCtx,
                           Collection* collection,
@@ -242,7 +244,7 @@ public:
 
     // ---- modify single index
 
-    void setMultikeyPaths(OperationContext* const opCtx,
+    void setMultikeyPaths(OperationContext* opCtx,
                           const CollectionPtr& coll,
                           const IndexDescriptor* desc,
                           const KeyStringSet& multikeyMetadataKeys,
@@ -264,13 +266,13 @@ public:
     /**
      * See IndexCatalog::updateRecord
      */
-    Status updateRecord(OperationContext* const opCtx,
+    Status updateRecord(OperationContext* opCtx,
                         const CollectionPtr& coll,
                         const BSONObj& oldDoc,
                         const BSONObj& newDoc,
                         const RecordId& recordId,
-                        int64_t* const keysInsertedOut,
-                        int64_t* const keysDeletedOut) const override;
+                        int64_t* keysInsertedOut,
+                        int64_t* keysDeletedOut) const override;
     /**
      * When 'keysDeletedOut' is not null, it will be set to the number of index keys removed by
      * this operation.
@@ -307,6 +309,13 @@ public:
                            Collection* coll,
                            IndexCatalogEntry* index) override;
 
+    /**
+     * Returns a status indicating whether 'expression' is valid for use in a partial index
+     * partialFilterExpression.
+     */
+    static Status checkValidFilterExpressions(const MatchExpression* expression,
+                                              bool timeseriesMetricIndexesFeatureFlagEnabled);
+
 private:
     static const BSONObj _idObj;  // { _id : 1 }
 
@@ -341,14 +350,14 @@ private:
                          const std::vector<BsonRecord>& bsonRecords,
                          int64_t* keysInsertedOut) const;
 
-    Status _updateRecord(OperationContext* const opCtx,
+    Status _updateRecord(OperationContext* opCtx,
                          const CollectionPtr& coll,
                          const IndexCatalogEntry* index,
                          const BSONObj& oldDoc,
                          const BSONObj& newDoc,
                          const RecordId& recordId,
-                         int64_t* const keysInsertedOut,
-                         int64_t* const keysDeletedOut) const;
+                         int64_t* keysInsertedOut,
+                         int64_t* keysDeletedOut) const;
 
     void _unindexKeys(OperationContext* opCtx,
                       const CollectionPtr& collection,
@@ -357,7 +366,7 @@ private:
                       const BSONObj& obj,
                       RecordId loc,
                       bool logIfError,
-                      int64_t* const keysDeletedOut) const;
+                      int64_t* keysDeletedOut) const;
 
     void _unindexRecord(OperationContext* opCtx,
                         const CollectionPtr& collection,
@@ -410,7 +419,7 @@ private:
     Status _doesSpecConflictWithExisting(OperationContext* opCtx,
                                          const CollectionPtr& collection,
                                          const BSONObj& spec,
-                                         const bool includeUnfinishedIndexes) const;
+                                         bool includeUnfinishedIndexes) const;
 
     /**
      * Returns true if the replica set member's config has {buildIndexes:false} set, which means
