@@ -289,12 +289,6 @@ public:
                 (cmd.getChangeStreamPreAndPostImages() &&
                  cmd.getChangeStreamPreAndPostImages()->getEnabled());
 
-            // Acquire shared lock on FCV if 'changeStreamPreAndPostImages' is enabled.
-            boost::optional<FixedFCVRegion> fcvRegion;
-            if (isChangeStreamPreAndPostImagesEnabled) {
-                fcvRegion.emplace(opCtx);
-            }
-
             if (feature_flags::gFeatureFlagChangeStreamPreAndPostImages.isEnabled(
                     serverGlobalParams.featureCompatibility)) {
                 const auto isRecordPreImagesEnabled = cmd.getRecordPreImages().get_value_or(false);
@@ -303,7 +297,7 @@ public:
                         "set to true simultaneously",
                         !(isChangeStreamPreAndPostImagesEnabled && isRecordPreImagesEnabled));
             } else {
-                uassert(5846900,
+                uassert(ErrorCodes::InvalidOptions,
                         "BSON field 'changeStreamPreAndPostImages' is an unknown field.",
                         !cmd.getChangeStreamPreAndPostImages().has_value());
             }
