@@ -538,12 +538,6 @@ public:
             (cmd->getChangeStreamPreAndPostImages() &&
              cmd->getChangeStreamPreAndPostImages()->getEnabled());
 
-        // Acquire shared lock on FCV if 'changeStreamPreAndPostImages' is enabled.
-        boost::optional<FixedFCVRegion> fcvRegion;
-        if (isChangeStreamPreAndPostImagesEnabled) {
-            fcvRegion.emplace(opCtx);
-        }
-
         if (feature_flags::gFeatureFlagChangeStreamPreAndPostImages.isEnabled(
                 serverGlobalParams.featureCompatibility)) {
             const auto isRecordPreImagesEnabled = cmd->getRecordPreImages().get_value_or(false);
@@ -552,7 +546,7 @@ public:
                     "to true simultaneously",
                     !(isChangeStreamPreAndPostImagesEnabled && isRecordPreImagesEnabled));
         } else {
-            uassert(5846901,
+            uassert(ErrorCodes::InvalidOptions,
                     "BSON field 'changeStreamPreAndPostImages' is an unknown field.",
                     !cmd->getChangeStreamPreAndPostImages().has_value());
         }
@@ -632,6 +626,7 @@ public:
             errmsg = "scale has to be a number > 0";
             return false;
         }
+        bool includeFreeStorage = jsobj["freeStorage"].trueValue();
 
         const std::string ns = parseNs(dbname, jsobj);
         uassert(ErrorCodes::InvalidNamespace,
@@ -679,7 +674,7 @@ public:
                     dbname.c_str(), CollectionCatalog::get(opCtx)->getDatabaseProfileLevel(dbname));
             }
 
-            db->getStats(opCtx, &result, scale);
+            db->getStats(opCtx, &result, includeFreeStorage, scale);
         }
 
         return true;
