@@ -86,17 +86,12 @@
 namespace mongo {
 
 using boost::intrusive_ptr;
-using std::endl;
 using std::shared_ptr;
 using std::string;
 using std::stringstream;
 using std::unique_ptr;
 
 namespace {
-Counter64 allowDiskUseCounter;
-ServerStatusMetricField<Counter64> allowDiskUseMetric{"commands.aggregate.allowDiskUseTrue",
-                                                      &allowDiskUseCounter};
-
 /**
  * If a pipeline is empty (assuming that a $cursor stage hasn't been created yet), it could mean
  * that we were able to absorb all pipeline stages and pull them into a single PlanExecutor. So,
@@ -199,8 +194,7 @@ bool handleCursorCommand(OperationContext* opCtx,
             // to the resume token of the invalidating event, and mark the cursor response as
             // invalidated. We expect ExtraInfo to always be present for this exception.
             const auto extraInfo = ex.extraInfo<ChangeStreamInvalidationInfo>();
-            tassert(
-                5493701, "Missing ChangeStreamInvalidationInfo on exception", extraInfo != nullptr);
+            tassert(5493701, "Missing ChangeStreamInvalidationInfo on exception", extraInfo);
 
             responseBuilder.setPostBatchResumeToken(extraInfo->getInvalidateResumeToken());
             responseBuilder.setInvalidated();
@@ -318,7 +312,7 @@ StatusWith<StringMap<ExpressionContext::ResolvedNamespace>> resolveInvolvedNames
         // If 'ns' refers to a view namespace, then we resolve its definition.
         auto resolveViewDefinition = [&](const NamespaceString& ns,
                                          std::shared_ptr<const ViewCatalog> vcp) -> Status {
-            auto resolvedView = vcp->resolveView(opCtx, ns);
+            auto resolvedView = vcp->resolveView(opCtx, ns, boost::none);
             if (!resolvedView.isOK()) {
                 return resolvedView.getStatus().withContext(
                     str::stream() << "Failed to resolve view '" << involvedNs.ns());
@@ -367,7 +361,7 @@ StatusWith<StringMap<ExpressionContext::ResolvedNamespace>> resolveInvolvedNames
                 // that the inverse scenario (mistaking a view for a collection) is not an issue
                 // because $merge/$out cannot target a view.
                 auto nssToCheck = NamespaceString(request.getNamespace().db(), involvedNs.coll());
-                if (viewCatalog && viewCatalog->lookup(opCtx, nssToCheck.ns())) {
+                if (viewCatalog && viewCatalog->lookup(opCtx, nssToCheck)) {
                     auto status = resolveViewDefinition(nssToCheck, viewCatalog);
                     if (!status.isOK()) {
                         return status;
@@ -384,7 +378,7 @@ StatusWith<StringMap<ExpressionContext::ResolvedNamespace>> resolveInvolvedNames
             // pipeline because 'involvedNs' doesn't refer to a view namespace in our consistent
             // snapshot of the view catalog.
             resolvedNamespaces[involvedNs.coll()] = {involvedNs, std::vector<BSONObj>{}};
-        } else if (viewCatalog->lookup(opCtx, involvedNs.ns())) {
+        } else if (viewCatalog->lookup(opCtx, involvedNs)) {
             auto status = resolveViewDefinition(involvedNs, viewCatalog);
             if (!status.isOK()) {
                 return status;
@@ -417,42 +411,25 @@ Status collatorCompatibleWithPipeline(OperationContext* opCtx,
             continue;
         }
 
-        auto view = viewCatalog->lookup(opCtx, potentialViewNs.ns());
+        auto view = viewCatalog->lookup(opCtx, potentialViewNs);
         if (!view) {
             continue;
         }
         if (!CollatorInterface::collatorsMatch(view->defaultCollator(), collator)) {
             return {ErrorCodes::OptionNotSupportedOnView,
-                    str::stream() << "Cannot override default collation of view "
+                    str::stream() << "Cannot override a view's default collation"
                                   << potentialViewNs.ns()};
         }
     }
     return Status::OK();
 }
 
-// A 4.7+ mongoS issues $mergeCursors pipelines with ChunkVersion::IGNORED. On the shard, this will
-// skip the versioning check but also marks the operation as versioned, so the shard knows that any
-// sub-operations executed by the merging pipeline should also be versioned. We manually set the
-// IGNORED version here if we are running a $mergeCursors pipeline and the operation is not already
-// versioned. This can happen in the case where we are running in a cluster with a 4.4 mongoS, which
-// does not set any shard version on a $mergeCursors pipeline.
-void setIgnoredShardVersionForMergeCursors(OperationContext* opCtx,
-                                           const AggregateCommandRequest& request) {
-    auto isMergeCursors = request.getFromMongos() && request.getPipeline().size() > 0 &&
-        request.getPipeline().front().firstElementFieldNameStringData() == "$mergeCursors"_sd;
-    if (isMergeCursors && !OperationShardingState::isOperationVersioned(opCtx)) {
-        OperationShardingState::get(opCtx).initializeClientRoutingVersions(
-            request.getNamespace(), ChunkVersion::IGNORED(), boost::none);
-    }
-}
-
 boost::intrusive_ptr<ExpressionContext> makeExpressionContext(
     OperationContext* opCtx,
     const AggregateCommandRequest& request,
     std::unique_ptr<CollatorInterface> collator,
     boost::optional<UUID> uuid,
     ExpressionContext::CollationMatchesDefault collationMatchesDefault) {
-    setIgnoredShardVersionForMergeCursors(opCtx, request);
     boost::intrusive_ptr<ExpressionContext> expCtx =
         new ExpressionContext(opCtx,
                               request,
@@ -462,8 +439,8 @@ boost::intrusive_ptr<ExpressionContext> makeExpressionContext(
                               uuid,
                               CurOp::get(opCtx)->dbProfileLevel() > 0);
     expCtx->tempDir = storageGlobalParams.dbpath + "/_tmp";
+    expCtx->inMultiDocumentTransaction = opCtx->inMultiDocumentTransaction();
     expCtx->collationMatchesDefault = collationMatchesDefault;
-    expCtx->forPerShardCursor = request.getPassthroughToShard().has_value();
 
     return expCtx;
 }
@@ -576,7 +553,6 @@ Status runAggregate(OperationContext* opCtx,
                     const BSONObj& cmdObj,
                     const PrivilegeVector& privileges,
                     rpc::ReplyBuilderInterface* result) {
-
     // Perform some validations on the LiteParsedPipeline and request before continuing with the
     // aggregation command.
     performValidationChecks(opCtx, request, liteParsedPipeline);
@@ -601,10 +577,11 @@ Status runAggregate(OperationContext* opCtx,
     boost::intrusive_ptr<ExpressionContext> expCtx;
     auto curOp = CurOp::get(opCtx);
     {
-        // If we are in a transaction, check whether the parsed pipeline supports
-        // being in a transaction.
+        // If we are in a transaction, check whether the parsed pipeline supports being in
+        // a transaction and if the transaction's read concern is supported.
         if (opCtx->inMultiDocumentTransaction()) {
             liteParsedPipeline.assertSupportsMultiDocumentTransaction(request.getExplain());
+            liteParsedPipeline.assertSupportsReadConcern(opCtx, request.getExplain());
         }
 
         const auto& pipelineInvolvedNamespaces = liteParsedPipeline.getInvolvedNamespaces();
@@ -631,7 +608,7 @@ Status runAggregate(OperationContext* opCtx,
             if (!origNss.isCollectionlessAggregateNS()) {
                 auto viewCatalog = DatabaseHolder::get(opCtx)->getViewCatalog(opCtx, origNss.db());
                 if (viewCatalog) {
-                    auto view = viewCatalog->lookup(opCtx, origNss.ns());
+                    auto view = viewCatalog->lookup(opCtx, origNss);
                     uassert(ErrorCodes::CommandNotSupportedOnView,
                             str::stream()
                                 << "Namespace " << origNss.ns() << " is a timeseries collection",
@@ -705,16 +682,23 @@ Status runAggregate(OperationContext* opCtx,
             if (!request.getCollation().get_value_or(BSONObj()).isEmpty()) {
                 invariant(collatorToUse);  // Should already be resolved at this point.
                 if (!CollatorInterface::collatorsMatch(ctx->getView()->defaultCollator(),
-                                                       collatorToUse->get())) {
+                                                       collatorToUse->get()) &&
+                    !ctx->getView()->timeseries()) {
+
                     return {ErrorCodes::OptionNotSupportedOnView,
                             "Cannot override a view's default collation"};
                 }
             }
 
-
+            // Queries on timeseries views may specify non-default collation whereas queries
+            // on all other types of views must match the default collator (the collation use
+            // to originally create that collections). Thus in the case of operations on TS
+            // views, we use the request's collation.
+            auto timeSeriesCollator =
+                ctx->getView()->timeseries() ? request.getCollation() : boost::none;
             auto resolvedView = uassertStatusOK(DatabaseHolder::get(opCtx)
                                                     ->getViewCatalog(opCtx, nss.db())
-                                                    ->resolveView(opCtx, nss));
+                                                    ->resolveView(opCtx, nss, timeSeriesCollator));
 
             // With the view & collation resolved, we can relinquish locks.
             ctx.reset();
@@ -774,10 +758,6 @@ Status runAggregate(OperationContext* opCtx,
         auto pipeline = Pipeline::parse(request.getPipeline(), expCtx);
         expCtx->stopExpressionCounters();
 
-        if (request.getAllowDiskUse()) {
-            allowDiskUseCounter.increment();
-        }
-
         // Check that the view's collation matches the collation of any views involved in the
         // pipeline.
         if (!pipelineInvolvedNamespaces.empty()) {
@@ -790,6 +770,9 @@ Status runAggregate(OperationContext* opCtx,
 
         pipeline->optimizePipeline();
 
+        constexpr bool alreadyOptimized = true;
+        pipeline->validateCommon(alreadyOptimized);
+
         // Check if the pipeline has a $geoNear stage, as it will be ripped away during the build
         // query executor phase below (to be replaced with a $geoNearCursorStage later during the
         // executor attach phase).
@@ -797,20 +780,8 @@ Status runAggregate(OperationContext* opCtx,
             dynamic_cast<DocumentSourceGeoNear*>(pipeline->peekFront());
 
         // Prepare a PlanExecutor to provide input into the pipeline, if needed.
-        std::pair<PipelineD::AttachExecutorCallback,
-                  std::unique_ptr<PlanExecutor, PlanExecutor::Deleter>>
-            attachExecutorCallback;
-        if (liteParsedPipeline.hasChangeStream()) {
-            // If we are using a change stream, the cursor stage should have a simple collation,
-            // regardless of what the user's collation was.
-            std::unique_ptr<CollatorInterface> collatorForCursor = nullptr;
-            auto collatorStash = expCtx->temporarilyChangeCollator(std::move(collatorForCursor));
-            attachExecutorCallback =
-                PipelineD::buildInnerQueryExecutor(collection, nss, &request, pipeline.get());
-        } else {
-            attachExecutorCallback =
-                PipelineD::buildInnerQueryExecutor(collection, nss, &request, pipeline.get());
-        }
+        auto attachExecutorCallback =
+            PipelineD::buildInnerQueryExecutor(collection, nss, &request, pipeline.get());
 
         if (canOptimizeAwayPipeline(pipeline.get(),
                                     attachExecutorCallback.second.get(),
@@ -823,7 +794,11 @@ Status runAggregate(OperationContext* opCtx,
             // PlanExecutor by itself. The resulting cursor will look like what the client would
             // have gotten from find command.
             execs.emplace_back(std::move(attachExecutorCallback.second));
+            // Mark that this query does not use DocumentSource.
+            curOp->debug().documentSourceUsed = false;
         } else {
+            // Mark that this query uses DocumentSource.
+            curOp->debug().documentSourceUsed = true;
             // Complete creation of the initial $cursor stage, if needed.
             PipelineD::attachInnerQueryExecutorToPipeline(collection,
                                                           attachExecutorCallback.first,
@@ -868,7 +843,7 @@ Status runAggregate(OperationContext* opCtx,
     std::vector<ClientCursorPin> pins;
     std::vector<ClientCursor*> cursors;
 
-    auto cursorFreer = makeGuard([&] {
+    ScopeGuard cursorFreer([&] {
         for (auto& p : pins) {
             p.deleteUnderlying();
         }
@@ -881,15 +856,10 @@ Status runAggregate(OperationContext* opCtx,
             APIParameters::get(opCtx),
             opCtx->getWriteConcern(),
             repl::ReadConcernArgs::get(opCtx),
+            ReadPreferenceSetting::get(opCtx),
             cmdObj,
-            privileges,
-            false /*isOpQueryExhaust*/);
-        if (expCtx->tailableMode == TailableModeEnum::kTailable) {
-            cursorParams.setTailable(true);
-        } else if (expCtx->tailableMode == TailableModeEnum::kTailableAndAwaitData) {
-            cursorParams.setTailable(true);
-            cursorParams.setAwaitData(true);
-        }
+            privileges);
+        cursorParams.setTailableMode(expCtx->tailableMode);
 
         auto pin = CursorManager::get(opCtx)->registerCursor(opCtx, std::move(cursorParams));
 
