@@ -49,18 +49,6 @@
 
 namespace mongo {
 
-Lock::TempRelease::TempRelease(Locker* lockState)
-    : _lockState(lockState),
-      _lockSnapshot(),
-      _locksReleased(_lockState->saveLockStateAndUnlock(&_lockSnapshot)) {}
-
-Lock::TempRelease::~TempRelease() {
-    if (_locksReleased) {
-        invariant(!_lockState->isLocked());
-        _lockState->restoreLockState(_lockSnapshot);
-    }
-}
-
 namespace {
 
 /**
@@ -154,7 +142,7 @@ Lock::GlobalLock::GlobalLock(OperationContext* opCtx,
         if (_opCtx->lockState()->shouldConflictWithSecondaryBatchApplication()) {
             _pbwm.lock(opCtx, MODE_IS, deadline);
         }
-        auto unlockPBWM = makeGuard([this] {
+        ScopeGuard unlockPBWM([this] {
             if (_opCtx->lockState()->shouldConflictWithSecondaryBatchApplication()) {
                 _pbwm.unlock();
             }
@@ -184,7 +172,7 @@ void Lock::GlobalLock::_takeGlobalLockOnly(LockMode lockMode, Date_t deadline) {
 
 void Lock::GlobalLock::_takeGlobalAndRSTLLocks(LockMode lockMode, Date_t deadline) {
     _opCtx->lockState()->lock(_opCtx, resourceIdReplicationStateTransitionLock, MODE_IX, deadline);
-    auto unlockRSTL = makeGuard(
+    ScopeGuard unlockRSTL(
         [this] { _opCtx->lockState()->unlock(resourceIdReplicationStateTransitionLock); });
 
     _opCtx->lockState()->lockGlobal(_opCtx, lockMode, deadline);
