@@ -27,7 +27,7 @@
  *    it in the license file.
  */
 
-#define MONGO_LOG_DEFAULT_COMPONENT ::mongo::logger::LogComponent::kDefault
+#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kDefault
 
 #include "mongo/platform/basic.h"
 
@@ -43,7 +43,6 @@
 #include "mongo/db/wire_version.h"
 #include "mongo/rpc/get_status_from_command_result.h"
 #include "mongo/transport/service_entry_point.h"
-#include "mongo/util/log.h"
 #include "mongo/util/scopeguard.h"
 
 namespace mongo {
@@ -75,8 +74,10 @@ private:
 }  // namespace
 
 
-DBDirectClient::DBDirectClient(OperationContext* opCtx) : _opCtx(opCtx) {
-    _setServerRPCProtocols(rpc::supports::kAll);
+DBDirectClient::DBDirectClient(OperationContext* opCtx) : _opCtx(opCtx) {}
+
+void DBDirectClient::_auth(const BSONObj& params) {
+    uasserted(2625701, "DBDirectClient should not authenticate");
 }
 
 bool DBDirectClient::isFailed() const {
@@ -97,12 +98,12 @@ std::string DBDirectClient::getServerAddress() const {
 
 // Returned version should match the incoming connections restrictions.
 int DBDirectClient::getMinWireVersion() {
-    return WireSpec::instance().incomingExternalClient.minWireVersion;
+    return WireSpec::instance().get()->incomingExternalClient.minWireVersion;
 }
 
 // Returned version should match the incoming connections restrictions.
 int DBDirectClient::getMaxWireVersion() {
-    return WireSpec::instance().incomingExternalClient.maxWireVersion;
+    return WireSpec::instance().get()->incomingExternalClient.maxWireVersion;
 }
 
 bool DBDirectClient::isReplicaSetMember() const {
@@ -111,45 +112,28 @@ bool DBDirectClient::isReplicaSetMember() const {
 }
 
 ConnectionString::ConnectionType DBDirectClient::type() const {
-    return ConnectionString::MASTER;
+    return ConnectionString::ConnectionType::kStandalone;
 }
 
 double DBDirectClient::getSoTimeout() const {
     return 0;
 }
 
-bool DBDirectClient::lazySupported() const {
-    return false;
-}
-
-void DBDirectClient::setOpCtx(OperationContext* opCtx) {
-    _opCtx = opCtx;
-}
-
-QueryOptions DBDirectClient::_lookupAvailableOptions() {
-    // Exhaust mode is not available in DBDirectClient.
-    return QueryOptions(DBClientBase::_lookupAvailableOptions() & ~QueryOption_Exhaust);
-}
-
 namespace {
-DbResponse loopbackBuildResponse(OperationContext* const opCtx,
-                                 LastError* lastError,
-                                 Message& toSend) {
+DbResponse loopbackBuildResponse(OperationContext* const opCtx, Message& toSend) {
     DirectClientScope directClientScope(opCtx);
-    boost::swap(*lastError, LastError::get(opCtx->getClient()));
-    ON_BLOCK_EXIT([&] { boost::swap(*lastError, LastError::get(opCtx->getClient())); });
 
-    LastError::get(opCtx->getClient()).startRequest();
     CurOp curOp(opCtx);
 
     toSend.header().setId(nextMessageId());
     toSend.header().setResponseToMsgId(0);
-    return opCtx->getServiceContext()->getServiceEntryPoint()->handleRequest(opCtx, toSend);
+    IgnoreAPIParametersBlock ignoreApiParametersBlock(opCtx);
+    return opCtx->getServiceContext()->getServiceEntryPoint()->handleRequest(opCtx, toSend).get();
 }
 }  // namespace
 
 bool DBDirectClient::call(Message& toSend, Message& response, bool assertOk, string* actualServer) {
-    auto dbResponse = loopbackBuildResponse(_opCtx, &_lastError, toSend);
+    auto dbResponse = loopbackBuildResponse(_opCtx, toSend);
     invariant(!dbResponse.response.empty());
     response = std::move(dbResponse.response);
 
@@ -157,30 +141,62 @@ bool DBDirectClient::call(Message& toSend, Message& response, bool assertOk, str
 }
 
 void DBDirectClient::say(Message& toSend, bool isRetry, string* actualServer) {
-    auto dbResponse = loopbackBuildResponse(_opCtx, &_lastError, toSend);
+    auto dbResponse = loopbackBuildResponse(_opCtx, toSend);
     invariant(dbResponse.response.empty());
 }
 
 unique_ptr<DBClientCursor> DBDirectClient::query(const NamespaceStringOrUUID& nsOrUuid,
-                                                 Query query,
-                                                 int nToReturn,
+                                                 const BSONObj& filter,
+                                                 const Query& querySettings,
+                                                 int limit,
                                                  int nToSkip,
                                                  const BSONObj* fieldsToReturn,
                                                  int queryOptions,
-                                                 int batchSize) {
+                                                 int batchSize,
+                                                 boost::optional<BSONObj> readConcernObj) {
+    invariant(!readConcernObj, "passing readConcern to DBDirectClient functions is not supported");
     return DBClientBase::query(
-        nsOrUuid, query, nToReturn, nToSkip, fieldsToReturn, queryOptions, batchSize);
+        nsOrUuid, filter, querySettings, limit, nToSkip, fieldsToReturn, queryOptions, batchSize);
 }
 
-unsigned long long DBDirectClient::count(
-    const string& ns, const BSONObj& query, int options, int limit, int skip) {
-    DirectClientScope directClientScope(_opCtx);
-    BSONObj cmdObj = _countCmd(ns, query, options, limit, skip);
+write_ops::FindAndModifyCommandReply DBDirectClient::findAndModify(
+    const write_ops::FindAndModifyCommandRequest& findAndModify) {
+    auto response = runCommand(findAndModify.serialize({}));
+    return FindAndModifyOp::parseResponse(response->getCommandReply());
+}
 
-    NamespaceString nsString(ns);
+write_ops::InsertCommandReply DBDirectClient::insert(
+    const write_ops::InsertCommandRequest& insert) {
+    auto response = runCommand(insert.serialize({}));
+    return InsertOp::parseResponse(response->getCommandReply());
+}
+
+write_ops::UpdateCommandReply DBDirectClient::update(
+    const write_ops::UpdateCommandRequest& update) {
+    auto response = runCommand(update.serialize({}));
+    return UpdateOp::parseResponse(response->getCommandReply());
+}
+
+write_ops::DeleteCommandReply DBDirectClient::remove(
+    const write_ops::DeleteCommandRequest& remove) {
+    auto response = runCommand(remove.serialize({}));
+    return DeleteOp::parseResponse(response->getCommandReply());
+}
+
+long long DBDirectClient::count(const NamespaceStringOrUUID nsOrUuid,
+                                const BSONObj& query,
+                                int options,
+                                int limit,
+                                int skip,
+                                boost::optional<BSONObj> readConcernObj) {
+    invariant(!readConcernObj, "passing readConcern to DBDirectClient functions is not supported");
+    DirectClientScope directClientScope(_opCtx);
+    BSONObj cmdObj = _countCmd(nsOrUuid, query, options, limit, skip, boost::none);
+
+    auto dbName = (nsOrUuid.uuid() ? nsOrUuid.dbname() : (*nsOrUuid.nss()).db().toString());
 
     auto result = CommandHelpers::runCommandDirectly(
-        _opCtx, OpMsgRequest::fromDBAndBody(nsString.db(), std::move(cmdObj)));
+        _opCtx, OpMsgRequest::fromDBAndBody(dbName, std::move(cmdObj)));
 
     uassertStatusOK(getStatusFromCommandResult(result));
     return static_cast<unsigned long long>(result["n"].numberLong());
