@@ -29,6 +29,7 @@
 
 #pragma once
 
+
 #include <algorithm>
 #include <set>
 
@@ -37,23 +38,82 @@
 
 namespace mongo {
 /**
- * Carries parameters for unpacking a bucket.
+ * Carries parameters for unpacking a bucket. The order of operations applied to determine which
+ * fields are in the final document are:
+ * If we are in include mode:
+ *   1. Unpack all fields from the bucket.
+ *   2. Remove any fields not in _fieldSet, since we are in include mode.
+ *   3. Add fields from _computedMetaProjFields.
+ * If we are in exclude mode:
+ *   1. Unpack all fields from the bucket.
+ *   2. Add fields from _computedMetaProjFields.
+ *   3. Remove any fields in _fieldSet, since we are in exclude mode.
  */
-struct BucketSpec {
+class BucketSpec {
+public:
+    BucketSpec() = default;
+    BucketSpec(const std::string& timeField,
+               const boost::optional<std::string>& metaField,
+               const std::set<std::string>& fields = {},
+               const std::set<std::string>& computedProjections = {});
+    BucketSpec(const BucketSpec&);
+    BucketSpec(BucketSpec&&);
+
+    BucketSpec& operator=(const BucketSpec&);
+
     // The user-supplied timestamp field name specified during time-series collection creation.
-    std::string timeField;
+    void setTimeField(std::string&& field);
+    const std::string& timeField() const;
+    HashedFieldName timeFieldHashed() const;
 
     // An optional user-supplied metadata field name specified during time-series collection
     // creation. This field name is used during materialization of metadata fields of a measurement
     // after unpacking.
-    boost::optional<std::string> metaField;
+    void setMetaField(boost::optional<std::string>&& field);
+    const boost::optional<std::string>& metaField() const;
+    boost::optional<HashedFieldName> metaFieldHashed() const;
 
+    void setFieldSet(std::set<std::string>& fieldSet) {
+        _fieldSet = std::move(fieldSet);
+    }
+
+    void addIncludeExcludeField(const StringData& field) {
+        _fieldSet.emplace(field);
+    }
+
+    void removeIncludeExcludeField(const std::string& field) {
+        _fieldSet.erase(field);
+    }
+
+    const std::set<std::string>& fieldSet() const {
+        return _fieldSet;
+    }
+
+    void addComputedMetaProjFields(const StringData& field) {
+        _computedMetaProjFields.emplace(field);
+    }
+
+    const std::set<std::string>& computedMetaProjFields() const {
+        return _computedMetaProjFields;
+    }
+
+    void eraseFromComputedMetaProjFields(const std::string& field) {
+        _computedMetaProjFields.erase(field);
+    }
+
+private:
     // The set of field names in the data region that should be included or excluded.
-    std::set<std::string> fieldSet;
+    std::set<std::string> _fieldSet;
 
-    // Vector of computed meta field projection names. Added at the end of materialized
+    // Set of computed meta field projection names. Added at the end of materialized
     // measurements.
-    std::vector<std::string> computedMetaProjFields;
+    std::set<std::string> _computedMetaProjFields;
+
+    std::string _timeField;
+    boost::optional<HashedFieldName> _timeFieldHashed;
+
+    boost::optional<std::string> _metaField = boost::none;
+    boost::optional<HashedFieldName> _metaFieldHashed = boost::none;
 };
 
 /**
@@ -147,9 +207,23 @@ public:
     // Add computed meta projection names to the bucket specification.
     void addComputedMetaProjFields(const std::vector<StringData>& computedFieldNames);
 
+    // Fill _spec.unpackFieldsToIncludeExclude with final list of fields to include/exclude during
+    // unpacking. Only calculates the list the first time it is called.
+    const std::set<std::string>& fieldsToIncludeExcludeDuringUnpack();
+
     class UnpackingImpl;
 
 private:
+    // Determines if timestamp values should be included in the materialized measurements.
+    void determineIncludeTimeField();
+
+    // Removes metaField from the field set and determines whether metaField should be
+    // included in the materialized measurements.
+    void eraseMetaFromFieldSetAndDetermineIncludeMeta();
+
+    // Erase computed meta projection fields if they are present in the exclusion field set.
+    void eraseExcludedComputedMetaProjFields();
+
     BucketSpec _spec;
     Behavior _unpackerBehavior;
 
@@ -167,10 +241,9 @@ private:
     BSONObj _bucket;
 
     // Since the metadata value is the same across all materialized measurements we can cache the
-    // metadata BSONElement in the reset phase and use it to materialize the metadata in each
+    // metadata Value in the reset phase and use it to materialize the metadata in each
     // measurement.
-    BSONElement _metaValue;
-
+    Value _metaValue;
 
     // Map <name, BSONElement> for the computed meta field projections. Updated for
     // every bucket upon reset().
@@ -178,44 +251,21 @@ private:
 
     // The number of measurements in the bucket.
     int32_t _numberOfMeasurements = 0;
+
+    // Final list of fields to include/exclude during unpacking. This is computed once during the
+    // first doGetNext call so we don't have to recalculate every time we reach a new bucket.
+    boost::optional<std::set<std::string>> _unpackFieldsToIncludeExclude = boost::none;
 };
 
-/**
- * Removes metaField from the field set and returns a boolean indicating whether metaField should be
- * included in the materialized measurements. Always returns false if metaField does not exist.
- */
-inline bool eraseMetaFromFieldSetAndDetermineIncludeMeta(BucketUnpacker::Behavior unpackerBehavior,
-                                                         BucketSpec* bucketSpec) {
-    if (!bucketSpec->metaField ||
-        std::find(bucketSpec->computedMetaProjFields.cbegin(),
-                  bucketSpec->computedMetaProjFields.cend(),
-                  *bucketSpec->metaField) != bucketSpec->computedMetaProjFields.cend()) {
-        return false;
-    } else if (auto itr = bucketSpec->fieldSet.find(*bucketSpec->metaField);
-               itr != bucketSpec->fieldSet.end()) {
-        bucketSpec->fieldSet.erase(itr);
-        return unpackerBehavior == BucketUnpacker::Behavior::kInclude;
-    } else {
-        return unpackerBehavior == BucketUnpacker::Behavior::kExclude;
-    }
-}
-
-/**
- * Determines if timestamp values should be included in the materialized measurements.
- */
-inline bool determineIncludeTimeField(BucketUnpacker::Behavior unpackerBehavior,
-                                      BucketSpec* bucketSpec) {
-    return (unpackerBehavior == BucketUnpacker::Behavior::kInclude) ==
-        (bucketSpec->fieldSet.find(bucketSpec->timeField) != bucketSpec->fieldSet.end());
-}
-
 /**
  * Determines if an arbitrary field should be included in the materialized measurements.
  */
 inline bool determineIncludeField(StringData fieldName,
                                   BucketUnpacker::Behavior unpackerBehavior,
-                                  const BucketSpec& bucketSpec) {
-    return (unpackerBehavior == BucketUnpacker::Behavior::kInclude) ==
-        (bucketSpec.fieldSet.find(fieldName.toString()) != bucketSpec.fieldSet.end());
+                                  const std::set<std::string>& unpackFieldsToIncludeExclude) {
+    const bool isInclude = unpackerBehavior == BucketUnpacker::Behavior::kInclude;
+    const bool unpackFieldsContains = unpackFieldsToIncludeExclude.find(fieldName.toString()) !=
+        unpackFieldsToIncludeExclude.cend();
+    return isInclude == unpackFieldsContains;
 }
 }  // namespace mongo
