@@ -29,8 +29,9 @@
 
 #include "mongo/db/exec/upsert_stage.h"
 
+#include "mongo/db/catalog/document_validation.h"
 #include "mongo/db/catalog/local_oplog_info.h"
-#include "mongo/db/concurrency/exception_util.h"
+#include "mongo/db/concurrency/write_conflict_exception.h"
 #include "mongo/db/curop_failpoint_helpers.h"
 #include "mongo/db/query/query_feature_flags_gen.h"
 #include "mongo/db/s/operation_sharding_state.h"
@@ -227,9 +228,11 @@ BSONObj UpsertStage::_produceNewDocumentForInsert() {
 
     // Fifth: validate that the newly-produced document does not exceed the maximum BSON user size.
     auto newDocument = _doc.getObject();
-    uassert(17420,
-            str::stream() << "Document to upsert is larger than " << BSONObjMaxUserSize,
-            newDocument.objsize() <= BSONObjMaxUserSize);
+    if (!DocumentValidationSettings::get(opCtx()).isInternalValidationDisabled()) {
+        uassert(17420,
+                str::stream() << "Document to upsert is larger than " << BSONObjMaxUserSize,
+                newDocument.objsize() <= BSONObjMaxUserSize);
+    }
 
     return newDocument;
 }
@@ -279,11 +282,14 @@ void UpsertStage::_assertDocumentToBeInsertedIsValid(const mb::Document& documen
         // should always have an _id here, since we generated one earlier if not already present.
         invariant(document.root().ok() && document.root()[idFieldName].ok());
         bool containsDotsAndDollarsField = false;
-        storage_validation::storageValid(
-            document,
-            feature_flags::gFeatureFlagDotsAndDollars.isEnabledAndIgnoreFCV(),
-            true, /* Should validate for storage */
-            &containsDotsAndDollarsField);
+        bool allowTopLevelDollarPrefixes =
+            serverGlobalParams.featureCompatibility.isVersionInitialized() &&
+            serverGlobalParams.featureCompatibility.isGreaterThanOrEqualTo(
+                multiversion::FeatureCompatibilityVersion::kFullyDowngradedTo_5_0);
+        storage_validation::scanDocument(document,
+                                         allowTopLevelDollarPrefixes,
+                                         true, /* Should validate for storage */
+                                         &containsDotsAndDollarsField);
         if (containsDotsAndDollarsField)
             _params.driver->setContainsDotsAndDollarsField(true);
 
