@@ -273,7 +273,7 @@ Status verifyFieldNameNotReserved(StringData s) {
 }  // namespace
 
 StatusWith<BSONObj> FTSSpec::fixSpec(const BSONObj& spec) {
-    if (spec["textIndexVersion"].numberInt() == TEXT_INDEX_VERSION_1) {
+    if (spec["textIndexVersion"].safeNumberInt() == TEXT_INDEX_VERSION_1) {
         return _fixSpecV1(spec);
     }
 
@@ -308,7 +308,8 @@ StatusWith<BSONObj> FTSSpec::fixSpec(const BSONObj& spec) {
 
                     m[e.fieldName()] = 1;
                 } else {
-                    if (e.numberInt() != 1 && e.numberInt() != -1) {
+                    auto intVal = e.safeNumberInt();
+                    if (intVal != 1 && intVal != -1) {
                         return {ErrorCodes::CannotCreateIndex,
                                 "expected value 1 or -1 for non-text key in compound index"};
                     }
@@ -384,7 +385,7 @@ StatusWith<BSONObj> FTSSpec::fixSpec(const BSONObj& spec) {
             if (!e.isNumber()) {
                 return {ErrorCodes::CannotCreateIndex, "weight for text index needs numeric type"};
             }
-            m[e.fieldName()] = e.numberInt();
+            m[e.fieldName()] = e.safeNumberInt();
         }
     } else if (spec["weights"].str() == WILDCARD) {
         m[WILDCARD] = 1;
@@ -511,5 +512,32 @@ StatusWith<BSONObj> FTSSpec::fixSpec(const BSONObj& spec) {
 
     return b.obj();
 }
+
+size_t FTSSpec::getApproximateSize() const {
+    auto computeVectorSize = [](const std::vector<std::string>& v) {
+        size_t size = 0;
+        for (const auto& str : v) {
+            size += sizeof(str) + str.size() + 1;
+        }
+        return size;
+    };
+
+    auto computeWeightsSize = [](const Weights& w) {
+        size_t size = 0;
+        for (const auto& p : w) {
+            size += sizeof(p) + p.first.size() + 1;
+        }
+        return size;
+    };
+
+    // _defaultLanguage is owned by the LanguageRegistry class and may be shared across many
+    // FTSSpec's, so we don't account for the size of _defaultLanguage here.
+    auto size = sizeof(FTSSpec);
+    size += _languageOverrideField.size() + 1;
+    size += computeWeightsSize(_weights);
+    size += computeVectorSize(_extraBefore);
+    size += computeVectorSize(_extraAfter);
+    return size;
+}
 }  // namespace fts
 }  // namespace mongo
