@@ -49,13 +49,23 @@ HashAccessMethod::HashAccessMethod(IndexCatalogEntry* btreeState,
     _collator = btreeState->getCollator();
 }
 
-void HashAccessMethod::doGetKeys(const BSONObj& obj,
+void HashAccessMethod::validateDocument(const CollectionPtr& collection,
+                                        const BSONObj& obj,
+                                        const BSONObj& keyPattern) const {
+    ExpressionKeysPrivate::validateDocumentCommon(collection, obj, keyPattern);
+}
+
+void HashAccessMethod::doGetKeys(OperationContext* opCtx,
+                                 const CollectionPtr& collection,
+                                 SharedBufferFragmentBuilder& pooledBufferBuilder,
+                                 const BSONObj& obj,
                                  GetKeysContext context,
                                  KeyStringSet* keys,
                                  KeyStringSet* multikeyMetadataKeys,
                                  MultikeyPaths* multikeyPaths,
                                  boost::optional<RecordId> id) const {
-    ExpressionKeysPrivate::getHashKeys(obj,
+    ExpressionKeysPrivate::getHashKeys(pooledBufferBuilder,
+                                       obj,
                                        _keyPattern,
                                        _seed,
                                        _hashVersion,
