@@ -42,6 +42,7 @@
 #include "mongo/db/record_id.h"
 #include "mongo/db/sorter/sorter.h"
 #include "mongo/db/storage/sorted_data_interface.h"
+#include "mongo/db/yieldable.h"
 
 namespace mongo {
 
@@ -87,6 +88,7 @@ public:
      * The behavior of the insertion can be specified through 'options'.
      */
     virtual Status insert(OperationContext* opCtx,
+                          SharedBufferFragmentBuilder& pooledBufferBuilder,
                           const CollectionPtr& coll,
                           const BSONObj& obj,
                           const RecordId& loc,
@@ -138,6 +140,7 @@ public:
      * Provides a ticket for actually performing the update.
      */
     virtual void prepareUpdate(OperationContext* opCtx,
+                               const CollectionPtr& collection,
                                const IndexCatalogEntry* index,
                                const BSONObj& from,
                                const BSONObj& to,
@@ -210,7 +213,9 @@ public:
      */
     virtual long long getFreeStorageBytes(OperationContext* opCtx) const = 0;
 
-    virtual RecordId findSingle(OperationContext* opCtx, const BSONObj& key) const = 0;
+    virtual RecordId findSingle(OperationContext* opCtx,
+                                const CollectionPtr& collection,
+                                const BSONObj& key) const = 0;
 
     /**
      * Attempt compaction to regain disk space if the indexed record store supports
@@ -248,6 +253,8 @@ public:
          * have to save/restore around each insert() call just in case there is a side table write.
          */
         virtual Status insert(OperationContext* opCtx,
+                              const CollectionPtr& collection,
+                              SharedBufferFragmentBuilder& pooledBuilder,
                               const BSONObj& obj,
                               const RecordId& loc,
                               const InsertDeleteOptions& options,
@@ -298,17 +305,20 @@ public:
      * Call this when you are ready to finish your bulk work.
      * Pass in the BulkBuilder returned from initiateBulk.
      * @param bulk - Something created from initiateBulk
-     * @param mayInterrupt - Is this commit interruptible (will cancel)
      * @param dupsAllowed - If false and 'dupRecords' is not null, append with the RecordIds of
      *                      the uninserted duplicates.
+     * @param yieldIterations - The number of iterations run before each yielding. Will not yield if
+     * zero.
      * @param onDuplicateKeyInserted - Will be called for each duplicate key inserted into the
      * index.
      * @param onDuplicateRecord - If not nullptr, will be called for each RecordId of uninserted
      * duplicate keys.
      */
     virtual Status commitBulk(OperationContext* opCtx,
+                              const CollectionPtr& collection,
                               BulkBuilder* bulk,
                               bool dupsAllowed,
+                              int32_t yieldIterations,
                               const KeyHandlerFn& onDuplicateKeyInserted,
                               const RecordIdHandlerFn& onDuplicateRecord) = 0;
 
@@ -353,7 +363,9 @@ public:
      */
     using OnSuppressedErrorFn =
         std::function<void(Status status, const BSONObj& obj, boost::optional<RecordId> loc)>;
-    virtual void getKeys(SharedBufferFragmentBuilder& pooledBufferBuilder,
+    virtual void getKeys(OperationContext* opCtx,
+                         const CollectionPtr& collection,
+                         SharedBufferFragmentBuilder& pooledBufferBuilder,
                          const BSONObj& obj,
                          GetKeysMode mode,
                          GetKeysContext context,
@@ -361,9 +373,7 @@ public:
                          KeyStringSet* multikeyMetadataKeys,
                          MultikeyPaths* multikeyPaths,
                          boost::optional<RecordId> id,
-                         OnSuppressedErrorFn onSuppressedError) const = 0;
-
-    static OnSuppressedErrorFn kNoopOnSuppressedErrorFn;
+                         OnSuppressedErrorFn&& onSuppressedError = nullptr) const = 0;
 
     /**
      * Given the set of keys, multikeyMetadataKeys and multikeyPaths generated by a particular
@@ -468,6 +478,7 @@ public:
                               std::unique_ptr<SortedDataInterface> btree);
 
     Status insert(OperationContext* opCtx,
+                  SharedBufferFragmentBuilder& pooledBufferBuilder,
                   const CollectionPtr& coll,
                   const BSONObj& obj,
                   const RecordId& loc,
@@ -500,6 +511,7 @@ public:
                       int64_t* numDeleted) final;
 
     void prepareUpdate(OperationContext* opCtx,
+                       const CollectionPtr& collection,
                        const IndexCatalogEntry* index,
                        const BSONObj& from,
                        const BSONObj& to,
@@ -531,7 +543,9 @@ public:
 
     long long getFreeStorageBytes(OperationContext* opCtx) const final;
 
-    RecordId findSingle(OperationContext* opCtx, const BSONObj& key) const final;
+    RecordId findSingle(OperationContext* opCtx,
+                        const CollectionPtr& collection,
+                        const BSONObj& key) const final;
 
     Status compact(OperationContext* opCtx) final;
 
@@ -545,12 +559,16 @@ public:
                                               StringData dbName) final;
 
     Status commitBulk(OperationContext* opCtx,
+                      const CollectionPtr& collection,
                       BulkBuilder* bulk,
                       bool dupsAllowed,
+                      int32_t yieldIterations,
                       const KeyHandlerFn& onDuplicateKeyInserted,
                       const RecordIdHandlerFn& onDuplicateRecord) final;
 
-    void getKeys(SharedBufferFragmentBuilder& pooledBufferBuilder,
+    void getKeys(OperationContext* opCtx,
+                 const CollectionPtr& collection,
+                 SharedBufferFragmentBuilder& pooledBufferBuilder,
                  const BSONObj& obj,
                  GetKeysMode mode,
                  GetKeysContext context,
@@ -558,7 +576,7 @@ public:
                  KeyStringSet* multikeyMetadataKeys,
                  MultikeyPaths* multikeyPaths,
                  boost::optional<RecordId> id,
-                 OnSuppressedErrorFn onSuppressedError) const final;
+                 OnSuppressedErrorFn&& onSuppressedError = nullptr) const final;
 
     bool shouldMarkIndexAsMultikey(size_t numberOfKeys,
                                    const KeyStringSet& multikeyMetadataKeys,
@@ -567,6 +585,14 @@ public:
     SortedDataInterface* getSortedDataInterface() const override final;
 
 protected:
+    /**
+     * Perform some initial validation on the document to ensure it can be indexed before calling
+     * the implementation-specific 'doGetKeys' method.
+     */
+    virtual void validateDocument(const CollectionPtr& collection,
+                                  const BSONObj& obj,
+                                  const BSONObj& keyPattern) const;
+
     /**
      * Fills 'keys' with the keys that should be generated for 'obj' on this index.
      *
@@ -581,7 +607,9 @@ protected:
      * keys are not associated with the document itself, but instead represent multi-key path
      * information that must be stored in a reserved keyspace within the index.
      */
-    virtual void doGetKeys(SharedBufferFragmentBuilder& pooledBufferBuilder,
+    virtual void doGetKeys(OperationContext* opCtx,
+                           const CollectionPtr& collection,
+                           SharedBufferFragmentBuilder& pooledBufferBuilder,
                            const BSONObj& obj,
                            GetKeysContext context,
                            KeyStringSet* keys,
@@ -613,6 +641,10 @@ private:
                                const KeyString::Value& dataKey,
                                const RecordIdHandlerFn& onDuplicateRecord);
 
+    void _yieldBulkLoad(OperationContext* opCtx,
+                        const Yieldable* yieldable,
+                        const NamespaceString& ns) const;
+
     const std::unique_ptr<SortedDataInterface> _newInterface;
 };
 
