@@ -33,6 +33,7 @@
 
 #include "mongo/db/logical_session_cache_impl.h"
 
+#include "mongo/db/internal_transactions_feature_flag_gen.h"
 #include "mongo/db/logical_session_id.h"
 #include "mongo/db/logical_session_id_helpers.h"
 #include "mongo/db/operation_context.h"
@@ -42,6 +43,7 @@
 #include "mongo/logv2/log.h"
 #include "mongo/logv2/log_severity_suppressor.h"
 #include "mongo/platform/atomic_word.h"
+#include "mongo/s/is_mongos.h"
 #include "mongo/util/duration.h"
 #include "mongo/util/scopeguard.h"
 
@@ -93,8 +95,16 @@ Status LogicalSessionCacheImpl::startSession(OperationContext* opCtx,
 }
 
 Status LogicalSessionCacheImpl::vivify(OperationContext* opCtx, const LogicalSessionId& lsid) {
+    auto parentLsid = getParentSessionId(lsid);
+    if (parentLsid) {
+        uassert(ErrorCodes::InvalidOptions,
+                "Internal transactions are only supported in sharded clusters",
+                isMongos() || serverGlobalParams.clusterRole != ClusterRole::None);
+    }
+
     stdx::lock_guard lg(_mutex);
-    auto it = _activeSessions.find(lsid);
+
+    auto it = _activeSessions.find(parentLsid ? *parentLsid : lsid);
     if (it == _activeSessions.end())
         return _addToCacheIfNotFull(lg, makeLogicalSessionRecord(opCtx, lsid, _service->now()));
 
@@ -182,19 +192,11 @@ Status LogicalSessionCacheImpl::_reap(Client* client) {
         try {
             _sessionsColl->checkSessionsCollectionExists(opCtx);
         } catch (const DBException& ex) {
-            if (ex.code() != ErrorCodes::NamespaceNotFound ||
-                ex.code() != ErrorCodes::NamespaceNotSharded) {
-                LOGV2(
-                    20712,
-                    "Sessions collection is not set up: {error}; waiting until next sessions reap "
-                    "interval",
-                    "Sessions collection is not set up; waiting until next sessions reap interval",
-                    "error"_attr = redact(ex));
-            } else {
-                LOGV2(20713,
-                      "Sessions collection is not set up because the namespace is either not found"
-                      "or not sharded; waiting until next sessions reap interval");
-            }
+            LOGV2(20712,
+                  "Sessions collection is not set up: {error}; waiting until next sessions reap "
+                  "interval",
+                  "Sessions collection is not set up; waiting until next sessions reap interval",
+                  "error"_attr = redact(ex));
             return Status::OK();
         }
         numReaped = _reapSessionsOlderThanFn(opCtx,
@@ -235,6 +237,8 @@ void LogicalSessionCacheImpl::_refresh(Client* client) {
 
     const auto replCoord = repl::ReplicationCoordinator::get(opCtx);
     if (replCoord && replCoord->isReplEnabled() && replCoord->getMemberState().arbiter()) {
+        stdx::lock_guard<Latch> lk(_mutex);
+        _activeSessions.clear();
         return;
     }
 
@@ -254,7 +258,7 @@ void LogicalSessionCacheImpl::_refresh(Client* client) {
     }
 
     // This will finish timing _refresh for our stats no matter when we return.
-    const auto timeRefreshJob = makeGuard([this] {
+    const ScopeGuard timeRefreshJob([this] {
         stdx::lock_guard<Latch> lk(_mutex);
         auto millis = _service->now() - _stats.getLastSessionsCollectionJobTimestamp();
         _stats.setLastSessionsCollectionJobDurationMillis(millis.count());
@@ -295,9 +299,9 @@ void LogicalSessionCacheImpl::_refresh(Client* client) {
             member.emplace(it);
         }
     };
-    auto activeSessionsBackSwapper = makeGuard([&] { backSwap(_activeSessions, activeSessions); });
+    ScopeGuard activeSessionsBackSwapper([&] { backSwap(_activeSessions, activeSessions); });
     auto explicitlyEndingBackSwaper =
-        makeGuard([&] { backSwap(_endingSessions, explicitlyEndingSessions); });
+        ScopeGuard([&] { backSwap(_endingSessions, explicitlyEndingSessions); });
 
     // remove all explicitlyEndingSessions from activeSessions
     for (const auto& lsid : explicitlyEndingSessions) {
@@ -380,6 +384,11 @@ void LogicalSessionCacheImpl::_refresh(Client* client) {
 }
 
 void LogicalSessionCacheImpl::endSessions(const LogicalSessionIdSet& sessions) {
+    for (const auto& lsid : sessions) {
+        uassert(ErrorCodes::InvalidOptions,
+                str::stream() << "Cannot specify a session id with parent session id " << lsid,
+                !getParentSessionId(lsid));
+    }
     stdx::lock_guard<Latch> lk(_mutex);
     _endingSessions.insert(begin(sessions), end(sessions));
 }
@@ -441,9 +450,13 @@ std::vector<LogicalSessionId> LogicalSessionCacheImpl::listIds(
 }
 
 boost::optional<LogicalSessionRecord> LogicalSessionCacheImpl::peekCached(
-    const LogicalSessionId& id) const {
+    const LogicalSessionId& lsid) const {
+    uassert(ErrorCodes::InvalidOptions,
+            str::stream() << "Cannot specify a session id with parent session id " << lsid,
+            !getParentSessionId(lsid));
+
     stdx::lock_guard<Latch> lk(_mutex);
-    const auto it = _activeSessions.find(id);
+    const auto it = _activeSessions.find(lsid);
     if (it == _activeSessions.end()) {
         return boost::none;
     }
