@@ -52,11 +52,6 @@ public:
     constexpr static size_t MaxNSCollectionLenFCV42 = 120U;
     constexpr static size_t MaxNsCollectionLen = 255;
 
-    // The maximum namespace length of sharded collections is less than that of unsharded ones since
-    // the namespace of the cached chunks metadata, local to each shard, is composed by the
-    // namespace of the related sharded collection (i.e., config.cache.chunks.<ns>).
-    constexpr static size_t MaxNsShardedCollectionLen = 235;  // 255 - len(ChunkType::ShardNSPrefix)
-
     // Reserved system namespaces
 
     // Namespace for the admin database
@@ -68,9 +63,6 @@ public:
     // Namespace for the sharding config database
     static constexpr StringData kConfigDb = "config"_sd;
 
-    // The $external database used by X.509, LDAP, etc...
-    static constexpr StringData kExternalDb = "$external"_sd;
-
     // Name for the system views collection
     static constexpr StringData kSystemDotViewsCollectionName = "system.views"_sd;
 
@@ -132,6 +124,9 @@ public:
     // Namespace for storing the persisted state of migration coordinators.
     static const NamespaceString kMigrationCoordinatorsNamespace;
 
+    // Namespace for storing the persisted state of migration recipients.
+    static const NamespaceString kMigrationRecipientsNamespace;
+
     // Namespace for storing the persisted state of tenant migration donors.
     static const NamespaceString kTenantMigrationDonorsNamespace;
 
@@ -141,9 +136,15 @@ public:
     // Namespace for view on local.oplog.rs for tenant migrations.
     static const NamespaceString kTenantMigrationOplogView;
 
+    // Namespace for storing the persisted state of tenant split donors.
+    static const NamespaceString kTenantSplitDonorsNamespace;
+
     // Namespace for replica set configuration settings.
     static const NamespaceString kSystemReplSetNamespace;
 
+    // Namespace for storing the last replica set election vote.
+    static const NamespaceString kLastVoteNamespace;
+
     // Namespace for change stream pre-images collection.
     static const NamespaceString kChangeStreamPreImagesNamespace;
 
@@ -192,9 +193,6 @@ public:
     // Namespace used for storing retryable findAndModify images.
     static const NamespaceString kConfigImagesNamespace;
 
-    // Namespace used for storing the list of shards on the CSRS
-    static const NamespaceString kConfigsvrShardsNamespace;
-
     /**
      * Constructs an empty NamespaceString.
      */
@@ -380,11 +378,6 @@ public:
      */
     bool isChangeStreamPreImagesCollection() const;
 
-    /**
-     * Returns whether the specified namespace is config.image_collection.
-     */
-    bool isConfigImagesCollection() const;
-
     /**
      * Returns the time-series buckets namespace for this view.
      */
