@@ -27,20 +27,22 @@
  *    it in the license file.
  */
 
-#define MONGO_LOG_DEFAULT_COMPONENT ::mongo::logger::LogComponent::kQuery
+#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kQuery
 
 #include "mongo/platform/basic.h"
 
 #include "mongo/db/pipeline/document_source_cursor.h"
 
 #include "mongo/db/catalog/collection.h"
+#include "mongo/db/exec/document_value/document.h"
 #include "mongo/db/exec/working_set_common.h"
-#include "mongo/db/pipeline/document.h"
+#include "mongo/db/query/collection_query_info.h"
 #include "mongo/db/query/explain.h"
 #include "mongo/db/query/find_common.h"
 #include "mongo/db/storage/storage_options.h"
-#include "mongo/util/fail_point_service.h"
-#include "mongo/util/log.h"
+#include "mongo/logv2/log.h"
+#include "mongo/s/resharding/resume_token_gen.h"
+#include "mongo/util/fail_point.h"
 #include "mongo/util/scopeguard.h"
 
 namespace mongo {
@@ -52,39 +54,49 @@ using std::shared_ptr;
 using std::string;
 
 const char* DocumentSourceCursor::getSourceName() const {
-    return "$cursor";
+    return kStageName.rawData();
 }
 
 bool DocumentSourceCursor::Batch::isEmpty() const {
-    if (shouldProduceEmptyDocs) {
-        return !_count;
-    } else {
-        return _batchOfDocs.empty();
+    switch (_type) {
+        case CursorType::kRegular:
+            return _batchOfDocs.empty();
+        case CursorType::kEmptyDocuments:
+            return !_count;
     }
     MONGO_UNREACHABLE;
 }
 
 void DocumentSourceCursor::Batch::enqueue(Document&& doc) {
-    if (shouldProduceEmptyDocs) {
-        ++_count;
-    } else {
-        _batchOfDocs.push_back(doc.getOwned());
-        _memUsageBytes += _batchOfDocs.back().getApproximateSize();
+    switch (_type) {
+        case CursorType::kRegular: {
+            invariant(doc.isOwned());
+            _batchOfDocs.push_back(std::move(doc));
+            _memUsageBytes += _batchOfDocs.back().getApproximateSize();
+            break;
+        }
+        case CursorType::kEmptyDocuments: {
+            ++_count;
+            break;
+        }
     }
 }
 
 Document DocumentSourceCursor::Batch::dequeue() {
     invariant(!isEmpty());
-    if (shouldProduceEmptyDocs) {
-        --_count;
-        return Document{};
-    } else {
-        Document out = std::move(_batchOfDocs.front());
-        _batchOfDocs.pop_front();
-        if (_batchOfDocs.empty()) {
-            _memUsageBytes = 0;
+    switch (_type) {
+        case CursorType::kRegular: {
+            Document out = std::move(_batchOfDocs.front());
+            _batchOfDocs.pop_front();
+            if (_batchOfDocs.empty()) {
+                _memUsageBytes = 0;
+            }
+            return out;
+        }
+        case CursorType::kEmptyDocuments: {
+            --_count;
+            return Document{};
         }
-        return out;
     }
     MONGO_UNREACHABLE;
 }
@@ -95,9 +107,7 @@ void DocumentSourceCursor::Batch::clear() {
     _memUsageBytes = 0;
 }
 
-DocumentSource::GetNextResult DocumentSourceCursor::getNext() {
-    pExpCtx->checkForInterrupt();
-
+DocumentSource::GetNextResult DocumentSourceCursor::doGetNext() {
     if (_currentBatch.isEmpty()) {
         loadBatch();
     }
@@ -112,86 +122,66 @@ DocumentSource::GetNextResult DocumentSourceCursor::getNext() {
     return _currentBatch.dequeue();
 }
 
-Document DocumentSourceCursor::transformBSONObjToDocument(const BSONObj& obj) const {
-    return _dependencies ? _dependencies->extractFields(obj) : Document::fromBsonWithMetaData(obj);
-}
-
 void DocumentSourceCursor::loadBatch() {
     if (!_exec || _exec->isDisposed()) {
         // No more documents.
         return;
     }
 
-    while (MONGO_FAIL_POINT(hangBeforeDocumentSourceCursorLoadBatch)) {
-        log() << "Hanging aggregation due to 'hangBeforeDocumentSourceCursorLoadBatch' failpoint";
+    while (MONGO_unlikely(hangBeforeDocumentSourceCursorLoadBatch.shouldFail())) {
+        LOGV2(20895,
+              "Hanging aggregation due to 'hangBeforeDocumentSourceCursorLoadBatch' failpoint");
         sleepmillis(10);
     }
 
     PlanExecutor::ExecState state;
-    BSONObj resultObj;
-    {
-        AutoGetCollectionForRead autoColl(pExpCtx->opCtx, _exec->nss());
-        uassertStatusOK(repl::ReplicationCoordinator::get(pExpCtx->opCtx)
-                            ->checkCanServeReadsFor(pExpCtx->opCtx, _exec->nss(), true));
+    Document resultObj;
 
-        _exec->restoreState();
+    boost::optional<AutoGetCollectionForReadMaybeLockFree> autoColl;
+    tassert(5565800,
+            "Expected PlanExecutor to use an external lock policy",
+            _exec->lockPolicy() == PlanExecutor::LockPolicy::kLockExternally);
+    autoColl.emplace(pExpCtx->opCtx, _exec->nss());
+    uassertStatusOK(repl::ReplicationCoordinator::get(pExpCtx->opCtx)
+                        ->checkCanServeReadsFor(pExpCtx->opCtx, _exec->nss(), true));
 
-        {
-            ON_BLOCK_EXIT([this] { recordPlanSummaryStats(); });
+    _exec->restoreState(autoColl ? &autoColl->getCollection() : nullptr);
 
-            while ((state = _exec->getNext(&resultObj, nullptr)) == PlanExecutor::ADVANCED) {
-                if (_currentBatch.shouldProduceEmptyDocs) {
-                    _currentBatch.enqueue(Document());
-                } else {
-                    _currentBatch.enqueue(transformBSONObjToDocument(resultObj));
-                }
+    try {
+        ON_BLOCK_EXIT([this] { recordPlanSummaryStats(); });
 
-                if (_limit) {
-                    if (++_docsAddedToBatches == _limit->getLimit()) {
-                        break;
-                    }
-                    verify(_docsAddedToBatches < _limit->getLimit());
-                }
+        while ((state = _exec->getNextDocument(&resultObj, nullptr)) == PlanExecutor::ADVANCED) {
+            _currentBatch.enqueue(transformDoc(std::move(resultObj)));
 
-                // As long as we're waiting for inserts, we shouldn't do any batching at this level
-                // we need the whole pipeline to see each document to see if we should stop waiting.
-                if (awaitDataState(pExpCtx->opCtx).shouldWaitForInserts ||
-                    static_cast<long long>(_currentBatch.memUsageBytes()) >
-                        internalDocumentSourceCursorBatchSizeBytes.load()) {
-                    // End this batch and prepare PlanExecutor for yielding.
-                    _exec->saveState();
-                    return;
-                }
-            }
-            // Special case for tailable cursor -- EOF doesn't preclude more results, so keep
-            // the PlanExecutor alive.
-            if (state == PlanExecutor::IS_EOF && pExpCtx->isTailableAwaitData()) {
+            // As long as we're waiting for inserts, we shouldn't do any batching at this level we
+            // need the whole pipeline to see each document to see if we should stop waiting.
+            if (awaitDataState(pExpCtx->opCtx).shouldWaitForInserts ||
+                static_cast<long long>(_currentBatch.memUsageBytes()) >
+                    internalDocumentSourceCursorBatchSizeBytes.load()) {
+                // End this batch and prepare PlanExecutor for yielding.
                 _exec->saveState();
                 return;
             }
         }
 
-        // If we got here, there won't be any more documents, so destroy our PlanExecutor. Note we
-        // must hold a collection lock to destroy '_exec', but we can only assume that our locks are
-        // still held if '_exec' did not end in an error. If '_exec' encountered an error during a
-        // yield, the locks might be yielded.
-        if (state != PlanExecutor::FAILURE) {
-            cleanupExecutor();
+        invariant(state == PlanExecutor::IS_EOF);
+
+        // Keep the inner PlanExecutor alive if the cursor is tailable, since more results may
+        // become available in the future, or if we are tracking the latest oplog timestamp, since
+        // we will need to retrieve the last timestamp the executor observed before hitting EOF.
+        if (_trackOplogTS || pExpCtx->isTailableAwaitData()) {
+            _exec->saveState();
+            return;
         }
+    } catch (...) {
+        // Record error details before re-throwing the exception.
+        _execStatus = exceptionToStatus().withContext("Error in $cursor stage");
+        throw;
     }
 
-    switch (state) {
-        case PlanExecutor::ADVANCED:
-        case PlanExecutor::IS_EOF:
-            return;  // We've reached our limit or exhausted the cursor.
-        case PlanExecutor::FAILURE: {
-            _execStatus = WorkingSetCommon::getMemberObjectStatus(resultObj).withContext(
-                "Error in $cursor stage");
-            uassertStatusOK(_execStatus);
-        }
-        default:
-            MONGO_UNREACHABLE;
-    }
+    // If we got here, there won't be any more documents and we no longer need our PlanExecutor, so
+    // destroy it.
+    cleanupExecutor();
 }
 
 void DocumentSourceCursor::_updateOplogTimestamp() {
@@ -207,39 +197,9 @@ void DocumentSourceCursor::_updateOplogTimestamp() {
     _latestOplogTimestamp = _exec->getLatestOplogTimestamp();
 }
 
-Pipeline::SourceContainer::iterator DocumentSourceCursor::doOptimizeAt(
-    Pipeline::SourceContainer::iterator itr, Pipeline::SourceContainer* container) {
-    invariant(*itr == this);
-
-    if (std::next(itr) == container->end()) {
-        return container->end();
-    }
-
-    auto nextLimit = dynamic_cast<DocumentSourceLimit*>((*std::next(itr)).get());
-
-    if (nextLimit) {
-        if (_limit) {
-            // We already have an internal limit, set it to the more restrictive of the two.
-            _limit->setLimit(std::min(_limit->getLimit(), nextLimit->getLimit()));
-        } else {
-            _limit = nextLimit;
-        }
-        container->erase(std::next(itr));
-        return itr;
-    }
-    return std::next(itr);
-}
-
 void DocumentSourceCursor::recordPlanSummaryStats() {
     invariant(_exec);
-    // Aggregation handles in-memory sort outside of the query sub-system. Given that we need to
-    // preserve the existing value of hasSortStage rather than overwrite with the underlying
-    // PlanExecutor's value.
-    auto hasSortStage = _planSummaryStats.hasSortStage;
-
-    Explain::getSummaryStats(*_exec, &_planSummaryStats);
-
-    _planSummaryStats.hasSortStage = hasSortStage;
+    _exec->getPlanExplainer().getSummaryStats(&_stats.planSummaryStats);
 }
 
 Value DocumentSourceCursor::serialize(boost::optional<ExplainOptions::Verbosity> verbosity) const {
@@ -254,16 +214,6 @@ Value DocumentSourceCursor::serialize(boost::optional<ExplainOptions::Verbosity>
             verbosity == pExpCtx->explain);
 
     MutableDocument out;
-    out["query"] = Value(_query);
-
-    if (!_sort.isEmpty())
-        out["sort"] = Value(_sort);
-
-    if (_limit)
-        out["limit"] = Value(_limit->getLimit());
-
-    if (!_projection.isEmpty())
-        out["fields"] = Value(_projection);
 
     BSONObjBuilder explainStatsBuilder;
 
@@ -272,14 +222,16 @@ Value DocumentSourceCursor::serialize(boost::optional<ExplainOptions::Verbosity>
         auto lockMode = getLockModeForQuery(opCtx, _exec->nss());
         AutoGetDb dbLock(opCtx, _exec->nss().db(), lockMode);
         Lock::CollectionLock collLock(opCtx, _exec->nss(), lockMode);
-        auto collection =
-            dbLock.getDb() ? dbLock.getDb()->getCollection(opCtx, _exec->nss()) : nullptr;
+        auto collection = dbLock.getDb()
+            ? CollectionCatalog::get(opCtx)->lookupCollectionByNamespace(opCtx, _exec->nss())
+            : nullptr;
 
         Explain::explainStages(_exec.get(),
                                collection,
                                verbosity.get(),
                                _execStatus,
-                               _winningPlanTrialStats.get(),
+                               _winningPlanTrialStats,
+                               BSONObj(),
                                BSONObj(),
                                &explainStatsBuilder);
     }
@@ -297,7 +249,8 @@ Value DocumentSourceCursor::serialize(boost::optional<ExplainOptions::Verbosity>
 }
 
 void DocumentSourceCursor::detachFromOperationContext() {
-    if (_exec && !_exec->isDetached()) {
+    // Only detach the underlying executor if it hasn't been detached already.
+    if (_exec && _exec->getOpCtx()) {
         _exec->detachFromOperationContext();
     }
 }
@@ -328,6 +281,13 @@ void DocumentSourceCursor::cleanupExecutor() {
     }
 }
 
+BSONObj DocumentSourceCursor::getPostBatchResumeToken() const {
+    if (_trackOplogTS) {
+        return ResumeTokenOplogTimestamp{getLatestOplogTimestamp()}.toBSON();
+    }
+    return BSONObj{};
+}
+
 DocumentSourceCursor::~DocumentSourceCursor() {
     if (pExpCtx->explain) {
         invariant(_exec->isDisposed());  // _exec should have at least been disposed.
@@ -337,38 +297,45 @@ DocumentSourceCursor::~DocumentSourceCursor() {
 }
 
 DocumentSourceCursor::DocumentSourceCursor(
-    Collection* collection,
+    const CollectionPtr& collection,
     std::unique_ptr<PlanExecutor, PlanExecutor::Deleter> exec,
     const intrusive_ptr<ExpressionContext>& pCtx,
+    CursorType cursorType,
     bool trackOplogTimestamp)
-    : DocumentSource(pCtx),
-      _docsAddedToBatches(0),
+    : DocumentSource(kStageName, pCtx),
+      _currentBatch(cursorType),
       _exec(std::move(exec)),
       _trackOplogTS(trackOplogTimestamp) {
+    // It is illegal for both 'kEmptyDocuments' and 'trackOplogTimestamp' to be set.
+    invariant(!(cursorType == CursorType::kEmptyDocuments && trackOplogTimestamp));
+
     // Later code in the DocumentSourceCursor lifecycle expects that '_exec' is in a saved state.
     _exec->saveState();
 
-    _planSummary = Explain::getPlanSummary(_exec.get());
+    auto&& explainer = _exec->getPlanExplainer();
+    _planSummary = explainer.getPlanSummary();
     recordPlanSummaryStats();
 
     if (pExpCtx->explain) {
         // It's safe to access the executor even if we don't have the collection lock since we're
         // just going to call getStats() on it.
-        _winningPlanTrialStats = Explain::getWinningPlanTrialStats(_exec.get());
+        _winningPlanTrialStats = explainer.getWinningPlanTrialStats();
     }
 
     if (collection) {
-        collection->infoCache()->notifyOfQuery(pExpCtx->opCtx, _planSummaryStats.indexesUsed);
+        CollectionQueryInfo::get(collection)
+            .notifyOfQuery(pExpCtx->opCtx, collection, _stats.planSummaryStats);
     }
 }
 
 intrusive_ptr<DocumentSourceCursor> DocumentSourceCursor::create(
-    Collection* collection,
+    const CollectionPtr& collection,
     std::unique_ptr<PlanExecutor, PlanExecutor::Deleter> exec,
     const intrusive_ptr<ExpressionContext>& pExpCtx,
+    CursorType cursorType,
     bool trackOplogTimestamp) {
-    intrusive_ptr<DocumentSourceCursor> source(
-        new DocumentSourceCursor(collection, std::move(exec), pExpCtx, trackOplogTimestamp));
+    intrusive_ptr<DocumentSourceCursor> source(new DocumentSourceCursor(
+        collection, std::move(exec), pExpCtx, cursorType, trackOplogTimestamp));
     return source;
 }
 }  // namespace mongo
