@@ -26,6 +26,7 @@
  *    exception statement from all source files in the program, then also delete
  *    it in the license file.
  */
+#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kQuery
 
 #include "mongo/platform/basic.h"
 
@@ -40,23 +41,51 @@
 #include "mongo/db/exec/document_value/value.h"
 #include "mongo/db/jsobj.h"
 #include "mongo/db/pipeline/document_path_support.h"
+#include "mongo/db/pipeline/document_source_merge_gen.h"
 #include "mongo/db/pipeline/expression.h"
 #include "mongo/db/pipeline/expression_context.h"
+#include "mongo/db/pipeline/lite_parsed_pipeline.h"
 #include "mongo/db/query/query_knobs_gen.h"
 #include "mongo/db/query/query_planner_common.h"
+#include "mongo/db/views/resolved_view.h"
+#include "mongo/logv2/log.h"
 
 namespace mongo {
 
 namespace {
-void assertIsValidCollectionState(const boost::intrusive_ptr<ExpressionContext>& expCtx) {
-    if (expCtx->mongoProcessInterface->isSharded(expCtx->opCtx, expCtx->ns)) {
-        const bool foreignShardedAllowed =
-            getTestCommandsEnabled() && internalQueryAllowShardedLookup.load();
-        if (!foreignShardedAllowed) {
-            uasserted(31428, "Cannot run $graphLookup with sharded foreign collection");
-        }
+
+// Parses $graphLookup 'from' field. The 'from' field must be a string with the exception of
+// 'local.system.tenantMigration.oplogView'.
+//
+// {from: {db: "local", coll: "system.tenantMigration.oplogView"}, ...}.
+NamespaceString parseGraphLookupFromAndResolveNamespace(const BSONElement& elem,
+                                                        StringData defaultDb) {
+    // The object syntax only works for 'local.system.tenantMigration.oplogView' which is not a user
+    // namespace so object type is omitted from the error message below.
+    uassert(ErrorCodes::FailedToParse,
+            str::stream() << "$graphLookup 'from' field must be a string, but found "
+                          << typeName(elem.type()),
+            elem.type() == String || elem.type() == Object);
+
+    if (elem.type() == BSONType::String) {
+        NamespaceString fromNss(defaultDb, elem.valueStringData());
+        uassert(ErrorCodes::InvalidNamespace,
+                str::stream() << "invalid $graphLookup namespace: " << fromNss.ns(),
+                fromNss.isValid());
+        return fromNss;
     }
+
+    // Valdate the db and coll names.
+    auto spec = NamespaceSpec::parse({elem.fieldNameStringData()}, elem.embeddedObject());
+    auto nss = NamespaceString(spec.getDb().value_or(""), spec.getColl().value_or(""));
+    uassert(ErrorCodes::FailedToParse,
+            str::stream()
+                << "$graphLookup with syntax {from: {db:<>, coll:<>},..} is not supported for db: "
+                << nss.db() << " and coll: " << nss.coll(),
+            nss == NamespaceString::kTenantMigrationOplogView);
+    return nss;
 }
+
 }  // namespace
 
 using boost::intrusive_ptr;
@@ -76,21 +105,15 @@ std::unique_ptr<DocumentSourceGraphLookUp::LiteParsed> DocumentSourceGraphLookUp
             str::stream() << "missing 'from' option to $graphLookup stage specification: "
                           << specObj,
             fromElement);
-    uassert(ErrorCodes::FailedToParse,
-            str::stream() << "'from' option to $graphLookup must be a string, but was type "
-                          << typeName(specObj["from"].type()),
-            fromElement.type() == BSONType::String);
 
-    NamespaceString fromNss(nss.db(), fromElement.valueStringData());
-    uassert(ErrorCodes::InvalidNamespace,
-            str::stream() << "invalid $graphLookup namespace: " << fromNss.ns(),
-            fromNss.isValid());
-    return std::make_unique<LiteParsed>(spec.fieldName(), std::move(fromNss));
+    return std::make_unique<LiteParsed>(
+        spec.fieldName(), parseGraphLookupFromAndResolveNamespace(fromElement, nss.db()));
 }
 
 REGISTER_DOCUMENT_SOURCE(graphLookup,
                          DocumentSourceGraphLookUp::LiteParsed::parse,
-                         DocumentSourceGraphLookUp::createFromBson);
+                         DocumentSourceGraphLookUp::createFromBson,
+                         AllowedWithApiStrict::kAlways);
 
 const char* DocumentSourceGraphLookUp::getSourceName() const {
     return kStageName.rawData();
@@ -184,11 +207,34 @@ void DocumentSourceGraphLookUp::doDispose() {
     _visited.clear();
 }
 
+bool DocumentSourceGraphLookUp::foreignShardedGraphLookupAllowed() const {
+    return feature_flags::gFeatureFlagShardedLookup.isEnabled(
+               serverGlobalParams.featureCompatibility) &&
+        !pExpCtx->opCtx->inMultiDocumentTransaction();
+}
+
+boost::optional<DocumentSource::DistributedPlanLogic>
+DocumentSourceGraphLookUp::distributedPlanLogic() {
+    // If $graphLookup into a sharded foreign collection is allowed, top-level $graphLookup
+    // stages can run in parallel on the shards.
+    if (foreignShardedGraphLookupAllowed() && pExpCtx->subPipelineDepth == 0) {
+        return boost::none;
+    }
+
+    // {shardsStage, mergingStage, sortPattern}
+    return DistributedPlanLogic{nullptr, this, boost::none};
+}
+
 void DocumentSourceGraphLookUp::doBreadthFirstSearch() {
     long long depth = 0;
     bool shouldPerformAnotherQuery;
-    assertIsValidCollectionState(_fromExpCtx);
     do {
+        const auto allowForeignSharded = foreignShardedGraphLookupAllowed();
+        if (!allowForeignSharded) {
+            // Enforce that the foreign collection must be unsharded for $graphLookup.
+            _fromExpCtx->mongoProcessInterface->setExpectedShardVersion(
+                _fromExpCtx->opCtx, _fromExpCtx->ns, ChunkVersion::UNSHARDED());
+        }
         shouldPerformAnotherQuery = false;
 
         // Check whether each key in the frontier exists in the cache or needs to be queried.
@@ -218,8 +264,50 @@ void DocumentSourceGraphLookUp::doBreadthFirstSearch() {
             pipelineOpts.optimize = true;
             pipelineOpts.attachCursorSource = true;
             // By default, $graphLookup doesn't support a sharded 'from' collection.
-            pipelineOpts.allowTargetingShards = internalQueryAllowShardedLookup.load();
-            auto pipeline = Pipeline::makePipeline(_fromPipeline, _fromExpCtx, pipelineOpts);
+            pipelineOpts.shardTargetingPolicy = allowForeignSharded
+                ? ShardTargetingPolicy::kAllowed
+                : ShardTargetingPolicy::kNotAllowed;
+            _variables.copyToExpCtx(_variablesParseState, _fromExpCtx.get());
+
+            std::unique_ptr<Pipeline, PipelineDeleter> pipeline;
+            try {
+                pipeline = Pipeline::makePipeline(_fromPipeline, _fromExpCtx, pipelineOpts);
+            } catch (const ExceptionFor<ErrorCodes::CommandOnShardedViewNotSupportedOnMongod>& e) {
+                // This exception returns the information we need to resolve a sharded view. Update
+                // the pipeline with the resolved view definition, but don't optimize or attach the
+                // cursor source yet.
+                MakePipelineOptions opts;
+                opts.optimize = false;
+                opts.attachCursorSource = false;
+                pipeline = Pipeline::makePipelineFromViewDefinition(
+                    _fromExpCtx,
+                    ExpressionContext::ResolvedNamespace{e->getNamespace(), e->getPipeline()},
+                    _fromPipeline,
+                    opts);
+
+                // Update '_fromPipeline' with the resolved view definition to avoid triggering this
+                // exception next time.
+                _fromPipeline = pipeline->serializeToBson();
+
+                // Update the expression context with any new namespaces the resolved pipeline has
+                // introduced.
+                LiteParsedPipeline liteParsedPipeline(e->getNamespace(), e->getPipeline());
+                _fromExpCtx = _fromExpCtx->copyWith(e->getNamespace());
+                _fromExpCtx->addResolvedNamespaces(liteParsedPipeline.getInvolvedNamespaces());
+
+                LOGV2_DEBUG(
+                    5865400,
+                    3,
+                    "$graphLookup found view definition. ns: {namespace}, pipeline: {pipeline}. "
+                    "New $graphLookup sub-pipeline: {new_pipe}",
+                    logAttrs(e->getNamespace()),
+                    "pipeline"_attr = Value(e->getPipeline()),
+                    "new_pipe"_attr = _fromPipeline);
+
+                // We can now safely optimize and reattempt attaching the cursor source.
+                pipeline = Pipeline::makePipeline(_fromPipeline, _fromExpCtx, pipelineOpts);
+            }
+
             while (auto next = pipeline->getNext()) {
                 uassert(40271,
                         str::stream()
@@ -314,6 +402,12 @@ boost::optional<BSONObj> DocumentSourceGraphLookUp::makeMatchStageFromFrontier(
     //
     // We wrap the query in a $match so that it can be parsed into a DocumentSourceMatch when
     // constructing a pipeline to execute.
+
+    // $match stages will conflate null, and undefined values. Keep track of which ones are
+    // present and eliminate documents that would match the others later.
+    bool matchNull = false;
+    bool matchUndefined = false;
+    bool seenMissing = false;
     BSONObjBuilder match;
     {
         BSONObjBuilder query(match.subobjStart("$match"));
@@ -330,11 +424,39 @@ boost::optional<BSONObj> DocumentSourceGraphLookUp::makeMatchStageFromFrontier(
                     {
                         BSONArrayBuilder in(subObj.subarrayStart("$in"));
                         for (auto&& value : _frontier) {
+                            if (value.getType() == BSONType::jstNULL) {
+                                matchNull = true;
+                            } else if (value.getType() == BSONType::Undefined) {
+                                matchUndefined = true;
+                            } else if (value.missing()) {
+                                seenMissing = true;
+                            }
                             in << value;
                         }
                     }
                 }
             }
+            // We never want to see documents where the 'connectToField' is missing. Only add a
+            // check for it in situations where we might match it accidentally.
+            if (matchNull || matchUndefined || seenMissing) {
+                auto existsMatch = BSON(_connectToField.fullPath() << BSON("$exists" << true));
+                andObj << existsMatch;
+            }
+            // If matching null or undefined, make sure we don't match the other one.
+            // If seenMissing is true, we've already filtered out missing values above.
+            if (matchNull || matchUndefined) {
+                if (!matchUndefined) {
+                    auto notUndefined =
+                        BSON(_connectToField.fullPath() << BSON("$not" << BSON("$type"
+                                                                               << "undefined")));
+                    andObj << notUndefined;
+                } else if (!matchNull) {
+                    auto notUndefined =
+                        BSON(_connectToField.fullPath() << BSON("$not" << BSON("$type"
+                                                                               << "null")));
+                    andObj << notUndefined;
+                }
+            }
         }
     }
 
@@ -358,7 +480,25 @@ void DocumentSourceGraphLookUp::performSearch() {
         _frontierUsageBytes += startingValue.getApproximateSize();
     }
 
-    doBreadthFirstSearch();
+    try {
+        doBreadthFirstSearch();
+    } catch (const ExceptionForCat<ErrorCategory::StaleShardVersionError>& ex) {
+        // If lookup on a sharded collection is disallowed and the foreign collection is sharded,
+        // throw a custom exception.
+        if (auto staleInfo = ex.extraInfo<StaleConfigInfo>(); staleInfo &&
+            staleInfo->getVersionWanted() &&
+            staleInfo->getVersionWanted() != ChunkVersion::UNSHARDED()) {
+            uassert(3904801,
+                    "Cannot run $graphLookup with a sharded foreign collection in a transaction",
+                    !feature_flags::gFeatureFlagShardedLookup.isEnabled(
+                        serverGlobalParams.featureCompatibility) ||
+                        !pExpCtx->opCtx->inMultiDocumentTransaction());
+            uassert(31428,
+                    "Cannot run $graphLookup with sharded foreign collection",
+                    foreignShardedGraphLookupAllowed());
+        }
+        throw;
+    }
 }
 
 DocumentSource::GetModPathsReturn DocumentSourceGraphLookUp::getModifiedPaths() const {
@@ -401,8 +541,12 @@ void DocumentSourceGraphLookUp::checkMemoryUsage() {
 
 void DocumentSourceGraphLookUp::serializeToArray(
     std::vector<Value>& array, boost::optional<ExplainOptions::Verbosity> explain) const {
+    auto fromValue = (pExpCtx->ns.db() == _from.db())
+        ? Value(_from.coll())
+        : Value(Document{{"db", _from.db()}, {"coll", _from.coll()}});
+
     // Serialize default options.
-    MutableDocument spec(DOC("from" << _from.coll() << "as" << _as.fullPath() << "connectToField"
+    MutableDocument spec(DOC("from" << fromValue << "as" << _as.fullPath() << "connectToField"
                                     << _connectToField.fullPath() << "connectFromField"
                                     << _connectFromField.fullPath() << "startWith"
                                     << _startWith->serialize(false)));
@@ -469,9 +613,11 @@ DocumentSourceGraphLookUp::DocumentSourceGraphLookUp(
       _frontier(pExpCtx->getValueComparator().makeUnorderedValueSet()),
       _visited(ValueComparator::kInstance.makeUnorderedValueMap<Document>()),
       _cache(pExpCtx->getValueComparator()),
-      _unwind(unwindSrc) {
+      _unwind(unwindSrc),
+      _variables(expCtx->variables),
+      _variablesParseState(expCtx->variablesParseState.copyWith(_variables.useIdGenerator())) {
     const auto& resolvedNamespace = pExpCtx->getResolvedNamespace(_from);
-    _fromExpCtx = pExpCtx->copyWith(resolvedNamespace.ns);
+    _fromExpCtx = pExpCtx->copyForSubPipeline(resolvedNamespace.ns);
 
     // We append an additional BSONObj to '_fromPipeline' as a placeholder for the $match stage
     // we'll eventually construct from the input document.
@@ -480,6 +626,28 @@ DocumentSourceGraphLookUp::DocumentSourceGraphLookUp(
     _fromPipeline.push_back(BSON("$match" << BSONObj()));
 }
 
+DocumentSourceGraphLookUp::DocumentSourceGraphLookUp(const DocumentSourceGraphLookUp& original)
+    : DocumentSource(kStageName, original.pExpCtx->copyWith(original.pExpCtx->ns)),
+      _from(original._from),
+      _as(original._as),
+      _connectFromField(original._connectFromField),
+      _connectToField(original._connectToField),
+      _startWith(original._startWith),
+      _additionalFilter(original._additionalFilter),
+      _depthField(original._depthField),
+      _maxDepth(original._maxDepth),
+      _fromExpCtx(original._fromExpCtx->copyWith(original.pExpCtx->getResolvedNamespace(_from).ns)),
+      _fromPipeline(original._fromPipeline),
+      _frontier(pExpCtx->getValueComparator().makeUnorderedValueSet()),
+      _visited(ValueComparator::kInstance.makeUnorderedValueMap<Document>()),
+      _cache(pExpCtx->getValueComparator()),
+      _variables(original._variables),
+      _variablesParseState(original._variablesParseState.copyWith(_variables.useIdGenerator())) {
+    if (original._unwind) {
+        _unwind = static_cast<DocumentSourceUnwind*>(original._unwind.get()->clone().get());
+    }
+}
+
 intrusive_ptr<DocumentSourceGraphLookUp> DocumentSourceGraphLookUp::create(
     const intrusive_ptr<ExpressionContext>& expCtx,
     NamespaceString fromNs,
@@ -522,7 +690,7 @@ intrusive_ptr<DocumentSource> DocumentSourceGraphLookUp::createFromBson(
         const auto argName = argument.fieldNameStringData();
 
         if (argName == "startWith") {
-            startWith = Expression::parseOperand(expCtx, argument, vps);
+            startWith = Expression::parseOperand(expCtx.get(), argument, vps);
             continue;
         } else if (argName == "maxDepth") {
             uassert(40100,
@@ -557,15 +725,18 @@ intrusive_ptr<DocumentSource> DocumentSourceGraphLookUp::createFromBson(
 
         if (argName == "from" || argName == "as" || argName == "connectFromField" ||
             argName == "depthField" || argName == "connectToField") {
-            // All remaining arguments to $graphLookup are expected to be strings.
+            // All remaining arguments to $graphLookup are expected to be strings or
+            // {db: "local", coll: "system.tenantMigration.oplogView"}.
+            // 'local.system.tenantMigration.oplogView' is not a user namespace so object
+            // type is omitted from the error message below.
             uassert(40103,
                     str::stream() << "expected string as argument for " << argName
-                                  << ", found: " << argument.toString(false, false),
-                    argument.type() == String);
+                                  << ", found: " << typeName(argument.type()),
+                    argument.type() == String || argument.type() == Object);
         }
 
         if (argName == "from") {
-            from = NamespaceString(expCtx->ns.db().toString() + '.' + argument.String());
+            from = parseGraphLookupFromAndResolveNamespace(argument, expCtx->ns.db().toString());
         } else if (argName == "as") {
             as = argument.String();
         } else if (argName == "connectFromField") {
@@ -601,7 +772,11 @@ intrusive_ptr<DocumentSource> DocumentSourceGraphLookUp::createFromBson(
                                       maxDepth,
                                       boost::none));
 
-    return std::move(newSource);
+    return newSource;
+}
+
+boost::intrusive_ptr<DocumentSource> DocumentSourceGraphLookUp::clone() const {
+    return make_intrusive<DocumentSourceGraphLookUp>(*this);
 }
 
 void DocumentSourceGraphLookUp::addInvolvedCollections(
