@@ -115,8 +115,13 @@ template <typename MatchType>
 auto constructObjectIdValue(const BSONElement& rhs, int bucketMaxSpanSeconds) {
     // Indicates whether to initialize an ObjectId with a max or min value for the non-date bytes.
     enum class OIDInit : bool { max, min };
-    // Make an ObjectId cooresponding to a date value.
-    auto makeDateOID = [](auto&& date, auto&& maxOrMin) {
+    // Make an ObjectId cooresponding to a date value. As a conversion from date to ObjectId will
+    // truncate milliseconds, we round up when needed to prevent missing results.
+    auto makeDateOID = [](auto&& date, auto&& maxOrMin, bool roundMillisUpToSecond = false) {
+        if (roundMillisUpToSecond && (date.toMillisSinceEpoch() % 1000 != 0)) {
+            date += Seconds{1};
+        }
+
         auto oid = OID{};
         oid.init(date, maxOrMin == OIDInit::max);
         return oid;
@@ -142,9 +147,9 @@ auto constructObjectIdValue(const BSONElement& rhs, int bucketMaxSpanSeconds) {
     // make this case faster by keeping the ObjectId as the lowest or highest possible value so as
     // to eliminate all buckets.
     if constexpr (std::is_same_v<MatchType, LTMatchExpression>) {
-        return Value{makeDateOID(rhs.date(), OIDInit::min)};
+        return Value{makeDateOID(rhs.date(), OIDInit::min, true /*roundMillisUpToSecond*/)};
     } else if constexpr (std::is_same_v<MatchType, LTEMatchExpression>) {
-        return Value{makeDateOID(rhs.date(), OIDInit::max)};
+        return Value{makeDateOID(rhs.date(), OIDInit::max, true /*roundMillisUpToSecond*/)};
     } else if constexpr (std::is_same_v<MatchType, GTMatchExpression>) {
         return Value{makeMaxAdjustedDateOID(rhs.date(), OIDInit::max)};
     } else if constexpr (std::is_same_v<MatchType, GTEMatchExpression>) {
@@ -209,7 +214,7 @@ void optimizePrefix(Pipeline::SourceContainer::iterator itr, Pipeline::SourceCon
 // Returns whether 'field' depends on a pushed down $addFields or computed $project.
 bool fieldIsComputed(BucketSpec spec, std::string field) {
     return std::any_of(
-        spec.computedMetaProjFields.begin(), spec.computedMetaProjFields.end(), [&](auto& s) {
+        spec.computedMetaProjFields().begin(), spec.computedMetaProjFields().end(), [&](auto& s) {
             return s == field || expression::isPathPrefixOf(field, s) ||
                 expression::isPathPrefixOf(s, field);
         });
@@ -263,7 +268,7 @@ boost::intrusive_ptr<DocumentSource> DocumentSourceInternalUnpackBucket::createF
                 uassert(5346503,
                         "include or exclude field element must be a single-element field path",
                         field.find('.') == std::string::npos);
-                bucketSpec.fieldSet.emplace(field);
+                bucketSpec.addIncludeExcludeField(field);
             }
             unpackerBehavior = fieldName == kInclude ? BucketUnpacker::Behavior::kInclude
                                                      : BucketUnpacker::Behavior::kExclude;
@@ -272,7 +277,7 @@ boost::intrusive_ptr<DocumentSource> DocumentSourceInternalUnpackBucket::createF
             uassert(5346504,
                     str::stream() << "timeField field must be a string, got: " << elem.type(),
                     elem.type() == BSONType::String);
-            bucketSpec.timeField = elem.str();
+            bucketSpec.setTimeField(elem.str());
             hasTimeField = true;
         } else if (fieldName == timeseries::kMetaFieldName) {
             uassert(5346505,
@@ -282,7 +287,7 @@ boost::intrusive_ptr<DocumentSource> DocumentSourceInternalUnpackBucket::createF
             uassert(5545700,
                     str::stream() << "metaField field must be a single-element field path",
                     metaField.find('.') == std::string::npos);
-            bucketSpec.metaField = std::move(metaField);
+            bucketSpec.setMetaField(std::move(metaField));
         } else if (fieldName == kBucketMaxSpanSeconds) {
             uassert(5510600,
                     str::stream() << "bucketMaxSpanSeconds field must be an integer, got: "
@@ -309,7 +314,7 @@ boost::intrusive_ptr<DocumentSource> DocumentSourceInternalUnpackBucket::createF
                 uassert(5509902,
                         "computedMetaProjFields field element must be a single-element field path",
                         field.find('.') == std::string::npos);
-                bucketSpec.computedMetaProjFields.emplace_back(field);
+                bucketSpec.addComputedMetaProjFields(field);
             }
         } else {
             uasserted(5346506,
@@ -345,7 +350,7 @@ boost::intrusive_ptr<DocumentSource> DocumentSourceInternalUnpackBucket::createF
             uassert(5612401,
                     str::stream() << "timeField field must be a string, got: " << elem.type(),
                     elem.type() == BSONType::String);
-            bucketSpec.timeField = elem.str();
+            bucketSpec.setTimeField(elem.str());
             hasTimeField = true;
         } else if (fieldName == timeseries::kMetaFieldName) {
             uassert(5612402,
@@ -355,7 +360,7 @@ boost::intrusive_ptr<DocumentSource> DocumentSourceInternalUnpackBucket::createF
             uassert(5612403,
                     str::stream() << "metaField field must be a single-element field path",
                     metaField.find('.') == std::string::npos);
-            bucketSpec.metaField = std::move(metaField);
+            bucketSpec.setMetaField(std::move(metaField));
         } else {
             uasserted(5612404,
                       str::stream() << "unrecognized parameter to $_unpackBucket: " << fieldName);
@@ -374,32 +379,32 @@ void DocumentSourceInternalUnpackBucket::serializeToArray(
     MutableDocument out;
     auto behavior =
         _bucketUnpacker.behavior() == BucketUnpacker::Behavior::kInclude ? kInclude : kExclude;
-    auto&& spec = _bucketUnpacker.bucketSpec();
+    const auto& spec = _bucketUnpacker.bucketSpec();
     std::vector<Value> fields;
-    for (auto&& field : spec.fieldSet) {
+    for (auto&& field : spec.fieldSet()) {
         fields.emplace_back(field);
     }
     if (((_bucketUnpacker.includeMetaField() &&
           _bucketUnpacker.behavior() == BucketUnpacker::Behavior::kInclude) ||
          (!_bucketUnpacker.includeMetaField() &&
-          _bucketUnpacker.behavior() == BucketUnpacker::Behavior::kExclude && spec.metaField)) &&
-        std::find(spec.computedMetaProjFields.cbegin(),
-                  spec.computedMetaProjFields.cend(),
-                  *spec.metaField) == spec.computedMetaProjFields.cend())
-        fields.emplace_back(*spec.metaField);
+          _bucketUnpacker.behavior() == BucketUnpacker::Behavior::kExclude && spec.metaField())) &&
+        std::find(spec.computedMetaProjFields().cbegin(),
+                  spec.computedMetaProjFields().cend(),
+                  *spec.metaField()) == spec.computedMetaProjFields().cend())
+        fields.emplace_back(*spec.metaField());
 
     out.addField(behavior, Value{std::move(fields)});
-    out.addField(timeseries::kTimeFieldName, Value{spec.timeField});
-    if (spec.metaField) {
-        out.addField(timeseries::kMetaFieldName, Value{*spec.metaField});
+    out.addField(timeseries::kTimeFieldName, Value{spec.timeField()});
+    if (spec.metaField()) {
+        out.addField(timeseries::kMetaFieldName, Value{*spec.metaField()});
     }
     out.addField(kBucketMaxSpanSeconds, Value{_bucketMaxSpanSeconds});
 
-    if (!spec.computedMetaProjFields.empty())
+    if (!spec.computedMetaProjFields().empty())
         out.addField("computedMetaProjFields", Value{[&] {
                          std::vector<Value> compFields;
-                         std::transform(spec.computedMetaProjFields.cbegin(),
-                                        spec.computedMetaProjFields.cend(),
+                         std::transform(spec.computedMetaProjFields().cbegin(),
+                                        spec.computedMetaProjFields().cend(),
                                         std::back_inserter(compFields),
                                         [](auto&& projString) { return Value{projString}; });
                          return compFields;
@@ -450,7 +455,7 @@ bool DocumentSourceInternalUnpackBucket::pushDownComputedMetaProjection(
     if (std::next(itr) == container->end()) {
         return nextStageWasRemoved;
     }
-    if (!_bucketUnpacker.bucketSpec().metaField) {
+    if (!_bucketUnpacker.bucketSpec().metaField()) {
         return nextStageWasRemoved;
     }
 
@@ -460,7 +465,7 @@ bool DocumentSourceInternalUnpackBucket::pushDownComputedMetaProjection(
         (nextTransform->getType() == TransformerInterface::TransformerType::kInclusionProjection ||
          nextTransform->getType() == TransformerInterface::TransformerType::kComputedProjection)) {
 
-        auto& metaName = _bucketUnpacker.bucketSpec().metaField.get();
+        auto& metaName = _bucketUnpacker.bucketSpec().metaField().get();
         auto [addFieldsSpec, deleteStage] =
             nextTransform->extractComputedProjections(metaName,
                                                       timeseries::kBucketMetaFieldName.toString(),
@@ -501,7 +506,7 @@ void DocumentSourceInternalUnpackBucket::internalizeProject(const BSONObj& proje
 
     // Update '_bucketUnpacker' state with the new fields and behavior.
     auto spec = _bucketUnpacker.bucketSpec();
-    spec.fieldSet = std::move(fields);
+    spec.setFieldSet(fields);
     _bucketUnpacker.setBucketSpecAndBehavior(std::move(spec),
                                              isInclusion ? BucketUnpacker::Behavior::kInclude
                                                          : BucketUnpacker::Behavior::kExclude);
@@ -509,7 +514,7 @@ void DocumentSourceInternalUnpackBucket::internalizeProject(const BSONObj& proje
 
 std::pair<BSONObj, bool> DocumentSourceInternalUnpackBucket::extractOrBuildProjectToInternalize(
     Pipeline::SourceContainer::iterator itr, Pipeline::SourceContainer* container) const {
-    if (std::next(itr) == container->end() || !_bucketUnpacker.bucketSpec().fieldSet.empty()) {
+    if (std::next(itr) == container->end() || !_bucketUnpacker.bucketSpec().fieldSet().empty()) {
         // There is no project to internalize or there are already fields being included/excluded.
         return {BSONObj{}, false};
     }
@@ -622,9 +627,9 @@ std::unique_ptr<MatchExpression> createComparisonPredicate(
     }
 
     // We must avoid mapping predicates on the meta field onto the control field.
-    if (bucketSpec.metaField &&
-        (matchExprPath == bucketSpec.metaField.get() ||
-         expression::isPathPrefixOf(bucketSpec.metaField.get(), matchExprPath)))
+    if (bucketSpec.metaField() &&
+        (matchExprPath == bucketSpec.metaField().get() ||
+         expression::isPathPrefixOf(bucketSpec.metaField().get(), matchExprPath)))
         return nullptr;
 
     // We must avoid mapping predicates on fields computed via $addFields or a computed $project.
@@ -632,7 +637,7 @@ std::unique_ptr<MatchExpression> createComparisonPredicate(
         return nullptr;
     }
 
-    const auto isTimeField = (matchExprPath == bucketSpec.timeField);
+    const auto isTimeField = (matchExprPath == bucketSpec.timeField());
     if (isTimeField && matchExprData.type() != BSONType::Date) {
         // Users are not allowed to insert non-date measurements into time field. So this query
         // would not match anything. We do not need to optimize for this case.
@@ -813,7 +818,7 @@ DocumentSourceInternalUnpackBucket::createPredicatesOnBucketLevelField(
 std::pair<boost::intrusive_ptr<DocumentSourceMatch>, boost::intrusive_ptr<DocumentSourceMatch>>
 DocumentSourceInternalUnpackBucket::splitMatchOnMetaAndRename(
     boost::intrusive_ptr<DocumentSourceMatch> match) {
-    if (auto&& metaField = _bucketUnpacker.bucketSpec().metaField) {
+    if (auto&& metaField = _bucketUnpacker.bucketSpec().metaField()) {
         return std::move(*match).extractMatchOnFieldsAndRemainder(
             {*metaField}, {{*metaField, timeseries::kBucketMetaFieldName.toString()}});
     }
@@ -823,10 +828,10 @@ DocumentSourceInternalUnpackBucket::splitMatchOnMetaAndRename(
 std::pair<BSONObj, bool> DocumentSourceInternalUnpackBucket::extractProjectForPushDown(
     DocumentSource* src) const {
     if (auto nextProject = dynamic_cast<DocumentSourceSingleDocumentTransformation*>(src);
-        _bucketUnpacker.bucketSpec().metaField && nextProject &&
+        _bucketUnpacker.bucketSpec().metaField() && nextProject &&
         nextProject->getType() == TransformerInterface::TransformerType::kExclusionProjection) {
         return nextProject->extractProjectOnFieldAndRename(
-            _bucketUnpacker.bucketSpec().metaField.get(), timeseries::kBucketMetaFieldName);
+            _bucketUnpacker.bucketSpec().metaField().get(), timeseries::kBucketMetaFieldName);
     }
 
     return {BSONObj{}, false};
@@ -841,7 +846,7 @@ DocumentSourceInternalUnpackBucket::rewriteGroupByMinMax(Pipeline::SourceContain
     }
 
     const auto& idFields = groupPtr->getIdFields();
-    if (idFields.size() != 1 || !_bucketUnpacker.bucketSpec().metaField.has_value()) {
+    if (idFields.size() != 1 || !_bucketUnpacker.bucketSpec().metaField().has_value()) {
         return {};
     }
 
@@ -853,7 +858,7 @@ DocumentSourceInternalUnpackBucket::rewriteGroupByMinMax(Pipeline::SourceContain
 
     const auto& idPath = exprIdPath->getFieldPath();
     if (idPath.getPathLength() < 2 ||
-        idPath.getFieldName(1) != _bucketUnpacker.bucketSpec().metaField.get()) {
+        idPath.getFieldName(1) != _bucketUnpacker.bucketSpec().metaField().get()) {
         return {};
     }
 
@@ -874,7 +879,7 @@ DocumentSourceInternalUnpackBucket::rewriteGroupByMinMax(Pipeline::SourceContain
         if (const auto* exprArgPath = dynamic_cast<const ExpressionFieldPath*>(exprArg)) {
             const auto& path = exprArgPath->getFieldPath();
             if (path.getPathLength() <= 1 ||
-                path.getFieldName(1) == _bucketUnpacker.bucketSpec().timeField) {
+                path.getFieldName(1) == _bucketUnpacker.bucketSpec().timeField()) {
                 // Rewrite not valid for time field. We want to eliminate the bucket
                 // unpack stage here.
                 suitable = false;
@@ -946,12 +951,13 @@ Pipeline::SourceContainer::iterator DocumentSourceInternalUnpackBucket::doOptimi
 
     // Some optimizations may not be safe to do if we have computed the metaField via an $addFields
     // or a computed $project. We won't do those optimizations if 'haveComputedMetaField' is true.
-    bool haveComputedMetaField = _bucketUnpacker.bucketSpec().metaField &&
-        fieldIsComputed(_bucketUnpacker.bucketSpec(), _bucketUnpacker.bucketSpec().metaField.get());
+    bool haveComputedMetaField = _bucketUnpacker.bucketSpec().metaField() &&
+        fieldIsComputed(_bucketUnpacker.bucketSpec(),
+                        _bucketUnpacker.bucketSpec().metaField().get());
 
     // Before any other rewrites for the current stage, consider reordering with $sort.
     if (auto sortPtr = dynamic_cast<DocumentSourceSort*>(std::next(itr)->get())) {
-        if (auto metaField = _bucketUnpacker.bucketSpec().metaField;
+        if (auto metaField = _bucketUnpacker.bucketSpec().metaField();
             metaField && !haveComputedMetaField) {
             if (checkMetadataSortReorder(sortPtr->getSortKeyPattern(), metaField.get())) {
                 // We have a sort on metadata field following this stage. Reorder the two stages
@@ -995,7 +1001,7 @@ Pipeline::SourceContainer::iterator DocumentSourceInternalUnpackBucket::doOptimi
             "Must not specify 'query' for $geoNear on a time-series collection; use $match instead",
             nextNear->getQuery().binaryEqual(BSONObj()));
 
-        auto metaField = _bucketUnpacker.bucketSpec().metaField;
+        auto metaField = _bucketUnpacker.bucketSpec().metaField();
         if (metaField && *metaField == keyField->front()) {
             // Make sure we actually re-write the key field for the buckets collection so we can
             // locate the index.
@@ -1049,8 +1055,8 @@ Pipeline::SourceContainer::iterator DocumentSourceInternalUnpackBucket::doOptimi
         auto deps = Pipeline::getDependenciesForContainer(
             pExpCtx, Pipeline::SourceContainer{std::next(itr), container->end()}, boost::none);
         if (deps.hasNoRequirements()) {
-            _bucketUnpacker.setBucketSpecAndBehavior({_bucketUnpacker.bucketSpec().timeField,
-                                                      _bucketUnpacker.bucketSpec().metaField,
+            _bucketUnpacker.setBucketSpecAndBehavior({_bucketUnpacker.bucketSpec().timeField(),
+                                                      _bucketUnpacker.bucketSpec().metaField(),
                                                       {}},
                                                      BucketUnpacker::Behavior::kInclude);
 
