@@ -724,55 +724,6 @@ TEST_F(OptimizePipeline, InternalizeProjectAndPushdownAddFields) {
                       serialized[1]);
 }
 
-TEST_F(OptimizePipeline, DoNotSwapAddFieldsIfDependencyIsExcluded) {
-    {
-        auto unpackSpecObj = fromjson(
-            "{$_internalUnpackBucket: { exclude: [], timeField: 'time', metaField: 'myMeta', "
-            "bucketMaxSpanSeconds: 3600}}");
-        auto projectSpecObj = fromjson("{$project: {x: true, _id: false}}");
-        auto addFieldsSpec = fromjson("{$addFields: {newMeta: '$myMeta'}}");
-
-        auto pipeline =
-            Pipeline::parse(makeVector(unpackSpecObj, projectSpecObj, addFieldsSpec), getExpCtx());
-
-        pipeline->optimizePipeline();
-
-        // We should internalize the $project but _not_ push down the $addFields because it's field
-        // dependency has been excluded. Theoretically we could remove the $addFields for this
-        // trivial except but not always.
-        auto serialized = pipeline->serializeToBson();
-        ASSERT_EQ(2u, serialized.size());
-        ASSERT_BSONOBJ_EQ(fromjson("{$_internalUnpackBucket: { include: ['x'], timeField: 'time', "
-                                   "metaField: 'myMeta', bucketMaxSpanSeconds: 3600}}"),
-                          serialized[0]);
-        ASSERT_BSONOBJ_EQ(fromjson("{$addFields: {newMeta: '$myMeta'}}"), serialized[1]);
-    }
-
-    // Similar test except the dependency is on an excluded non-meta field.
-    {
-        auto unpackSpecObj = fromjson(
-            "{$_internalUnpackBucket: { exclude: [], timeField: 'time', metaField: 'myMeta', "
-            "bucketMaxSpanSeconds: 3600}}");
-        auto projectSpecObj = fromjson("{$project: {x: true, _id: false}}");
-        auto addFieldsSpec = fromjson("{$addFields: {newMeta: '$excluded'}}");
-
-        auto pipeline =
-            Pipeline::parse(makeVector(unpackSpecObj, projectSpecObj, addFieldsSpec), getExpCtx());
-
-        pipeline->optimizePipeline();
-
-        // We should internalize the $project but _not_ push down the $addFields because it's field
-        // dependency has been excluded. Theoretically we could remove the $addFields for this
-        // trivial except but not always.
-        auto serialized = pipeline->serializeToBson();
-        ASSERT_EQ(2u, serialized.size());
-        ASSERT_BSONOBJ_EQ(fromjson("{$_internalUnpackBucket: { include: ['x'], timeField: 'time', "
-                                   "metaField: 'myMeta', bucketMaxSpanSeconds: 3600}}"),
-                          serialized[0]);
-        ASSERT_BSONOBJ_EQ(fromjson("{$addFields: {newMeta: '$excluded'}}"), serialized[1]);
-    }
-}
-
 TEST_F(OptimizePipeline, PushdownSortAndAddFields) {
     auto unpackSpecObj = fromjson(
         "{$_internalUnpackBucket: { exclude: [], timeField: 'time', metaField: 'myMeta', "
@@ -817,5 +768,118 @@ TEST_F(OptimizePipeline, PushdownMatchAndAddFields) {
         serialized[2]);
     ASSERT_BSONOBJ_EQ(fromjson("{$addFields: {z: {$add : ['$x', '$y']}}}"), serialized[3]);
 }
+
+TEST_F(OptimizePipeline, MatchWithGeoWithinOnMeasurementsPushedDownUsingInternalBucketGeoWithin) {
+    auto pipeline = Pipeline::parse(
+        makeVector(fromjson("{$_internalUnpackBucket: {exclude: [], timeField: "
+                            "'time', bucketMaxSpanSeconds: 3600}}"),
+                   fromjson("{$match: {loc: {$geoWithin: {$geometry: {type: \"Polygon\", "
+                            "coordinates: [ [ [ 0, 0 ], [ 3, 6 ], [ 6, 1 ], [ 0, 0 ] ] ]}}}}}")),
+        getExpCtx());
+
+    ASSERT_EQ(pipeline->getSources().size(), 2U);
+
+    pipeline->optimizePipeline();
+
+    auto serialized = pipeline->serializeToBson();
+    ASSERT_EQ(serialized.size(), 3U);
+
+    // $match with $geoWithin on a non-metadata field is pushed down and $_internalBucketGeoWithin
+    // is used.
+    ASSERT_BSONOBJ_EQ(
+        fromjson("{ $match: {$_internalBucketGeoWithin: { withinRegion: { $geometry: { type : "
+                 "\"Polygon\" ,coordinates: [ [ [ 0, 0 ], [ 3, 6 ], [ 6, 1 ], [ 0, 0 "
+                 "] ] ]}},field: \"loc\"}}}"),
+        serialized[0]);
+    ASSERT_BSONOBJ_EQ(fromjson("{$_internalUnpackBucket: {exclude: [], timeField: "
+                               "'time', bucketMaxSpanSeconds: 3600}}"),
+                      serialized[1]);
+    ASSERT_BSONOBJ_EQ(fromjson("{$match: {loc: {$geoWithin: {$geometry: {type: \"Polygon\", "
+                               "coordinates: [ [ [ 0, 0 ], [ 3, 6 ], [ 6, 1 ], [ 0, 0 ] ] ]}}}}}"),
+                      serialized[2]);
+}
+
+TEST_F(OptimizePipeline, MatchWithGeoWithinOnMetaFieldIsPushedDown) {
+    auto pipeline = Pipeline::parse(
+        makeVector(fromjson("{$_internalUnpackBucket: {exclude: [], timeField: "
+                            "'time', metaField: 'myMeta', bucketMaxSpanSeconds: 3600}}"),
+                   fromjson("{$match: {'myMeta.loc': {$geoWithin: {$geometry: {type: \"Polygon\", "
+                            "coordinates: [ [ [ 0, 0 ], [ 3, 6 ], [ 6, 1 ], [ 0, 0 ] ] ]}}}}}")),
+        getExpCtx());
+
+    ASSERT_EQ(pipeline->getSources().size(), 2U);
+
+    pipeline->optimizePipeline();
+
+    auto serialized = pipeline->serializeToBson();
+    ASSERT_EQ(serialized.size(), 2U);
+
+    // $match with $geoWithin on the metadata field is pushed down without using
+    // $_internalBucketGeoWithin.
+    ASSERT_BSONOBJ_EQ(fromjson("{$match: {'meta.loc': {$geoWithin: {$geometry: {type: \"Polygon\", "
+                               "coordinates: [ [ [ 0, 0 ], [ 3, 6 ], [ 6, 1 ], [ 0, 0 ] ] ]}}}}}"),
+                      serialized[0]);
+    ASSERT_BSONOBJ_EQ(fromjson("{$_internalUnpackBucket: {exclude: [], timeField: "
+                               "'time', metaField: 'myMeta', bucketMaxSpanSeconds: 3600}}"),
+                      serialized[1]);
+}
+
+TEST_F(OptimizePipeline,
+       MatchWithGeoIntersectsOnMeasurementsPushedDownUsingInternalBucketGeoWithin) {
+    auto pipeline = Pipeline::parse(
+        makeVector(fromjson("{$_internalUnpackBucket: {exclude: [], timeField: "
+                            "'time', bucketMaxSpanSeconds: 3600}}"),
+                   fromjson("{$match: {loc: {$geoIntersects: {$geometry: {type: \"Polygon\", "
+                            "coordinates: [ [ [ 0, 0 ], [ 3, 6 ], [ 6, 1 ], [ 0, 0 ] ] ]}}}}}")),
+        getExpCtx());
+
+    ASSERT_EQ(pipeline->getSources().size(), 2U);
+
+    pipeline->optimizePipeline();
+
+    auto serialized = pipeline->serializeToBson();
+    ASSERT_EQ(serialized.size(), 3U);
+
+    // $match with $geoIntersects on a non-metadata field is pushed down and
+    // $_internalBucketGeoWithin is used.
+    ASSERT_BSONOBJ_EQ(
+        fromjson("{ $match: {$_internalBucketGeoWithin: { withinRegion: { $geometry: { type : "
+                 "\"Polygon\" ,coordinates: [ [ [ 0, 0 ], [ 3, 6 ], [ 6, 1 ], [ 0, 0 "
+                 "] ] ]}},field: \"loc\"}}}"),
+        serialized[0]);
+    ASSERT_BSONOBJ_EQ(fromjson("{$_internalUnpackBucket: {exclude: [], timeField: "
+                               "'time', bucketMaxSpanSeconds: 3600}}"),
+                      serialized[1]);
+    ASSERT_BSONOBJ_EQ(fromjson("{$match: {loc: {$geoIntersects: {$geometry: {type: \"Polygon\", "
+                               "coordinates: [ [ [ 0, 0 ], [ 3, 6 ], [ 6, 1 ], [ 0, 0 ] ] ]}}}}}"),
+                      serialized[2]);
+}
+
+TEST_F(OptimizePipeline, MatchWithGeoIntersectsOnMetaFieldIsPushedDown) {
+    auto pipeline = Pipeline::parse(
+        makeVector(
+            fromjson("{$_internalUnpackBucket: {exclude: [], timeField: "
+                     "'time', metaField: 'myMeta', bucketMaxSpanSeconds: 3600}}"),
+            fromjson("{$match: {'myMeta.loc': {$geoIntersects: {$geometry: {type: \"Polygon\", "
+                     "coordinates: [ [ [ 0, 0 ], [ 3, 6 ], [ 6, 1 ], [ 0, 0 ] ] ]}}}}}")),
+        getExpCtx());
+
+    ASSERT_EQ(pipeline->getSources().size(), 2U);
+
+    pipeline->optimizePipeline();
+
+    auto serialized = pipeline->serializeToBson();
+    ASSERT_EQ(serialized.size(), 2U);
+
+    // $match with $geoIntersects on the metadata field is pushed down without using
+    // $_internalBucketGeoWithin.
+    ASSERT_BSONOBJ_EQ(
+        fromjson("{$match: {'meta.loc': {$geoIntersects: {$geometry: {type: \"Polygon\", "
+                 "coordinates: [ [ [ 0, 0 ], [ 3, 6 ], [ 6, 1 ], [ 0, 0 ] ] ]}}}}}"),
+        serialized[0]);
+    ASSERT_BSONOBJ_EQ(fromjson("{$_internalUnpackBucket: {exclude: [], timeField: "
+                               "'time', metaField: 'myMeta', bucketMaxSpanSeconds: 3600}}"),
+                      serialized[1]);
+}
 }  // namespace
 }  // namespace mongo
