@@ -74,20 +74,23 @@ public:
         return _isCommutative;
     }
 
-    void acceptVisitor(ExpressionVisitor* visitor) final {
+    void acceptVisitor(ExpressionMutableVisitor* visitor) final {
         return visitor->visit(this);
     }
 
-    static intrusive_ptr<Testable> create(bool associative, bool commutative) {
-        return new Testable(associative, commutative);
+    void acceptVisitor(ExpressionConstVisitor* visitor) const final {
+        return visitor->visit(this);
+    }
+
+    static intrusive_ptr<Testable> create(ExpressionContext* const expCtx,
+                                          bool associative,
+                                          bool commutative) {
+        return new Testable(expCtx, associative, commutative);
     }
 
 private:
-    Testable(bool isAssociative, bool isCommutative)
-        : ExpressionNary(
-              boost::intrusive_ptr<ExpressionContextForTest>(new ExpressionContextForTest())),
-          _isAssociative(isAssociative),
-          _isCommutative(isCommutative) {}
+    Testable(ExpressionContext* const expCtx, bool isAssociative, bool isCommutative)
+        : ExpressionNary(expCtx), _isAssociative(isAssociative), _isCommutative(isCommutative) {}
     bool _isAssociative;
     bool _isCommutative;
 };
@@ -123,9 +126,11 @@ static BSONObj expressionToBson(const intrusive_ptr<Expression>& expression) {
 class ExpressionBaseTest : public unittest::Test {
 public:
     void addOperand(intrusive_ptr<ExpressionNary> expr, Value arg) {
-        intrusive_ptr<ExpressionContextForTest> expCtx(new ExpressionContextForTest());
-        expr->addOperand(ExpressionConstant::create(expCtx, arg));
+        expr->addOperand(ExpressionConstant::create(&expCtx, arg));
     }
+
+protected:
+    ExpressionContextForTest expCtx;
 };
 
 class ExpressionNaryTestOneArg : public ExpressionBaseTest {
@@ -161,9 +166,9 @@ public:
 class ExpressionNaryTest : public unittest::Test {
 public:
     virtual void setUp() override {
-        _notAssociativeNorCommutative = Testable::create(false, false);
-        _associativeOnly = Testable::create(true, false);
-        _associativeAndCommutative = Testable::create(true, true);
+        _notAssociativeNorCommutative = Testable::create(&expCtx, false, false);
+        _associativeOnly = Testable::create(&expCtx, true, false);
+        _associativeAndCommutative = Testable::create(&expCtx, true, true);
     }
 
 protected:
@@ -187,29 +192,28 @@ protected:
     }
 
     void addOperandArrayToExpr(const intrusive_ptr<Testable>& expr, const BSONArray& operands) {
-        intrusive_ptr<ExpressionContextForTest> expCtx(new ExpressionContextForTest());
-        VariablesParseState vps = expCtx->variablesParseState;
+        VariablesParseState vps = expCtx.variablesParseState;
         BSONObjIterator i(operands);
         while (i.more()) {
             BSONElement element = i.next();
-            expr->addOperand(Expression::parseOperand(expCtx, element, vps));
+            expr->addOperand(Expression::parseOperand(&expCtx, element, vps));
         }
     }
 
+    ExpressionContextForTest expCtx;
     intrusive_ptr<Testable> _notAssociativeNorCommutative;
     intrusive_ptr<Testable> _associativeOnly;
     intrusive_ptr<Testable> _associativeAndCommutative;
 };
 
 TEST_F(ExpressionNaryTest, AddedConstantOperandIsSerialized) {
-    intrusive_ptr<ExpressionContextForTest> expCtx(new ExpressionContextForTest());
-    _notAssociativeNorCommutative->addOperand(ExpressionConstant::create(expCtx, Value(9)));
+    _notAssociativeNorCommutative->addOperand(ExpressionConstant::create(&expCtx, Value(9)));
     assertContents(_notAssociativeNorCommutative, BSON_ARRAY(9));
 }
 
 TEST_F(ExpressionNaryTest, AddedFieldPathOperandIsSerialized) {
-    intrusive_ptr<ExpressionContextForTest> expCtx(new ExpressionContextForTest());
-    _notAssociativeNorCommutative->addOperand(ExpressionFieldPath::create(expCtx, "ab.c"));
+    _notAssociativeNorCommutative->addOperand(
+        ExpressionFieldPath::deprecatedCreate(&expCtx, "ab.c"));
     assertContents(_notAssociativeNorCommutative, BSON_ARRAY("$ab.c"));
 }
 
@@ -218,14 +222,13 @@ TEST_F(ExpressionNaryTest, ValidateEmptyDependencies) {
 }
 
 TEST_F(ExpressionNaryTest, ValidateConstantExpressionDependency) {
-    intrusive_ptr<ExpressionContextForTest> expCtx(new ExpressionContextForTest());
-    _notAssociativeNorCommutative->addOperand(ExpressionConstant::create(expCtx, Value(1)));
+    _notAssociativeNorCommutative->addOperand(ExpressionConstant::create(&expCtx, Value(1)));
     assertDependencies(_notAssociativeNorCommutative, BSONArray());
 }
 
 TEST_F(ExpressionNaryTest, ValidateFieldPathExpressionDependency) {
-    intrusive_ptr<ExpressionContextForTest> expCtx(new ExpressionContextForTest());
-    _notAssociativeNorCommutative->addOperand(ExpressionFieldPath::create(expCtx, "ab.c"));
+    _notAssociativeNorCommutative->addOperand(
+        ExpressionFieldPath::deprecatedCreate(&expCtx, "ab.c"));
     assertDependencies(_notAssociativeNorCommutative, BSON_ARRAY("ab.c"));
 }
 
@@ -234,26 +237,23 @@ TEST_F(ExpressionNaryTest, ValidateObjectExpressionDependency) {
                                    << "$x"
                                    << "q"
                                    << "$r"));
-    intrusive_ptr<ExpressionContextForTest> expCtx(new ExpressionContextForTest());
     BSONElement specElement = spec.firstElement();
-    VariablesParseState vps = expCtx->variablesParseState;
+    VariablesParseState vps = expCtx.variablesParseState;
     _notAssociativeNorCommutative->addOperand(
-        Expression::parseObject(expCtx, specElement.Obj(), vps));
+        Expression::parseObject(&expCtx, specElement.Obj(), vps));
     assertDependencies(_notAssociativeNorCommutative,
                        BSON_ARRAY("r"
                                   << "x"));
 }
 
 TEST_F(ExpressionNaryTest, SerializationToBsonObj) {
-    intrusive_ptr<ExpressionContextForTest> expCtx(new ExpressionContextForTest());
-    _notAssociativeNorCommutative->addOperand(ExpressionConstant::create(expCtx, Value(5)));
+    _notAssociativeNorCommutative->addOperand(ExpressionConstant::create(&expCtx, Value(5)));
     ASSERT_BSONOBJ_EQ(BSON("foo" << BSON("$testable" << BSON_ARRAY(BSON("$const" << 5)))),
                       BSON("foo" << _notAssociativeNorCommutative->serialize(false)));
 }
 
 TEST_F(ExpressionNaryTest, SerializationToBsonArr) {
-    intrusive_ptr<ExpressionContextForTest> expCtx(new ExpressionContextForTest());
-    _notAssociativeNorCommutative->addOperand(ExpressionConstant::create(expCtx, Value(5)));
+    _notAssociativeNorCommutative->addOperand(ExpressionConstant::create(&expCtx, Value(5)));
     ASSERT_BSONOBJ_EQ(constify(BSON_ARRAY(BSON("$testable" << BSON_ARRAY(5)))),
                       BSON_ARRAY(_notAssociativeNorCommutative->serialize(false)));
 }
@@ -366,7 +366,7 @@ TEST_F(ExpressionNaryTest, FlattenOptimizationNotDoneOnOtherExpressionsForAssoci
 TEST_F(ExpressionNaryTest, FlattenOptimizationNotDoneOnSameButNotAssociativeExpression) {
     BSONArrayBuilder specBuilder;
 
-    intrusive_ptr<Testable> innerOperand = Testable::create(false, false);
+    intrusive_ptr<Testable> innerOperand = Testable::create(&expCtx, false, false);
     addOperandArrayToExpr(innerOperand, BSON_ARRAY(100 << "$path1" << 101));
     specBuilder.append(expressionToBson(innerOperand));
     _associativeOnly->addOperand(innerOperand);
@@ -389,7 +389,7 @@ TEST_F(ExpressionNaryTest, FlattenOptimizationNotDoneOnSameButNotAssociativeExpr
 TEST_F(ExpressionNaryTest, FlattenInnerOperandsOptimizationOnNotCommutativeNorAssociative) {
     BSONArrayBuilder specBuilder;
 
-    intrusive_ptr<Testable> innerOperand = Testable::create(false, false);
+    intrusive_ptr<Testable> innerOperand = Testable::create(&expCtx, false, false);
     addOperandArrayToExpr(innerOperand, BSON_ARRAY(100 << "$path1"));
     specBuilder.append(expressionToBson(innerOperand));
     _notAssociativeNorCommutative->addOperand(innerOperand);
@@ -413,7 +413,7 @@ TEST_F(ExpressionNaryTest, FlattenInnerOperandsOptimizationOnNotCommutativeNorAs
 TEST_F(ExpressionNaryTest, FlattenInnerOperandsOptimizationOnAssociativeOnlyFrontOperandNoGroup) {
     BSONArrayBuilder specBuilder;
 
-    intrusive_ptr<Testable> innerOperand = Testable::create(true, false);
+    intrusive_ptr<Testable> innerOperand = Testable::create(&expCtx, true, false);
     addOperandArrayToExpr(innerOperand, BSON_ARRAY(100 << "$path1"));
     specBuilder.append(expressionToBson(innerOperand));
     _associativeOnly->addOperand(innerOperand);
@@ -438,7 +438,7 @@ TEST_F(ExpressionNaryTest, FlattenInnerOperandsOptimizationOnAssociativeOnlyFron
 TEST_F(ExpressionNaryTest, FlattenInnerOperandsOptimizationOnAssociativeOnlyFrontOperandAndGroup) {
     BSONArrayBuilder specBuilder;
 
-    intrusive_ptr<Testable> innerOperand = Testable::create(true, false);
+    intrusive_ptr<Testable> innerOperand = Testable::create(&expCtx, true, false);
     addOperandArrayToExpr(innerOperand, BSON_ARRAY(100 << "$path1" << 101));
     specBuilder.append(expressionToBson(innerOperand));
     _associativeOnly->addOperand(innerOperand);
@@ -466,7 +466,7 @@ TEST_F(ExpressionNaryTest, FlattenInnerOperandsOptimizationOnAssociativeOnlyMidd
     addOperandArrayToExpr(_associativeOnly, BSON_ARRAY(200 << "$path3"));
     specBuilder << 200 << "$path3";
 
-    intrusive_ptr<Testable> innerOperand = Testable::create(true, false);
+    intrusive_ptr<Testable> innerOperand = Testable::create(&expCtx, true, false);
     addOperandArrayToExpr(innerOperand, BSON_ARRAY(100 << "$path1"));
     specBuilder.append(expressionToBson(innerOperand));
     _associativeOnly->addOperand(innerOperand);
@@ -494,7 +494,7 @@ TEST_F(ExpressionNaryTest, FlattenInnerOperandsOptimizationOnAssociativeOnlyMidd
     addOperandArrayToExpr(_associativeOnly, BSON_ARRAY(200 << "$path3" << 201));
     specBuilder << 200 << "$path3" << 201;
 
-    intrusive_ptr<Testable> innerOperand = Testable::create(true, false);
+    intrusive_ptr<Testable> innerOperand = Testable::create(&expCtx, true, false);
     addOperandArrayToExpr(innerOperand, BSON_ARRAY(100 << "$path1" << 101));
     specBuilder.append(expressionToBson(innerOperand));
     _associativeOnly->addOperand(innerOperand);
@@ -523,7 +523,7 @@ TEST_F(ExpressionNaryTest, FlattenInnerOperandsOptimizationOnAssociativeOnlyBack
     addOperandArrayToExpr(_associativeOnly, BSON_ARRAY(200 << "$path3"));
     specBuilder << 200 << "$path3";
 
-    intrusive_ptr<Testable> innerOperand = Testable::create(true, false);
+    intrusive_ptr<Testable> innerOperand = Testable::create(&expCtx, true, false);
     addOperandArrayToExpr(innerOperand, BSON_ARRAY(100 << "$path1"));
     specBuilder.append(expressionToBson(innerOperand));
     _associativeOnly->addOperand(innerOperand);
@@ -548,7 +548,7 @@ TEST_F(ExpressionNaryTest, FlattenInnerOperandsOptimizationOnAssociativeOnlyBack
     addOperandArrayToExpr(_associativeOnly, BSON_ARRAY(200 << "$path3" << 201));
     specBuilder << 200 << "$path3" << 201;
 
-    intrusive_ptr<Testable> innerOperand = Testable::create(true, false);
+    intrusive_ptr<Testable> innerOperand = Testable::create(&expCtx, true, false);
     addOperandArrayToExpr(innerOperand, BSON_ARRAY(100 << "$path1" << 101));
     specBuilder.append(expressionToBson(innerOperand));
     _associativeOnly->addOperand(innerOperand);
@@ -571,12 +571,12 @@ TEST_F(ExpressionNaryTest, FlattenInnerOperandsOptimizationOnAssociativeOnlyBack
 TEST_F(ExpressionNaryTest, FlattenConsecutiveInnerOperandsOptimizationOnAssociativeOnlyNoGroup) {
     BSONArrayBuilder specBuilder;
 
-    intrusive_ptr<Testable> innerOperand = Testable::create(true, false);
+    intrusive_ptr<Testable> innerOperand = Testable::create(&expCtx, true, false);
     addOperandArrayToExpr(innerOperand, BSON_ARRAY(100 << "$path1"));
     specBuilder.append(expressionToBson(innerOperand));
     _associativeOnly->addOperand(innerOperand);
 
-    intrusive_ptr<Testable> innerOperand2 = Testable::create(true, false);
+    intrusive_ptr<Testable> innerOperand2 = Testable::create(&expCtx, true, false);
     addOperandArrayToExpr(innerOperand2, BSON_ARRAY(200 << "$path2"));
     specBuilder.append(expressionToBson(innerOperand2));
     _associativeOnly->addOperand(innerOperand2);
@@ -598,12 +598,12 @@ TEST_F(ExpressionNaryTest, FlattenConsecutiveInnerOperandsOptimizationOnAssociat
 TEST_F(ExpressionNaryTest, FlattenConsecutiveInnerOperandsOptimizationOnAssociativeAndGroup) {
     BSONArrayBuilder specBuilder;
 
-    intrusive_ptr<Testable> innerOperand = Testable::create(true, false);
+    intrusive_ptr<Testable> innerOperand = Testable::create(&expCtx, true, false);
     addOperandArrayToExpr(innerOperand, BSON_ARRAY(100 << "$path1" << 101));
     specBuilder.append(expressionToBson(innerOperand));
     _associativeOnly->addOperand(innerOperand);
 
-    intrusive_ptr<Testable> innerOperand2 = Testable::create(true, false);
+    intrusive_ptr<Testable> innerOperand2 = Testable::create(&expCtx, true, false);
     addOperandArrayToExpr(innerOperand2, BSON_ARRAY(200 << "$path2"));
     specBuilder.append(expressionToBson(innerOperand2));
     _associativeOnly->addOperand(innerOperand2);
@@ -627,7 +627,7 @@ TEST_F(ExpressionNaryTest, FlattenInnerOperandsOptimizationOnCommutativeAndAssoc
     addOperandArrayToExpr(_associativeAndCommutative, BSON_ARRAY(200 << "$path3" << 201));
     specBuilder << 200 << "$path3" << 201;
 
-    intrusive_ptr<Testable> innerOperand = Testable::create(true, true);
+    intrusive_ptr<Testable> innerOperand = Testable::create(&expCtx, true, true);
     addOperandArrayToExpr(innerOperand, BSON_ARRAY(100 << "$path1" << 101));
     specBuilder.append(expressionToBson(innerOperand));
     _associativeAndCommutative->addOperand(innerOperand);
@@ -653,8 +653,7 @@ TEST_F(ExpressionNaryTest, FlattenInnerOperandsOptimizationOnCommutativeAndAssoc
 class ExpressionTruncOneArgTest : public ExpressionNaryTestOneArg {
 public:
     void assertEval(ImplicitValue input, ImplicitValue output) {
-        intrusive_ptr<ExpressionContextForTest> expCtx(new ExpressionContextForTest());
-        _expr = new ExpressionTrunc(expCtx);
+        _expr = new ExpressionTrunc(&expCtx);
         ExpressionNaryTestOneArg::assertEvaluates(input, output);
     }
 };
@@ -662,8 +661,7 @@ public:
 class ExpressionTruncTwoArgTest : public ExpressionNaryTestTwoArg {
 public:
     void assertEval(ImplicitValue input1, ImplicitValue input2, ImplicitValue output) {
-        intrusive_ptr<ExpressionContextForTest> expCtx(new ExpressionContextForTest());
-        _expr = new ExpressionTrunc(expCtx);
+        _expr = new ExpressionTrunc(&expCtx);
         ExpressionNaryTestTwoArg::assertEvaluates(input1, input2, output);
     }
 };
@@ -713,7 +711,7 @@ TEST_F(ExpressionTruncOneArgTest, DoubleArg1) {
 
     // Outside the range of long longs (there isn't enough precision for decimals in this range, so
     // should just preserve the number).
-    double largerThanLong = numeric_limits<long long>::max() * 2.0;
+    double largerThanLong = static_cast<double>(numeric_limits<long long>::max()) * 2.0;
     assertEval(largerThanLong, largerThanLong);
     double smallerThanLong = numeric_limits<long long>::min() * 2.0;
     assertEval(smallerThanLong, smallerThanLong);
@@ -820,8 +818,7 @@ TEST_F(ExpressionTruncTwoArgTest, NullArg2) {
 class ExpressionSqrtTest : public ExpressionNaryTestOneArg {
 public:
     virtual void assertEvaluates(Value input, Value output) override {
-        intrusive_ptr<ExpressionContextForTest> expCtx(new ExpressionContextForTest());
-        _expr = new ExpressionSqrt(expCtx);
+        _expr = new ExpressionSqrt(&expCtx);
         ExpressionNaryTestOneArg::assertEvaluates(input, output);
     }
 };
@@ -866,8 +863,7 @@ TEST_F(ExpressionSqrtTest, SqrtNaNArg) {
 class ExpressionExpTest : public ExpressionNaryTestOneArg {
 public:
     virtual void assertEvaluates(Value input, Value output) override {
-        intrusive_ptr<ExpressionContextForTest> expCtx(new ExpressionContextForTest());
-        _expr = new ExpressionExp(expCtx);
+        _expr = new ExpressionExp(&expCtx);
         ExpressionNaryTestOneArg::assertEvaluates(input, output);
     }
 
@@ -908,8 +904,7 @@ TEST_F(ExpressionExpTest, ExpNaNArg) {
 class ExpressionCeilTest : public ExpressionNaryTestOneArg {
 public:
     virtual void assertEvaluates(Value input, Value output) override {
-        intrusive_ptr<ExpressionContextForTest> expCtx(new ExpressionContextForTest());
-        _expr = new ExpressionCeil(expCtx);
+        _expr = new ExpressionCeil(&expCtx);
         ExpressionNaryTestOneArg::assertEvaluates(input, output);
     }
 };
@@ -938,7 +933,7 @@ TEST_F(ExpressionCeilTest, DoubleArg) {
 
     // Outside the range of long longs (there isn't enough precision for decimals in this range, so
     // ceil should just preserve the number).
-    double largerThanLong = numeric_limits<long long>::max() * 2.0;
+    double largerThanLong = static_cast<double>(numeric_limits<long long>::max()) * 2.0;
     assertEvaluates(Value(largerThanLong), Value(largerThanLong));
     double smallerThanLong = numeric_limits<long long>::min() * 2.0;
     assertEvaluates(Value(smallerThanLong), Value(smallerThanLong));
@@ -967,8 +962,7 @@ TEST_F(ExpressionCeilTest, NullArg) {
 class ExpressionFloorTest : public ExpressionNaryTestOneArg {
 public:
     virtual void assertEvaluates(Value input, Value output) override {
-        intrusive_ptr<ExpressionContextForTest> expCtx(new ExpressionContextForTest());
-        _expr = new ExpressionFloor(expCtx);
+        _expr = new ExpressionFloor(&expCtx);
         ExpressionNaryTestOneArg::assertEvaluates(input, output);
     }
 };
@@ -997,7 +991,7 @@ TEST_F(ExpressionFloorTest, DoubleArg) {
 
     // Outside the range of long longs (there isn't enough precision for decimals in this range, so
     // floor should just preserve the number).
-    double largerThanLong = numeric_limits<long long>::max() * 2.0;
+    double largerThanLong = static_cast<double>(numeric_limits<long long>::max()) * 2.0;
     assertEvaluates(Value(largerThanLong), Value(largerThanLong));
     double smallerThanLong = numeric_limits<long long>::min() * 2.0;
     assertEvaluates(Value(smallerThanLong), Value(smallerThanLong));
@@ -1026,8 +1020,7 @@ TEST_F(ExpressionFloorTest, NullArg) {
 class ExpressionRoundOneArgTest : public ExpressionNaryTestOneArg {
 public:
     void assertEval(ImplicitValue input, ImplicitValue output) {
-        intrusive_ptr<ExpressionContextForTest> expCtx(new ExpressionContextForTest());
-        _expr = new ExpressionRound(expCtx);
+        _expr = new ExpressionRound(&expCtx);
         ExpressionNaryTestOneArg::assertEvaluates(input, output);
     }
 };
@@ -1035,8 +1028,7 @@ public:
 class ExpressionRoundTwoArgTest : public ExpressionNaryTestTwoArg {
 public:
     void assertEval(ImplicitValue input1, ImplicitValue input2, ImplicitValue output) {
-        intrusive_ptr<ExpressionContextForTest> expCtx(new ExpressionContextForTest());
-        _expr = new ExpressionRound(expCtx);
+        _expr = new ExpressionRound(&expCtx);
         ExpressionNaryTestTwoArg::assertEvaluates(input1, input2, output);
     }
 };
@@ -1084,7 +1076,7 @@ TEST_F(ExpressionRoundOneArgTest, DoubleArg1) {
 
     // Outside the range of long longs (there isn't enough precision for decimals in this range, so
     // should just preserve the number).
-    double largerThanLong = numeric_limits<long long>::max() * 2.0;
+    double largerThanLong = static_cast<double>(numeric_limits<long long>::max()) * 2.0;
     assertEval(largerThanLong, largerThanLong);
     double smallerThanLong = numeric_limits<long long>::min() * 2.0;
     assertEval(smallerThanLong, smallerThanLong);
@@ -1196,8 +1188,7 @@ TEST_F(ExpressionRoundTwoArgTest, NullArg2) {
 class ExpressionBinarySizeTest : public ExpressionNaryTestOneArg {
 public:
     void assertEval(ImplicitValue input, ImplicitValue output) {
-        intrusive_ptr<ExpressionContextForTest> expCtx(new ExpressionContextForTest());
-        _expr = new ExpressionBinarySize(expCtx);
+        _expr = new ExpressionBinarySize(&expCtx);
         ExpressionNaryTestOneArg::assertEvaluates(input, output);
     }
 };
@@ -1227,13 +1218,11 @@ TEST_F(ExpressionBinarySizeTest, HandlesNullish) {
 class ExpressionFirstTest : public ExpressionNaryTestOneArg {
 public:
     void assertEval(ImplicitValue input, ImplicitValue output) {
-        intrusive_ptr<ExpressionContextForTest> expCtx(new ExpressionContextForTest());
-        _expr = new ExpressionFirst(expCtx);
+        _expr = new ExpressionFirst(&expCtx);
         ExpressionNaryTestOneArg::assertEvaluates(input, output);
     }
     void assertEvalFails(ImplicitValue input) {
-        intrusive_ptr<ExpressionContextForTest> expCtx(new ExpressionContextForTest());
-        _expr = new ExpressionFirst(expCtx);
+        _expr = new ExpressionFirst(&expCtx);
         ASSERT_THROWS_CODE(eval(input), DBException, 28689);
     }
 };
@@ -1264,13 +1253,11 @@ TEST_F(ExpressionFirstTest, RejectsNonArrays) {
 class ExpressionLastTest : public ExpressionNaryTestOneArg {
 public:
     void assertEval(ImplicitValue input, ImplicitValue output) {
-        intrusive_ptr<ExpressionContextForTest> expCtx(new ExpressionContextForTest());
-        _expr = new ExpressionLast(expCtx);
+        _expr = new ExpressionLast(&expCtx);
         ExpressionNaryTestOneArg::assertEvaluates(input, output);
     }
     void assertEvalFails(ImplicitValue input) {
-        intrusive_ptr<ExpressionContextForTest> expCtx(new ExpressionContextForTest());
-        _expr = new ExpressionLast(expCtx);
+        _expr = new ExpressionLast(&expCtx);
         ASSERT_THROWS_CODE(eval(input), DBException, 28689);
     }
 };
@@ -1296,5 +1283,61 @@ TEST_F(ExpressionLastTest, RejectsNonArrays) {
     assertEvalFails(BSONBinData("asdf", 4, BinDataGeneral));
 }
 
+/* ------------------------- ExpressionTsSecond -------------------------- */
+
+class ExpressionTsSecondTest : public ExpressionNaryTestOneArg {
+public:
+    void assertEval(ImplicitValue input, ImplicitValue output) {
+        _expr = new ExpressionTsSecond(&expCtx);
+        ExpressionNaryTestOneArg::assertEvaluates(input, output);
+    }
+    void assertEvalFails(ImplicitValue input) {
+        _expr = new ExpressionTsSecond(&expCtx);
+        ASSERT_THROWS_CODE(eval(input), DBException, 5687301);
+    }
+};
+
+TEST_F(ExpressionTsSecondTest, HandlesTimestamp) {
+    assertEval(Timestamp(1622731060, 10), static_cast<long long>(1622731060));
+}
+
+TEST_F(ExpressionTsSecondTest, HandlesNullish) {
+    assertEval(BSONNULL, BSONNULL);
+    assertEval(BSONUndefined, BSONNULL);
+    assertEval(Value(), BSONNULL);
+}
+
+TEST_F(ExpressionTsSecondTest, HandlesInvalidTimestamp) {
+    assertEvalFails(1622731060);
+}
+
+/* ------------------------- ExpressionTsIncrement -------------------------- */
+
+class ExpressionTsIncrementTest : public ExpressionNaryTestOneArg {
+public:
+    void assertEval(ImplicitValue input, ImplicitValue output) {
+        _expr = new ExpressionTsIncrement(&expCtx);
+        ExpressionNaryTestOneArg::assertEvaluates(input, output);
+    }
+    void assertEvalFails(ImplicitValue input) {
+        _expr = new ExpressionTsIncrement(&expCtx);
+        ASSERT_THROWS_CODE(eval(input), DBException, 5687302);
+    }
+};
+
+TEST_F(ExpressionTsIncrementTest, HandlesTimestamp) {
+    assertEval(Timestamp(1622731060, 10), static_cast<long long>(10));
+}
+
+TEST_F(ExpressionTsIncrementTest, HandlesNullish) {
+    assertEval(BSONNULL, BSONNULL);
+    assertEval(BSONUndefined, BSONNULL);
+    assertEval(Value(), BSONNULL);
+}
+
+TEST_F(ExpressionTsIncrementTest, HandlesInvalidTimestamp) {
+    assertEvalFails(10);
+}
+
 }  // anonymous namespace
 }  // namespace ExpressionTests
