@@ -118,10 +118,13 @@ public:
      * Returns false if at least one of the stages does not allow the involved namespace 'nss' to be
      * sharded.
      */
-    bool allowShardedForeignCollection(NamespaceString nss) const {
-        return std::all_of(_stageSpecs.begin(), _stageSpecs.end(), [&nss](auto&& spec) {
-            return spec->allowShardedForeignCollection(nss);
-        });
+    bool allowShardedForeignCollection(NamespaceString nss, bool isMultiDocumentTransaction) const {
+        return std::all_of(_stageSpecs.begin(),
+                           _stageSpecs.end(),
+                           [&nss, isMultiDocumentTransaction](auto&& spec) {
+                               return spec->allowShardedForeignCollection(
+                                   nss, isMultiDocumentTransaction);
+                           });
     }
 
     /**
@@ -132,6 +135,13 @@ public:
                                                  boost::optional<ExplainOptions::Verbosity> explain,
                                                  bool enableMajorityReadConcern) const;
 
+    /**
+     * Checks that all of the stages in this pipeline are allowed to run with the specified read
+     * concern level. Does not do any pipeline global checks.
+     */
+    ReadConcernSupportResult sourcesSupportReadConcern(repl::ReadConcernLevel level,
+                                                       bool isImplicitDefault) const;
+
     /**
      * Verifies that this pipeline is allowed to run in a multi-document transaction. This ensures
      * that each stage is compatible, and throws a UserException if not. This should only be called
@@ -140,16 +150,22 @@ public:
     void assertSupportsMultiDocumentTransaction(
         boost::optional<ExplainOptions::Verbosity> explain) const;
 
+    /**
+     * Verifies that this pipeline is allowed to run with the read concern from the provided opCtx.
+     * Used only when asserting is the desired behavior, otherwise use supportsReadConcern instead.
+     */
+    void assertSupportsReadConcern(OperationContext* opCtx,
+                                   boost::optional<ExplainOptions::Verbosity> explain) const;
+
     /**
      * Perform checks that verify that the LitePipe is valid. Note that this function must be called
      * before forwarding an aggregation command on an unsharded collection, in order to verify that
      * the involved namespaces are allowed to be sharded.
      */
-    void verifyIsSupported(
-        OperationContext* opCtx,
-        const std::function<bool(OperationContext*, const NamespaceString&)> isSharded,
-        const boost::optional<ExplainOptions::Verbosity> explain,
-        bool enableMajorityReadConcern) const;
+    void verifyIsSupported(OperationContext* opCtx,
+                           std::function<bool(OperationContext*, const NamespaceString&)> isSharded,
+                           boost::optional<ExplainOptions::Verbosity> explain,
+                           bool enableMajorityReadConcern) const;
 
     /**
      * Returns true if the first stage in the pipeline does not require an input source.
