@@ -44,7 +44,7 @@
 #include "mongo/db/catalog/list_indexes.h"
 #include "mongo/db/catalog/rename_collection.h"
 #include "mongo/db/concurrency/d_concurrency.h"
-#include "mongo/db/concurrency/exception_util.h"
+#include "mongo/db/concurrency/write_conflict_exception.h"
 #include "mongo/db/curop.h"
 #include "mongo/db/cursor_manager.h"
 #include "mongo/db/db_raii.h"
@@ -54,8 +54,8 @@
 #include "mongo/db/pipeline/pipeline_d.h"
 #include "mongo/db/query/collection_index_usage_tracker_decoration.h"
 #include "mongo/db/query/collection_query_info.h"
+#include "mongo/db/query/sbe_plan_cache.h"
 #include "mongo/db/repl/primary_only_service.h"
-#include "mongo/db/repl/speculative_majority_read_info.h"
 #include "mongo/db/s/collection_sharding_state.h"
 #include "mongo/db/s/sharding_state.h"
 #include "mongo/db/s/transaction_coordinator_curop.h"
@@ -142,17 +142,17 @@ bool supportsUniqueKey(const boost::intrusive_ptr<ExpressionContext>& expCtx,
             CollatorInterface::collatorsMatch(index->getCollator(), expCtx->getCollator()));
 }
 
-// In an operation across GetMore requests we need to check that ignore conflicts is set for each
-// write to the RecordStore.
-void setIgnoreConflictsWriteBehavior(const boost::intrusive_ptr<ExpressionContext>& expCtx) {
-    if (expCtx->opCtx->recoveryUnit()->getPrepareConflictBehavior() !=
-        PrepareConflictBehavior::kIgnoreConflictsAllowWrites) {
-        expCtx->opCtx->recoveryUnit()->abandonSnapshot();
-        expCtx->opCtx->recoveryUnit()->setPrepareConflictBehavior(
-            PrepareConflictBehavior::kIgnoreConflictsAllowWrites);
-    }
+// Proactively assert that this operation can safely write before hitting an assertion in the
+// storage engine. We can safely write if we are enforcing prepare conflicts by blocking or if we
+// are ignoring prepare conflicts and explicitly allowing writes. Ignoring prepare conflicts
+// without allowing writes will cause this operation to fail in the storage engine.
+void assertIgnorePrepareConflictsBehavior(const boost::intrusive_ptr<ExpressionContext>& expCtx) {
+    tassert(5996900,
+            "Expected operation to either be blocking on prepare conflicts or ignoring prepare "
+            "conflicts and allowing writes",
+            expCtx->opCtx->recoveryUnit()->getPrepareConflictBehavior() !=
+                PrepareConflictBehavior::kIgnoreConflicts);
 }
-
 }  // namespace
 
 std::unique_ptr<TransactionHistoryIteratorBase>
@@ -337,19 +337,12 @@ std::vector<GenericCursor> CommonMongodProcessInterface::getIdleCursors(
     return CursorManager::get(expCtx->opCtx)->getIdleCursors(expCtx->opCtx, userMode);
 }
 
-boost::optional<Document> CommonMongodProcessInterface::lookupSingleDocument(
+boost::optional<Document> CommonMongodProcessInterface::doLookupSingleDocument(
     const boost::intrusive_ptr<ExpressionContext>& expCtx,
     const NamespaceString& nss,
     UUID collectionUUID,
     const Document& documentKey,
-    boost::optional<BSONObj> readConcern,
-    bool allowSpeculativeMajorityRead) {
-    invariant(!readConcern);  // We don't currently support a read concern on mongod - it's only
-                              // expected to be necessary on mongos.
-    invariant(!allowSpeculativeMajorityRead);  // We don't expect 'allowSpeculativeMajorityRead' on
-                                               // mongod - it's only expected to be necessary on
-                                               // mongos.
-
+    MakePipelineOptions opts) {
     std::unique_ptr<Pipeline, PipelineDeleter> pipeline;
     try {
         // Be sure to do the lookup using the collection default collation
@@ -357,11 +350,12 @@ boost::optional<Document> CommonMongodProcessInterface::lookupSingleDocument(
             nss,
             collectionUUID,
             _getCollectionDefaultCollator(expCtx->opCtx, nss.db(), collectionUUID));
-        // When looking up on a mongoD, we only ever want to read from the local collection. By
-        // default, makePipeline will attach a cursor source which may read from remote if the
-        // collection is sharded, so we configure it to not allow that here.
-        MakePipelineOptions opts;
-        opts.allowTargetingShards = false;
+
+        // If we are here, we are either executing the pipeline normally or running in one of the
+        // execution stat explain verbosities. In either case, we disable explain on the foreign
+        // context so that we actually retrieve the document.
+        foreignExpCtx->explain = boost::none;
+
         pipeline = Pipeline::makePipeline({BSON("$match" << documentKey)}, foreignExpCtx, opts);
     } catch (const ExceptionFor<ErrorCodes::NamespaceNotFound>&) {
         return boost::none;
@@ -375,22 +369,6 @@ boost::optional<Document> CommonMongodProcessInterface::lookupSingleDocument(
                                 << ", " << next->toString() << "]");
     }
 
-    // Set the speculative read timestamp appropriately after we do a document lookup locally. We
-    // set the speculative read timestamp based on the timestamp used by the transaction.
-    repl::SpeculativeMajorityReadInfo& speculativeMajorityReadInfo =
-        repl::SpeculativeMajorityReadInfo::get(expCtx->opCtx);
-    if (speculativeMajorityReadInfo.isSpeculativeRead()) {
-        // Speculative majority reads are required to use the 'kNoOverlap' read source.
-        // Storage engine operations require at least Global IS.
-        Lock::GlobalLock lk(expCtx->opCtx, MODE_IS);
-        invariant(expCtx->opCtx->recoveryUnit()->getTimestampReadSource() ==
-                  RecoveryUnit::ReadSource::kNoOverlap);
-        boost::optional<Timestamp> readTs =
-            expCtx->opCtx->recoveryUnit()->getPointInTimeReadTimestamp(expCtx->opCtx);
-        invariant(readTs);
-        speculativeMajorityReadInfo.setSpeculativeReadTimestampForward(*readTs);
-    }
-
     return lookedUpDocument;
 }
 
@@ -426,7 +404,7 @@ BackupCursorExtendState CommonMongodProcessInterface::extendBackupCursor(
 
 std::vector<BSONObj> CommonMongodProcessInterface::getMatchingPlanCacheEntryStats(
     OperationContext* opCtx, const NamespaceString& nss, const MatchExpression* matchExp) const {
-    const auto serializer = [](const PlanCacheEntry& entry) {
+    const auto serializer = [](const auto& entry) {
         BSONObjBuilder out;
         Explain::planCacheEntryToBSON(entry, &out);
         return out.obj();
@@ -440,10 +418,30 @@ std::vector<BSONObj> CommonMongodProcessInterface::getMatchingPlanCacheEntryStat
     uassert(
         50933, str::stream() << "collection '" << nss.toString() << "' does not exist", collection);
 
-    const auto planCache = CollectionQueryInfo::get(collection.getCollection()).getPlanCache();
+    const auto& collQueryInfo = CollectionQueryInfo::get(collection.getCollection());
+    const auto planCache = collQueryInfo.getPlanCache();
     invariant(planCache);
 
-    return planCache->getMatchingStats(serializer, predicate);
+    auto planCacheEntries =
+        planCache->getMatchingStats({} /* cacheKeyFilterFunc */, serializer, predicate);
+
+    if (feature_flags::gFeatureFlagSbePlanCache.isEnabledAndIgnoreFCV()) {
+        // Retrieve plan cache entries from the SBE plan cache.
+        const auto cacheKeyFilter = [uuid = collection->uuid(),
+                                     collVersion = collQueryInfo.getPlanCacheInvalidatorVersion()](
+                                        const sbe::PlanCacheKey& key) {
+            // Only fetch plan cache entries with keys matching given UUID and collectionVersion.
+            return uuid == key.getCollectionUuid() && collVersion == key.getCollectionVersion();
+        };
+
+        auto planCacheEntriesSBE =
+            sbe::getPlanCache(opCtx).getMatchingStats(cacheKeyFilter, serializer, predicate);
+
+        planCacheEntries.insert(
+            planCacheEntries.end(), planCacheEntriesSBE.begin(), planCacheEntriesSBE.end());
+    }
+
+    return planCacheEntries;
 }
 
 bool CommonMongodProcessInterface::fieldsHaveSupportingUniqueIndex(
@@ -682,7 +680,7 @@ void CommonMongodProcessInterface::writeRecordsToRecordStore(
     std::vector<Record>* records,
     const std::vector<Timestamp>& ts) const {
     tassert(5643012, "Attempted to write to record store with nullptr", records);
-    setIgnoreConflictsWriteBehavior(expCtx);
+    assertIgnorePrepareConflictsBehavior(expCtx);
     writeConflictRetry(expCtx->opCtx, "MPI::writeRecordsToRecordStore", expCtx->ns.ns(), [&] {
         AutoGetCollection autoColl(expCtx->opCtx, expCtx->ns, MODE_IX);
         WriteUnitOfWork wuow(expCtx->opCtx);
@@ -693,13 +691,12 @@ void CommonMongodProcessInterface::writeRecordsToRecordStore(
         wuow.commit();
     });
 }
+
 std::unique_ptr<TemporaryRecordStore> CommonMongodProcessInterface::createTemporaryRecordStore(
-    const boost::intrusive_ptr<ExpressionContext>& expCtx) const {
-    expCtx->opCtx->recoveryUnit()->abandonSnapshot();
-    expCtx->opCtx->recoveryUnit()->setPrepareConflictBehavior(
-        PrepareConflictBehavior::kIgnoreConflictsAllowWrites);
+    const boost::intrusive_ptr<ExpressionContext>& expCtx, KeyFormat keyFormat) const {
+    assertIgnorePrepareConflictsBehavior(expCtx);
     return expCtx->opCtx->getServiceContext()->getStorageEngine()->makeTemporaryRecordStore(
-        expCtx->opCtx);
+        expCtx->opCtx, keyFormat);
 }
 
 Document CommonMongodProcessInterface::readRecordFromRecordStore(
@@ -713,7 +710,7 @@ Document CommonMongodProcessInterface::readRecordFromRecordStore(
 
 void CommonMongodProcessInterface::deleteRecordFromRecordStore(
     const boost::intrusive_ptr<ExpressionContext>& expCtx, RecordStore* rs, RecordId rID) const {
-    setIgnoreConflictsWriteBehavior(expCtx);
+    assertIgnorePrepareConflictsBehavior(expCtx);
     writeConflictRetry(expCtx->opCtx, "MPI::deleteFromRecordStore", expCtx->ns.ns(), [&] {
         AutoGetCollection autoColl(expCtx->opCtx, expCtx->ns, MODE_IX);
         WriteUnitOfWork wuow(expCtx->opCtx);
@@ -724,7 +721,7 @@ void CommonMongodProcessInterface::deleteRecordFromRecordStore(
 
 void CommonMongodProcessInterface::truncateRecordStore(
     const boost::intrusive_ptr<ExpressionContext>& expCtx, RecordStore* rs) const {
-    setIgnoreConflictsWriteBehavior(expCtx);
+    assertIgnorePrepareConflictsBehavior(expCtx);
     writeConflictRetry(expCtx->opCtx, "MPI::truncateRecordStore", expCtx->ns.ns(), [&] {
         AutoGetCollection autoColl(expCtx->opCtx, expCtx->ns, MODE_IX);
         WriteUnitOfWork wuow(expCtx->opCtx);
@@ -734,12 +731,4 @@ void CommonMongodProcessInterface::truncateRecordStore(
     });
 }
 
-void CommonMongodProcessInterface::deleteTemporaryRecordStore(
-    const boost::intrusive_ptr<ExpressionContext>& expCtx,
-    std::unique_ptr<TemporaryRecordStore> rs) const {
-    setIgnoreConflictsWriteBehavior(expCtx);
-    AutoGetCollection autoColl(expCtx->opCtx, expCtx->ns, MODE_IX);
-    rs->finalizeTemporaryTable(expCtx->opCtx, TemporaryRecordStore::FinalizationAction::kDelete);
-}
-
 }  // namespace mongo
