@@ -27,7 +27,7 @@
  *    it in the license file.
  */
 
-#define MONGO_LOG_DEFAULT_COMPONENT ::mongo::logger::LogComponent::kQuery
+#define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kQuery
 
 #include "mongo/platform/basic.h"
 
@@ -42,7 +42,7 @@
 #include "mongo/db/query/query_knobs_gen.h"
 #include "mongo/db/query/query_planner.h"
 #include "mongo/db/query/query_planner_test_lib.h"
-#include "mongo/util/log.h"
+#include "mongo/logv2/log.h"
 #include "mongo/util/transitional_tools_do_not_use/vector_spooling.h"
 
 namespace mongo {
@@ -53,20 +53,25 @@ const NamespaceString QueryPlannerTest::nss("test.collection");
 
 void QueryPlannerTest::setUp() {
     opCtx = serviceContext.makeOperationContext();
+    expCtx = make_intrusive<ExpressionContext>(
+        opCtx.get(), std::unique_ptr<CollatorInterface>(nullptr), nss);
     internalQueryPlannerEnableHashIntersection.store(true);
     params.options = QueryPlannerParams::INCLUDE_COLLSCAN;
     addIndex(BSON("_id" << 1));
 }
 
 void QueryPlannerTest::clearState() {
+    plannerStatus = Status::OK();
     solns.clear();
     cq.reset();
+    expCtx.reset();
     relaxBoundsCheck = false;
 }
 
 void QueryPlannerTest::addIndex(BSONObj keyPattern, bool multikey) {
     params.indices.push_back({keyPattern,
                               IndexNames::nameToType(IndexNames::findPluginName(keyPattern)),
+                              IndexDescriptor::kLatestIndexVersion,
                               multikey,
                               {},
                               {},
@@ -82,6 +87,7 @@ void QueryPlannerTest::addIndex(BSONObj keyPattern, bool multikey) {
 void QueryPlannerTest::addIndex(BSONObj keyPattern, bool multikey, bool sparse) {
     params.indices.push_back({keyPattern,
                               IndexNames::nameToType(IndexNames::findPluginName(keyPattern)),
+                              IndexDescriptor::kLatestIndexVersion,
                               multikey,
                               {},
                               {},
@@ -95,24 +101,31 @@ void QueryPlannerTest::addIndex(BSONObj keyPattern, bool multikey, bool sparse)
 }
 
 void QueryPlannerTest::addIndex(BSONObj keyPattern, bool multikey, bool sparse, bool unique) {
-    params.indices.push_back(
-        {keyPattern,
-         IndexNames::nameToType(IndexNames::findPluginName(keyPattern)),
-         multikey,
-         {},
-         {},
-         sparse,
-         unique,
-         IndexEntry::Identifier{"sql_query_walks_into_bar_and_says_can_i_join_you?"},
-         nullptr,  // filterExpr
-         BSONObj(),
-         nullptr,
-         nullptr});
+    addIndex(
+        keyPattern, multikey, sparse, unique, "sql_query_walks_into_bar_and_says_can_i_join_you?");
+}
+
+void QueryPlannerTest::addIndex(
+    BSONObj keyPattern, bool multikey, bool sparse, bool unique, const std::string& name) {
+    params.indices.push_back({keyPattern,
+                              IndexNames::nameToType(IndexNames::findPluginName(keyPattern)),
+                              IndexDescriptor::kLatestIndexVersion,
+                              multikey,
+                              {},
+                              {},
+                              sparse,
+                              unique,
+                              IndexEntry::Identifier{name},
+                              nullptr,  // filterExpr
+                              BSONObj(),
+                              nullptr,
+                              nullptr});
 }
 
 void QueryPlannerTest::addIndex(BSONObj keyPattern, BSONObj infoObj) {
     params.indices.push_back({keyPattern,
                               IndexNames::nameToType(IndexNames::findPluginName(keyPattern)),
+                              IndexDescriptor::kLatestIndexVersion,
                               false,  // multikey
                               {},
                               {},
@@ -128,6 +141,7 @@ void QueryPlannerTest::addIndex(BSONObj keyPattern, BSONObj infoObj) {
 void QueryPlannerTest::addIndex(BSONObj keyPattern, MatchExpression* filterExpr) {
     params.indices.push_back({keyPattern,
                               IndexNames::nameToType(IndexNames::findPluginName(keyPattern)),
+                              IndexDescriptor::kLatestIndexVersion,
                               false,  // multikey
                               {},
                               {},
@@ -147,7 +161,7 @@ void QueryPlannerTest::addIndex(BSONObj keyPattern, const MultikeyPaths& multike
     const bool multikey =
         std::any_of(multikeyPaths.cbegin(),
                     multikeyPaths.cend(),
-                    [](const std::set<size_t>& components) { return !components.empty(); });
+                    [](const MultikeyComponents& components) { return !components.empty(); });
     const bool sparse = false;
     const bool unique = false;
     const char name[] = "my_index_with_path_level_multikey_info";
@@ -155,6 +169,7 @@ void QueryPlannerTest::addIndex(BSONObj keyPattern, const MultikeyPaths& multike
     const BSONObj infoObj;
     IndexEntry entry(keyPattern,
                      type,
+                     IndexDescriptor::kLatestIndexVersion,
                      multikey,
                      {},
                      {},
@@ -179,6 +194,7 @@ void QueryPlannerTest::addIndex(BSONObj keyPattern, const CollatorInterface* col
     const BSONObj infoObj;
     IndexEntry entry(keyPattern,
                      type,
+                     IndexDescriptor::kLatestIndexVersion,
                      multikey,
                      {},
                      {},
@@ -205,6 +221,7 @@ void QueryPlannerTest::addIndex(BSONObj keyPattern,
     const BSONObj infoObj;
     IndexEntry entry(keyPattern,
                      type,
+                     IndexDescriptor::kLatestIndexVersion,
                      multikey,
                      {},
                      {},
@@ -230,6 +247,7 @@ void QueryPlannerTest::addIndex(BSONObj keyPattern,
     const BSONObj infoObj;
     IndexEntry entry(keyPattern,
                      type,
+                     IndexDescriptor::kLatestIndexVersion,
                      multikey,
                      {},
                      {},
@@ -249,37 +267,48 @@ void QueryPlannerTest::addIndex(const IndexEntry& ie) {
 }
 
 void QueryPlannerTest::runQuery(BSONObj query) {
-    runQuerySortProjSkipNToReturn(query, BSONObj(), BSONObj(), 0, 0);
+    runQuerySortProjSkipLimit(query, BSONObj(), BSONObj(), 0, 0);
+}
+
+void QueryPlannerTest::runQueryWithPipeline(
+    BSONObj query, std::vector<std::unique_ptr<InnerPipelineStageInterface>> queryLayerPipeline) {
+    runQueryFull(query,
+                 BSONObj(),
+                 BSONObj(),
+                 0,
+                 0,
+                 BSONObj(),
+                 BSONObj(),
+                 BSONObj(),
+                 std::move(queryLayerPipeline));
 }
 
 void QueryPlannerTest::runQuerySortProj(const BSONObj& query,
                                         const BSONObj& sort,
                                         const BSONObj& proj) {
-    runQuerySortProjSkipNToReturn(query, sort, proj, 0, 0);
+    runQuerySortProjSkipLimit(query, sort, proj, 0, 0);
 }
 
-void QueryPlannerTest::runQuerySkipNToReturn(const BSONObj& query,
-                                             long long skip,
-                                             long long ntoreturn) {
-    runQuerySortProjSkipNToReturn(query, BSONObj(), BSONObj(), skip, ntoreturn);
+void QueryPlannerTest::runQuerySkipLimit(const BSONObj& query, long long skip, long long limit) {
+    runQuerySortProjSkipLimit(query, BSONObj(), BSONObj(), skip, limit);
 }
 
 void QueryPlannerTest::runQueryHint(const BSONObj& query, const BSONObj& hint) {
-    runQuerySortProjSkipNToReturnHint(query, BSONObj(), BSONObj(), 0, 0, hint);
+    runQuerySortProjSkipLimitHint(query, BSONObj(), BSONObj(), 0, 0, hint);
 }
 
-void QueryPlannerTest::runQuerySortProjSkipNToReturn(const BSONObj& query,
-                                                     const BSONObj& sort,
-                                                     const BSONObj& proj,
-                                                     long long skip,
-                                                     long long ntoreturn) {
-    runQuerySortProjSkipNToReturnHint(query, sort, proj, skip, ntoreturn, BSONObj());
+void QueryPlannerTest::runQuerySortProjSkipLimit(const BSONObj& query,
+                                                 const BSONObj& sort,
+                                                 const BSONObj& proj,
+                                                 long long skip,
+                                                 long long limit) {
+    runQuerySortProjSkipLimitHint(query, sort, proj, skip, limit, BSONObj());
 }
 
 void QueryPlannerTest::runQuerySortHint(const BSONObj& query,
                                         const BSONObj& sort,
                                         const BSONObj& hint) {
-    runQuerySortProjSkipNToReturnHint(query, sort, BSONObj(), 0, 0, hint);
+    runQuerySortProjSkipLimitHint(query, sort, BSONObj(), 0, 0, hint);
 }
 
 void QueryPlannerTest::runQueryHintMinMax(const BSONObj& query,
@@ -289,78 +318,77 @@ void QueryPlannerTest::runQueryHintMinMax(const BSONObj& query,
     runQueryFull(query, BSONObj(), BSONObj(), 0, 0, hint, minObj, maxObj);
 }
 
-void QueryPlannerTest::runQuerySortProjSkipNToReturnHint(const BSONObj& query,
-                                                         const BSONObj& sort,
-                                                         const BSONObj& proj,
-                                                         long long skip,
-                                                         long long ntoreturn,
-                                                         const BSONObj& hint) {
-    runQueryFull(query, sort, proj, skip, ntoreturn, hint, BSONObj(), BSONObj());
+void QueryPlannerTest::runQuerySortProjSkipLimitHint(const BSONObj& query,
+                                                     const BSONObj& sort,
+                                                     const BSONObj& proj,
+                                                     long long skip,
+                                                     long long limit,
+                                                     const BSONObj& hint) {
+    runQueryFull(query, sort, proj, skip, limit, hint, BSONObj(), BSONObj());
 }
 
-void QueryPlannerTest::runQueryFull(const BSONObj& query,
-                                    const BSONObj& sort,
-                                    const BSONObj& proj,
-                                    long long skip,
-                                    long long ntoreturn,
-                                    const BSONObj& hint,
-                                    const BSONObj& minObj,
-                                    const BSONObj& maxObj) {
+void QueryPlannerTest::runQueryFull(
+    const BSONObj& query,
+    const BSONObj& sort,
+    const BSONObj& proj,
+    long long skip,
+    long long limit,
+    const BSONObj& hint,
+    const BSONObj& minObj,
+    const BSONObj& maxObj,
+    std::vector<std::unique_ptr<InnerPipelineStageInterface>> pipeline) {
     clearState();
 
-    auto qr = stdx::make_unique<QueryRequest>(nss);
-    qr->setFilter(query);
-    qr->setSort(sort);
-    qr->setProj(proj);
+    auto findCommand = std::make_unique<FindCommandRequest>(nss);
+    findCommand->setFilter(query);
+    findCommand->setSort(sort);
+    findCommand->setProjection(proj);
     if (skip) {
-        qr->setSkip(skip);
+        findCommand->setSkip(skip);
     }
-    if (ntoreturn) {
-        if (ntoreturn < 0) {
-            ASSERT_NE(ntoreturn, std::numeric_limits<long long>::min());
-            ntoreturn = -ntoreturn;
-            qr->setWantMore(false);
-        }
-        qr->setNToReturn(ntoreturn);
+    if (limit) {
+        findCommand->setLimit(limit);
     }
-    qr->setHint(hint);
-    qr->setMin(minObj);
-    qr->setMax(maxObj);
-    const boost::intrusive_ptr<ExpressionContext> expCtx;
+    findCommand->setHint(hint);
+    findCommand->setMin(minObj);
+    findCommand->setMax(maxObj);
     auto statusWithCQ =
         CanonicalQuery::canonicalize(opCtx.get(),
-                                     std::move(qr),
+                                     std::move(findCommand),
+                                     false,
                                      expCtx,
                                      ExtensionsCallbackNoop(),
-                                     MatchExpressionParser::kAllowAllSpecialFeatures);
+                                     MatchExpressionParser::kAllowAllSpecialFeatures,
+                                     ProjectionPolicies::findProjectionPolicies(),
+                                     std::move(pipeline));
     ASSERT_OK(statusWithCQ.getStatus());
     cq = std::move(statusWithCQ.getValue());
 
-    auto statusWithSolutions = QueryPlanner::plan(*cq, params);
-    ASSERT_OK(statusWithSolutions.getStatus());
-    solns = std::move(statusWithSolutions.getValue());
+    auto statusWithMultiPlanSolns = QueryPlanner::plan(*cq, params);
+    ASSERT_OK(statusWithMultiPlanSolns.getStatus());
+    solns = std::move(statusWithMultiPlanSolns.getValue());
 }
 
 void QueryPlannerTest::runInvalidQuery(const BSONObj& query) {
-    runInvalidQuerySortProjSkipNToReturn(query, BSONObj(), BSONObj(), 0, 0);
+    runInvalidQuerySortProjSkipLimit(query, BSONObj(), BSONObj(), 0, 0);
 }
 
 void QueryPlannerTest::runInvalidQuerySortProj(const BSONObj& query,
                                                const BSONObj& sort,
                                                const BSONObj& proj) {
-    runInvalidQuerySortProjSkipNToReturn(query, sort, proj, 0, 0);
+    runInvalidQuerySortProjSkipLimit(query, sort, proj, 0, 0);
 }
 
-void QueryPlannerTest::runInvalidQuerySortProjSkipNToReturn(const BSONObj& query,
-                                                            const BSONObj& sort,
-                                                            const BSONObj& proj,
-                                                            long long skip,
-                                                            long long ntoreturn) {
-    runInvalidQuerySortProjSkipNToReturnHint(query, sort, proj, skip, ntoreturn, BSONObj());
+void QueryPlannerTest::runInvalidQuerySortProjSkipLimit(const BSONObj& query,
+                                                        const BSONObj& sort,
+                                                        const BSONObj& proj,
+                                                        long long skip,
+                                                        long long limit) {
+    runInvalidQuerySortProjSkipLimitHint(query, sort, proj, skip, limit, BSONObj());
 }
 
 void QueryPlannerTest::runInvalidQueryHint(const BSONObj& query, const BSONObj& hint) {
-    runInvalidQuerySortProjSkipNToReturnHint(query, BSONObj(), BSONObj(), 0, 0, hint);
+    runInvalidQuerySortProjSkipLimitHint(query, BSONObj(), BSONObj(), 0, 0, hint);
 }
 
 void QueryPlannerTest::runInvalidQueryHintMinMax(const BSONObj& query,
@@ -370,79 +398,79 @@ void QueryPlannerTest::runInvalidQueryHintMinMax(const BSONObj& query,
     runInvalidQueryFull(query, BSONObj(), BSONObj(), 0, 0, hint, minObj, maxObj);
 }
 
-void QueryPlannerTest::runInvalidQuerySortProjSkipNToReturnHint(const BSONObj& query,
-                                                                const BSONObj& sort,
-                                                                const BSONObj& proj,
-                                                                long long skip,
-                                                                long long ntoreturn,
-                                                                const BSONObj& hint) {
-    runInvalidQueryFull(query, sort, proj, skip, ntoreturn, hint, BSONObj(), BSONObj());
+void QueryPlannerTest::runInvalidQuerySortProjSkipLimitHint(const BSONObj& query,
+                                                            const BSONObj& sort,
+                                                            const BSONObj& proj,
+                                                            long long skip,
+                                                            long long limit,
+                                                            const BSONObj& hint) {
+    runInvalidQueryFull(query, sort, proj, skip, limit, hint, BSONObj(), BSONObj());
 }
 
 void QueryPlannerTest::runInvalidQueryFull(const BSONObj& query,
                                            const BSONObj& sort,
                                            const BSONObj& proj,
                                            long long skip,
-                                           long long ntoreturn,
+                                           long long limit,
                                            const BSONObj& hint,
                                            const BSONObj& minObj,
                                            const BSONObj& maxObj) {
     clearState();
 
-    auto qr = stdx::make_unique<QueryRequest>(nss);
-    qr->setFilter(query);
-    qr->setSort(sort);
-    qr->setProj(proj);
+    auto findCommand = std::make_unique<FindCommandRequest>(nss);
+    findCommand->setFilter(query);
+    findCommand->setSort(sort);
+    findCommand->setProjection(proj);
     if (skip) {
-        qr->setSkip(skip);
+        findCommand->setSkip(skip);
     }
-    if (ntoreturn) {
-        if (ntoreturn < 0) {
-            ASSERT_NE(ntoreturn, std::numeric_limits<long long>::min());
-            ntoreturn = -ntoreturn;
-            qr->setWantMore(false);
-        }
-        qr->setNToReturn(ntoreturn);
+    if (limit) {
+        findCommand->setLimit(limit);
     }
-    qr->setHint(hint);
-    qr->setMin(minObj);
-    qr->setMax(maxObj);
-    const boost::intrusive_ptr<ExpressionContext> expCtx;
+    findCommand->setHint(hint);
+    findCommand->setMin(minObj);
+    findCommand->setMax(maxObj);
     auto statusWithCQ =
         CanonicalQuery::canonicalize(opCtx.get(),
-                                     std::move(qr),
+                                     std::move(findCommand),
+                                     false,
                                      expCtx,
                                      ExtensionsCallbackNoop(),
                                      MatchExpressionParser::kAllowAllSpecialFeatures);
     ASSERT_OK(statusWithCQ.getStatus());
     cq = std::move(statusWithCQ.getValue());
 
-    auto statusWithSolutions = QueryPlanner::plan(*cq, params);
-    ASSERT_NOT_OK(statusWithSolutions.getStatus());
+    auto statusWithMultiPlanSolns = QueryPlanner::plan(*cq, params);
+    plannerStatus = statusWithMultiPlanSolns.getStatus();
+    ASSERT_NOT_OK(plannerStatus);
 }
 
+
 void QueryPlannerTest::runQueryAsCommand(const BSONObj& cmdObj) {
     clearState();
 
     invariant(nss.isValid());
 
     const bool isExplain = false;
-    std::unique_ptr<QueryRequest> qr(
-        assertGet(QueryRequest::makeFromFindCommand(nss, cmdObj, isExplain)));
 
-    const boost::intrusive_ptr<ExpressionContext> expCtx;
+    // If there is no '$db', append it.
+    auto cmd = OpMsgRequest::fromDBAndBody(nss.db(), cmdObj).body;
+    std::unique_ptr<FindCommandRequest> findCommand(
+        query_request_helper::makeFromFindCommandForTests(cmd, nss));
+
     auto statusWithCQ =
         CanonicalQuery::canonicalize(opCtx.get(),
-                                     std::move(qr),
+                                     std::move(findCommand),
+                                     isExplain,
                                      expCtx,
                                      ExtensionsCallbackNoop(),
                                      MatchExpressionParser::kAllowAllSpecialFeatures);
     ASSERT_OK(statusWithCQ.getStatus());
     cq = std::move(statusWithCQ.getValue());
 
-    auto statusWithSolutions = QueryPlanner::plan(*cq, params);
-    ASSERT_OK(statusWithSolutions.getStatus());
-    solns = std::move(statusWithSolutions.getValue());
+    auto statusWithMultiPlanSolns = QueryPlanner::plan(*cq, params);
+    ASSERT_OK(statusWithMultiPlanSolns.getStatus());
+    solns = std::move(statusWithMultiPlanSolns.getValue());
 }
 
 void QueryPlannerTest::runInvalidQueryAsCommand(const BSONObj& cmdObj) {
@@ -451,21 +479,25 @@ void QueryPlannerTest::runInvalidQueryAsCommand(const BSONObj& cmdObj) {
     invariant(nss.isValid());
 
     const bool isExplain = false;
-    std::unique_ptr<QueryRequest> qr(
-        assertGet(QueryRequest::makeFromFindCommand(nss, cmdObj, isExplain)));
 
-    const boost::intrusive_ptr<ExpressionContext> expCtx;
+    // If there is no '$db', append it.
+    auto cmd = OpMsgRequest::fromDBAndBody(nss.db(), cmdObj).body;
+    std::unique_ptr<FindCommandRequest> findCommand(
+        query_request_helper::makeFromFindCommandForTests(cmd, nss));
+
     auto statusWithCQ =
         CanonicalQuery::canonicalize(opCtx.get(),
-                                     std::move(qr),
+                                     std::move(findCommand),
+                                     isExplain,
                                      expCtx,
                                      ExtensionsCallbackNoop(),
                                      MatchExpressionParser::kAllowAllSpecialFeatures);
     ASSERT_OK(statusWithCQ.getStatus());
     cq = std::move(statusWithCQ.getValue());
 
-    auto statusWithSolutions = QueryPlanner::plan(*cq, params);
-    ASSERT_NOT_OK(statusWithSolutions.getStatus());
+    auto statusWithMultiPlanSolns = QueryPlanner::plan(*cq, params);
+    plannerStatus = statusWithMultiPlanSolns.getStatus();
+    ASSERT_NOT_OK(plannerStatus);
 }
 
 size_t QueryPlannerTest::getNumSolutions() const {
@@ -475,7 +507,7 @@ size_t QueryPlannerTest::getNumSolutions() const {
 void QueryPlannerTest::dumpSolutions() const {
     str::stream ost;
     dumpSolutions(ost);
-    log() << std::string(ost);
+    LOGV2(20985, "Solutions", "value"_attr = std::string(ost));
 }
 
 void QueryPlannerTest::dumpSolutions(str::stream& ost) const {
@@ -490,7 +522,8 @@ void QueryPlannerTest::assertNumSolutions(size_t expectSolutions) const {
     }
     str::stream ss;
     ss << "expected " << expectSolutions << " solutions but got " << getNumSolutions()
-       << " instead. solutions generated: " << '\n';
+       << " instead. Run with --verbose=vv to see reasons for mismatch. Solutions generated: "
+       << '\n';
     dumpSolutions(ss);
     FAIL(ss);
 }
@@ -499,9 +532,13 @@ size_t QueryPlannerTest::numSolutionMatches(const std::string& solnJson) const {
     BSONObj testSoln = fromjson(solnJson);
     size_t matches = 0;
     for (auto&& soln : solns) {
-        QuerySolutionNode* root = soln->root.get();
-        if (QueryPlannerTestLib::solutionMatches(testSoln, root, relaxBoundsCheck)) {
+        auto matchStatus =
+            QueryPlannerTestLib::solutionMatches(testSoln, soln->root(), relaxBoundsCheck);
+        if (matchStatus.isOK()) {
             ++matches;
+        } else {
+            LOGV2_DEBUG(
+                5676408, 2, "Mismatching solution: {reason}", "reason"_attr = matchStatus.reason());
         }
     }
     return matches;
@@ -514,7 +551,9 @@ void QueryPlannerTest::assertSolutionExists(const std::string& solnJson, size_t
     }
     str::stream ss;
     ss << "expected " << numMatches << " matches for solution " << solnJson << " but got "
-       << matches << " instead. all solutions generated: " << '\n';
+       << matches
+       << " instead. Run with --verbose=vv to see reasons for mismatch. All solutions generated: "
+       << '\n';
     dumpSolutions(ss);
     FAIL(ss);
 }
@@ -532,21 +571,30 @@ void QueryPlannerTest::assertHasOneSolutionOf(const std::vector<std::string>& so
     }
     str::stream ss;
     ss << "assertHasOneSolutionOf expected one matching solution"
-       << " but got " << matches << " instead. all solutions generated: " << '\n';
+       << " but got " << matches
+       << " instead. Run with --verbose=vv to see reasons for mismatch. All solutions generated: "
+       << '\n';
     dumpSolutions(ss);
     FAIL(ss);
 }
 
+void QueryPlannerTest::assertNoSolutions() const {
+    ASSERT_EQUALS(plannerStatus.code(), ErrorCodes::NoQueryExecutionPlans);
+}
+
 void QueryPlannerTest::assertHasOnlyCollscan() const {
     assertNumSolutions(1U);
     assertSolutionExists("{cscan: {dir: 1}}");
 }
 
 std::unique_ptr<MatchExpression> QueryPlannerTest::parseMatchExpression(
-    const BSONObj& obj, const CollatorInterface* collator) {
-    boost::intrusive_ptr<ExpressionContextForTest> expCtx(new ExpressionContextForTest());
-    expCtx->setCollator(collator);
-    StatusWithMatchExpression status = MatchExpressionParser::parse(obj, std::move(expCtx));
+    const BSONObj& obj, const boost::intrusive_ptr<ExpressionContext>& optionalExpCtx) {
+    auto expCtx = optionalExpCtx;
+    if (!expCtx.get()) {
+        expCtx = make_intrusive<ExpressionContextForTest>();
+    }
+
+    StatusWithMatchExpression status = MatchExpressionParser::parse(obj, expCtx);
     if (!status.isOK()) {
         FAIL(str::stream() << "failed to parse query: " << obj.toString()
                            << ". Reason: " << status.getStatus().toString());
