@@ -46,10 +46,13 @@ namespace mongo::sbe {
 CandidatePlans MultiPlanner::plan(
     std::vector<std::unique_ptr<QuerySolution>> solutions,
     std::vector<std::pair<std::unique_ptr<PlanStage>, stage_builder::PlanStageData>> roots) {
-    auto candidates =
-        collectExecutionStats(std::move(solutions),
-                              std::move(roots),
-                              trial_period::getTrialPeriodMaxWorks(_opCtx, _collection));
+    auto candidates = collectExecutionStats(
+        std::move(solutions),
+        std::move(roots),
+        trial_period::getTrialPeriodMaxWorks(_opCtx,
+                                             _collection,
+                                             internalQueryPlanEvaluationWorksSbe.load(),
+                                             internalQueryPlanEvaluationCollFractionSbe.load()));
     auto decision = uassertStatusOK(mongo::plan_ranker::pickBestPlan<PlanStageStats>(candidates));
     return finalizeExecutionPlans(std::move(decision), std::move(candidates));
 }
