@@ -29,11 +29,11 @@
 
 #define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kReplicationInitialSync
 
-#include "mongo/db/index/index_descriptor_fwd.h"
-#include "mongo/db/service_context.h"
 #include "mongo/platform/basic.h"
 
 #include "mongo/base/string_data.h"
+#include "mongo/db/catalog/clustered_collection_options_gen.h"
+#include "mongo/db/catalog/clustered_collection_util.h"
 #include "mongo/db/commands/list_collections_filter.h"
 #include "mongo/db/index_build_entry_helpers.h"
 #include "mongo/db/index_builds_coordinator.h"
@@ -109,6 +109,13 @@ CollectionCloner::CollectionCloner(const NamespaceString& sourceNss,
 }
 
 BaseCloner::ClonerStages CollectionCloner::getStages() {
+    if (_sourceNss.isChangeStreamPreImagesCollection()) {
+        // Only the change stream pre-images collection needs to be created - its documents should
+        // not be copied.
+        return {&_listIndexesStage,
+                &_createCollectionStage,
+                &_setupIndexBuildersForUnfinishedIndexesStage};
+    }
     return {&_countStage,
             &_listIndexesStage,
             &_createCollectionStage,
@@ -135,7 +142,7 @@ void CollectionCloner::preStage() {
                     1,
                     "Skipping the recording of some initial sync metrics due to failure in the "
                     "'collStats' command",
-                    "ns"_attr = _sourceNss.coll().toString(),
+                    logAttrs(_sourceNss),
                     "status"_attr = status);
     }
 }
@@ -202,21 +209,16 @@ BaseCloner::AfterStageBehavior CollectionCloner::listIndexesStage() {
                       "source"_attr = getSource());
     }
 
-    const auto storageEngine = getGlobalServiceContext()->getStorageEngine();
     // Parse the index specs into their respective state, ready or unfinished.
     for (auto&& spec : indexSpecs) {
-        // Sanitize storage engine options to remove options which might not apply to this node. See
-        // SERVER-68122.
-        if (auto storageEngineElem = spec.getField(IndexDescriptor::kStorageEngineFieldName)) {
-            auto sanitizedStorageEngineOpts =
-                storageEngine->getSanitizedStorageOptionsForSecondaryReplication(
-                    storageEngineElem.embeddedObject());
-            fassert(6812200, sanitizedStorageEngineOpts);
-            spec = spec.addFields(BSON(IndexDescriptor::kStorageEngineFieldName
-                                       << sanitizedStorageEngineOpts.getValue()));
-        }
-
-        if (spec.hasField("buildUUID")) {
+        if (spec.hasField("clustered")) {
+            invariant(_collectionOptions.clusteredIndex);
+            invariant(spec.getBoolField("clustered") == true);
+            invariant(clustered_util::formatClusterKeyForListIndexes(
+                          _collectionOptions.clusteredIndex.get())
+                          .woCompare(spec) == 0);
+            // Skip if the spec is for the collection's clusteredIndex.
+        } else if (spec.hasField("buildUUID")) {
             _unfinishedIndexSpecs.push_back(spec.getOwned());
         } else if (spec.hasField("name") && spec.getStringField("name") == "_id_"_sd) {
             _idIndexSpec = spec.getOwned();
@@ -322,12 +324,11 @@ void CollectionCloner::runQuery() {
         if (_resumeToken) {
             // Resume the query from where we left off.
             LOGV2_DEBUG(21133, 1, "Collection cloner will resume the last successful query");
-            query = QUERY("query" << BSONObj() << "$_requestResumeToken" << true << "$_resumeAfter"
-                                  << _resumeToken.get());
+            query.requestResumeToken(true).resumeAfter(_resumeToken.get());
         } else {
             // New attempt at a resumable query.
             LOGV2_DEBUG(21134, 1, "Collection cloner will run a new query");
-            query = QUERY("query" << BSONObj() << "$_requestResumeToken" << true);
+            query.requestResumeToken(true);
         }
         query.hint(BSON("$natural" << 1));
     }
@@ -340,6 +341,7 @@ void CollectionCloner::runQuery() {
     try {
         getClient()->query([this](DBClientCursorBatchIterator& iter) { handleNextBatch(iter); },
                            _sourceDbAndUuid,
+                           BSONObj{},
                            query,
                            nullptr /* fieldsToReturn */,
                            QueryOption_NoCursorTimeout | QueryOption_SecondaryOk |
