@@ -100,7 +100,7 @@ void OplogApplier::enqueue(OperationContext* opCtx,
                            std::vector<OplogEntry>::const_iterator end) {
     OplogBuffer::Batch batch;
     for (auto i = begin; i != end; ++i) {
-        batch.push_back(i->getRaw());
+        batch.push_back(i->getEntry().getRaw());
     }
     enqueue(opCtx, batch.cbegin(), batch.cend());
 }
@@ -137,6 +137,8 @@ const OplogApplier::Options& OplogApplier::getOptions() const {
 }
 
 std::unique_ptr<ThreadPool> makeReplWriterPool() {
+    // Reduce content pinned in cache by single oplog batch on small machines by reducing the number
+    // of threads of ReplWriter to reduce the number of concurrent open WT transactions.
     if (replWriterThreadCount < replWriterMinThreadCount) {
         LOGV2_FATAL_NOTRACE(
             5605400,
@@ -144,24 +146,33 @@ std::unique_ptr<ThreadPool> makeReplWriterPool() {
             "replWriterMinThreadCount"_attr = replWriterMinThreadCount,
             "replWriterThreadCount"_attr = replWriterThreadCount);
     }
-
-    // Reduce content pinned in cache by single oplog batch on small machines by reducing the number
-    // of threads of ReplWriter to reduce the number of concurrent open WT transactions.
     auto numberOfThreads =
         std::min(replWriterThreadCount, 2 * static_cast<int>(ProcessInfo::getNumAvailableCores()));
     return makeReplWriterPool(numberOfThreads);
 }
 
 std::unique_ptr<ThreadPool> makeReplWriterPool(int threadCount) {
+    return makeReplWriterPool(threadCount, "ReplWriterWorker"_sd);
+}
+
+std::unique_ptr<ThreadPool> makeReplWriterPool(int threadCount,
+                                               StringData name,
+                                               bool isKillableByStepdown) {
     ThreadPool::Options options;
-    options.threadNamePrefix = "ReplWriterWorker-";
-    options.poolName = "ReplWriterWorkerThreadPool";
+    options.threadNamePrefix = name + "-";
+    options.poolName = name + "ThreadPool";
     options.minThreads =
         replWriterMinThreadCount < threadCount ? replWriterMinThreadCount : threadCount;
     options.maxThreads = static_cast<size_t>(threadCount);
-    options.onCreateThread = [](const std::string&) {
+    options.onCreateThread = [isKillableByStepdown](const std::string&) {
         Client::initThread(getThreadName());
-        AuthorizationSession::get(cc())->grantInternalAuthorization(&cc());
+        auto client = Client::getCurrent();
+        AuthorizationSession::get(*client)->grantInternalAuthorization(client);
+
+        if (isKillableByStepdown) {
+            stdx::lock_guard<Client> lk(*client);
+            client->setSystemOperationKillableByStepdown(lk);
+        }
     };
     auto pool = std::make_unique<ThreadPool>(options);
     pool->startup();
