@@ -44,6 +44,7 @@
 #include "mongo/db/storage/storage_engine_init.h"
 #include "mongo/executor/task_executor.h"
 #include "mongo/rpc/topology_version_gen.h"
+#include "mongo/util/duration.h"
 #include "mongo/util/net/hostandport.h"
 #include "mongo/util/time_support.h"
 
@@ -195,8 +196,11 @@ public:
      * Returns OK if member state is 'state'.
      * Returns ErrorCodes::ExceededTimeLimit if we timed out waiting for the state change.
      * Returns ErrorCodes::BadValue if timeout is negative.
+     * Throws if interrupted (pass Interruptible::notInterruptible() if interruption is not desired.
      */
-    virtual Status waitForMemberState(MemberState expectedState, Milliseconds timeout) = 0;
+    virtual Status waitForMemberState(Interruptible* interruptible,
+                                      MemberState expectedState,
+                                      Milliseconds timeout) = 0;
 
     /**
      * Returns true if this node is in state PRIMARY or SECONDARY.
@@ -590,7 +594,8 @@ public:
      * steps down and steps up so quickly that the applier signals drain complete in the wrong
      * term.
      */
-    virtual void signalDrainComplete(OperationContext* opCtx, long long termWhenBufferIsEmpty) = 0;
+    virtual void signalDrainComplete(OperationContext* opCtx,
+                                     long long termWhenBufferIsEmpty) noexcept = 0;
 
     /**
      * Signals the sync source feedback thread to wake up and send a handshake and
@@ -610,7 +615,8 @@ public:
      * Handles an incoming replSetGetStatus command. Adds BSON to 'result'. If kInitialSync is
      * requested but initial sync is not running, kBasic will be used.
      */
-    virtual Status processReplSetGetStatus(BSONObjBuilder* result,
+    virtual Status processReplSetGetStatus(OperationContext* opCtx,
+                                           BSONObjBuilder* result,
                                            ReplSetGetStatusResponseStyle responseStyle) = 0;
 
     /**
@@ -621,9 +627,78 @@ public:
 
     /**
      * Returns a copy of the current ReplSetConfig.
+     *
+     * To avoid unnecessarily copying the ReplSetConfig through this method, please use the
+     * getConfig* methods below if you are only accessing a specific field of the ReplSetConfig,
+     * i.e. replCoord->getConfigNumMembers() instead of replCoord->getConfig().getNumMembers(). See
+     * SERVER-47828.
      */
     virtual ReplSetConfig getConfig() const = 0;
 
+    /**
+     * Returns the current ReplSetConfig's connection string.
+     */
+    virtual ConnectionString getConfigConnectionString() const = 0;
+
+    /**
+     * Returns the current ReplSetConfig's election timeout period.
+     */
+    virtual Milliseconds getConfigElectionTimeoutPeriod() const = 0;
+
+    /**
+     * Returns the current ReplSetConfig's voting members.
+     */
+    virtual std::vector<MemberConfig> getConfigVotingMembers() const = 0;
+
+    /**
+     * Returns the current ReplSetConfig's term.
+     */
+    virtual std::int64_t getConfigTerm() const = 0;
+
+    /**
+     * Returns the current ReplSetConfig's version.
+     */
+    virtual std::int64_t getConfigVersion() const = 0;
+
+    /**
+     * Returns the (version, term) pair of the current ReplSetConfig.
+     */
+    virtual ConfigVersionAndTerm getConfigVersionAndTerm() const = 0;
+
+    /**
+     * Returns the number of members in the current ReplSetConfig.
+     */
+    virtual int getConfigNumMembers() const = 0;
+
+    /**
+     * Returns the amount of time to wait for a response to heartbeats sent to other
+     * nodes in the current ReplSetConfig.
+     */
+    virtual Milliseconds getConfigHeartbeatTimeoutPeriodMillis() const = 0;
+
+    /**
+     * Returns the BSON of the current ReplSetConfig.
+     */
+    virtual BSONObj getConfigBSON() const = 0;
+
+    /**
+     * Returns a pointer to the MemberConfig corresponding to the member with the given
+     * HostAndPort in the current ReplSetConfig, or NULL if there is no member with that address.
+     */
+    virtual const MemberConfig* findConfigMemberByHostAndPort(const HostAndPort& hap) const = 0;
+
+    /**
+     * Returns whether all members of the current ReplSetConfig set have hostname localhost.
+     */
+    virtual bool isConfigLocalHostAllowed() const = 0;
+
+    /**
+     * Returns the interval between the time the last heartbeat from a node was received
+     * successfully, or the time when we gave up retrying, and when the next heartbeat should be
+     * sent to a target, for the current ReplSetConfig.
+     */
+    virtual Milliseconds getConfigHeartbeatInterval() const = 0;
+
     /**
      * Handles an incoming replSetGetConfig command. Adds BSON to 'result'.
      *
@@ -667,9 +742,12 @@ public:
     /**
      * Toggles maintenanceMode to the value expressed by 'activate'
      * return Status::OK if the change worked, NotSecondary if it failed because we are
-     * PRIMARY, and OperationFailed if we are not currently in maintenance mode
+     * PRIMARY, and OperationFailed if we are not currently in maintenance mode.
+     *
+     * Takes the ReplicationStateTransitionLock (RSTL) in X mode, since the state can potentially
+     * change to and from RECOVERING.
      */
-    virtual Status setMaintenanceMode(bool activate) = 0;
+    virtual Status setMaintenanceMode(OperationContext* opCtx, bool activate) = 0;
 
     /**
      * Retrieves the current count of maintenanceMode and returns 'true' if greater than 0.
@@ -953,12 +1031,6 @@ public:
      */
     virtual bool setContainsArbiter() const = 0;
 
-    /**
-     * Returns true if the current replica set config has at least one member with 'newlyAdded'
-     * field set to true.
-     */
-    virtual bool replSetContainsNewlyAddedMembers() const = 0;
-
     /**
      * Instructs the ReplicationCoordinator to recalculate the stable timestamp and advance it for
      * storage if needed.
@@ -991,9 +1063,9 @@ public:
      * internal operations occurs (i.e. step up, step down, or rollback). Also logs the metrics.
      */
     virtual void updateAndLogStateTransitionMetrics(
-        const ReplicationCoordinator::OpsKillingStateTransitionEnum stateTransition,
-        const size_t numOpsKilled,
-        const size_t numOpsRunning) const = 0;
+        ReplicationCoordinator::OpsKillingStateTransitionEnum stateTransition,
+        size_t numOpsKilled,
+        size_t numOpsRunning) const = 0;
 
     /**
      * Increment the server TopologyVersion and fulfill the promise of any currently waiting
@@ -1069,6 +1141,13 @@ public:
      */
     virtual void restartScheduledHeartbeats_forTest() = 0;
 
+    /**
+     * Records if the cluster-wide write concern is set during sharding initialization.
+     *
+     * This function will assert if the shard can't talk to config server.
+     */
+    virtual void recordIfCWWCIsSetOnConfigServerOnStartup(OperationContext* opCtx) = 0;
+
 protected:
     ReplicationCoordinator();
 };
