@@ -42,6 +42,7 @@
 #include "mongo/bson/util/bson_extract.h"
 #include "mongo/db/auth/authorization_session.h"
 #include "mongo/db/catalog/coll_mod.h"
+#include "mongo/db/catalog/create_collection.h"
 #include "mongo/db/catalog/database.h"
 #include "mongo/db/catalog/database_holder.h"
 #include "mongo/db/catalog/local_oplog_info.h"
@@ -50,7 +51,7 @@
 #include "mongo/db/commands/rwc_defaults_commands_gen.h"
 #include "mongo/db/commands/server_status_metric.h"
 #include "mongo/db/concurrency/d_concurrency.h"
-#include "mongo/db/concurrency/exception_util.h"
+#include "mongo/db/concurrency/write_conflict_exception.h"
 #include "mongo/db/db_raii.h"
 #include "mongo/db/dbdirectclient.h"
 #include "mongo/db/dbhelpers.h"
@@ -60,6 +61,7 @@
 #include "mongo/db/kill_sessions_local.h"
 #include "mongo/db/logical_time_validator.h"
 #include "mongo/db/op_observer.h"
+#include "mongo/db/query/query_feature_flags_gen.h"
 #include "mongo/db/repl/always_allow_non_local_writes.h"
 #include "mongo/db/repl/bgsync.h"
 #include "mongo/db/repl/drop_pending_collection_reaper.h"
@@ -96,7 +98,6 @@
 #include "mongo/db/system_index.h"
 #include "mongo/db/vector_clock.h"
 #include "mongo/db/vector_clock_metadata_hook.h"
-#include "mongo/db/vector_clock_mutable.h"
 #include "mongo/executor/network_connection_hook.h"
 #include "mongo/executor/network_interface.h"
 #include "mongo/executor/network_interface_factory.h"
@@ -126,7 +127,6 @@ namespace {
 const char localDbName[] = "local";
 const char configCollectionName[] = "local.system.replset";
 const auto configDatabaseName = localDbName;
-const char lastVoteCollectionName[] = "local.replset.election";
 const auto lastVoteDatabaseName = localDbName;
 const char meCollectionName[] = "local.me";
 const auto meDatabaseName = localDbName;
@@ -333,6 +333,10 @@ void ReplicationCoordinatorExternalStateImpl::_stopDataReplication_inlock(
 }
 
 
+JournalListener* ReplicationCoordinatorExternalStateImpl::getReplicationJournalListener() {
+    return this;
+}
+
 void ReplicationCoordinatorExternalStateImpl::startThreads() {
     stdx::lock_guard<Latch> lk(_threadMutex);
     if (_startedThreads) {
@@ -458,11 +462,18 @@ Status ReplicationCoordinatorExternalStateImpl::initializeReplSetStorage(Operati
                                // Permit writing to the oplog before we step up to primary.
                                AllowNonLocalWritesBlock allowNonLocalWrites(opCtx);
                                Lock::GlobalWrite globalWrite(opCtx);
-                               WriteUnitOfWork wuow(opCtx);
-                               Helpers::putSingleton(opCtx, configCollectionName, config);
-                               const auto msgObj = BSON("msg" << kInitiatingSetMsg);
-                               _service->getOpObserver()->onOpMessage(opCtx, msgObj);
-                               wuow.commit();
+                               {
+                                   // Writes to 'local.system.replset' must be untimestamped.
+                                   WriteUnitOfWork wuow(opCtx);
+                                   Helpers::putSingleton(opCtx, configCollectionName, config);
+                                   wuow.commit();
+                               }
+                               {
+                                   WriteUnitOfWork wuow(opCtx);
+                                   const auto msgObj = BSON("msg" << kInitiatingSetMsg);
+                                   _service->getOpObserver()->onOpMessage(opCtx, msgObj);
+                                   wuow.commit();
+                               }
                            });
 
         // ReplSetTest assumes that immediately after the replSetInitiate command returns, it can
@@ -573,6 +584,12 @@ OpTime ReplicationCoordinatorExternalStateImpl::onTransitionToPrimary(OperationC
         }
     });
 
+    // Create the pre-images collection if it doesn't exist yet.
+    if (::mongo::feature_flags::gFeatureFlagChangeStreamPreAndPostImages.isEnabled(
+            serverGlobalParams.featureCompatibility)) {
+        createChangeStreamPreImagesCollection(opCtx);
+    }
+
     serverGlobalParams.validateFeaturesAsPrimary.store(true);
 
     return opTimeToReturn;
@@ -605,17 +622,27 @@ Status ReplicationCoordinatorExternalStateImpl::storeLocalConfigDocument(Operati
                                                                          bool writeOplog) {
     try {
         writeConflictRetry(opCtx, "save replica set config", configCollectionName, [&] {
-            WriteUnitOfWork wuow(opCtx);
-            Lock::DBLock dbWriteLock(opCtx, configDatabaseName, MODE_X);
-            Helpers::putSingleton(opCtx, configCollectionName, config);
+            {
+                // Writes to 'local.system.replset' must be untimestamped.
+                WriteUnitOfWork wuow(opCtx);
+                Lock::DBLock dbWriteLock(opCtx, configDatabaseName, MODE_X);
+                Helpers::putSingleton(opCtx, configCollectionName, config);
+                wuow.commit();
+            }
 
             if (writeOplog) {
+                // The no-op write doesn't affect the correctness of the safe reconfig protocol and
+                // so it doesn't have to be written in the same WUOW as the config write. In fact,
+                // the no-op write is only needed for some corner cases where the committed snapshot
+                // is dropped after a force reconfig that changes the config content or a safe
+                // reconfig that changes writeConcernMajorityJournalDefault.
+                WriteUnitOfWork wuow(opCtx);
                 auto msgObj = BSON("msg"
                                    << "Reconfig set"
                                    << "version" << config["version"]);
                 _service->getOpObserver()->onOpMessage(opCtx, msgObj);
+                wuow.commit();
             }
-            wuow.commit();
         });
         return Status::OK();
     } catch (const DBException& ex) {
@@ -626,23 +653,29 @@ Status ReplicationCoordinatorExternalStateImpl::storeLocalConfigDocument(Operati
 Status ReplicationCoordinatorExternalStateImpl::createLocalLastVoteCollection(
     OperationContext* opCtx) {
     auto status = _storageInterface->createCollection(
-        opCtx, NamespaceString(lastVoteCollectionName), CollectionOptions());
+        opCtx, NamespaceString::kLastVoteNamespace, CollectionOptions());
     if (!status.isOK() && status.code() != ErrorCodes::NamespaceExists) {
         return {ErrorCodes::CannotCreateCollection,
                 str::stream() << "Failed to create local last vote collection. Ns: "
-                              << lastVoteCollectionName << " Error: " << status.toString()};
+                              << NamespaceString::kLastVoteNamespace.toString()
+                              << " Error: " << status.toString()};
     }
 
     // Make sure there's always a last vote document.
     try {
         writeConflictRetry(
-            opCtx, "create initial replica set lastVote", lastVoteCollectionName, [opCtx] {
-                AutoGetCollection coll(opCtx, NamespaceString(lastVoteCollectionName), MODE_X);
+            opCtx,
+            "create initial replica set lastVote",
+            NamespaceString::kLastVoteNamespace.toString(),
+            [opCtx] {
+                AutoGetCollection coll(opCtx, NamespaceString::kLastVoteNamespace, MODE_X);
                 BSONObj result;
-                bool exists = Helpers::getSingleton(opCtx, lastVoteCollectionName, result);
+                bool exists = Helpers::getSingleton(
+                    opCtx, NamespaceString::kLastVoteNamespace.ns().c_str(), result);
                 if (!exists) {
                     LastVote lastVote{OpTime::kInitialTerm, -1};
-                    Helpers::putSingleton(opCtx, lastVoteCollectionName, lastVote.toBSON());
+                    Helpers::putSingleton(
+                        opCtx, NamespaceString::kLastVoteNamespace.ns().c_str(), lastVote.toBSON());
                 }
             });
     } catch (const DBException& ex) {
@@ -656,13 +689,18 @@ StatusWith<LastVote> ReplicationCoordinatorExternalStateImpl::loadLocalLastVoteD
     OperationContext* opCtx) {
     try {
         return writeConflictRetry(
-            opCtx, "load replica set lastVote", lastVoteCollectionName, [opCtx] {
+            opCtx,
+            "load replica set lastVote",
+            NamespaceString::kLastVoteNamespace.toString(),
+            [opCtx] {
                 BSONObj lastVoteObj;
-                if (!Helpers::getSingleton(opCtx, lastVoteCollectionName, lastVoteObj)) {
+                if (!Helpers::getSingleton(opCtx,
+                                           NamespaceString::kLastVoteNamespace.toString().c_str(),
+                                           lastVoteObj)) {
                     return StatusWith<LastVote>(
                         ErrorCodes::NoMatchingDocument,
                         str::stream() << "Did not find replica set lastVote document in "
-                                      << lastVoteCollectionName);
+                                      << NamespaceString::kLastVoteNamespace.toString());
                 }
                 return LastVote::readFromLastVote(lastVoteObj);
             });
@@ -685,9 +723,17 @@ Status ReplicationCoordinatorExternalStateImpl::storeLocalLastVoteDocument(
         // acquisition and the "waitUntilDurable" lock acquisition must be uninterruptible.
         UninterruptibleLockGuard noInterrupt(opCtx->lockState());
 
-        Status status =
-            writeConflictRetry(opCtx, "save replica set lastVote", lastVoteCollectionName, [&] {
-                AutoGetCollection coll(opCtx, NamespaceString(lastVoteCollectionName), MODE_IX);
+        Status status = writeConflictRetry(
+            opCtx,
+            "save replica set lastVote",
+            NamespaceString::kLastVoteNamespace.toString(),
+            [&] {
+                // Writes to non-replicated collections do not need concurrency control with the
+                // OplogApplier that never accesses them. Skip taking the PBWM.
+                ShouldNotConflictWithSecondaryBatchApplicationBlock shouldNotConflictBlock(
+                    opCtx->lockState());
+
+                AutoGetCollection coll(opCtx, NamespaceString::kLastVoteNamespace, MODE_IX);
                 WriteUnitOfWork wunit(opCtx);
 
                 // We only want to replace the last vote document if the new last vote document
@@ -696,14 +742,16 @@ Status ReplicationCoordinatorExternalStateImpl::storeLocalLastVoteDocument(
                 // operations. We have already ensured at startup time that there is an old
                 // document.
                 BSONObj result;
-                bool exists = Helpers::getSingleton(opCtx, lastVoteCollectionName, result);
+                bool exists = Helpers::getSingleton(
+                    opCtx, NamespaceString::kLastVoteNamespace.ns().c_str(), result);
                 fassert(51241, exists);
                 StatusWith<LastVote> oldLastVoteDoc = LastVote::readFromLastVote(result);
                 if (!oldLastVoteDoc.isOK()) {
                     return oldLastVoteDoc.getStatus();
                 }
                 if (lastVote.getTerm() > oldLastVoteDoc.getValue().getTerm()) {
-                    Helpers::putSingleton(opCtx, lastVoteCollectionName, lastVoteObj);
+                    Helpers::putSingleton(
+                        opCtx, NamespaceString::kLastVoteNamespace.ns().c_str(), lastVoteObj);
                 }
                 wunit.commit();
                 return Status::OK();
@@ -768,16 +816,6 @@ bool ReplicationCoordinatorExternalStateImpl::isSelf(const HostAndPort& host, Se
     return repl::isSelf(host, ctx);
 }
 
-bool ReplicationCoordinatorExternalStateImpl::isSelfFastPath(const HostAndPort& host) {
-    return repl::isSelfFastPath(host);
-}
-
-bool ReplicationCoordinatorExternalStateImpl::isSelfSlowPath(const HostAndPort& host,
-                                                             ServiceContext* ctx,
-                                                             Milliseconds timeout) {
-    return repl::isSelfSlowPath(host, ctx, timeout);
-}
-
 HostAndPort ReplicationCoordinatorExternalStateImpl::getClientHostAndPort(
     const OperationContext* opCtx) {
     return HostAndPort(opCtx->getClient()->clientAddress(true));
@@ -903,7 +941,6 @@ void ReplicationCoordinatorExternalStateImpl::_shardingOnTransitionToPrimaryHook
         TransactionCoordinatorService::get(_service)->onStepUp(opCtx);
     } else if (ShardingState::get(opCtx)->enabled()) {
         Status status = ShardingStateRecovery::recover(opCtx);
-        VectorClockMutable::get(opCtx)->recoverDirect(opCtx);
 
         // If the node is shutting down or it lost quorum just as it was becoming primary, don't
         // run the sharding onStepUp machinery. The onStepDown counterpart to these methods is
@@ -929,6 +966,7 @@ void ReplicationCoordinatorExternalStateImpl::_shardingOnTransitionToPrimaryHook
         // refreshes which should use the recovered configOpTime.
         migrationutil::resubmitRangeDeletionsOnStepUp(_service);
         migrationutil::resumeMigrationCoordinationsOnStepUp(opCtx);
+        migrationutil::resumeMigrationRecipientsOnStepUp(opCtx);
 
         const bool scheduleAsyncRefresh = true;
         resharding::clearFilteringMetadata(opCtx, scheduleAsyncRefresh);
