@@ -38,8 +38,8 @@
 #include "mongo/bson/timestamp.h"
 #include "mongo/db/concurrency/d_concurrency.h"
 #include "mongo/db/concurrency/replication_state_transition_lock_guard.h"
-#include "mongo/db/repl/delayable_timeout_callback.h"
 #include "mongo/db/repl/initial_syncer.h"
+#include "mongo/db/repl/initial_syncer_interface.h"
 #include "mongo/db/repl/member_state.h"
 #include "mongo/db/repl/optime.h"
 #include "mongo/db/repl/repl_set_config.h"
@@ -126,7 +126,9 @@ public:
 
     virtual bool canAcceptNonLocalWrites() const override;
 
-    virtual Status waitForMemberState(MemberState expectedState, Milliseconds timeout) override;
+    virtual Status waitForMemberState(Interruptible* interruptible,
+                                      MemberState expectedState,
+                                      Milliseconds timeout) override;
 
     virtual bool isInPrimaryOrSecondaryState(OperationContext* opCtx) const override;
 
@@ -218,19 +220,45 @@ public:
     virtual ApplierState getApplierState() override;
 
     virtual void signalDrainComplete(OperationContext* opCtx,
-                                     long long termWhenBufferIsEmpty) override;
+                                     long long termWhenBufferIsEmpty) noexcept override;
 
     virtual void signalUpstreamUpdater() override;
 
     virtual StatusWith<BSONObj> prepareReplSetUpdatePositionCommand() const override;
 
-    virtual Status processReplSetGetStatus(BSONObjBuilder* result,
+    virtual Status processReplSetGetStatus(OperationContext* opCtx,
+                                           BSONObjBuilder* result,
                                            ReplSetGetStatusResponseStyle responseStyle) override;
 
     virtual void appendSecondaryInfoData(BSONObjBuilder* result) override;
 
     virtual ReplSetConfig getConfig() const override;
 
+    virtual ConnectionString getConfigConnectionString() const override;
+
+    virtual Milliseconds getConfigElectionTimeoutPeriod() const override;
+
+    virtual std::vector<MemberConfig> getConfigVotingMembers() const override;
+
+    virtual std::int64_t getConfigTerm() const override;
+
+    virtual std::int64_t getConfigVersion() const override;
+
+    virtual ConfigVersionAndTerm getConfigVersionAndTerm() const override;
+
+    virtual int getConfigNumMembers() const override;
+
+    virtual Milliseconds getConfigHeartbeatTimeoutPeriodMillis() const override;
+
+    virtual BSONObj getConfigBSON() const override;
+
+    virtual const MemberConfig* findConfigMemberByHostAndPort(
+        const HostAndPort& hap) const override;
+
+    virtual bool isConfigLocalHostAllowed() const override;
+
+    virtual Milliseconds getConfigHeartbeatInterval() const override;
+
     virtual void processReplSetGetConfig(BSONObjBuilder* result,
                                          bool commitmentStatus = false,
                                          bool includeNewlyAdded = false) override;
@@ -288,15 +316,11 @@ public:
 
     virtual void resetLastOpTimesFromOplog(OperationContext* opCtx) override;
 
-    virtual ChangeSyncSourceAction shouldChangeSyncSource(
-        const HostAndPort& currentSource,
-        const rpc::ReplSetMetadata& replMetadata,
-        const rpc::OplogQueryMetadata& oqMetadata,
-        const OpTime& previousOpTimeFetched,
-        const OpTime& lastOpTimeFetched) const override;
-
-    virtual ChangeSyncSourceAction shouldChangeSyncSourceOnError(
-        const HostAndPort& currentSource, const OpTime& lastOpTimeFetched) const override;
+    virtual ChangeSyncSourceAction shouldChangeSyncSource(const HostAndPort& currentSource,
+                                                          const rpc::ReplSetMetadata& replMetadata,
+                                                          const rpc::OplogQueryMetadata& oqMetadata,
+                                                          const OpTime& previousOpTimeFetched,
+                                                          const OpTime& lastOpTimeFetched) override;
 
     virtual OpTime getLastCommittedOpTime() const override;
     virtual OpTimeAndWallTime getLastCommittedOpTimeAndWallTime() const override;
@@ -353,16 +377,14 @@ public:
 
     virtual bool setContainsArbiter() const override;
 
-    virtual bool replSetContainsNewlyAddedMembers() const override;
-
     virtual void attemptToAdvanceStableTimestamp() override;
 
     virtual void finishRecoveryIfEligible(OperationContext* opCtx) override;
 
     virtual void updateAndLogStateTransitionMetrics(
-        const ReplicationCoordinator::OpsKillingStateTransitionEnum stateTransition,
-        const size_t numOpsKilled,
-        const size_t numOpsRunning) const override;
+        ReplicationCoordinator::OpsKillingStateTransitionEnum stateTransition,
+        size_t numOpsKilled,
+        size_t numOpsRunning) const override;
 
     virtual TopologyVersion getTopologyVersion() const override;
 
@@ -1049,13 +1071,6 @@ private:
      */
     Milliseconds _getRandomizedElectionOffset_inlock();
 
-    /*
-     * Return the upper bound of the offset amount returned by _getRandomizedElectionOffset
-     * This is actually off by one, that is, the election offset is in the half-open range
-     * [0, electionOffsetUpperBound)
-     */
-    long long _getElectionOffsetUpperBound_inlock();
-
     /**
      * Starts a heartbeat for each member in the current config.  Called while holding _mutex.
      */
@@ -1115,8 +1130,20 @@ private:
     /**
      * Start replicating data, and does an initial sync if needed first.
      */
-    void _startDataReplication(OperationContext* opCtx,
-                               std::function<void()> startCompleted = nullptr);
+    void _startDataReplication(OperationContext* opCtx);
+
+    /**
+     * Start initial sync.
+     */
+    void _startInitialSync(OperationContext* opCtx,
+                           InitialSyncerInterface::OnCompletionFn onCompletionFn,
+                           bool fallbackToLogical = false);
+
+
+    /**
+     * Function to be called on completion of initial sync.
+     */
+    void _initialSyncerCompletionFunction(const StatusWith<OpTimeAndWallTime>& opTimeStatus);
 
     /**
      * Finishes the work of processReplSetInitiate() in the event of a successful quorum check.
@@ -1257,7 +1284,7 @@ private:
      */
     std::shared_ptr<HelloResponse> _makeHelloResponse(boost::optional<StringData> horizonString,
                                                       WithLock,
-                                                      const bool hasValidConfig) const;
+                                                      bool hasValidConfig) const;
 
     /**
      * Creates a semi-future for HelloResponse. horizonString should be passed in if and only if
@@ -1370,10 +1397,8 @@ private:
     /**
      * Bottom half of _scheduleNextLivenessUpdate.
      * Must be called with _mutex held.
-     * If reschedule is true, will recompute the liveness update even if a timeout is
-     * already pending.
      */
-    void _scheduleNextLivenessUpdate_inlock(bool reschedule);
+    void _scheduleNextLivenessUpdate_inlock();
 
     /**
      * Callback which marks downed nodes as down, triggers a stepdown if a majority of nodes are no
@@ -1382,11 +1407,11 @@ private:
     void _handleLivenessTimeout(const executor::TaskExecutor::CallbackArgs& cbData);
 
     /**
-     * If "updatedMemberId" is the current _earliestMemberId, calls _scheduleNextLivenessUpdate to
-     * schedule a new one.
+     * If "updatedMemberId" is the current _earliestMemberId, cancels the current
+     * _handleLivenessTimeout callback and calls _scheduleNextLivenessUpdate to schedule a new one.
      * Returns immediately otherwise.
      */
-    void _rescheduleLivenessUpdate_inlock(int updatedMemberId);
+    void _cancelAndRescheduleLivenessUpdate_inlock(int updatedMemberId);
 
     /**
      * Cancels all outstanding _priorityTakeover callbacks.
@@ -1498,15 +1523,10 @@ private:
      */
     void _setImplicitDefaultWriteConcern(OperationContext* opCtx, WithLock lk);
 
-    /**
-     * Checks whether replication coordinator supports automatic reconfig.
-     */
-    bool _supportsAutomaticReconfig() const;
-
     /*
      * Calculates and returns the read preference for the node.
      */
-    const ReadPreference _getSyncSourceReadPreference(WithLock) const;
+    const ReadPreference _getSyncSourceReadPreference(WithLock);
 
     /*
      * Performs the replica set reconfig procedure. Certain consensus safety checks are omitted when
@@ -1629,7 +1649,7 @@ private:
     // Storage interface used by initial syncer.
     StorageInterface* _storage;  // (PS)
     // InitialSyncer used for initial sync.
-    std::shared_ptr<InitialSyncer>
+    std::shared_ptr<InitialSyncerInterface>
         _initialSyncer;  // (I) pointer set under mutex, copied by callers.
 
     // The non-null OpTime used for committed reads, if there is one.
@@ -1643,10 +1663,15 @@ private:
     stdx::condition_variable _currentCommittedSnapshotCond;  // (M)
 
     // Callback Handle used to cancel a scheduled LivenessTimeout callback.
-    DelayableTimeoutCallback _handleLivenessTimeoutCallback;  // (S)
+    executor::TaskExecutor::CallbackHandle _handleLivenessTimeoutCbh;  // (M)
 
-    // Used to manage scheduling and canceling election timeouts.
-    DelayableTimeoutCallbackWithJitter _handleElectionTimeoutCallback;  // (M)
+    // Callback Handle used to cancel a scheduled ElectionTimeout callback.
+    executor::TaskExecutor::CallbackHandle _handleElectionTimeoutCbh;  // (M)
+
+    // Election timeout callback will not run before this time.
+    // If this date is Date_t(), the callback is either unscheduled or canceled.
+    // Used for testing only.
+    Date_t _handleElectionTimeoutWhen;  // (M)
 
     // Callback Handle used to cancel a scheduled PriorityTakeover callback.
     executor::TaskExecutor::CallbackHandle _priorityTakeoverCbh;  // (M)
@@ -1718,6 +1743,8 @@ private:
 
     // This should be set during sharding initialization.
     boost::optional<bool> _wasCWWCSetOnConfigServerOnStartup;
+
+    InitialSyncerInterface::OnCompletionFn _onCompletion;
 };
 
 }  // namespace repl
