@@ -165,7 +165,7 @@ bool ReplicationCoordinatorNoOp::canAcceptNonLocalWrites() const {
     MONGO_UNREACHABLE;
 }
 
-Status ReplicationCoordinatorNoOp::waitForMemberState(MemberState, Milliseconds) {
+Status ReplicationCoordinatorNoOp::waitForMemberState(Interruptible*, MemberState, Milliseconds) {
     MONGO_UNREACHABLE;
 }
 
@@ -189,7 +189,7 @@ ReplicationCoordinator::ApplierState ReplicationCoordinatorNoOp::getApplierState
     MONGO_UNREACHABLE;
 }
 
-void ReplicationCoordinatorNoOp::signalDrainComplete(OperationContext*, long long) {
+void ReplicationCoordinatorNoOp::signalDrainComplete(OperationContext*, long long) noexcept {
     MONGO_UNREACHABLE;
 }
 
@@ -285,7 +285,8 @@ StatusWith<BSONObj> ReplicationCoordinatorNoOp::prepareReplSetUpdatePositionComm
     MONGO_UNREACHABLE;
 }
 
-Status ReplicationCoordinatorNoOp::processReplSetGetStatus(BSONObjBuilder*,
+Status ReplicationCoordinatorNoOp::processReplSetGetStatus(OperationContext* opCtx,
+                                                           BSONObjBuilder*,
                                                            ReplSetGetStatusResponseStyle) {
     MONGO_UNREACHABLE;
 }
@@ -298,6 +299,55 @@ ReplSetConfig ReplicationCoordinatorNoOp::getConfig() const {
     MONGO_UNREACHABLE;
 }
 
+ConnectionString ReplicationCoordinatorNoOp::getConfigConnectionString() const {
+    MONGO_UNREACHABLE;
+}
+
+Milliseconds ReplicationCoordinatorNoOp::getConfigElectionTimeoutPeriod() const {
+    MONGO_UNREACHABLE;
+}
+
+std::vector<MemberConfig> ReplicationCoordinatorNoOp::getConfigVotingMembers() const {
+    MONGO_UNREACHABLE;
+}
+
+std::int64_t ReplicationCoordinatorNoOp::getConfigTerm() const {
+    MONGO_UNREACHABLE;
+}
+
+std::int64_t ReplicationCoordinatorNoOp::getConfigVersion() const {
+    MONGO_UNREACHABLE;
+}
+
+ConfigVersionAndTerm ReplicationCoordinatorNoOp::getConfigVersionAndTerm() const {
+    MONGO_UNREACHABLE;
+}
+
+int ReplicationCoordinatorNoOp::getConfigNumMembers() const {
+    MONGO_UNREACHABLE;
+}
+
+Milliseconds ReplicationCoordinatorNoOp::getConfigHeartbeatTimeoutPeriodMillis() const {
+    MONGO_UNREACHABLE;
+}
+
+BSONObj ReplicationCoordinatorNoOp::getConfigBSON() const {
+    MONGO_UNREACHABLE;
+}
+
+const MemberConfig* ReplicationCoordinatorNoOp::findConfigMemberByHostAndPort(
+    const HostAndPort& hap) const {
+    MONGO_UNREACHABLE;
+}
+
+bool ReplicationCoordinatorNoOp::isConfigLocalHostAllowed() const {
+    MONGO_UNREACHABLE;
+}
+
+Milliseconds ReplicationCoordinatorNoOp::getConfigHeartbeatInterval() const {
+    MONGO_UNREACHABLE;
+}
+
 void ReplicationCoordinatorNoOp::processReplSetGetConfig(BSONObjBuilder* result,
                                                          bool commitmentStatus,
                                                          bool includeNewlyAdded) {
@@ -479,10 +529,6 @@ bool ReplicationCoordinatorNoOp::setContainsArbiter() const {
     MONGO_UNREACHABLE;
 }
 
-bool ReplicationCoordinatorNoOp::replSetContainsNewlyAddedMembers() const {
-    MONGO_UNREACHABLE;
-}
-
 void ReplicationCoordinatorNoOp::attemptToAdvanceStableTimestamp() {
     MONGO_UNREACHABLE;
 }
