@@ -52,9 +52,6 @@
 
 namespace mongo {
 
-using MigrateInfoVector = BalancerChunkSelectionPolicy::MigrateInfoVector;
-using SplitInfoVector = BalancerChunkSelectionPolicy::SplitInfoVector;
-
 namespace {
 
 /**
@@ -76,7 +73,7 @@ StatusWith<DistributionStatus> createCollectionDistributionStatus(
 
     chunkMgr.forEachChunk([&](const auto& chunkEntry) {
         ChunkType chunk;
-        chunk.setCollectionUUID(*chunkMgr.getUUID());
+        chunk.setCollectionUUID(chunkMgr.getUUID());
         chunk.setMin(chunkEntry.getMin());
         chunk.setMax(chunkEntry.getMax());
         chunk.setJumbo(chunkEntry.isJumbo());
@@ -88,28 +85,14 @@ StatusWith<DistributionStatus> createCollectionDistributionStatus(
         return true;
     });
 
-    const auto swCollectionTags =
-        Grid::get(opCtx)->catalogClient()->getTagsForCollection(opCtx, nss);
-    if (!swCollectionTags.isOK()) {
-        return swCollectionTags.getStatus().withContext(
-            str::stream() << "Unable to load tags for collection " << nss);
-    }
-    const auto& collectionTags = swCollectionTags.getValue();
-
     DistributionStatus distribution(nss, std::move(shardToChunksMap));
 
-    // Cache the collection tags
     const auto& keyPattern = chunkMgr.getShardKeyPattern().getKeyPattern();
 
-    for (const auto& tag : collectionTags) {
-        auto status = distribution.addRangeToZone(
-            ZoneRange(keyPattern.extendRangeBound(tag.getMinKey(), false),
-                      keyPattern.extendRangeBound(tag.getMaxKey(), false),
-                      tag.getTag()));
-
-        if (!status.isOK()) {
-            return status;
-        }
+    // Cache the collection tags
+    auto status = ZoneInfo::addTagsFromCatalog(opCtx, nss, keyPattern, distribution.zoneInfo());
+    if (!status.isOK()) {
+        return status;
     }
 
     return {std::move(distribution)};
@@ -129,8 +112,7 @@ public:
     SplitCandidatesBuffer(NamespaceString nss, ChunkVersion collectionVersion)
         : _nss(std::move(nss)),
           _collectionVersion(collectionVersion),
-          _chunkSplitPoints(SimpleBSONObjComparator::kInstance
-                                .makeBSONObjIndexedMap<BalancerChunkSelectionPolicy::SplitInfo>()) {
+          _chunkSplitPoints(SimpleBSONObjComparator::kInstance.makeBSONObjIndexedMap<SplitInfo>()) {
     }
 
     /**
@@ -141,13 +123,13 @@ public:
         auto it = _chunkSplitPoints.find(chunk.getMin());
         if (it == _chunkSplitPoints.end()) {
             _chunkSplitPoints.emplace(chunk.getMin(),
-                                      BalancerChunkSelectionPolicy::SplitInfo(chunk.getShardId(),
-                                                                              _nss,
-                                                                              _collectionVersion,
-                                                                              chunk.getLastmod(),
-                                                                              chunk.getMin(),
-                                                                              chunk.getMax(),
-                                                                              {splitPoint}));
+                                      SplitInfo(chunk.getShardId(),
+                                                _nss,
+                                                _collectionVersion,
+                                                chunk.getLastmod(),
+                                                chunk.getMin(),
+                                                chunk.getMax(),
+                                                {splitPoint}));
         } else if (splitPoint.woCompare(it->second.splitKeys.back()) > 0) {
             it->second.splitKeys.push_back(splitPoint);
         } else {
@@ -161,7 +143,7 @@ public:
      * a vector of split infos to be passed to the split call.
      */
     SplitInfoVector done() {
-        BalancerChunkSelectionPolicy::SplitInfoVector splitPoints;
+        SplitInfoVector splitPoints;
         for (const auto& entry : _chunkSplitPoints) {
             splitPoints.push_back(std::move(entry.second));
         }
@@ -175,7 +157,7 @@ private:
     const ChunkVersion _collectionVersion;
 
     // Chunk min key and split vector associated with that chunk
-    BSONObjIndexedMap<BalancerChunkSelectionPolicy::SplitInfo> _chunkSplitPoints;
+    BSONObjIndexedMap<SplitInfo> _chunkSplitPoints;
 };
 
 /**
@@ -365,15 +347,16 @@ StatusWith<MigrateInfoVector> BalancerChunkSelectionPolicyImpl::selectChunksToMo
     for (const auto& coll : collections) {
         const NamespaceString& nss(coll.getNss());
 
-        if (!coll.getAllowBalance() || !coll.getAllowMigrations() || !coll.getPermitMigrations()) {
-            LOGV2_DEBUG(21851,
+        if (!coll.getAllowBalance() || !coll.getAllowMigrations() || !coll.getPermitMigrations() ||
+            coll.getBalancerShouldMergeChunks()) {
+            LOGV2_DEBUG(5966401,
                         1,
-                        "Not balancing collection {namespace}; explicitly disabled.",
                         "Not balancing explicitly disabled collection",
                         "namespace"_attr = nss,
                         "allowBalance"_attr = coll.getAllowBalance(),
                         "allowMigrations"_attr = coll.getAllowMigrations(),
-                        "timeseriesFields"_attr = coll.getTimeseriesFields());
+                        "permitMigrations"_attr = coll.getPermitMigrations(),
+                        "balancerShouldMergeChunks"_attr = coll.getBalancerShouldMergeChunks());
             continue;
         }
 
