@@ -121,13 +121,13 @@ void moveChunk(OperationContext* opCtx, const NamespaceString& nss, const BSONOb
     const auto suggestedChunk = cm.findIntersectingChunkWithSimpleCollation(minKey);
 
     ChunkType chunkToMove;
-    chunkToMove.setNS(nss);
+    chunkToMove.setCollectionUUID(cm.getUUID());
     chunkToMove.setShard(suggestedChunk.getShardId());
     chunkToMove.setMin(suggestedChunk.getMin());
     chunkToMove.setMax(suggestedChunk.getMax());
     chunkToMove.setVersion(suggestedChunk.getLastmod());
 
-    uassertStatusOK(configsvr_client::rebalanceChunk(opCtx, chunkToMove));
+    uassertStatusOK(configsvr_client::rebalanceChunk(opCtx, nss, chunkToMove));
 }
 
 /**
@@ -145,10 +145,10 @@ BSONObj findExtremeKeyForShard(OperationContext* opCtx,
                                const NamespaceString& nss,
                                const ShardKeyPattern& shardKeyPattern,
                                bool doSplitAtLower) {
-    Query q;
+    FindCommandRequest findCmd{nss};
 
     if (doSplitAtLower) {
-        q.sort(shardKeyPattern.toBSON());
+        findCmd.setSort(shardKeyPattern.toBSON());
     } else {
         // need to invert shard key pattern to sort backwards
         BSONObjBuilder r;
@@ -160,7 +160,7 @@ BSONObj findExtremeKeyForShard(OperationContext* opCtx,
             r.append(e.fieldName(), -1 * e.number());
         }
 
-        q.sort(r.obj());
+        findCmd.setSort(r.obj());
     }
 
     DBDirectClient client(opCtx);
@@ -168,13 +168,12 @@ BSONObj findExtremeKeyForShard(OperationContext* opCtx,
     BSONObj end;
 
     if (doSplitAtLower) {
-        // Splitting close to the lower bound means that the split point will be the
-        // upper bound. Chunk range upper bounds are exclusive so skip a document to
-        // make the lower half of the split end up with a single document.
-        std::unique_ptr<DBClientCursor> cursor = client.query(nss,
-                                                              q,
-                                                              1, /* nToReturn */
-                                                              1 /* nToSkip */);
+        // Splitting close to the lower bound means that the split point will be the upper bound.
+        // Chunk range upper bounds are exclusive so skip a document to make the lower half of the
+        // split end up with a single document.
+        findCmd.setLimit(1);
+        findCmd.setSkip(1);
+        std::unique_ptr<DBClientCursor> cursor = client.find(std::move(findCmd));
 
         uassert(40618,
                 str::stream() << "failed to initialize cursor during auto split due to "
@@ -185,7 +184,7 @@ BSONObj findExtremeKeyForShard(OperationContext* opCtx,
             end = cursor->next().getOwned();
         }
     } else {
-        end = client.findOne(nss.ns(), q);
+        end = client.findOne(std::move(findCmd));
     }
 
     if (end.isEmpty()) {
@@ -195,27 +194,6 @@ BSONObj findExtremeKeyForShard(OperationContext* opCtx,
     return shardKeyPattern.extractShardKeyFromDoc(end);
 }
 
-/**
- * Checks if autobalance is enabled on the current sharded collection.
- */
-bool isAutoBalanceEnabled(OperationContext* opCtx,
-                          const NamespaceString& nss,
-                          BalancerConfiguration* balancerConfig) {
-    if (!balancerConfig->shouldBalanceForAutoSplit())
-        return false;
-
-    try {
-        return Grid::get(opCtx)->catalogClient()->getCollection(opCtx, nss).getAllowBalance();
-    } catch (const DBException& ex) {
-        LOGV2(21903,
-              "Auto-split for {namespace} failed to load collection metadata: {error}",
-              "Auto-split failed to load collection metadata",
-              "namespace"_attr = nss,
-              "error"_attr = redact(ex));
-        return false;
-    }
-}
-
 const auto getChunkSplitter = ServiceContext::declareDecoration<ChunkSplitter>();
 
 }  // namespace
@@ -285,6 +263,28 @@ void ChunkSplitter::trySplitting(std::shared_ptr<ChunkSplitStateDriver> chunkSpl
         });
 }
 
+/**
+ * Checks if autobalance is enabled on the current sharded collection.
+ */
+bool isAutoBalanceEnabled(OperationContext* opCtx,
+                          const NamespaceString& nss,
+                          BalancerConfiguration* balancerConfig) {
+    if (!balancerConfig->shouldBalanceForAutoSplit()) {
+        return false;
+    }
+
+    try {
+        return Grid::get(opCtx)->catalogClient()->getCollection(opCtx, nss).getAllowBalance();
+    } catch (const DBException& ex) {
+        LOGV2(21903,
+              "Auto-split for {namespace} failed to load collection metadata: {error}",
+              "Auto-split failed to load collection metadata",
+              "namespace"_attr = nss,
+              "error"_attr = redact(ex));
+        return false;
+    }
+}
+
 void ChunkSplitter::_runAutosplit(std::shared_ptr<ChunkSplitStateDriver> chunkSplitStateDriver,
                                   const NamespaceString& nss,
                                   const BSONObj& min,
@@ -320,11 +320,17 @@ void ChunkSplitter::_runAutosplit(std::shared_ptr<ChunkSplitStateDriver> chunkSp
         // Ensure we have the most up-to-date balancer configuration
         uassertStatusOK(balancerConfig->refreshAndCheck(opCtx.get()));
 
-        if (!balancerConfig->getShouldAutoSplit()) {
+        if (!balancerConfig->getShouldAutoSplit() || !cm.allowAutoSplit()) {
             return;
         }
 
-        const uint64_t maxChunkSizeBytes = balancerConfig->getMaxChunkSizeBytes();
+        const uint64_t maxChunkSizeBytes = [&] {
+            if (cm.maxChunkSizeBytes()) {
+                return *cm.maxChunkSizeBytes();
+            }
+            return balancerConfig->getMaxChunkSizeBytes();
+        }();
+        invariant(ChunkSizeSettingsType::checkMaxChunkSizeValid(maxChunkSizeBytes));
 
         LOGV2_DEBUG(21906,
                     1,
@@ -398,11 +404,13 @@ void ChunkSplitter::_runAutosplit(std::shared_ptr<ChunkSplitStateDriver> chunkSp
         const bool shouldBalance = isAutoBalanceEnabled(opCtx.get(), nss, balancerConfig);
 
         LOGV2(21908,
-              "autosplitted {namespace} chunk: {chunk} with {splitPoints} split points "
-              "(maxChunkSizeBytes: {maxChunkSizeBytes}). {extraInfo}",
+              "autosplitted {namespace} chunk: [{minKey},{maxKey}) with {splitPoints} split points "
+              "(maxChunkSizeBytes: {maxChunkSizeBytes}, lastmod: {lastmod}). {extraInfo}",
               "autosplitted chunk",
               "namespace"_attr = nss,
-              "chunk"_attr = redact(chunk.toString()),
+              "minKey"_attr = redact(chunk.getMin()),
+              "maxKey"_attr = redact(chunk.getMax()),
+              "lastmod"_attr = redact(chunk.getLastmod().toBSON()),
               "splitPoints"_attr = splitPoints.size(),
               "maxChunkSizeBytes"_attr = maxChunkSizeBytes,
               "extraInfo"_attr =
