@@ -70,6 +70,8 @@
 namespace mongo {
 namespace {
 
+MONGO_FAIL_POINT_DEFINE(shardingCatalogManagerWithTransactionFailWCAfterCommit);
+
 const WriteConcernOptions kNoWaitWriteConcern(1, WriteConcernOptions::SyncMode::UNSET, Seconds(0));
 
 // This value is initialized only if the node is running as a config server
@@ -131,7 +133,8 @@ void startTransactionWithNoopFind(OperationContext* opCtx,
 
 BSONObj commitOrAbortTransaction(OperationContext* opCtx,
                                  TxnNumber txnNumber,
-                                 std::string cmdName) {
+                                 std::string cmdName,
+                                 const WriteConcernOptions& writeConcern) {
     // Swap out the clients in order to get a fresh opCtx. Previous operations in this transaction
     // that have been run on this opCtx would have set the timeout in the locker on the opCtx, but
     // commit should not have a lock timeout.
@@ -152,7 +155,7 @@ BSONObj commitOrAbortTransaction(OperationContext* opCtx,
     bob.append(cmdName, true);
     bob.append("autocommit", false);
     bob.append(OperationSessionInfo::kTxnNumberFieldName, txnNumber);
-    bob.append(WriteConcernOptions::kWriteConcernField, WriteConcernOptions::Majority);
+    bob.append(WriteConcernOptions::kWriteConcernField, writeConcern.toBSON());
 
     BSONObjBuilder lsidBuilder(bob.subobjStart("lsid"));
     newOpCtx->getLogicalSessionId()->serialize(&bob);
@@ -173,15 +176,19 @@ BSONObj commitOrAbortTransaction(OperationContext* opCtx,
 }
 
 // Runs commit for the transaction with 'txnNumber'.
-auto commitTransaction(OperationContext* opCtx, TxnNumber txnNumber) {
-    auto response = commitOrAbortTransaction(opCtx, txnNumber, "commitTransaction");
+auto commitTransaction(OperationContext* opCtx,
+                       TxnNumber txnNumber,
+                       const WriteConcernOptions& writeConcern) {
+    auto response = commitOrAbortTransaction(opCtx, txnNumber, "commitTransaction", writeConcern);
     return std::make_tuple(getStatusFromCommandResult(response),
                            getWriteConcernStatusFromCommandResult(response));
 }
 
 // Runs abort for the transaction with 'txnNumber'.
-void abortTransaction(OperationContext* opCtx, TxnNumber txnNumber) {
-    auto response = commitOrAbortTransaction(opCtx, txnNumber, "abortTransaction");
+void abortTransaction(OperationContext* opCtx,
+                      TxnNumber txnNumber,
+                      const WriteConcernOptions& writeConcern) {
+    auto response = commitOrAbortTransaction(opCtx, txnNumber, "abortTransaction", writeConcern);
 
     // It is safe to ignore write concern errors in the presence of a NoSuchTransaction command
     // error because the transaction being aborted was both generated by and run locally on this
@@ -845,6 +852,18 @@ void ShardingCatalogManager::withTransaction(
     OperationContext* opCtx,
     const NamespaceString& namespaceForInitialFind,
     unique_function<void(OperationContext*, TxnNumber)> func) {
+    withTransaction(opCtx,
+                    namespaceForInitialFind,
+                    std::move(func),
+                    ShardingCatalogClient::kMajorityWriteConcern);
+}
+
+void ShardingCatalogManager::withTransaction(
+    OperationContext* opCtx,
+    const NamespaceString& namespaceForInitialFind,
+    unique_function<void(OperationContext*, TxnNumber)> func,
+    const WriteConcernOptions& writeConcern) {
+
     AlternativeSessionRegion asr(opCtx);
     auto* const client = asr.opCtx()->getClient();
     {
@@ -855,9 +874,9 @@ void ShardingCatalogManager::withTransaction(
     AuthorizationSession::get(client)->grantInternalAuthorization(client);
     TxnNumber txnNumber = 0;
 
-    ScopeGuard guard([opCtx = asr.opCtx(), &txnNumber] {
+    ScopeGuard guard([opCtx = asr.opCtx(), &txnNumber, &writeConcern] {
         try {
-            abortTransaction(opCtx, txnNumber);
+            abortTransaction(opCtx, txnNumber, writeConcern);
         } catch (DBException& e) {
             LOGV2_WARNING(5192100,
                           "Failed to abort transaction in AlternativeSessionRegion",
@@ -899,7 +918,7 @@ void ShardingCatalogManager::withTransaction(
             throw;
         }
 
-        auto [cmdStatus, wcStatus] = commitTransaction(asr.opCtx(), txnNumber);
+        auto [cmdStatus, wcStatus] = commitTransaction(asr.opCtx(), txnNumber, writeConcern);
         if (!cmdStatus.isOK() && !cmdStatus.isA<ErrorCategory::NotPrimaryError>() &&
             !cmdStatus.isA<ErrorCategory::ShutdownError>() &&
             isTransientTransactionError(
@@ -920,6 +939,15 @@ void ShardingCatalogManager::withTransaction(
         // or shutting down or asr.opCtx() is killed. We throw because all of those cases are
         // terminal for the caller running a local replica set transaction anyway.
         uassertStatusOK(wcStatus);
+        shardingCatalogManagerWithTransactionFailWCAfterCommit.execute([&](const BSONObj& data) {
+            // Simulates the case described in the above comment where the transaction commits, but
+            // fails to replicate due to some interruption.
+            if (!writeConcern.needToWaitForOtherNodes()) {
+                return;
+            }
+            uasserted(ErrorCodes::Interrupted,
+                      "Failpoint shardingCatalogManagerWithTransactionFailWCAfterCommit");
+        });
 
         guard.dismiss();
         return;
