@@ -194,6 +194,10 @@ public:
     static void withTransaction(OperationContext* opCtx,
                                 const NamespaceString& namespaceForInitialFind,
                                 unique_function<void(OperationContext*, TxnNumber)> func);
+    static void withTransaction(OperationContext* opCtx,
+                                const NamespaceString& namespaceForInitialFind,
+                                unique_function<void(OperationContext*, TxnNumber)> func,
+                                const WriteConcernOptions& writeConcern);
 
     /**
      * Runs the write 'request' on namespace 'nss' in a transaction with 'txnNumber'. Write must be
@@ -324,6 +328,11 @@ public:
         OperationContext* opCtx,
         const NamespaceString& nss,
         unique_function<void(OperationContext*, TxnNumber)> changeMetadataFunc);
+    void bumpCollectionVersionAndChangeMetadataInTxn(
+        OperationContext* opCtx,
+        const NamespaceString& nss,
+        unique_function<void(OperationContext*, TxnNumber)> changeMetadataFunc,
+        const WriteConcernOptions& writeConcern);
 
     /**
      * Same as bumpCollectionVersionAndChangeMetadataInTxn, but bumps the version for several
@@ -333,6 +342,11 @@ public:
         OperationContext* opCtx,
         const std::vector<NamespaceString>& collNames,
         unique_function<void(OperationContext*, TxnNumber)> changeMetadataFunc);
+    void bumpMultipleCollectionVersionsAndChangeMetadataInTxn(
+        OperationContext* opCtx,
+        const std::vector<NamespaceString>& collNames,
+        unique_function<void(OperationContext*, TxnNumber)> changeMetadataFunc,
+        const WriteConcernOptions& writeConcern);
 
     /**
      * Performs a split on the chunk with min value "minKey". If the split fails, it is marked as
@@ -513,6 +527,18 @@ public:
      */
     static void clearForTests(ServiceContext* serviceContext);
 
+    //
+    // Upgrade/downgrade
+    //
+
+    /**
+     * Upgrade the chunk metadata to include the history field.
+     */
+    void upgradeChunksHistory(OperationContext* opCtx,
+                              const NamespaceString& nss,
+                              bool force,
+                              const Timestamp& validAfter);
+
 private:
     /**
      * Performs the necessary checks for version compatibility and creates a new config.version
