@@ -34,14 +34,13 @@
 
 #include "mongo/bson/json.h"
 #include "mongo/db/catalog_raii.h"
-#include "mongo/db/concurrency/exception_util.h"
+#include "mongo/db/concurrency/write_conflict_exception.h"
 #include "mongo/db/curop.h"
 #include "mongo/db/namespace_string.h"
 #include "mongo/db/operation_context.h"
 #include "mongo/db/repl/replication_coordinator_mock.h"
 #include "mongo/db/s/config/config_server_test_fixture.h"
 #include "mongo/db/s/config/sharding_catalog_manager.h"
-#include "mongo/db/s/sharding_ddl_50_upgrade_downgrade.h"
 #include "mongo/db/s/type_lockpings.h"
 #include "mongo/db/s/type_locks.h"
 #include "mongo/s/catalog/config_server_version.h"
@@ -287,35 +286,18 @@ TEST_F(ConfigInitializationTest, BuildsNecessaryIndexes) {
     ASSERT_OK(ShardingCatalogManager::get(operationContext())
                   ->initializeConfigDatabaseIfNeeded(operationContext()));
 
-    std::vector<BSONObj> expectedChunksIndexes;
-
-    if (serverGlobalParams.featureCompatibility.isGreaterThanOrEqualTo(
-            ServerGlobalParams::FeatureCompatibility::Version::kVersion50)) {
-        expectedChunksIndexes = std::vector<BSONObj>{
-            BSON("v" << 2 << "key" << BSON("_id" << 1) << "name"
-                     << "_id_"),
-            BSON("v" << 2 << "key" << BSON("uuid" << 1 << "min" << 1) << "name"
-                     << "uuid_1_min_1"
-                     << "unique" << true),
-            BSON("v" << 2 << "key" << BSON("uuid" << 1 << "shard" << 1 << "min" << 1) << "name"
-                     << "uuid_1_shard_1_min_1"
-                     << "unique" << true),
-            BSON("v" << 2 << "key" << BSON("uuid" << 1 << "lastmod" << 1) << "name"
-                     << "uuid_1_lastmod_1"
-                     << "unique" << true)};
-    } else {
-        expectedChunksIndexes = std::vector<BSONObj>{
-            BSON("v" << 2 << "key" << BSON("_id" << 1) << "name"
-                     << "_id_"),
-            BSON("v" << 2 << "unique" << true << "key" << BSON("ns" << 1 << "min" << 1) << "name"
-                     << "ns_1_min_1"),
-            BSON("v" << 2 << "unique" << true << "key"
-                     << BSON("ns" << 1 << "shard" << 1 << "min" << 1) << "name"
-                     << "ns_1_shard_1_min_1"),
-            BSON("v" << 2 << "unique" << true << "key" << BSON("ns" << 1 << "lastmod" << 1)
-                     << "name"
-                     << "ns_1_lastmod_1")};
-    }
+    std::vector<BSONObj> expectedChunksIndexes = std::vector<BSONObj>{
+        BSON("v" << 2 << "key" << BSON("_id" << 1) << "name"
+                 << "_id_"),
+        BSON("v" << 2 << "key" << BSON("uuid" << 1 << "min" << 1) << "name"
+                 << "uuid_1_min_1"
+                 << "unique" << true),
+        BSON("v" << 2 << "key" << BSON("uuid" << 1 << "shard" << 1 << "min" << 1) << "name"
+                 << "uuid_1_shard_1_min_1"
+                 << "unique" << true),
+        BSON("v" << 2 << "key" << BSON("uuid" << 1 << "lastmod" << 1) << "name"
+                 << "uuid_1_lastmod_1"
+                 << "unique" << true)};
 
     auto expectedLockpingsIndexes =
         std::vector<BSONObj>{BSON("v" << 2 << "key" << BSON("_id" << 1) << "name"
