@@ -37,16 +37,18 @@
 #include "mongo/db/catalog_raii.h"
 #include "mongo/db/commands.h"
 #include "mongo/db/field_parser.h"
-#include "mongo/db/logical_clock.h"
 #include "mongo/db/namespace_string.h"
-#include "mongo/db/s/collection_sharding_state.h"
+#include "mongo/db/s/active_migrations_registry.h"
+#include "mongo/db/s/collection_sharding_runtime.h"
+#include "mongo/db/s/operation_sharding_state.h"
 #include "mongo/db/s/shard_filtering_metadata_refresh.h"
 #include "mongo/db/s/sharding_state.h"
+#include "mongo/db/vector_clock.h"
 #include "mongo/logv2/log.h"
 #include "mongo/s/catalog/type_chunk.h"
 #include "mongo/s/client/shard_registry.h"
 #include "mongo/s/grid.h"
-#include "mongo/s/request_types/merge_chunk_request_type.h"
+#include "mongo/s/request_types/merge_chunk_request_gen.h"
 #include "mongo/util/str.h"
 
 namespace mongo {
@@ -56,177 +58,112 @@ bool checkMetadataForSuccess(OperationContext* opCtx,
                              const NamespaceString& nss,
                              const OID& epoch,
                              const ChunkRange& chunkRange) {
-    const auto metadataAfterMerge = [&] {
-        AutoGetCollection autoColl(opCtx, nss, MODE_IS);
-        return CollectionShardingState::get(opCtx, nss)->getCollectionDescription();
-    }();
+    AutoGetCollection autoColl(opCtx, nss, MODE_IS);
+    const auto metadataAfterMerge =
+        CollectionShardingRuntime::get(opCtx, nss)->getCurrentMetadataIfKnown();
 
     uassert(ErrorCodes::StaleEpoch,
             str::stream() << "Collection " << nss.ns() << " changed since merge start",
-            metadataAfterMerge.getCollVersion().epoch() == epoch);
+            metadataAfterMerge && metadataAfterMerge->getShardVersion().epoch() == epoch);
 
     ChunkType chunk;
-    if (!metadataAfterMerge->getNextChunk(chunkRange.getMin(), &chunk)) {
+    if (!metadataAfterMerge->getNextChunk(chunkRange.getMin(), &chunk))
         return false;
-    }
 
     return chunk.getMin().woCompare(chunkRange.getMin()) == 0 &&
         chunk.getMax().woCompare(chunkRange.getMax()) == 0;
 }
 
+Shard::CommandResponse commitMergeOnConfigServer(OperationContext* opCtx,
+                                                 const NamespaceString& nss,
+                                                 const ChunkRange& chunkRange,
+                                                 const CollectionMetadata& metadata) {
+    auto const shardingState = ShardingState::get(opCtx);
+    const auto currentTime = VectorClock::get(opCtx)->getTime();
+
+    ConfigSvrMergeChunks request{nss, shardingState->shardId(), metadata.getUUID(), chunkRange};
+    request.setValidAfter(currentTime.clusterTime().asTimestamp());
+    request.setWriteConcern(ShardingCatalogClient::kMajorityWriteConcern.toBSON());
+
+    auto cmdResponse =
+        uassertStatusOK(Grid::get(opCtx)->shardRegistry()->getConfigShard()->runCommand(
+            opCtx,
+            ReadPreferenceSetting{ReadPreference::PrimaryOnly},
+            NamespaceString::kAdminDb.toString(),
+            request.toBSON(BSONObj()),
+            Shard::RetryPolicy::kIdempotent));
+
+    return cmdResponse;
+}
+
 void mergeChunks(OperationContext* opCtx,
                  const NamespaceString& nss,
                  const BSONObj& minKey,
                  const BSONObj& maxKey,
-                 const OID& epoch) {
-    const std::string whyMessage = str::stream() << "merging chunks in " << nss.ns() << " from "
-                                                 << redact(minKey) << " to " << redact(maxKey);
-    auto scopedDistLock = uassertStatusOKWithContext(
-        Grid::get(opCtx)->catalogClient()->getDistLockManager()->lock(
-            opCtx, nss.ns(), whyMessage, DistLockManager::kSingleLockAttemptTimeout),
-        str::stream() << "could not acquire collection lock for " << nss.ns()
-                      << " to merge chunks in [" << redact(minKey) << ", " << redact(maxKey)
-                      << ")");
+                 const OID& expectedEpoch) {
+    auto scopedSplitOrMergeChunk(
+        uassertStatusOK(ActiveMigrationsRegistry::get(opCtx).registerSplitOrMergeChunk(
+            opCtx, nss, ChunkRange(minKey, maxKey))));
 
-    auto const shardingState = ShardingState::get(opCtx);
+    const bool isVersioned = OperationShardingState::isOperationVersioned(opCtx);
+    if (!isVersioned) {
+        onShardVersionMismatch(opCtx, nss, boost::none);
+    }
 
-    // We now have the collection distributed lock, refresh metadata to latest version and sanity
-    // check
-    forceShardFilteringMetadataRefresh(opCtx, nss, true /* forceRefreshFromThisThread */);
-
-    const auto collDesc = [&] {
+    const auto metadataBeforeMerge = [&] {
         AutoGetCollection autoColl(opCtx, nss, MODE_IS);
-        return CollectionShardingState::get(opCtx, nss)->getCollectionDescription();
+        auto csr = CollectionShardingRuntime::get(opCtx, nss);
+        // If there is a version attached to the OperationContext, validate it
+        if (isVersioned) {
+            csr->checkShardVersionOrThrow(opCtx);
+        }
+        return csr->getCurrentMetadataIfKnown();
     }();
 
     uassert(ErrorCodes::StaleEpoch,
-            str::stream() << "Collection " << nss.ns() << " became unsharded",
-            collDesc.isSharded());
+            str::stream() << "Collection " << nss.ns() << " is not sharded",
+            metadataBeforeMerge && metadataBeforeMerge->isSharded());
 
-    const auto shardVersion = collDesc.getShardVersion();
+    const auto epoch = metadataBeforeMerge->getShardVersion().epoch();
     uassert(ErrorCodes::StaleEpoch,
             str::stream() << "could not merge chunks, collection " << nss.ns()
-                          << " has changed since merge was sent (sent epoch: " << epoch.toString()
-                          << ", current epoch: " << shardVersion.epoch() << ")",
-            shardVersion.epoch() == epoch);
+                          << " has changed since merge was sent (sent epoch: " << expectedEpoch
+                          << ", current epoch: " << epoch << ")",
+            expectedEpoch == epoch);
+
+    ChunkRange chunkRange(minKey, maxKey);
 
     uassert(ErrorCodes::IllegalOperation,
-            str::stream() << "could not merge chunks, the range "
-                          << redact(ChunkRange(minKey, maxKey).toString()) << " is not valid"
+            str::stream() << "could not merge chunks, the range " << redact(chunkRange.toString())
+                          << " is not valid"
                           << " for collection " << nss.ns() << " with key pattern "
-                          << collDesc.getKeyPattern().toString(),
-            collDesc.isValidKey(minKey) && collDesc.isValidKey(maxKey));
+                          << metadataBeforeMerge->getKeyPattern().toString(),
+            metadataBeforeMerge->isValidKey(minKey) && metadataBeforeMerge->isValidKey(maxKey));
 
-    //
-    // Get merged chunk information
-    //
-    std::vector<ChunkType> chunksToMerge;
-    std::vector<BSONObj> chunkBoundaries;
-    chunkBoundaries.push_back(minKey);
+    auto cmdResponse = commitMergeOnConfigServer(opCtx, nss, chunkRange, metadataBeforeMerge.get());
 
-    ChunkType itChunk;
-    itChunk.setMin(minKey);
-    itChunk.setMax(minKey);
-
-    while (itChunk.getMax().woCompare(maxKey) < 0 &&
-           collDesc->getNextChunk(itChunk.getMax(), &itChunk)) {
-        chunkBoundaries.push_back(itChunk.getMax());
-        chunksToMerge.push_back(itChunk);
-    }
-
-    uassert(ErrorCodes::IllegalOperation,
-            str::stream() << "could not merge chunks, collection " << nss.ns()
-                          << " range starting at " << redact(minKey) << " and ending at "
-                          << redact(maxKey) << " does not belong to shard "
-                          << shardingState->shardId(),
-            !chunksToMerge.empty());
-
-    //
-    // Validate the range starts and ends at chunks and has no holes, error if not valid
-    //
-
-    BSONObj firstDocMin = chunksToMerge.front().getMin();
-    BSONObj firstDocMax = chunksToMerge.front().getMax();
-    // minKey is inclusive
-    bool minKeyInRange = rangeContains(firstDocMin, firstDocMax, minKey);
-
-    uassert(ErrorCodes::IllegalOperation,
-            str::stream() << "could not merge chunks, collection " << nss.ns()
-                          << " range starting at " << redact(minKey) << " does not belong to shard "
-                          << shardingState->shardId(),
-            minKeyInRange);
-
-    BSONObj lastDocMin = chunksToMerge.back().getMin();
-    BSONObj lastDocMax = chunksToMerge.back().getMax();
-    // maxKey is exclusive
-    bool maxKeyInRange = lastDocMin.woCompare(maxKey) < 0 && lastDocMax.woCompare(maxKey) >= 0;
-
-    uassert(ErrorCodes::IllegalOperation,
-            str::stream() << "could not merge chunks, collection " << nss.ns()
-                          << " range ending at " << redact(maxKey) << " does not belong to shard "
-                          << shardingState->shardId(),
-            maxKeyInRange);
-
-    bool validRangeStartKey = firstDocMin.woCompare(minKey) == 0;
-    bool validRangeEndKey = lastDocMax.woCompare(maxKey) == 0;
-
-    uassert(ErrorCodes::IllegalOperation,
-            str::stream() << "could not merge chunks, collection " << nss.ns()
-                          << " does not contain a chunk "
-                          << (!validRangeStartKey ? "starting at " + redact(minKey.toString()) : "")
-                          << (!validRangeStartKey && !validRangeEndKey ? " or " : "")
-                          << (!validRangeEndKey ? "ending at " + redact(maxKey.toString()) : ""),
-            validRangeStartKey && validRangeEndKey);
-
-    uassert(ErrorCodes::IllegalOperation,
-            str::stream() << "could not merge chunks, collection " << nss.ns()
-                          << " already contains chunk for "
-                          << ChunkRange(minKey, maxKey).toString(),
-            chunksToMerge.size() > 1);
-
-    // Look for hole in range
-    for (size_t i = 1; i < chunksToMerge.size(); ++i) {
-        uassert(
-            ErrorCodes::IllegalOperation,
-            str::stream()
-                << "could not merge chunks, collection " << nss.ns() << " has a hole in the range "
-                << ChunkRange(minKey, maxKey).toString() << " at "
-                << ChunkRange(chunksToMerge[i - 1].getMax(), chunksToMerge[i].getMin()).toString(),
-            chunksToMerge[i - 1].getMax().woCompare(chunksToMerge[i].getMin()) == 0);
-    }
-
-    //
-    // Run _configsvrCommitChunkMerge.
-    //
-    MergeChunkRequest request{nss,
-                              shardingState->shardId().toString(),
-                              shardVersion.epoch(),
-                              chunkBoundaries,
-                              LogicalClock::get(opCtx)->getClusterTime().asTimestamp()};
-
-    auto configCmdObj =
-        request.toConfigCommandBSON(ShardingCatalogClient::kMajorityWriteConcern.toBSON());
-    auto cmdResponse =
-        uassertStatusOK(Grid::get(opCtx)->shardRegistry()->getConfigShard()->runCommand(
-            opCtx,
-            ReadPreferenceSetting{ReadPreference::PrimaryOnly},
-            "admin",
-            configCmdObj,
-            Shard::RetryPolicy::kIdempotent));
+    boost::optional<ChunkVersion> shardVersionReceived = [&]() -> boost::optional<ChunkVersion> {
+        // old versions might not have the shardVersion field
+        if (cmdResponse.response[ChunkVersion::kShardVersionField]) {
+            return uassertStatusOK(ChunkVersion::parseWithField(cmdResponse.response,
+                                                                ChunkVersion::kShardVersionField));
+        }
+        return boost::none;
+    }();
 
     // Refresh metadata to pick up new chunk definitions (regardless of the results returned from
-    // running _configsvrCommitChunkMerge).
-    forceShardFilteringMetadataRefresh(opCtx, nss, true /* forceRefreshFromThisThread */);
+    // running _configsvrCommitChunksMerge).
+    onShardVersionMismatch(opCtx, nss, std::move(shardVersionReceived));
 
-    // If _configsvrCommitChunkMerge returned an error, look at this shard's metadata to determine
+    // If _configsvrCommitChunksMerge returned an error, look at this shard's metadata to determine
     // if the merge actually did happen. This can happen if there's a network error getting the
-    // response from the first call to _configsvrCommitChunkMerge, but it actually succeeds, thus
+    // response from the first call to _configsvrCommitChunksMerge, but it actually succeeds, thus
     // the automatic retry fails with a precondition violation, for example.
     auto commandStatus = std::move(cmdResponse.commandStatus);
     auto writeConcernStatus = std::move(cmdResponse.writeConcernStatus);
 
     if ((!commandStatus.isOK() || !writeConcernStatus.isOK()) &&
-        checkMetadataForSuccess(opCtx, nss, epoch, ChunkRange(minKey, maxKey))) {
+        checkMetadataForSuccess(opCtx, nss, epoch, chunkRange)) {
         LOGV2_DEBUG(21983,
                     1,
                     "mergeChunk interval [{minKey},{maxKey}) has already been committed",
