@@ -33,15 +33,10 @@
 
 #include "mongo/db/s/migration_chunk_cloner_source_legacy.h"
 
-#include <fmt/format.h>
-
 #include "mongo/base/status.h"
-#include "mongo/bson/bsonobj.h"
 #include "mongo/client/read_preference.h"
 #include "mongo/db/catalog/index_catalog.h"
 #include "mongo/db/catalog_raii.h"
-#include "mongo/db/db_raii.h"
-#include "mongo/db/dbdirectclient.h"
 #include "mongo/db/dbhelpers.h"
 #include "mongo/db/exec/working_set_common.h"
 #include "mongo/db/index/index_access_method.h"
@@ -50,7 +45,6 @@
 #include "mongo/db/repl/replication_process.h"
 #include "mongo/db/s/collection_sharding_runtime.h"
 #include "mongo/db/s/migration_source_manager.h"
-#include "mongo/db/s/operation_sharding_state.h"
 #include "mongo/db/s/sharding_runtime_d_params_gen.h"
 #include "mongo/db/s/sharding_statistics.h"
 #include "mongo/db/s/start_chunk_clone_request.h"
@@ -72,8 +66,6 @@
 namespace mongo {
 namespace {
 
-using namespace fmt::literals;
-
 const char kRecvChunkStatus[] = "_recvChunkStatus";
 const char kRecvChunkCommit[] = "_recvChunkCommit";
 const char kRecvChunkAbort[] = "_recvChunkAbort";
@@ -83,22 +75,12 @@ const Hours kMaxWaitToCommitCloneForJumboChunk(6);
 
 MONGO_FAIL_POINT_DEFINE(failTooMuchMemoryUsed);
 
-/**
- * Returns true if the given BSON object in the shard key value pair format is within the given
- * range.
- */
-bool isShardKeyValueInRange(const BSONObj& shardKeyValue, const BSONObj& min, const BSONObj& max) {
-    return shardKeyValue.woCompare(min) >= 0 && shardKeyValue.woCompare(max) < 0;
-}
-
-/**
- * Returns true if the given BSON document is within the given chunk range.
- */
-bool isDocInRange(const BSONObj& obj,
-                  const BSONObj& min,
-                  const BSONObj& max,
-                  const ShardKeyPattern& shardKeyPattern) {
-    return isShardKeyValueInRange(shardKeyPattern.extractShardKeyFromDoc(obj), min, max);
+bool isInRange(const BSONObj& obj,
+               const BSONObj& min,
+               const BSONObj& max,
+               const ShardKeyPattern& shardKeyPattern) {
+    BSONObj k = shardKeyPattern.extractShardKeyFromDoc(obj);
+    return k.woCompare(min) >= 0 && k.woCompare(max) < 0;
 }
 
 BSONObj createRequestWithSessionId(StringData commandName,
@@ -112,8 +94,9 @@ BSONObj createRequestWithSessionId(StringData commandName,
     return builder.obj();
 }
 
-BSONObj getDocumentKeyFromReplOperation(repl::ReplOperation replOperation) {
-    switch (replOperation.getOpType()) {
+BSONObj getDocumentKeyFromReplOperation(repl::ReplOperation replOperation,
+                                        repl::OpTypeEnum opType) {
+    switch (opType) {
         case repl::OpTypeEnum::kInsert:
         case repl::OpTypeEnum::kDelete:
             return replOperation.getObject();
@@ -178,28 +161,6 @@ private:
     const repl::OpTime _prePostImageOpTime;
 };
 
-LogTransactionOperationsForShardingHandler::LogTransactionOperationsForShardingHandler(
-    ServiceContext* service,
-    const std::vector<repl::OplogEntry>& stmts,
-    repl::OpTime prepareOrCommitOpTime)
-    : _svcCtx(service), _prepareOrCommitOpTime(std::move(prepareOrCommitOpTime)) {
-    _stmts.reserve(stmts.size());
-    _ownedReplBSONObj.reserve(stmts.size());
-
-    for (const auto& op : stmts) {
-        auto ownedBSON = op.getDurableReplOperation().toBSON().getOwned();
-        _ownedReplBSONObj.push_back(ownedBSON);
-        _stmts.push_back(
-            repl::ReplOperation::parse({"MigrationChunkClonerSource_toReplOperation"}, ownedBSON));
-    }
-}
-
-LogTransactionOperationsForShardingHandler::LogTransactionOperationsForShardingHandler(
-    ServiceContext* service,
-    const std::vector<repl::ReplOperation>& stmts,
-    repl::OpTime prepareOrCommitOpTime)
-    : _svcCtx(service), _stmts(stmts), _prepareOrCommitOpTime(std::move(prepareOrCommitOpTime)) {}
-
 void LogTransactionOperationsForShardingHandler::commit(boost::optional<Timestamp>) {
     std::set<NamespaceString> namespacesTouchedByTransaction;
 
@@ -211,51 +172,41 @@ void LogTransactionOperationsForShardingHandler::commit(boost::optional<Timestam
         UninterruptibleLockGuard noInterrupt(opCtx->lockState());
         auto csrLock = CollectionShardingRuntime::CSRLock::lockShared(opCtx, csr);
 
-        auto msm = MigrationSourceManager::get(csr, csrLock);
-        if (!msm) {
+        const auto clonerPtr = MigrationSourceManager::getCurrentCloner(csr, csrLock);
+        if (!clonerPtr) {
             continue;
         }
-
-        auto cloner = dynamic_cast<MigrationChunkClonerSourceLegacy*>(msm->getCloner().get());
+        auto* const cloner = dynamic_cast<MigrationChunkClonerSourceLegacy*>(clonerPtr.get());
 
         auto opType = stmt.getOpType();
-        auto preImageDocKey = getDocumentKeyFromReplOperation(stmt);
+        auto documentKey = getDocumentKeyFromReplOperation(stmt, opType);
 
-        auto idElement = preImageDocKey["_id"];
+        auto idElement = documentKey["_id"];
         if (idElement.eoo()) {
             LOGV2_WARNING(21994,
+                          "Received a document without an _id field, ignoring: {documentKey}",
                           "Received a document without an _id and will ignore that document",
-                          "documentKey"_attr = redact(preImageDocKey));
+                          "documentKey"_attr = redact(documentKey));
             continue;
         }
 
-        if (opType == repl::OpTypeEnum::kUpdate) {
-            auto const& shardKeyPattern = cloner->_shardKeyPattern;
-            auto preImageShardKeyValues =
-                shardKeyPattern.extractShardKeyFromDocumentKey(preImageDocKey);
+        auto const& minKey = cloner->_args.getMinKey();
+        auto const& maxKey = cloner->_args.getMaxKey();
+        auto const& shardKeyPattern = cloner->_shardKeyPattern;
 
-            // If prepare was performed from another term, we will not have the post image doc key
-            // since it is not persisted in the oplog.
-            auto postImageDocKey = stmt.getPostImageDocumentKey();
-            if (!postImageDocKey.isEmpty()) {
-                if (!cloner->_processUpdateForXferMod(preImageDocKey, postImageDocKey)) {
-                    // We don't need to add this op to session migration if neither post or pre
-                    // image doc falls within the chunk range.
-                    continue;
-                }
+        if (!isInRange(documentKey, minKey, maxKey, shardKeyPattern)) {
+            // If the preImageDoc is not in range but the postImageDoc was, we know that the
+            // document has changed shard keys and no longer belongs in the chunk being cloned.
+            // We will model the deletion of the preImage document so that the destination chunk
+            // does not receive an outdated version of this document.
+            if (opType == repl::OpTypeEnum::kUpdate &&
+                isInRange(stmt.getPreImageDocumentKey(), minKey, maxKey, shardKeyPattern) &&
+                !stmt.getPreImageDocumentKey()["_id"].eoo()) {
+                opType = repl::OpTypeEnum::kDelete;
+                idElement = stmt.getPreImageDocumentKey()["id"];
             } else {
-                // We can't perform reads here using the same recovery unit because the transaction
-                // is already committed. We instead defer performing the reads when xferMods command
-                // is called. Also allow this op to be added to session migration since we can't
-                // tell whether post image doc will fall within the chunk range. If it turns out
-                // both preImage and postImage doc don't fall into the chunk range, it is not wrong
-                // for this op to be added to session migration, but it will result in wasted work
-                // and unneccesary extra oplog storage on the destination.
-                cloner->_deferProcessingForXferMod(preImageDocKey);
+                continue;
             }
-        } else {
-            cloner->_addToTransferModsQueue(
-                idElement.wrap(), getOpCharForCrudOpType(opType), {}, {});
         }
 
         // Inform the session migration subsystem that a transaction has committed for all involved
@@ -387,7 +338,8 @@ Status MigrationChunkClonerSourceLegacy::awaitUntilCriticalSectionIsAppropriate(
     return _checkRecipientCloningStatus(opCtx, maxTimeToWait);
 }
 
-StatusWith<BSONObj> MigrationChunkClonerSourceLegacy::commitClone(OperationContext* opCtx) {
+StatusWith<BSONObj> MigrationChunkClonerSourceLegacy::commitClone(OperationContext* opCtx,
+                                                                  bool acquireCSOnRecipient) {
     invariant(_state == kCloning);
     invariant(!opCtx->lockState()->isLocked());
     if (_jumboChunkCloneState && _forceJumbo) {
@@ -398,7 +350,7 @@ StatusWith<BSONObj> MigrationChunkClonerSourceLegacy::commitClone(OperationConte
             }
         } else {
             invariant(PlanExecutor::IS_EOF == _jumboChunkCloneState->clonerState);
-            invariant(!_cloneList.hasMore());
+            invariant(_cloneLocs.empty());
         }
     }
 
@@ -406,11 +358,16 @@ StatusWith<BSONObj> MigrationChunkClonerSourceLegacy::commitClone(OperationConte
         _sessionCatalogSource->onCommitCloneStarted();
     }
 
-    auto responseStatus = _callRecipient(
-        opCtx, createRequestWithSessionId(kRecvChunkCommit, _args.getNss(), _sessionId));
+    auto responseStatus = _callRecipient(opCtx, [&] {
+        BSONObjBuilder builder;
+        builder.append(kRecvChunkCommit, _args.getNss().ns());
+        builder.append("acquireCSOnRecipient", acquireCSOnRecipient);
+        _sessionId.append(&builder);
+        return builder.obj();
+    }());
 
     if (responseStatus.isOK()) {
-        _cleanup(opCtx);
+        _cleanup();
 
         if (_sessionCatalogSource && _sessionCatalogSource->hasMoreOplog()) {
             return {ErrorCodes::SessionTransferIncomplete,
@@ -425,7 +382,7 @@ StatusWith<BSONObj> MigrationChunkClonerSourceLegacy::commitClone(OperationConte
     return responseStatus.getStatus();
 }
 
-void MigrationChunkClonerSourceLegacy::cancelClone(OperationContext* opCtx) {
+void MigrationChunkClonerSourceLegacy::cancelClone(OperationContext* opCtx) noexcept {
     invariant(!opCtx->lockState()->isLocked());
 
     if (_sessionCatalogSource) {
@@ -449,7 +406,7 @@ void MigrationChunkClonerSourceLegacy::cancelClone(OperationContext* opCtx) {
         }
         // Intentional fall through
         case kNew:
-            _cleanup(opCtx);
+            _cleanup();
             break;
         default:
             MONGO_UNREACHABLE;
@@ -457,7 +414,7 @@ void MigrationChunkClonerSourceLegacy::cancelClone(OperationContext* opCtx) {
 }
 
 bool MigrationChunkClonerSourceLegacy::isDocumentInMigratingChunk(const BSONObj& doc) {
-    return isDocInRange(doc, _args.getMinKey(), _args.getMaxKey(), _shardKeyPattern);
+    return isInRange(doc, _args.getMinKey(), _args.getMaxKey(), _shardKeyPattern);
 }
 
 void MigrationChunkClonerSourceLegacy::onInsertOp(OperationContext* opCtx,
@@ -476,7 +433,7 @@ void MigrationChunkClonerSourceLegacy::onInsertOp(OperationContext* opCtx,
         return;
     }
 
-    if (!isDocInRange(insertedDoc, _args.getMinKey(), _args.getMaxKey(), _shardKeyPattern)) {
+    if (!isInRange(insertedDoc, _args.getMinKey(), _args.getMaxKey(), _shardKeyPattern)) {
         return;
     }
 
@@ -511,13 +468,13 @@ void MigrationChunkClonerSourceLegacy::onUpdateOp(OperationContext* opCtx,
         return;
     }
 
-    if (!isDocInRange(postImageDoc, _args.getMinKey(), _args.getMaxKey(), _shardKeyPattern)) {
+    if (!isInRange(postImageDoc, _args.getMinKey(), _args.getMaxKey(), _shardKeyPattern)) {
         // If the preImageDoc is not in range but the postImageDoc was, we know that the document
         // has changed shard keys and no longer belongs in the chunk being cloned. We will model
         // the deletion of the preImage document so that the destination chunk does not receive an
         // outdated version of this document.
         if (preImageDoc &&
-            isDocInRange(*preImageDoc, _args.getMinKey(), _args.getMaxKey(), _shardKeyPattern)) {
+            isInRange(*preImageDoc, _args.getMinKey(), _args.getMaxKey(), _shardKeyPattern)) {
             onDeleteOp(opCtx, *preImageDoc, opTime, prePostImageOpTime);
         }
         return;
@@ -704,55 +661,38 @@ void MigrationChunkClonerSourceLegacy::_nextCloneBatchFromCloneLocs(OperationCon
                            internalQueryExecYieldIterations.load(),
                            Milliseconds(internalQueryExecYieldPeriodMS.load()));
 
-    while (true) {
-        int recordsNoLongerExist = 0;
-        auto docInFlight = _cloneList.getNextDoc(opCtx, collection, &recordsNoLongerExist);
-
-        if (recordsNoLongerExist) {
-            stdx::lock_guard lk(_mutex);
-            _numRecordsPassedOver += recordsNoLongerExist;
-        }
-
-        const auto& doc = docInFlight->getDoc();
-        if (!doc) {
-            break;
-        }
+    stdx::unique_lock<Latch> lk(_mutex);
+    auto iter = _cloneLocs.begin();
 
+    for (; iter != _cloneLocs.end(); ++iter) {
         // We must always make progress in this method by at least one document because empty
         // return indicates there is no more initial clone data.
         if (arrBuilder->arrSize() && tracker.intervalHasElapsed()) {
-            _cloneList.insertOverflowDoc(*doc);
             break;
         }
 
-        // Do not send documents that are no longer in the chunk range being moved. This can
-        // happen when document shard key value of the document changed after the initial
-        // index scan during cloning. This is needed because the destination is very
-        // conservative in processing xferMod deletes and won't delete docs that are not in
-        // the range of the chunk being migrated.
-        if (!isDocInRange(doc->value(), _args.getMinKey(), _args.getMaxKey(), _shardKeyPattern)) {
-            {
-                stdx::lock_guard lk(_mutex);
-                _numRecordsPassedOver++;
+        auto nextRecordId = *iter;
+
+        lk.unlock();
+
+        Snapshotted<BSONObj> doc;
+        if (collection->findDoc(opCtx, nextRecordId, &doc)) {
+            // Use the builder size instead of accumulating the document sizes directly so
+            // that we take into consideration the overhead of BSONArray indices.
+            if (arrBuilder->arrSize() &&
+                (arrBuilder->len() + doc.value().objsize() + 1024) > BSONObjMaxUserSize) {
+
+                break;
             }
-            continue;
+
+            arrBuilder->append(doc.value());
+            ShardingStatistics::get(opCtx).countDocsClonedOnDonor.addAndFetch(1);
         }
 
-        // Use the builder size instead of accumulating the document sizes directly so
-        // that we take into consideration the overhead of BSONArray indices.
-        if (arrBuilder->arrSize() &&
-            (arrBuilder->len() + doc->value().objsize() + 1024) > BSONObjMaxUserSize) {
-            _cloneList.insertOverflowDoc(*doc);
-            break;
-        }
-
-        {
-            stdx::lock_guard lk(_mutex);
-            _numRecordsCloned++;
-        }
-        arrBuilder->append(doc->value());
-        ShardingStatistics::get(opCtx).countDocsClonedOnDonor.addAndFetch(1);
+        lk.lock();
     }
+
+    _cloneLocs.erase(_cloneLocs.begin(), iter);
 }
 
 uint64_t MigrationChunkClonerSourceLegacy::getCloneBatchBufferAllocationSize() {
@@ -761,7 +701,7 @@ uint64_t MigrationChunkClonerSourceLegacy::getCloneBatchBufferAllocationSize() {
         return static_cast<uint64_t>(BSONObjMaxUserSize);
 
     return std::min(static_cast<uint64_t>(BSONObjMaxUserSize),
-                    _averageObjectSizeForCloneLocs * _cloneList.size());
+                    _averageObjectSizeForCloneLocs * _cloneLocs.size());
 }
 
 Status MigrationChunkClonerSourceLegacy::nextCloneBatch(OperationContext* opCtx,
@@ -784,86 +724,18 @@ Status MigrationChunkClonerSourceLegacy::nextCloneBatch(OperationContext* opCtx,
     return Status::OK();
 }
 
-bool MigrationChunkClonerSourceLegacy::_processUpdateForXferMod(const BSONObj& preImageDocKey,
-                                                                const BSONObj& postImageDocKey) {
-    auto const& minKey = _args.getMinKey();
-    auto const& maxKey = _args.getMaxKey();
-
-    auto postShardKeyValues = _shardKeyPattern.extractShardKeyFromDocumentKey(postImageDocKey);
-    fassert(6836100, !postShardKeyValues.isEmpty());
-
-    auto opType = repl::OpTypeEnum::kUpdate;
-    auto idElement = preImageDocKey["_id"];
-
-    if (!isShardKeyValueInRange(postShardKeyValues, minKey, maxKey)) {
-        // If the preImageDoc is not in range but the postImageDoc was, we know that the
-        // document has changed shard keys and no longer belongs in the chunk being cloned.
-        // We will model the deletion of the preImage document so that the destination chunk
-        // does not receive an outdated version of this document.
-
-        auto preImageShardKeyValues =
-            _shardKeyPattern.extractShardKeyFromDocumentKey(preImageDocKey);
-        fassert(6836101, !preImageShardKeyValues.isEmpty());
-
-        if (!isShardKeyValueInRange(preImageShardKeyValues, minKey, maxKey)) {
-            return false;
-        }
-
-        opType = repl::OpTypeEnum::kDelete;
-        idElement = postImageDocKey["_id"];
-    }
-
-    _addToTransferModsQueue(idElement.wrap(), getOpCharForCrudOpType(opType), {}, {});
-
-    return true;
-}
-
-void MigrationChunkClonerSourceLegacy::_deferProcessingForXferMod(const BSONObj& preImageDocKey) {
-    stdx::lock_guard<Latch> sl(_mutex);
-    _deferredReloadOrDeletePreImageDocKeys.push_back(preImageDocKey.getOwned());
-    _deferredUntransferredOpsCounter++;
-}
-
-void MigrationChunkClonerSourceLegacy::_processDeferredXferMods(OperationContext* opCtx,
-                                                                Database* db) {
-    std::vector<BSONObj> deferredReloadOrDeletePreImageDocKeys;
-
-    {
-        stdx::unique_lock lk(_mutex);
-        deferredReloadOrDeletePreImageDocKeys.swap(_deferredReloadOrDeletePreImageDocKeys);
-    }
-
-    for (const auto& preImageDocKey : deferredReloadOrDeletePreImageDocKeys) {
-        auto idElement = preImageDocKey["_id"];
-        BSONObj newerVersionDoc;
-        if (!Helpers::findById(
-                opCtx, db, _args.getNss().ns(), BSON("_id" << idElement), newerVersionDoc)) {
-            // If the document can no longer be found, this means that another later op must have
-            // deleted it. That delete would have been captured by the xferMods so nothing else to
-            // do here.
-            continue;
-        }
-
-        auto postImageDocKey =
-            CollectionMetadata::extractDocumentKey(&_shardKeyPattern, newerVersionDoc);
-        static_cast<void>(_processUpdateForXferMod(preImageDocKey, postImageDocKey));
-    }
-}
-
 Status MigrationChunkClonerSourceLegacy::nextModsBatch(OperationContext* opCtx,
                                                        Database* db,
                                                        BSONObjBuilder* builder) {
     dassert(opCtx->lockState()->isCollectionLockedForMode(_args.getNss(), MODE_IS));
 
-    _processDeferredXferMods(opCtx, db);
-
     std::list<BSONObj> deleteList;
     std::list<BSONObj> updateList;
 
     {
         // All clone data must have been drained before starting to fetch the incremental changes.
         stdx::unique_lock<Latch> lk(_mutex);
-        invariant(!_cloneList.hasMore());
+        invariant(_cloneLocs.empty());
 
         // The "snapshot" for delete and update list must be taken under a single lock. This is to
         // ensure that we will preserve the causal order of writes. Always consume the delete
@@ -875,8 +747,23 @@ Status MigrationChunkClonerSourceLegacy::nextModsBatch(OperationContext* opCtx,
         updateList.splice(updateList.cbegin(), _reload);
     }
 
-    auto totalDocSize = _xferDeletes(builder, &deleteList, 0);
-    totalDocSize = _xferUpdates(opCtx, db, builder, &updateList, totalDocSize);
+    StringData ns = _args.getNss().ns().c_str();
+    BSONArrayBuilder arrDel(builder->subarrayStart("deleted"));
+    auto noopFn = [](BSONObj idDoc, BSONObj* fullDoc) {
+        *fullDoc = idDoc;
+        return true;
+    };
+    long long totalDocSize = xferMods(&arrDel, &deleteList, 0, noopFn);
+    arrDel.done();
+
+    if (deleteList.empty()) {
+        BSONArrayBuilder arrUpd(builder->subarrayStart("reload"));
+        auto findByIdWrapper = [opCtx, db, ns](BSONObj idDoc, BSONObj* fullDoc) {
+            return Helpers::findById(opCtx, db, ns, idDoc, *fullDoc);
+        };
+        totalDocSize = xferMods(&arrUpd, &updateList, totalDocSize, findByIdWrapper);
+        arrUpd.done();
+    }
 
     builder->append("size", totalDocSize);
 
@@ -886,12 +773,11 @@ Status MigrationChunkClonerSourceLegacy::nextModsBatch(OperationContext* opCtx,
     _untransferredDeletesCounter = _deleted.size();
     _reload.splice(_reload.cbegin(), updateList);
     _untransferredUpsertsCounter = _reload.size();
-    _deferredUntransferredOpsCounter = _deferredReloadOrDeletePreImageDocKeys.size();
 
     return Status::OK();
 }
 
-void MigrationChunkClonerSourceLegacy::_cleanup(OperationContext* opCtx) {
+void MigrationChunkClonerSourceLegacy::_cleanup() {
     stdx::unique_lock<Latch> lk(_mutex);
     _state = kDone;
 
@@ -901,8 +787,6 @@ void MigrationChunkClonerSourceLegacy::_cleanup(OperationContext* opCtx) {
     _untransferredUpsertsCounter = 0;
     _deleted.clear();
     _untransferredDeletesCounter = 0;
-    _deferredReloadOrDeletePreImageDocKeys.clear();
-    _deferredUntransferredOpsCounter = 0;
 }
 
 StatusWith<BSONObj> MigrationChunkClonerSourceLegacy::_callRecipient(OperationContext* opCtx,
@@ -956,10 +840,9 @@ MigrationChunkClonerSourceLegacy::_getIndexScanExecutor(
     InternalPlanner::IndexScanOptions scanOption) {
     // Allow multiKey based on the invariant that shard keys must be single-valued. Therefore, any
     // multi-key index prefixed by shard key cannot be multikey over the shard key fields.
-    const IndexDescriptor* shardKeyIdx =
-        collection->getIndexCatalog()->findShardKeyPrefixedIndex(opCtx,
-                                                                 _shardKeyPattern.toBSON(),
-                                                                 false);  // requireSingleKey
+    auto catalog = collection->getIndexCatalog();
+    auto shardKeyIdx = catalog->findShardKeyPrefixedIndex(
+        opCtx, collection, _shardKeyPattern.toBSON(), /*requireSingleKey=*/false);
     if (!shardKeyIdx) {
         return {ErrorCodes::IndexNotFound,
                 str::stream() << "can't find index with prefix " << _shardKeyPattern.toBSON()
@@ -1014,7 +897,7 @@ Status MigrationChunkClonerSourceLegacy::_storeCurrentLocs(OperationContext* opC
         if (avgRecSize == 0) {
             avgRecSize = BSONObj::kMinBSONLength;
         }
-        maxRecsWhenFull = std::max(_args.getMaxChunkSizeBytes() / avgRecSize, 1LL);
+        maxRecsWhenFull = _args.getMaxChunkSizeBytes() / avgRecSize;
         maxRecsWhenFull = 2 * maxRecsWhenFull;  // pad some slack
     } else {
         avgRecSize = 0;
@@ -1029,8 +912,6 @@ Status MigrationChunkClonerSourceLegacy::_storeCurrentLocs(OperationContext* opC
     try {
         BSONObj obj;
         RecordId recordId;
-        RecordIdSet recordIdSet;
-
         while (PlanExecutor::ADVANCED == exec->getNext(&obj, &recordId)) {
             Status interruptStatus = opCtx->checkForInterruptNoAssert();
             if (!interruptStatus.isOK()) {
@@ -1038,20 +919,19 @@ Status MigrationChunkClonerSourceLegacy::_storeCurrentLocs(OperationContext* opC
             }
 
             if (!isLargeChunk) {
-                recordIdSet.insert(recordId);
+                stdx::lock_guard<Latch> lk(_mutex);
+                _cloneLocs.insert(recordId);
             }
 
             if (++recCount > maxRecsWhenFull) {
                 isLargeChunk = true;
 
                 if (_forceJumbo) {
-                    recordIdSet.clear();
+                    _cloneLocs.clear();
                     break;
                 }
             }
         }
-
-        _cloneList.populateList(std::move(recordIdSet));
     } catch (DBException& exception) {
         exception.addContext("Executor error while scanning for documents belonging to chunk");
         throw;
@@ -1090,60 +970,38 @@ Status MigrationChunkClonerSourceLegacy::_storeCurrentLocs(OperationContext* opC
     return Status::OK();
 }
 
-long long MigrationChunkClonerSourceLegacy::_xferDeletes(BSONObjBuilder* builder,
-                                                         std::list<BSONObj>* removeList,
-                                                         long long initialSize) {
-    const long long maxSize = 1024 * 1024;
+long long xferMods(BSONArrayBuilder* arr,
+                   std::list<BSONObj>* modsList,
+                   long long initialSize,
+                   std::function<bool(BSONObj, BSONObj*)> extractDocToAppendFn) {
+    const long long maxSize = BSONObjMaxUserSize;
 
-    if (removeList->empty() || initialSize > maxSize) {
+    if (modsList->empty() || initialSize > maxSize) {
         return initialSize;
     }
 
-    long long totalSize = initialSize;
-    BSONArrayBuilder arr(builder->subarrayStart("deleted"));
-
-    auto docIdIter = removeList->begin();
-    for (; docIdIter != removeList->end() && totalSize < maxSize; ++docIdIter) {
-        BSONObj idDoc = *docIdIter;
-        arr.append(idDoc);
-        totalSize += idDoc.objsize();
-    }
-
-    removeList->erase(removeList->begin(), docIdIter);
-
-    arr.done();
-    return totalSize;
-}
-
-long long MigrationChunkClonerSourceLegacy::_xferUpdates(OperationContext* opCtx,
-                                                         Database* db,
-                                                         BSONObjBuilder* builder,
-                                                         std::list<BSONObj>* updateList,
-                                                         long long initialSize) {
-    const long long maxSize = 1024 * 1024;
-
-    if (updateList->empty() || initialSize > maxSize) {
-        return initialSize;
-    }
-
-    const auto& nss = _args.getNss();
-    BSONArrayBuilder arr(builder->subarrayStart("reload"));
-    long long totalSize = initialSize;
-
-    auto iter = updateList->begin();
-    for (; iter != updateList->end() && totalSize < maxSize; ++iter) {
+    stdx::unordered_set<absl::string_view> addedSet;
+    auto iter = modsList->begin();
+    for (; iter != modsList->end(); ++iter) {
         auto idDoc = *iter;
+        absl::string_view idDocView(idDoc.objdata(), idDoc.objsize());
 
-        BSONObj fullDoc;
-        if (Helpers::findById(opCtx, db, nss.ns().c_str(), idDoc, fullDoc)) {
-            arr.append(fullDoc);
-            totalSize += fullDoc.objsize();
+        if (addedSet.find(idDocView) == addedSet.end()) {
+            addedSet.insert(idDocView);
+            BSONObj fullDoc;
+            if (extractDocToAppendFn(idDoc, &fullDoc)) {
+                if (arr->arrSize() &&
+                    (arr->len() + fullDoc.objsize() + kFixedCommandOverhead) > maxSize) {
+                    break;
+                }
+                arr->append(fullDoc);
+            }
         }
     }
 
-    updateList->erase(updateList->begin(), iter);
+    long long totalSize = arr->len();
+    modsList->erase(modsList->begin(), iter);
 
-    arr.done();
     return totalSize;
 }
 
@@ -1172,33 +1030,35 @@ Status MigrationChunkClonerSourceLegacy::_checkRecipientCloningStatus(OperationC
 
         stdx::lock_guard<Latch> sl(_mutex);
 
+        const std::size_t cloneLocsRemaining = _cloneLocs.size();
         int64_t untransferredModsSizeBytes = _untransferredDeletesCounter * _averageObjectIdSize +
-            (_untransferredUpsertsCounter + _deferredUntransferredOpsCounter) *
-                _averageObjectSizeForCloneLocs;
+            _untransferredUpsertsCounter * _averageObjectSizeForCloneLocs;
 
         if (_forceJumbo && _jumboChunkCloneState) {
             LOGV2(21992,
                   "moveChunk data transfer progress: {response} mem used: {memoryUsedBytes} "
-                  "documents cloned so far: {docsCloned}",
+                  "documents cloned so far: {docsCloned} remaining amount: "
+                  "{untransferredModsSizeBytes}",
                   "moveChunk data transfer progress",
                   "response"_attr = redact(res),
                   "memoryUsedBytes"_attr = _memoryUsed,
-                  "docsCloned"_attr = _jumboChunkCloneState->docsCloned);
+                  "docsCloned"_attr = _jumboChunkCloneState->docsCloned,
+                  "untransferredModsSizeBytes"_attr = untransferredModsSizeBytes);
         } else {
             LOGV2(21993,
                   "moveChunk data transfer progress: {response} mem used: {memoryUsedBytes} "
-                  "documents remaining to clone: {docsRemainingToClone}",
+                  "documents remaining to clone: {docsRemainingToClone} estimated remaining size "
+                  "{untransferredModsSizeBytes}",
                   "moveChunk data transfer progress",
                   "response"_attr = redact(res),
                   "memoryUsedBytes"_attr = _memoryUsed,
-                  "docsRemainingToClone"_attr =
-                      _cloneList.size() - _numRecordsCloned - _numRecordsPassedOver,
+                  "docsRemainingToClone"_attr = cloneLocsRemaining,
                   "untransferredModsSizeBytes"_attr = untransferredModsSizeBytes);
         }
 
         if (res["state"].String() == "steady" && sessionCatalogSourceInCatchupPhase &&
             estimateUntransferredSessionsSize == 0) {
-            if (_cloneList.hasMore() ||
+            if (cloneLocsRemaining != 0 ||
                 (_jumboChunkCloneState && _forceJumbo &&
                  PlanExecutor::IS_EOF != _jumboChunkCloneState->clonerState)) {
                 return {ErrorCodes::OperationIncomplete,
@@ -1224,11 +1084,8 @@ Status MigrationChunkClonerSourceLegacy::_checkRecipientCloningStatus(OperationC
 
         if ((res["state"].String() == "steady" || res["state"].String() == "catchup") &&
             sessionCatalogSourceInCatchupPhase && supportsCriticalSectionDuringCatchUp) {
-            int64_t estimatedUntransferredModsSize =
-                _untransferredDeletesCounter * _averageObjectIdSize +
-                _untransferredUpsertsCounter * _averageObjectSizeForCloneLocs;
             auto estimatedUntransferredChunkPercentage =
-                (std::min(_args.getMaxChunkSizeBytes(), estimatedUntransferredModsSize) * 100) /
+                (std::min(_args.getMaxChunkSizeBytes(), untransferredModsSizeBytes) * 100) /
                 _args.getMaxChunkSizeBytes();
             int64_t maxUntransferredSessionsSize = BSONObjMaxUserSize *
                 _args.getMaxChunkSizeBytes() / ChunkSizeSettingsType::kDefaultMaxChunkSizeBytes;
@@ -1236,18 +1093,16 @@ Status MigrationChunkClonerSourceLegacy::_checkRecipientCloningStatus(OperationC
                 estimateUntransferredSessionsSize < maxUntransferredSessionsSize) {
                 // The recipient is sufficiently caught-up with the writes on the donor.
                 // Block writes, so that it can drain everything.
-                LOGV2_DEBUG(
-                    5630700,
-                    1,
-                    "moveChunk data transfer within threshold to allow write blocking",
-                    "_untransferredUpsertsCounter"_attr = _untransferredUpsertsCounter,
-                    "_untransferredDeletesCounter"_attr = _untransferredDeletesCounter,
-                    "_deferredUntransferredOpsCounter"_attr = _deferredUntransferredOpsCounter,
-                    "_averageObjectSizeForCloneLocs"_attr = _averageObjectSizeForCloneLocs,
-                    "_averageObjectIdSize"_attr = _averageObjectIdSize,
-                    "untransferredSessionDataInBytes"_attr = estimateUntransferredSessionsSize,
-                    "maxChunksSizeBytes"_attr = _args.getMaxChunkSizeBytes(),
-                    "_sessionId"_attr = _sessionId.toString());
+                LOGV2(5630700,
+                      "moveChunk data transfer within threshold to allow write blocking",
+                      "_untransferredUpsertsCounter"_attr = _untransferredUpsertsCounter,
+                      "_untransferredDeletesCounter"_attr = _untransferredDeletesCounter,
+                      "_averageObjectSizeForCloneLocs"_attr = _averageObjectSizeForCloneLocs,
+                      "_averageObjectIdSize"_attr = _averageObjectIdSize,
+                      "untransferredModsSizeBytes"_attr = untransferredModsSizeBytes,
+                      "untransferredSessionDataInBytes"_attr = estimateUntransferredSessionsSize,
+                      "maxChunksSizeBytes"_attr = _args.getMaxChunkSizeBytes(),
+                      "_sessionId"_attr = _sessionId.toString());
                 return Status::OK();
             }
         }
@@ -1347,135 +1202,4 @@ MigrationChunkClonerSourceLegacy::getNotificationForNextSessionMigrationBatch()
     return _sessionCatalogSource->getNotificationForNewOplog();
 }
 
-MigrationChunkClonerSourceLegacy::CloneList::DocumentInFlightWithLock::DocumentInFlightWithLock(
-    WithLock lock, MigrationChunkClonerSourceLegacy::CloneList& clonerList)
-    : _inProgressReadToken(
-          std::make_unique<MigrationChunkClonerSourceLegacy::CloneList::InProgressReadToken>(
-              lock, clonerList)) {}
-
-void MigrationChunkClonerSourceLegacy::CloneList::DocumentInFlightWithLock::setDoc(
-    boost::optional<Snapshotted<BSONObj>> doc) {
-    _doc = std::move(doc);
-}
-
-std::unique_ptr<MigrationChunkClonerSourceLegacy::CloneList::DocumentInFlightWhileNotInLock>
-MigrationChunkClonerSourceLegacy::CloneList::DocumentInFlightWithLock::release() {
-    invariant(_inProgressReadToken);
-
-    return std::make_unique<
-        MigrationChunkClonerSourceLegacy::CloneList::DocumentInFlightWhileNotInLock>(
-        std::move(_inProgressReadToken), std::move(_doc));
-}
-
-MigrationChunkClonerSourceLegacy::CloneList::DocumentInFlightWhileNotInLock::
-    DocumentInFlightWhileNotInLock(
-        std::unique_ptr<CloneList::InProgressReadToken> inProgressReadToken,
-        boost::optional<Snapshotted<BSONObj>> doc)
-    : _inProgressReadToken(std::move(inProgressReadToken)), _doc(std::move(doc)) {}
-
-void MigrationChunkClonerSourceLegacy::CloneList::DocumentInFlightWhileNotInLock::setDoc(
-    boost::optional<Snapshotted<BSONObj>> doc) {
-    _doc = std::move(doc);
-}
-
-const boost::optional<Snapshotted<BSONObj>>&
-MigrationChunkClonerSourceLegacy::CloneList::DocumentInFlightWhileNotInLock::getDoc() {
-    return _doc;
-}
-
-MigrationChunkClonerSourceLegacy::CloneList::InProgressReadToken::InProgressReadToken(
-    WithLock withLock, CloneList& cloneList)
-    : _cloneList(cloneList) {
-    _cloneList._startedOneInProgressRead(withLock);
-}
-
-MigrationChunkClonerSourceLegacy::CloneList::InProgressReadToken::~InProgressReadToken() {
-    _cloneList._finishedOneInProgressRead();
-}
-
-MigrationChunkClonerSourceLegacy::CloneList::CloneList() {
-    _recordIdsIter = _recordIds.begin();
-}
-
-void MigrationChunkClonerSourceLegacy::CloneList::populateList(RecordIdSet recordIds) {
-    stdx::lock_guard lk(_mutex);
-    _recordIds = std::move(recordIds);
-    _recordIdsIter = _recordIds.begin();
-}
-
-void MigrationChunkClonerSourceLegacy::CloneList::insertOverflowDoc(Snapshotted<BSONObj> doc) {
-    stdx::lock_guard lk(_mutex);
-    invariant(_inProgressReads >= 1);
-    _overflowDocs.push_back(std::move(doc));
-}
-
-bool MigrationChunkClonerSourceLegacy::CloneList::hasMore() const {
-    stdx::lock_guard lk(_mutex);
-    return _recordIdsIter != _recordIds.cend() && _inProgressReads > 0;
-}
-
-std::unique_ptr<MigrationChunkClonerSourceLegacy::CloneList::DocumentInFlightWhileNotInLock>
-MigrationChunkClonerSourceLegacy::CloneList::getNextDoc(OperationContext* opCtx,
-                                                        const CollectionPtr& collection,
-                                                        int* numRecordsNoLongerExist) {
-    while (true) {
-        stdx::unique_lock lk(_mutex);
-        invariant(_inProgressReads >= 0);
-        RecordId nextRecordId;
-
-        opCtx->waitForConditionOrInterrupt(_moreDocsCV, lk, [&]() {
-            return _recordIdsIter != _recordIds.end() || !_overflowDocs.empty() ||
-                _inProgressReads == 0;
-        });
-
-        DocumentInFlightWithLock docInFlight(lk, *this);
-
-        // One of the following must now be true (corresponding to the three if conditions):
-        //   1.  There is a document in the overflow set
-        //   2.  The iterator has not reached the end of the record id set
-        //   3.  The overflow set is empty, the iterator is at the end, and
-        //       no threads are holding a document.  This condition indicates
-        //       that there are no more docs to return for the cloning phase.
-        if (!_overflowDocs.empty()) {
-            docInFlight.setDoc(std::move(_overflowDocs.front()));
-            _overflowDocs.pop_front();
-            return docInFlight.release();
-        } else if (_recordIdsIter != _recordIds.end()) {
-            nextRecordId = *_recordIdsIter;
-            ++_recordIdsIter;
-        } else {
-            return docInFlight.release();
-        }
-
-        lk.unlock();
-
-        auto docInFlightWhileNotLocked = docInFlight.release();
-
-        Snapshotted<BSONObj> doc;
-        if (collection->findDoc(opCtx, nextRecordId, &doc)) {
-            docInFlightWhileNotLocked->setDoc(std::move(doc));
-            return docInFlightWhileNotLocked;
-        }
-
-        if (numRecordsNoLongerExist) {
-            (*numRecordsNoLongerExist)++;
-        }
-    }
-}
-
-size_t MigrationChunkClonerSourceLegacy::CloneList::size() const {
-    stdx::unique_lock lk(_mutex);
-    return _recordIds.size();
-}
-
-void MigrationChunkClonerSourceLegacy::CloneList::_startedOneInProgressRead(WithLock) {
-    _inProgressReads++;
-}
-
-void MigrationChunkClonerSourceLegacy::CloneList::_finishedOneInProgressRead() {
-    stdx::lock_guard lk(_mutex);
-    _inProgressReads--;
-    _moreDocsCV.notify_one();
-}
-
 }  // namespace mongo
