@@ -79,7 +79,7 @@ NamespaceString MovePrimarySourceManager::getNss() const {
 Status MovePrimarySourceManager::clone(OperationContext* opCtx) {
     invariant(!opCtx->lockState()->isLocked());
     invariant(_state == kCreated);
-    auto scopedGuard = makeGuard([&] { cleanupOnError(opCtx); });
+    ScopeGuard scopedGuard([&] { cleanupOnError(opCtx); });
 
     LOGV2(22042,
           "Moving {db} primary from: {fromShard} to: {toShard}",
@@ -155,7 +155,7 @@ Status MovePrimarySourceManager::clone(OperationContext* opCtx) {
 Status MovePrimarySourceManager::enterCriticalSection(OperationContext* opCtx) {
     invariant(!opCtx->lockState()->isLocked());
     invariant(_state == kCloneCaughtUp);
-    auto scopedGuard = makeGuard([&] { cleanupOnError(opCtx); });
+    ScopeGuard scopedGuard([&] { cleanupOnError(opCtx); });
 
     // Mark the shard as running a critical operation that requires recovery on crash.
     auto startMetadataOpStatus = ShardingStateRecovery::startMetadataOp(opCtx);
@@ -213,7 +213,7 @@ Status MovePrimarySourceManager::enterCriticalSection(OperationContext* opCtx) {
 Status MovePrimarySourceManager::commitOnConfig(OperationContext* opCtx) {
     invariant(!opCtx->lockState()->isLocked());
     invariant(_state == kCriticalSection);
-    auto scopedGuard = makeGuard([&] { cleanupOnError(opCtx); });
+    ScopeGuard scopedGuard([&] { cleanupOnError(opCtx); });
 
     {
         AutoGetDb autoDb(opCtx, getNss().toString(), MODE_X);
@@ -332,56 +332,42 @@ Status MovePrimarySourceManager::commitOnConfig(OperationContext* opCtx) {
 Status MovePrimarySourceManager::_commitOnConfig(OperationContext* opCtx) {
     auto const configShard = Grid::get(opCtx)->shardRegistry()->getConfigShard();
 
-    auto getDatabaseEntry = [&]() {
-        auto findResponse = uassertStatusOK(
-            configShard->exhaustiveFindOnConfig(opCtx,
-                                                ReadPreferenceSetting{ReadPreference::PrimaryOnly},
-                                                repl::ReadConcernLevel::kMajorityReadConcern,
-                                                DatabaseType::ConfigNS,
-                                                BSON(DatabaseType::name << _dbname),
-                                                BSON(DatabaseType::name << -1),
-                                                1));
+    auto findResponse = uassertStatusOK(
+        configShard->exhaustiveFindOnConfig(opCtx,
+                                            ReadPreferenceSetting{ReadPreference::PrimaryOnly},
+                                            repl::ReadConcernLevel::kMajorityReadConcern,
+                                            DatabaseType::ConfigNS,
+                                            BSON(DatabaseType::name << _dbname),
+                                            BSON(DatabaseType::name << -1),
+                                            1));
 
-        const auto databasesVector = std::move(findResponse.docs);
-        uassert(ErrorCodes::IncompatibleShardingMetadata,
-                str::stream() << "Tried to find max database version for database '" << _dbname
-                              << "', but found no databases",
-                !databasesVector.empty());
+    const auto databasesVector = std::move(findResponse.docs);
+    uassert(ErrorCodes::IncompatibleShardingMetadata,
+            str::stream() << "Tried to find max database version for database '" << _dbname
+                          << "', but found no databases",
+            !databasesVector.empty());
 
-        return uassertStatusOK(DatabaseType::fromBSON(databasesVector.front()));
-    };
+    const auto dbType = uassertStatusOK(DatabaseType::fromBSON(databasesVector.front()));
 
-    const auto originalDbType = getDatabaseEntry();
-
-    if (originalDbType.getPrimary() == _toShard) {
+    if (dbType.getPrimary() == _toShard) {
         return Status::OK();
     }
 
-    auto const newDbVersion = [&]() {
-        auto version = originalDbType.getVersion();
-        return version.makeUpdated();
-    }();
+    auto newDbType = dbType;
+    newDbType.setPrimary(_toShard);
 
-    auto getDottedVersionField = [](const StringData& fieldName) {
-        return DatabaseType::version.name() + "." + fieldName;
-    };
+    auto const currentDatabaseVersion = dbType.getVersion();
 
-    // Include the version in the update filter to be resilient to potential network retries
-    auto const updateQuery =
-        BSON(DatabaseType::name << _dbname << getDottedVersionField(DatabaseVersion::kUuidFieldName)
-                                << newDbVersion.getUuid()
-                                << getDottedVersionField(DatabaseVersion::kLastModFieldName)
-                                << originalDbType.getVersion().getLastMod());
-    auto const update =
-        BSON("$set" << BSON(DatabaseType::primary
-                            << _toShard << getDottedVersionField(DatabaseVersion::kLastModFieldName)
-                            << newDbVersion.getLastMod()));
+    newDbType.setVersion(currentDatabaseVersion.makeUpdated());
+
+    auto updateQueryBuilder = BSONObjBuilder(BSON(DatabaseType::name << _dbname));
+    updateQueryBuilder.append(DatabaseType::version.name(), currentDatabaseVersion.toBSON());
 
     auto updateStatus = Grid::get(opCtx)->catalogClient()->updateConfigDocument(
         opCtx,
         DatabaseType::ConfigNS,
-        updateQuery,
-        update,
+        updateQueryBuilder.obj(),
+        newDbType.toBSON(),
         false,
         ShardingCatalogClient::kMajorityWriteConcern);
 
@@ -394,15 +380,6 @@ Status MovePrimarySourceManager::_commitOnConfig(OperationContext* opCtx) {
         return updateStatus.getStatus();
     }
 
-    const auto updatedDbType = getDatabaseEntry();
-    tassert(6851100,
-            "Error committing movePrimary: database version went backwards",
-            updatedDbType.getVersion().getUuid() != originalDbType.getVersion().getUuid() ||
-                updatedDbType.getVersion().getLastMod() > originalDbType.getVersion().getLastMod());
-    uassert(6851101,
-            "Error committing movePrimary: update of `config.databases` failed",
-            updatedDbType.getPrimary() != _fromShard);
-
     return Status::OK();
 }
 
