@@ -34,13 +34,14 @@
 #include "mongo/db/s/range_deletion_util.h"
 
 #include <algorithm>
-#include <boost/optional.hpp>
 #include <utility>
 
+#include <boost/optional.hpp>
+
 #include "mongo/db/catalog/index_catalog.h"
 #include "mongo/db/catalog_raii.h"
 #include "mongo/db/client.h"
-#include "mongo/db/concurrency/exception_util.h"
+#include "mongo/db/concurrency/write_conflict_exception.h"
 #include "mongo/db/dbhelpers.h"
 #include "mongo/db/exec/delete.h"
 #include "mongo/db/exec/working_set_common.h"
@@ -55,7 +56,6 @@
 #include "mongo/db/repl/repl_client_info.h"
 #include "mongo/db/repl/wait_for_majority_service.h"
 #include "mongo/db/s/migration_util.h"
-#include "mongo/db/s/sharding_runtime_d_params_gen.h"
 #include "mongo/db/s/sharding_statistics.h"
 #include "mongo/db/service_context.h"
 #include "mongo/db/storage/remove_saver.h"
@@ -66,7 +66,11 @@
 #include "mongo/util/future_util.h"
 
 namespace mongo {
+
 namespace {
+const WriteConcernOptions kMajorityWriteConcern(WriteConcernOptions::kMajority,
+                                                WriteConcernOptions::SyncMode::UNSET,
+                                                WriteConcernOptions::kWriteConcernTimeoutSharding);
 
 MONGO_FAIL_POINT_DEFINE(hangBeforeDoingDeletion);
 MONGO_FAIL_POINT_DEFINE(suspendRangeDeletion);
@@ -128,26 +132,19 @@ StatusWith<int> deleteNextBatch(OperationContext* opCtx,
     // The IndexChunk has a keyPattern that may apply to more than one index - we need to
     // select the index and get the full index keyPattern here.
     auto catalog = collection->getIndexCatalog();
-    const IndexDescriptor* idx = catalog->findShardKeyPrefixedIndex(opCtx, keyPattern, false);
-    if (!idx) {
-        LOGV2_ERROR(23765,
-                    "Unable to find shard key index",
-                    "keyPattern"_attr = keyPattern,
-                    "namespace"_attr = nss.ns());
-
-        // When a shard key index is not found, the range deleter gets stuck and indefinitely logs
-        // an error message. This sleep is aimed at avoiding logging too aggressively in order to
-        // prevent log files to increase too much in size.
-        opCtx->sleepFor(Seconds(5));
-
-        uasserted(ErrorCodes::IndexNotFound,
-                  str::stream() << "Unable to find shard key index"
-                                << " for " << nss.ns() << " and key pattern `"
-                                << keyPattern.toString() << "'");
+    auto shardKeyIdx = catalog->findShardKeyPrefixedIndex(
+        opCtx, collection, keyPattern, /*requireSingleKey=*/false);
+    if (!shardKeyIdx) {
+        LOGV2_ERROR_OPTIONS(23765,
+                            {logv2::UserAssertAfterLog(ErrorCodes::InternalError)},
+                            "Unable to find shard key index for {keyPattern} in {namespace}",
+                            "Unable to find shard key index",
+                            "keyPattern"_attr = keyPattern,
+                            "namespace"_attr = nss.ns());
     }
 
     // Extend bounds to match the index we found
-    const KeyPattern indexKeyPattern(idx->keyPattern());
+    const KeyPattern indexKeyPattern(shardKeyIdx->keyPattern());
     const auto extend = [&](const auto& key) {
         return Helpers::toKeyFormat(indexKeyPattern.extendRangeBound(key, false));
     };
@@ -163,7 +160,7 @@ StatusWith<int> deleteNextBatch(OperationContext* opCtx,
                 "max"_attr = max,
                 "namespace"_attr = nss.ns());
 
-    const auto indexName = idx->indexName();
+    const auto indexName = shardKeyIdx->indexName();
     const IndexDescriptor* descriptor =
         collection->getIndexCatalog()->findIndexByName(opCtx, indexName);
     if (!descriptor) {
@@ -243,6 +240,7 @@ StatusWith<int> deleteNextBatch(OperationContext* opCtx,
     return numDeleted;
 }
 
+
 template <typename Callable>
 auto withTemporaryOperationContext(Callable&& callable, const NamespaceString& nss) {
     ThreadClient tc(migrationutil::kRangeDeletionThreadName, getGlobalServiceContext());
@@ -253,14 +251,9 @@ auto withTemporaryOperationContext(Callable&& callable, const NamespaceString& n
     auto uniqueOpCtx = Client::getCurrent()->makeOperationContext();
     auto opCtx = uniqueOpCtx.get();
 
-    {
-        // We acquire the global IX lock and then immediately release it to ensure this operation
-        // would be killed by the RstlKillOpThread during step-up or stepdown. Note that the
-        // RstlKillOpThread kills any operations on step-up or stepdown for which
-        // Locker::wasGlobalLockTakenInModeConflictingWithWrites() returns true.
-        Lock::GlobalLock lk(opCtx, MODE_IX);
-    }
-    invariant(opCtx->lockState()->wasGlobalLockTakenInModeConflictingWithWrites());
+    // Ensure that this operation will be killed by the RstlKillOpThread during step-up or stepdown.
+    opCtx->setAlwaysInterruptAtStepDownOrUp();
+    invariant(opCtx->shouldAlwaysInterruptAtStepDownOrUp());
 
     {
         auto replCoord = repl::ReplicationCoordinator::get(opCtx);
@@ -268,7 +261,7 @@ auto withTemporaryOperationContext(Callable&& callable, const NamespaceString& n
         uassert(ErrorCodes::PrimarySteppedDown,
                 str::stream() << "Not primary while running range deletion task for collection"
                               << nss,
-                replCoord->getReplicationMode() != repl::ReplicationCoordinator::modeReplSet ||
+                replCoord->getReplicationMode() == repl::ReplicationCoordinator::modeReplSet &&
                     replCoord->canAcceptWritesFor(opCtx, nss));
     }
 
@@ -287,9 +280,9 @@ void ensureRangeDeletionTaskStillExists(OperationContext* opCtx, const UUID& mig
     // for deleting the range deletion task document.
     PersistentTaskStore<RangeDeletionTask> store(NamespaceString::kRangeDeletionNamespace);
     auto count = store.count(opCtx,
-                             QUERY(RangeDeletionTask::kIdFieldName
-                                   << migrationId << RangeDeletionTask::kPendingFieldName
-                                   << BSON("$exists" << false)));
+                             BSON(RangeDeletionTask::kIdFieldName
+                                  << migrationId << RangeDeletionTask::kPendingFieldName
+                                  << BSON("$exists" << false)));
     invariant(count == 0 || count == 1, "found duplicate range deletion tasks");
     uassert(ErrorCodes::RangeDeletionAbandonedBecauseTaskDocumentDoesNotExist,
             "Range deletion task no longer exists",
@@ -302,8 +295,8 @@ void ensureRangeDeletionTaskStillExists(OperationContext* opCtx, const UUID& mig
 }
 
 /**
- * Delete the range in a sequence of batches until there are no more documents to delete or deletion
- * returns an error.
+ * Delete the range in a sequence of batches until there are no more documents to
+ * delete or deletion returns an error.
  */
 ExecutorFuture<void> deleteRangeInBatches(const std::shared_ptr<executor::TaskExecutor>& executor,
                                           const NamespaceString& nss,
@@ -313,87 +306,72 @@ ExecutorFuture<void> deleteRangeInBatches(const std::shared_ptr<executor::TaskEx
                                           const boost::optional<UUID>& migrationId,
                                           int numDocsToRemovePerBatch,
                                           Milliseconds delayBetweenBatches) {
-    return ExecutorFuture<void>(executor)
-        .then([=] {
-            bool allDocsRemoved = false;
-            // Delete all batches in this range unless a stepdown error occurs. Do not yield the
-            // executor to ensure that this range is fully deleted before another range is
-            // processed.
-            while (!allDocsRemoved) {
-                try {
-                    allDocsRemoved = withTemporaryOperationContext(
-                        [=](OperationContext* opCtx) {
-                            LOGV2_DEBUG(5346200,
-                                        1,
-                                        "Starting batch deletion",
-                                        "namespace"_attr = nss,
-                                        "range"_attr = redact(range.toString()),
-                                        "numDocsToRemovePerBatch"_attr = numDocsToRemovePerBatch,
-                                        "delayBetweenBatches"_attr = delayBetweenBatches);
+    return AsyncTry([=] {
+               return withTemporaryOperationContext(
+                   [=](OperationContext* opCtx) {
+                       LOGV2_DEBUG(5346200,
+                                   1,
+                                   "Starting batch deletion",
+                                   "namespace"_attr = nss,
+                                   "range"_attr = redact(range.toString()),
+                                   "numDocsToRemovePerBatch"_attr = numDocsToRemovePerBatch,
+                                   "delayBetweenBatches"_attr = delayBetweenBatches);
 
-                            if (migrationId) {
-                                ensureRangeDeletionTaskStillExists(opCtx, *migrationId);
-                            }
+                       if (migrationId) {
+                           ensureRangeDeletionTaskStillExists(opCtx, *migrationId);
+                       }
 
-                            int numDeleted;
+                       AutoGetCollection collection(opCtx, nss, MODE_IX);
 
-                            {
-                                AutoGetCollection collection(opCtx, nss, MODE_IX);
+                       // Ensure the collection exists and has not been dropped or dropped and
+                       // recreated.
+                       uassert(
+                           ErrorCodes::RangeDeletionAbandonedBecauseCollectionWithUUIDDoesNotExist,
+                           "Collection has been dropped since enqueuing this range "
+                           "deletion task. No need to delete documents.",
+                           !collectionUuidHasChanged(
+                               nss, collection.getCollection(), collectionUuid));
 
-                                // Ensure the collection exists and has not been dropped or dropped
-                                // and recreated
-                                uassert(
-                                    ErrorCodes::
-                                        RangeDeletionAbandonedBecauseCollectionWithUUIDDoesNotExist,
-                                    "Collection has been dropped since enqueuing this range "
-                                    "deletion task. No need to delete documents.",
-                                    !collectionUuidHasChanged(
-                                        nss, collection.getCollection(), collectionUuid));
+                       auto numDeleted = uassertStatusOK(deleteNextBatch(opCtx,
+                                                                         collection.getCollection(),
+                                                                         keyPattern,
+                                                                         range,
+                                                                         numDocsToRemovePerBatch));
+                       LOGV2_DEBUG(
+                           23769,
+                           1,
+                           "Deleted {numDeleted} documents in pass in namespace {namespace} with "
+                           "UUID  {collectionUUID} for range {range}",
+                           "Deleted documents in pass",
+                           "numDeleted"_attr = numDeleted,
+                           "namespace"_attr = nss.ns(),
+                           "collectionUUID"_attr = collectionUuid,
+                           "range"_attr = range.toString());
 
-                                numDeleted =
-                                    uassertStatusOK(deleteNextBatch(opCtx,
-                                                                    collection.getCollection(),
-                                                                    keyPattern,
-                                                                    range,
-                                                                    numDocsToRemovePerBatch));
-                            }
+                       if (numDeleted > 0) {
+                           // (SERVER-62368) The range-deleter executor is mono-threaded, so
+                           // sleeping synchronously for `delayBetweenBatches` ensures that no other
+                           // batch is going to be cleared up before the expected delay.
+                           opCtx->sleepFor(delayBetweenBatches);
+                       }
 
-                            LOGV2_DEBUG(23769,
-                                        1,
-                                        "Deleted documents in pass",
-                                        "numDeleted"_attr = numDeleted,
-                                        "namespace"_attr = nss.ns(),
-                                        "collectionUUID"_attr = collectionUuid,
-                                        "range"_attr = range.toString());
-
-                            if (numDeleted > 0) {
-                                // (SERVER-62368) The range-deleter executor is mono-threaded, so
-                                // sleeping synchronously for `delayBetweenBatches` ensures that no
-                                // other batch is going to be cleared up before the expected delay.
-                                opCtx->sleepFor(delayBetweenBatches);
-                            }
-
-                            return numDeleted < numDocsToRemovePerBatch;
-                        },
-                        nss);
-                } catch (const DBException& ex) {
-                    // Errors other than those indicating stepdown and those that indicate that the
-                    // range deletion can no longer occur should be retried.
-                    auto errorCode = ex.code();
-                    if (errorCode ==
-                            ErrorCodes::
-                                RangeDeletionAbandonedBecauseCollectionWithUUIDDoesNotExist ||
-                        errorCode ==
-                            ErrorCodes::RangeDeletionAbandonedBecauseTaskDocumentDoesNotExist ||
-                        errorCode == ErrorCodes::KeyPatternShorterThanBound ||
-                        ErrorCodes::isShutdownError(errorCode) ||
-                        ErrorCodes::isNotPrimaryError(errorCode)) {
-                        return ex.toStatus();
-                    };
-                }
-            }
-            return Status::OK();
+                       return numDeleted;
+                   },
+                   nss);
+           })
+        .until([=](StatusWith<int> swNumDeleted) {
+            // Continue iterating until there are no more documents to delete, retrying on
+            // any error that doesn't indicate that this node is stepping down.
+            return (swNumDeleted.isOK() && swNumDeleted.getValue() < numDocsToRemovePerBatch) ||
+                swNumDeleted.getStatus() ==
+                ErrorCodes::RangeDeletionAbandonedBecauseCollectionWithUUIDDoesNotExist ||
+                swNumDeleted.getStatus() ==
+                ErrorCodes::RangeDeletionAbandonedBecauseTaskDocumentDoesNotExist ||
+                swNumDeleted.getStatus().code() == ErrorCodes::KeyPatternShorterThanBound ||
+                ErrorCodes::isShutdownError(swNumDeleted.getStatus()) ||
+                ErrorCodes::isNotPrimaryError(swNumDeleted.getStatus());
         })
+        .on(executor, CancellationToken::uncancelable())
         .ignoreValue();
 }
 
@@ -424,7 +402,7 @@ void removePersistentRangeDeletionTask(const NamespaceString& nss, UUID migratio
         [&](OperationContext* opCtx) {
             PersistentTaskStore<RangeDeletionTask> store(NamespaceString::kRangeDeletionNamespace);
 
-            store.remove(opCtx, QUERY(RangeDeletionTask::kIdFieldName << migrationId));
+            store.remove(opCtx, BSON(RangeDeletionTask::kIdFieldName << migrationId));
         },
         nss);
 }
@@ -460,7 +438,7 @@ std::vector<RangeDeletionTask> getPersistentRangeDeletionTasks(OperationContext*
     std::vector<RangeDeletionTask> tasks;
 
     PersistentTaskStore<RangeDeletionTask> store(NamespaceString::kRangeDeletionNamespace);
-    auto query = QUERY(RangeDeletionTask::kNssFieldName << nss.ns());
+    auto query = BSON(RangeDeletionTask::kNssFieldName << nss.ns());
 
     store.forEach(opCtx, query, [&](const RangeDeletionTask& deletionTask) {
         tasks.push_back(std::move(deletionTask));
@@ -478,7 +456,7 @@ void snapshotRangeDeletionsForRename(OperationContext* opCtx,
     // Clear out eventual snapshots associated with the target collection: always restart from a
     // clean state in case of stepdown or primary killed.
     PersistentTaskStore<RangeDeletionTask> store(NamespaceString::kRangeDeletionForRenameNamespace);
-    store.remove(opCtx, QUERY(RangeDeletionTask::kNssFieldName << toNss.ns()));
+    store.remove(opCtx, BSON(RangeDeletionTask::kNssFieldName << toNss.ns()));
 
     auto rangeDeletionTasks = getPersistentRangeDeletionTasks(opCtx, fromNss);
     for (auto& task : rangeDeletionTasks) {
@@ -496,7 +474,7 @@ void restoreRangeDeletionTasksForRename(OperationContext* opCtx, const Namespace
     PersistentTaskStore<RangeDeletionTask> rangeDeletionsStore(
         NamespaceString::kRangeDeletionNamespace);
 
-    const auto query = QUERY(RangeDeletionTask::kNssFieldName << nss.ns());
+    const auto query = BSON(RangeDeletionTask::kNssFieldName << nss.ns());
 
     rangeDeletionsForRenameStore.forEach(opCtx, query, [&](const RangeDeletionTask& deletionTask) {
         try {
@@ -514,15 +492,16 @@ void deleteRangeDeletionTasksForRename(OperationContext* opCtx,
     // Delete range deletion tasks associated to the source collection
     PersistentTaskStore<RangeDeletionTask> rangeDeletionsStore(
         NamespaceString::kRangeDeletionNamespace);
-    rangeDeletionsStore.remove(opCtx, QUERY(RangeDeletionTask::kNssFieldName << fromNss.ns()));
+    rangeDeletionsStore.remove(opCtx, BSON(RangeDeletionTask::kNssFieldName << fromNss.ns()));
 
     // Delete already restored snapshots associated to the target collection
     PersistentTaskStore<RangeDeletionTask> rangeDeletionsForRenameStore(
         NamespaceString::kRangeDeletionForRenameNamespace);
     rangeDeletionsForRenameStore.remove(opCtx,
-                                        QUERY(RangeDeletionTask::kNssFieldName << toNss.ns()));
+                                        BSON(RangeDeletionTask::kNssFieldName << toNss.ns()));
 }
 
+
 SharedSemiFuture<void> removeDocumentsInRange(
     const std::shared_ptr<executor::TaskExecutor>& executor,
     SemiFuture<void> waitForActiveQueriesToComplete,
