@@ -43,18 +43,14 @@
 #include "mongo/db/s/recoverable_critical_section_service.h"
 #include "mongo/db/s/rename_collection_participant_service.h"
 #include "mongo/db/s/shard_metadata_util.h"
-#include "mongo/db/s/sharding_ddl_util.h"
 #include "mongo/logv2/log.h"
 #include "mongo/s/catalog/sharding_catalog_client.h"
 #include "mongo/s/grid.h"
-#include "mongo/util/future_util.h"
 
 namespace mongo {
 
 namespace {
 
-const Backoff kExponentialBackoff(Seconds(1), Milliseconds::max());
-
 /*
  * Drop the collection locally and clear stale metadata from cache collections.
  */
@@ -202,23 +198,12 @@ void RenameParticipantInstance::_enterPhase(Phase newPhase) {
     PersistentTaskStore<StateDoc> store(NamespaceString::kShardingRenameParticipantsNamespace);
 
     if (_doc.getPhase() == Phase::kUnset) {
-        try {
-            store.add(opCtx.get(), newDoc, WriteConcerns::kMajorityWriteConcernNoTimeout);
-        } catch (const ExceptionFor<ErrorCodes::DuplicateKey>&) {
-            // A series of step-up and step-down events can cause a node to try and insert the
-            // document when it has already been persisted locally, but we must still wait for
-            // majority commit.
-            const auto replCoord = repl::ReplicationCoordinator::get(opCtx.get());
-            const auto lastLocalOpTime = replCoord->getMyLastAppliedOpTime();
-            WaitForMajorityService::get(opCtx->getServiceContext())
-                .waitUntilMajority(lastLocalOpTime, opCtx.get()->getCancellationToken())
-                .get(opCtx.get());
-        }
+        store.add(opCtx.get(), newDoc, WriteConcerns::kMajorityWriteConcernShardingTimeout);
     } else {
         store.update(opCtx.get(),
                      BSON(StateDoc::kFromNssFieldName << fromNss().ns()),
                      newDoc.toBSON(),
-                     WriteConcerns::kMajorityWriteConcernNoTimeout);
+                     WriteConcerns::kMajorityWriteConcernShardingTimeout);
     }
 
     _doc = std::move(newDoc);
@@ -234,7 +219,7 @@ void RenameParticipantInstance::_removeStateDocument(OperationContext* opCtx) {
     PersistentTaskStore<StateDoc> store(NamespaceString::kShardingRenameParticipantsNamespace);
     store.remove(opCtx,
                  BSON(StateDoc::kFromNssFieldName << fromNss().ns()),
-                 WriteConcerns::kMajorityWriteConcernNoTimeout);
+                 WriteConcerns::kMajorityWriteConcernShardingTimeout);
 
     _doc = {};
 }
@@ -256,47 +241,6 @@ void RenameParticipantInstance::_invalidateFutures(const Status& errStatus) {
 }
 
 SemiFuture<void> RenameParticipantInstance::run(
-    std::shared_ptr<executor::ScopedTaskExecutor> executor,
-    const CancellationToken& token) noexcept {
-    return ExecutorFuture<void>(**executor)
-        .then([this, executor, token, anchor = shared_from_this()] {
-            return AsyncTry([this, executor, token] { return _runImpl(executor, token); })
-                .until([this, token](Status status) { return status.isOK() || token.isCanceled(); })
-                .withBackoffBetweenIterations(kExponentialBackoff)
-                .on(**executor, CancellationToken::uncancelable());
-        })
-        .onCompletion([this, anchor = shared_from_this()](const Status& status) {
-            if (!status.isOK()) {
-                // The token gets canceled in case of stepdown/shutdown
-                _invalidateFutures(status);
-                return;
-            }
-
-            try {
-                auto opCtxHolder = cc().makeOperationContext();
-                auto* opCtx = opCtxHolder.get();
-                _removeStateDocument(opCtx);
-            } catch (DBException& ex) {
-                LOGV2_WARNING(5515108,
-                              "Failed to remove rename participant state document",
-                              "error"_attr = redact(ex));
-                ex.addContext("Failed to remove rename participant state document"_sd);
-                stdx::lock_guard<Latch> lg(_mutex);
-                if (!_unblockCRUDPromise.getFuture().isReady()) {
-                    _unblockCRUDPromise.setError(ex.toStatus());
-                }
-                throw;
-            }
-
-            stdx::lock_guard<Latch> lg(_mutex);
-            if (!_unblockCRUDPromise.getFuture().isReady()) {
-                _unblockCRUDPromise.emplaceValue();
-            }
-        })
-        .semi();
-}
-
-SemiFuture<void> RenameParticipantInstance::_runImpl(
     std::shared_ptr<executor::ScopedTaskExecutor> executor,
     const CancellationToken& token) noexcept {
     return ExecutorFuture<void>(**executor)
@@ -308,7 +252,9 @@ SemiFuture<void> RenameParticipantInstance::_runImpl(
 
                 // Acquire source/target critical sections
                 const auto reason =
-                    sharding_ddl_util::getCriticalSectionReasonForRename(fromNss(), toNss());
+                    BSON("command"
+                         << "rename"
+                         << "from" << fromNss().toString() << "to" << toNss().toString());
                 auto service = RecoverableCriticalSectionService::get(opCtx);
                 service->acquireRecoverableCriticalSectionBlockWrites(
                     opCtx, fromNss(), reason, ShardingCatalogClient::kLocalWriteConcern);
@@ -319,23 +265,10 @@ SemiFuture<void> RenameParticipantInstance::_runImpl(
                 service->promoteRecoverableCriticalSectionToBlockAlsoReads(
                     opCtx, toNss(), reason, ShardingCatalogClient::kLocalWriteConcern);
 
-                // Clear the filtering metadata before releasing the critical section (it will be
-                // recovered the next time is accessed) and to safely create new range deletion
-                // tasks (the submission will serialize on the renamed collection's metadata
-                // refresh).
-                {
-                    Lock::DBLock dbLock(opCtx, fromNss().db(), MODE_IX);
-                    Lock::CollectionLock collLock(opCtx, fromNss(), MODE_IX);
-                    auto* csr = CollectionShardingRuntime::get(opCtx, fromNss());
-                    csr->clearFilteringMetadataForDroppedCollection(opCtx);
-                }
-
-                {
-                    Lock::DBLock dbLock(opCtx, toNss().db(), MODE_IX);
-                    Lock::CollectionLock collLock(opCtx, toNss(), MODE_IX);
-                    auto* csr = CollectionShardingRuntime::get(opCtx, toNss());
-                    csr->clearFilteringMetadata(opCtx);
-                }
+                // Clear the filtering metadata to safely create new range deletion tasks: the
+                // submission will serialize on the renamed collection's metadata refresh.
+                clearFilteringMetadata(opCtx, fromNss());
+                clearFilteringMetadata(opCtx, toNss());
 
                 snapshotRangeDeletionsForRename(opCtx, fromNss(), toNss());
             }))
@@ -410,17 +343,45 @@ SemiFuture<void> RenameParticipantInstance::_runImpl(
                 service->releaseRecoverableCriticalSection(
                     opCtx, fromNss(), reason, ShardingCatalogClient::kLocalWriteConcern);
                 service->releaseRecoverableCriticalSection(
-                    opCtx, toNss(), reason, WriteConcerns::kMajorityWriteConcernNoTimeout);
+                    opCtx, toNss(), reason, ShardingCatalogClient::kMajorityWriteConcern);
 
                 LOGV2(5515107, "CRUD unblocked", "fromNs"_attr = fromNss(), "toNs"_attr = toNss());
             }))
-        .onError([this, anchor = shared_from_this()](const Status& status) {
-            LOGV2_ERROR(6372200,
-                        "Error executing rename collection participant. Going to be retried.",
-                        "fromNs"_attr = fromNss(),
-                        "toNs"_attr = toNss(),
-                        "error"_attr = redact(status));
-            return status;
+        .onCompletion([this, anchor = shared_from_this()](const Status& status) {
+            if (!status.isOK()) {
+                if (!status.isA<ErrorCategory::NotPrimaryError>() &&
+                    !status.isA<ErrorCategory::ShutdownError>()) {
+                    LOGV2_ERROR(5515109,
+                                "Error executing rename collection participant",
+                                "fromNs"_attr = fromNss(),
+                                "toNs"_attr = toNss(),
+                                "error"_attr = redact(status));
+                }
+
+                _invalidateFutures(status);
+                return;
+            }
+
+            try {
+                auto opCtxHolder = cc().makeOperationContext();
+                auto* opCtx = opCtxHolder.get();
+                _removeStateDocument(opCtx);
+            } catch (DBException& ex) {
+                LOGV2_WARNING(5515108,
+                              "Failed to remove rename participant state document",
+                              "error"_attr = redact(ex));
+                ex.addContext("Failed to remove rename participant state document"_sd);
+                stdx::lock_guard<Latch> lg(_mutex);
+                if (!_unblockCRUDPromise.getFuture().isReady()) {
+                    _unblockCRUDPromise.setError(ex.toStatus());
+                }
+                throw;
+            }
+
+            stdx::lock_guard<Latch> lg(_mutex);
+            if (!_unblockCRUDPromise.getFuture().isReady()) {
+                _unblockCRUDPromise.emplaceValue();
+            }
         })
         .semi();
 }
