@@ -58,6 +58,12 @@ public:
         return ThreadPool::Limits();
     }
 
+    // The service implemented its own conflict check before this method was added.
+    void checkIfConflictsWithOtherInstances(
+        OperationContext* opCtx,
+        BSONObj initialState,
+        const std::vector<const PrimaryOnlyService::Instance*>& existingInstances) override{};
+
     std::shared_ptr<Instance> constructInstance(BSONObj initialState) override;
 };
 
