@@ -27,15 +27,13 @@
  *    it in the license file.
  */
 
-#include "mongo/base/string_data.h"
-#include "mongo/s/write_ops/batched_command_request.h"
-
 #define MONGO_LOGV2_DEFAULT_COMPONENT ::mongo::logv2::LogComponent::kResharding
 
 #include "mongo/platform/basic.h"
 
 #include "mongo/db/s/resharding/resharding_coordinator_service.h"
 
+#include "mongo/base/string_data.h"
 #include "mongo/bson/bsonobj.h"
 #include "mongo/bson/json.h"
 #include "mongo/db/auth/authorization_session_impl.h"
@@ -43,6 +41,8 @@
 #include "mongo/db/logical_session_cache.h"
 #include "mongo/db/ops/write_ops.h"
 #include "mongo/db/repl/primary_only_service.h"
+#include "mongo/db/repl/repl_client_info.h"
+#include "mongo/db/repl/wait_for_majority_service.h"
 #include "mongo/db/s/balancer/balance_stats.h"
 #include "mongo/db/s/balancer/balancer_policy.h"
 #include "mongo/db/s/config/initial_split_policy.h"
@@ -69,6 +69,7 @@
 #include "mongo/s/request_types/flush_resharding_state_change_gen.h"
 #include "mongo/s/request_types/flush_routing_table_cache_updates_gen.h"
 #include "mongo/s/shard_id.h"
+#include "mongo/s/write_ops/batched_command_request.h"
 #include "mongo/s/write_ops/batched_command_response.h"
 #include "mongo/util/assert_util.h"
 #include "mongo/util/fail_point.h"
@@ -82,6 +83,7 @@ namespace {
 using namespace fmt::literals;
 
 MONGO_FAIL_POINT_DEFINE(reshardingPauseCoordinatorAfterPreparingToDonate);
+MONGO_FAIL_POINT_DEFINE(reshardingPauseCoordinatorBeforeInitializing);
 MONGO_FAIL_POINT_DEFINE(reshardingPauseCoordinatorBeforeCloning);
 MONGO_FAIL_POINT_DEFINE(reshardingPauseCoordinatorBeforeBlockingWrites);
 MONGO_FAIL_POINT_DEFINE(reshardingPauseCoordinatorBeforeDecisionPersisted);
@@ -542,15 +544,15 @@ void updateChunkAndTagsDocsForTempNss(OperationContext* opCtx,
 void executeMetadataChangesInTxn(
     OperationContext* opCtx,
     unique_function<void(OperationContext*, TxnNumber)> changeMetadataFunc) {
-    ShardingCatalogManager::withTransaction(opCtx,
-                                            NamespaceString::kConfigReshardingOperationsNamespace,
-                                            [&](OperationContext* opCtx, TxnNumber txnNumber) {
-                                                changeMetadataFunc(opCtx, txnNumber);
-                                            });
+    ShardingCatalogManager::withTransaction(
+        opCtx,
+        NamespaceString::kConfigReshardingOperationsNamespace,
+        [&](OperationContext* opCtx, TxnNumber txnNumber) { changeMetadataFunc(opCtx, txnNumber); },
+        ShardingCatalogClient::kLocalWriteConcern);
 }
 
 BSONObj makeFlushRoutingTableCacheUpdatesCmd(const NamespaceString& nss) {
-    auto cmd = _flushRoutingTableCacheUpdatesWithWriteConcern(nss);
+    auto cmd = FlushRoutingTableCacheUpdatesWithWriteConcern(nss);
     cmd.setSyncFromConfig(true);
     cmd.setDbName(nss.db());
     return cmd.toBSON(
@@ -633,9 +635,10 @@ void writeDecisionPersistedState(OperationContext* opCtx,
 
 void insertCoordDocAndChangeOrigCollEntry(OperationContext* opCtx,
                                           const ReshardingCoordinatorDocument& coordinatorDoc) {
-
     ShardingCatalogManager::get(opCtx)->bumpCollectionVersionAndChangeMetadataInTxn(
-        opCtx, coordinatorDoc.getSourceNss(), [&](OperationContext* opCtx, TxnNumber txnNumber) {
+        opCtx,
+        coordinatorDoc.getSourceNss(),
+        [&](OperationContext* opCtx, TxnNumber txnNumber) {
             auto doc = ShardingCatalogManager::get(opCtx)->findOneConfigDocumentInTxn(
                 opCtx,
                 CollectionType::ConfigNS,
@@ -674,7 +677,8 @@ void insertCoordDocAndChangeOrigCollEntry(OperationContext* opCtx,
             // 'reshardingFields'
             updateConfigCollectionsForOriginalNss(
                 opCtx, coordinatorDoc, boost::none, boost::none, txnNumber);
-        });
+        },
+        ShardingCatalogClient::kLocalWriteConcern);
 }
 
 void writeParticipantShardsAndTempCollInfo(
@@ -701,7 +705,8 @@ void writeParticipantShardsAndTempCollInfo(
             writeToCoordinatorStateNss(opCtx, updatedCoordinatorDoc, txnNumber);
             updateConfigCollectionsForOriginalNss(
                 opCtx, updatedCoordinatorDoc, boost::none, boost::none, txnNumber);
-        });
+        },
+        ShardingCatalogClient::kLocalWriteConcern);
 }
 
 void writeStateTransitionAndCatalogUpdatesThenBumpShardVersions(
@@ -715,7 +720,9 @@ void writeStateTransitionAndCatalogUpdatesThenBumpShardVersions(
     }
 
     ShardingCatalogManager::get(opCtx)->bumpMultipleCollectionVersionsAndChangeMetadataInTxn(
-        opCtx, collNames, [&](OperationContext* opCtx, TxnNumber txnNumber) {
+        opCtx,
+        collNames,
+        [&](OperationContext* opCtx, TxnNumber txnNumber) {
             // Update the config.reshardingOperations entry
             writeToCoordinatorStateNss(opCtx, coordinatorDoc, txnNumber);
 
@@ -731,7 +738,8 @@ void writeStateTransitionAndCatalogUpdatesThenBumpShardVersions(
                 writeToConfigCollectionsForTempNss(
                     opCtx, coordinatorDoc, boost::none, boost::none, txnNumber);
             }
-        });
+        },
+        ShardingCatalogClient::kLocalWriteConcern);
 }
 
 void removeCoordinatorDocAndReshardingFields(OperationContext* opCtx,
@@ -775,7 +783,8 @@ void removeCoordinatorDocAndReshardingFields(OperationContext* opCtx,
             // Remove the resharding fields from the config.collections entry
             updateConfigCollectionsForOriginalNss(
                 opCtx, updatedCoordinatorDoc, boost::none, boost::none, txnNumber);
-        });
+        },
+        ShardingCatalogClient::kLocalWriteConcern);
 }
 }  // namespace resharding
 
@@ -1057,7 +1066,7 @@ ReshardingCoordinatorService::ReshardingCoordinator::_tellAllParticipantsReshard
                        _cancelableOpCtxFactory.emplace(_ctHolder->getStepdownToken(),
                                                        _markKilledExecutor);
                    })
-                   .then([this, executor] {
+                   .then([this, executor]() {
                        pauseBeforeTellDonorToRefresh.pauseWhileSet();
                        _establishAllDonorsAsParticipants(executor);
                    })
@@ -1087,7 +1096,8 @@ ExecutorFuture<void> ReshardingCoordinatorService::ReshardingCoordinator::_initi
     return resharding::WithAutomaticRetry([this, executor] {
                return ExecutorFuture<void>(**executor)
                    .then([this, executor] { _insertCoordDocAndChangeOrigCollEntry(); })
-                   .then([this, executor] { _calculateParticipantsAndChunksThenWriteToDisk(); });
+                   .then([this, executor] { _calculateParticipantsAndChunksThenWriteToDisk(); })
+                   .then([this] { return _waitForMajority(_ctHolder->getAbortToken()); });
            })
         .onTransientError([](const Status& status) {
             LOGV2(5093703,
@@ -1229,6 +1239,8 @@ ReshardingCoordinatorService::ReshardingCoordinator::_commitAndFinishReshardOper
                    .then([this, executor, updatedCoordinatorDoc] {
                        return _commit(updatedCoordinatorDoc);
                    })
+                   .then([this] { return _waitForMajority(_ctHolder->getStepdownToken()); })
+                   .thenRunOn(**executor)
                    .then([this, executor] {
                        _tellAllParticipantsToCommit(_coordinatorDoc.getSourceNss(), executor);
                    })
@@ -1386,23 +1398,30 @@ ReshardingCoordinatorService::ReshardingCoordinator::_onAbortCoordinatorAndParti
     invariant(_coordinatorDoc.getState() >= CoordinatorStateEnum::kPreparingToDonate);
 
     return resharding::WithAutomaticRetry([this, executor, status] {
-               if (_coordinatorDoc.getState() != CoordinatorStateEnum::kAborting) {
-                   // The coordinator only transitions into kAborting if there are participants to
-                   // wait on before transitioning to kDone.
-                   _updateCoordinatorDocStateAndCatalogEntries(CoordinatorStateEnum::kAborting,
-                                                               _coordinatorDoc,
-                                                               boost::none,
-                                                               boost::none,
-                                                               status);
-               }
+               return ExecutorFuture<void>(**executor)
+                   .then([this, executor, status] {
+                       if (_coordinatorDoc.getState() != CoordinatorStateEnum::kAborting) {
+                           // The coordinator only transitions into kAborting if there are
+                           // participants to wait on before transitioning to kDone.
+                           _updateCoordinatorDocStateAndCatalogEntries(
+                               CoordinatorStateEnum::kAborting,
+                               _coordinatorDoc,
+                               boost::none,
+                               boost::none,
+                               status);
+                       }
+                   })
+                   .then([this] { return _waitForMajority(_ctHolder->getStepdownToken()); })
+                   .thenRunOn(**executor)
+                   .then([this, executor, status] {
+                       _tellAllParticipantsToAbort(executor,
+                                                   status == ErrorCodes::ReshardCollectionAborted);
 
-               _tellAllParticipantsToAbort(executor,
-                                           status == ErrorCodes::ReshardCollectionAborted);
-
-               // Wait for all participants to acknowledge the operation reached an unrecoverable
-               // error.
-               return future_util::withCancellation(_awaitAllParticipantShardsDone(executor),
-                                                    _ctHolder->getStepdownToken());
+                       // Wait for all participants to acknowledge the operation reached an
+                       // unrecoverable error.
+                       return future_util::withCancellation(
+                           _awaitAllParticipantShardsDone(executor), _ctHolder->getStepdownToken());
+                   });
            })
         .onTransientError([](const Status& retryStatus) {
             LOGV2(5093707,
@@ -1455,6 +1474,16 @@ void ReshardingCoordinatorService::ReshardingCoordinator::_fulfillOkayToEnterCri
     }
 }
 
+SemiFuture<void> ReshardingCoordinatorService::ReshardingCoordinator::_waitForMajority(
+    const CancellationToken& token) {
+    auto opCtx = _cancelableOpCtxFactory->makeOperationContext(&cc());
+    auto client = opCtx->getClient();
+    repl::ReplClientInfo::forClient(client).setLastOpToSystemLastOpTime(opCtx.get());
+    auto opTime = repl::ReplClientInfo::forClient(client).getLastOp();
+    return WaitForMajorityService::get(client->getServiceContext())
+        .waitUntilMajority(opTime, token);
+}
+
 void ReshardingCoordinatorService::ReshardingCoordinator::_insertCoordDocAndChangeOrigCollEntry() {
     if (_coordinatorDoc.getState() > CoordinatorStateEnum::kUnused) {
         if (!_coordinatorDocWrittenPromise.getFuture().isReady()) {
@@ -1471,6 +1500,8 @@ void ReshardingCoordinatorService::ReshardingCoordinator::_insertCoordDocAndChan
     }
 
     auto opCtx = _cancelableOpCtxFactory->makeOperationContext(&cc());
+    reshardingPauseCoordinatorBeforeInitializing.pauseWhileSetAndNotCanceled(
+        opCtx.get(), _ctHolder->getStepdownToken());
     ReshardingCoordinatorDocument updatedCoordinatorDoc = _coordinatorDoc;
     updatedCoordinatorDoc.setState(CoordinatorStateEnum::kInitializing);
     resharding::insertCoordDocAndChangeOrigCollEntry(opCtx.get(), updatedCoordinatorDoc);
@@ -1573,7 +1604,8 @@ ReshardingCoordinatorService::ReshardingCoordinator::_awaitAllDonorsReadyToDonat
                 coordinatorDocChangedOnDisk,
                 highestMinFetchTimestamp,
                 computeApproxCopySize(coordinatorDocChangedOnDisk));
-        });
+        })
+        .then([this] { return _waitForMajority(_ctHolder->getAbortToken()); });
 }
 
 ExecutorFuture<void>
@@ -1590,7 +1622,8 @@ ReshardingCoordinatorService::ReshardingCoordinator::_awaitAllRecipientsFinished
         .then([this](ReshardingCoordinatorDocument coordinatorDocChangedOnDisk) {
             this->_updateCoordinatorDocStateAndCatalogEntries(CoordinatorStateEnum::kApplying,
                                                               coordinatorDocChangedOnDisk);
-        });
+        })
+        .then([this] { return _waitForMajority(_ctHolder->getAbortToken()); });
 }
 
 void ReshardingCoordinatorService::ReshardingCoordinator::_startCommitMonitor(
@@ -1646,6 +1679,10 @@ ReshardingCoordinatorService::ReshardingCoordinator::_awaitAllRecipientsFinished
 
             this->_updateCoordinatorDocStateAndCatalogEntries(CoordinatorStateEnum::kBlockingWrites,
                                                               _coordinatorDoc);
+        })
+        .then([this] { return _waitForMajority(_ctHolder->getAbortToken()); })
+        .thenRunOn(**executor)
+        .then([this, executor] {
             const auto criticalSectionTimeout =
                 Milliseconds(resharding::gReshardingCriticalSectionTimeoutMillis.load());
             const auto criticalSectionExpiresAt = (*executor)->now() + criticalSectionTimeout;
