@@ -58,10 +58,12 @@ public:
     const ShardKeyPattern kShardKey = ShardKeyPattern(BSON("oldKey" << 1));
     const OID kOrigEpoch = OID::gen();
     const UUID kOrigUUID = UUID::gen();
+    const Timestamp kOrigTimestamp = Timestamp(1);
     const NamespaceString kOrigNss = NamespaceString("db.foo");
     const ShardKeyPattern kReshardingKey = ShardKeyPattern(BSON("newKey" << 1));
     const OID kReshardingEpoch = OID::gen();
     const UUID kReshardingUUID = UUID::gen();
+    const Timestamp kReshardingTimestamp = Timestamp(2);
     const NamespaceString kReshardingNss = NamespaceString(
         str::stream() << "db." << NamespaceString::kTemporaryReshardingCollectionPrefix
                       << kOrigUUID);
@@ -139,11 +141,12 @@ public:
                                                         const ShardKeyPattern& skey,
                                                         UUID uuid,
                                                         OID epoch,
+                                                        Timestamp timestamp,
                                                         const BSONObj& collation = {}) {
         auto future = scheduleRoutingInfoForcedRefresh(tempNss);
 
         expectFindSendBSONObjVector(kConfigHostAndPort, [&]() {
-            CollectionType coll(tempNss, epoch, Date_t::now(), uuid);
+            CollectionType coll(tempNss, epoch, timestamp, Date_t::now(), uuid);
             coll.setKeyPattern(skey.getKeyPattern());
             coll.setUnique(false);
             coll.setDefaultCollation(collation);
@@ -161,9 +164,9 @@ public:
             reshardingFields.setRecipientFields(recipientFields);
             coll.setReshardingFields(reshardingFields);
 
-            ChunkVersion version(1, 0, epoch, boost::none /* timestamp */);
+            ChunkVersion version(1, 0, epoch, timestamp);
 
-            ChunkType chunk(tempNss,
+            ChunkType chunk(uuid,
                             {skey.getKeyPattern().globalMin(), skey.getKeyPattern().globalMax()},
                             version,
                             {"0"});
@@ -180,15 +183,16 @@ public:
     void expectRefreshReturnForOriginalColl(const NamespaceString& origNss,
                                             const ShardKeyPattern& skey,
                                             UUID uuid,
-                                            OID epoch) {
+                                            OID epoch,
+                                            Timestamp timestamp) {
         expectFindSendBSONObjVector(kConfigHostAndPort, [&]() {
-            CollectionType coll(origNss, epoch, Date_t::now(), uuid);
+            CollectionType coll(origNss, epoch, timestamp, Date_t::now(), uuid);
             coll.setKeyPattern(skey.getKeyPattern());
             coll.setUnique(false);
 
-            ChunkVersion version(1, 0, epoch, boost::none /* timestamp */);
+            ChunkVersion version(1, 0, epoch, timestamp);
 
-            ChunkType chunk(origNss,
+            ChunkType chunk(uuid,
                             {skey.getKeyPattern().globalMin(), skey.getKeyPattern().globalMax()},
                             version,
                             {"0"});
@@ -204,8 +208,9 @@ public:
         onCommand([&](const executor::RemoteCommandRequest& request) {
             ASSERT_EQ(request.cmdObj.firstElementFieldNameStringData(), expectedCmdName);
             return createErrorCursorResponse(
-                Status(StaleDbRoutingVersion(
-                           nss.db().toString(), DatabaseVersion(UUID::gen()), boost::none),
+                Status(StaleDbRoutingVersion(nss.db().toString(),
+                                             DatabaseVersion(UUID::gen(), Timestamp(1, 1)),
+                                             boost::none),
                        "dummy stale db version error"));
         });
     }
@@ -243,15 +248,17 @@ public:
         }
         ASSERT_EQ(indexesCopy.size(), 0);
     }
+
+    void verifyTempReshardingCollectionAndMetadata() {
+        RecipientStateMachineExternalStateImpl externalState;
+        externalState.ensureTempReshardingCollectionExistsWithIndexes(
+            operationContext(), kMetadata, kDefaultFetchTimestamp);
+        CollectionShardingRuntime csr(getServiceContext(), kOrigNss, executor());
+        ASSERT(csr.getCurrentMetadataIfKnown() == boost::none);
+    }
 };
 
 TEST_F(RecipientServiceExternalStateTest, ReshardingConfigServerUpdatesHaveNoTimeout) {
-    // TODO (SERVER-57194): enable lock-free reads.
-    bool disableLockFreeReadsOriginalValue = storageGlobalParams.disableLockFreeReads;
-    storageGlobalParams.disableLockFreeReads = true;
-    ON_BLOCK_EXIT(
-        [&] { storageGlobalParams.disableLockFreeReads = disableLockFreeReadsOriginalValue; });
-
     RecipientStateMachineExternalStateImpl externalState;
 
     auto future = launchAsync([&] {
@@ -286,8 +293,12 @@ TEST_F(RecipientServiceExternalStateTest, CreateLocalReshardingCollectionBasic)
     }
 
     // Simulate a refresh for the temporary resharding collection.
-    loadOneChunkMetadataForTemporaryReshardingColl(
-        kReshardingNss, kOrigNss, kReshardingKey, kReshardingUUID, kReshardingEpoch);
+    loadOneChunkMetadataForTemporaryReshardingColl(kReshardingNss,
+                                                   kOrigNss,
+                                                   kReshardingKey,
+                                                   kReshardingUUID,
+                                                   kReshardingEpoch,
+                                                   kReshardingTimestamp);
 
     const std::vector<BSONObj> indexes = {BSON("v" << 2 << "key" << BSON("_id" << 1) << "name"
                                                    << "_id_"),
@@ -297,7 +308,8 @@ TEST_F(RecipientServiceExternalStateTest, CreateLocalReshardingCollectionBasic)
                                                    << "name"
                                                    << "indexOne")};
     auto future = launchAsync([&] {
-        expectRefreshReturnForOriginalColl(kOrigNss, kShardKey, kOrigUUID, kOrigEpoch);
+        expectRefreshReturnForOriginalColl(
+            kOrigNss, kShardKey, kOrigUUID, kOrigEpoch, kOrigTimestamp);
         expectListCollections(
             kOrigNss,
             kOrigUUID,
@@ -309,9 +321,7 @@ TEST_F(RecipientServiceExternalStateTest, CreateLocalReshardingCollectionBasic)
         expectListIndexes(kOrigNss, kOrigUUID, indexes, HostAndPort(shards[0].getHost()));
     });
 
-    RecipientStateMachineExternalStateImpl externalState;
-    externalState.ensureTempReshardingCollectionExistsWithIndexes(
-        operationContext(), kMetadata, kDefaultFetchTimestamp);
+    verifyTempReshardingCollectionAndMetadata();
 
     future.default_timed_get();
 
@@ -334,8 +344,12 @@ TEST_F(RecipientServiceExternalStateTest,
     }
 
     // Simulate a refresh for the temporary resharding collection.
-    loadOneChunkMetadataForTemporaryReshardingColl(
-        kReshardingNss, kOrigNss, kReshardingKey, kReshardingUUID, kReshardingEpoch);
+    loadOneChunkMetadataForTemporaryReshardingColl(kReshardingNss,
+                                                   kOrigNss,
+                                                   kReshardingKey,
+                                                   kReshardingUUID,
+                                                   kReshardingEpoch,
+                                                   kReshardingTimestamp);
 
     const std::vector<BSONObj> indexes = {BSON("v" << 2 << "key" << BSON("_id" << 1) << "name"
                                                    << "_id_"),
@@ -345,7 +359,8 @@ TEST_F(RecipientServiceExternalStateTest,
                                                    << "name"
                                                    << "indexOne")};
     auto future = launchAsync([&] {
-        expectRefreshReturnForOriginalColl(kOrigNss, kShardKey, kOrigUUID, kOrigEpoch);
+        expectRefreshReturnForOriginalColl(
+            kOrigNss, kShardKey, kOrigUUID, kOrigEpoch, kOrigTimestamp);
         expectStaleDbVersionError(kOrigNss, "listCollections");
         expectGetDatabase(kOrigNss, shards[1].getHost());
         expectListCollections(
@@ -358,13 +373,12 @@ TEST_F(RecipientServiceExternalStateTest,
             HostAndPort(shards[1].getHost()));
 
         expectStaleEpochError(kOrigNss, "listIndexes");
-        expectRefreshReturnForOriginalColl(kOrigNss, kShardKey, kOrigUUID, kOrigEpoch);
+        expectRefreshReturnForOriginalColl(
+            kOrigNss, kShardKey, kOrigUUID, kOrigEpoch, kOrigTimestamp);
         expectListIndexes(kOrigNss, kOrigUUID, indexes, HostAndPort(shards[0].getHost()));
     });
 
-    RecipientStateMachineExternalStateImpl externalState;
-    externalState.ensureTempReshardingCollectionExistsWithIndexes(
-        operationContext(), kMetadata, kDefaultFetchTimestamp);
+    verifyTempReshardingCollectionAndMetadata();
 
     future.default_timed_get();
 
@@ -387,8 +401,12 @@ TEST_F(RecipientServiceExternalStateTest,
     }
 
     // Simulate a refresh for the temporary resharding collection.
-    loadOneChunkMetadataForTemporaryReshardingColl(
-        kReshardingNss, kOrigNss, kReshardingKey, kReshardingUUID, kReshardingEpoch);
+    loadOneChunkMetadataForTemporaryReshardingColl(kReshardingNss,
+                                                   kOrigNss,
+                                                   kReshardingKey,
+                                                   kReshardingUUID,
+                                                   kReshardingEpoch,
+                                                   kReshardingTimestamp);
 
     const std::vector<BSONObj> indexes = {BSON("v" << 2 << "key" << BSON("_id" << 1) << "name"
                                                    << "_id_"),
@@ -413,7 +431,8 @@ TEST_F(RecipientServiceExternalStateTest,
     }
 
     auto future = launchAsync([&] {
-        expectRefreshReturnForOriginalColl(kOrigNss, kShardKey, kOrigUUID, kOrigEpoch);
+        expectRefreshReturnForOriginalColl(
+            kOrigNss, kShardKey, kOrigUUID, kOrigEpoch, kOrigTimestamp);
         expectListCollections(
             kOrigNss,
             kOrigUUID,
@@ -425,9 +444,7 @@ TEST_F(RecipientServiceExternalStateTest,
         expectListIndexes(kOrigNss, kOrigUUID, indexes, HostAndPort(shards[0].getHost()));
     });
 
-    RecipientStateMachineExternalStateImpl externalState;
-    externalState.ensureTempReshardingCollectionExistsWithIndexes(
-        operationContext(), kMetadata, kDefaultFetchTimestamp);
+    verifyTempReshardingCollectionAndMetadata();
 
     future.default_timed_get();
 
@@ -450,8 +467,12 @@ TEST_F(RecipientServiceExternalStateTest,
     }
 
     // Simulate a refresh for the temporary resharding collection.
-    loadOneChunkMetadataForTemporaryReshardingColl(
-        kReshardingNss, kOrigNss, kReshardingKey, kReshardingUUID, kReshardingEpoch);
+    loadOneChunkMetadataForTemporaryReshardingColl(kReshardingNss,
+                                                   kOrigNss,
+                                                   kReshardingKey,
+                                                   kReshardingUUID,
+                                                   kReshardingEpoch,
+                                                   kReshardingTimestamp);
 
     const std::vector<BSONObj> indexes = {BSON("v" << 2 << "key" << BSON("_id" << 1) << "name"
                                                    << "_id_"),
@@ -478,7 +499,8 @@ TEST_F(RecipientServiceExternalStateTest,
     }
 
     auto future = launchAsync([&] {
-        expectRefreshReturnForOriginalColl(kOrigNss, kShardKey, kOrigUUID, kOrigEpoch);
+        expectRefreshReturnForOriginalColl(
+            kOrigNss, kShardKey, kOrigUUID, kOrigEpoch, kOrigTimestamp);
         expectListCollections(
             kOrigNss,
             kOrigUUID,
@@ -490,9 +512,7 @@ TEST_F(RecipientServiceExternalStateTest,
         expectListIndexes(kOrigNss, kOrigUUID, indexes, HostAndPort(shards[0].getHost()));
     });
 
-    RecipientStateMachineExternalStateImpl externalState;
-    externalState.ensureTempReshardingCollectionExistsWithIndexes(
-        operationContext(), kMetadata, kDefaultFetchTimestamp);
+    verifyTempReshardingCollectionAndMetadata();
 
     future.default_timed_get();
 
@@ -515,8 +535,12 @@ TEST_F(RecipientServiceExternalStateTest,
     }
 
     // Simulate a refresh for the temporary resharding collection.
-    loadOneChunkMetadataForTemporaryReshardingColl(
-        kReshardingNss, kOrigNss, kReshardingKey, kReshardingUUID, kReshardingEpoch);
+    loadOneChunkMetadataForTemporaryReshardingColl(kReshardingNss,
+                                                   kOrigNss,
+                                                   kReshardingKey,
+                                                   kReshardingUUID,
+                                                   kReshardingEpoch,
+                                                   kReshardingTimestamp);
 
     const std::vector<BSONObj> indexes = {BSON("v" << 2 << "key" << BSON("_id" << 1) << "name"
                                                    << "_id_"),
@@ -533,7 +557,8 @@ TEST_F(RecipientServiceExternalStateTest,
         operationContext(), kReshardingNss, optionsAndIndexes);
 
     auto future = launchAsync([&] {
-        expectRefreshReturnForOriginalColl(kOrigNss, kShardKey, kOrigUUID, kOrigEpoch);
+        expectRefreshReturnForOriginalColl(
+            kOrigNss, kShardKey, kOrigUUID, kOrigEpoch, kOrigTimestamp);
         expectListCollections(
             kOrigNss,
             kOrigUUID,
@@ -545,9 +570,7 @@ TEST_F(RecipientServiceExternalStateTest,
         expectListIndexes(kOrigNss, kOrigUUID, indexes, HostAndPort(shards[0].getHost()));
     });
 
-    RecipientStateMachineExternalStateImpl externalState;
-    externalState.ensureTempReshardingCollectionExistsWithIndexes(
-        operationContext(), kMetadata, kDefaultFetchTimestamp);
+    verifyTempReshardingCollectionAndMetadata();
 
     future.default_timed_get();
 
