@@ -61,9 +61,8 @@ boost::optional<repl::OplogEntry> forgeNoopEntryFromImageCollection(
 
     DBDirectClient client(opCtx);
     BSONObj imageObj =
-        client.findOne(NamespaceString::kConfigImagesNamespace.ns(),
-                       BSON("_id" << retryableFindAndModifyOplogEntry.getSessionId()->toBSON()),
-                       nullptr);
+        client.findOne(NamespaceString::kConfigImagesNamespace,
+                       BSON("_id" << retryableFindAndModifyOplogEntry.getSessionId()->toBSON()));
     if (imageObj.isEmpty()) {
         return boost::none;
     }
@@ -123,8 +122,7 @@ boost::optional<repl::OplogEntry> fetchPrePostImageOplog(OperationContext* opCtx
 
     auto opTime = opTimeToFetch.value();
     DBDirectClient client(opCtx);
-    auto oplogBSON =
-        client.findOne(NamespaceString::kRsOplogNamespace.ns(), opTime.asQuery(), nullptr);
+    auto oplogBSON = client.findOne(NamespaceString::kRsOplogNamespace, opTime.asQuery());
 
     return uassertStatusOK(repl::OplogEntry::parse(oplogBSON));
 }
@@ -192,13 +190,12 @@ SessionCatalogMigrationSource::SessionCatalogMigrationSource(OperationContext* o
       _rollbackIdAtInit(repl::ReplicationProcess::get(opCtx)->getRollbackID()),
       _chunkRange(std::move(chunk)),
       _keyPattern(shardKey) {
-    Query query;
     // Sort is not needed for correctness. This is just for making it easier to write deterministic
     // tests.
-    query.sort(BSON("_id" << 1));
-
     DBDirectClient client(opCtx);
-    auto cursor = client.query(NamespaceString::kSessionTransactionsTableNamespace, query);
+    auto cursor = client.query(NamespaceString::kSessionTransactionsTableNamespace,
+                               BSONObj{},
+                               Query().sort(BSON("_id" << 1)));
 
     while (cursor->more()) {
         auto nextSession = SessionTxnRecord::parse(
@@ -437,8 +434,8 @@ bool SessionCatalogMigrationSource::_fetchNextNewWriteOplog(OperationContext* op
     }
 
     DBDirectClient client(opCtx);
-    const auto& newWriteOplogDoc = client.findOne(
-        NamespaceString::kRsOplogNamespace.ns(), nextOpTimeToFetch.asQuery(), nullptr);
+    const auto& newWriteOplogDoc =
+        client.findOne(NamespaceString::kRsOplogNamespace, nextOpTimeToFetch.asQuery());
 
     uassert(40620,
             str::stream() << "Unable to fetch oplog entry with opTime: "
