@@ -35,42 +35,42 @@
 
 #include "mongo/client/connection_string.h"
 #include "mongo/client/global_conn_pool.h"
-#include "mongo/client/remote_command_targeter.h"
 #include "mongo/client/remote_command_targeter_factory_impl.h"
 #include "mongo/client/replica_set_monitor.h"
+#include "mongo/db/audit.h"
 #include "mongo/db/catalog_raii.h"
 #include "mongo/db/client_metadata_propagation_egress_hook.h"
 #include "mongo/db/concurrency/d_concurrency.h"
 #include "mongo/db/dbhelpers.h"
-#include "mongo/db/logical_time_metadata_hook.h"
 #include "mongo/db/logical_time_validator.h"
 #include "mongo/db/operation_context.h"
 #include "mongo/db/ops/update.h"
 #include "mongo/db/repl/replication_coordinator.h"
 #include "mongo/db/s/chunk_splitter.h"
+#include "mongo/db/s/dist_lock_catalog_replset.h"
+#include "mongo/db/s/dist_lock_manager_replset.h"
 #include "mongo/db/s/periodic_balancer_config_refresher.h"
 #include "mongo/db/s/read_only_catalog_cache_loader.h"
+#include "mongo/db/s/shard_local.h"
 #include "mongo/db/s/shard_server_catalog_cache_loader.h"
-#include "mongo/db/s/sharding_config_optime_gossip.h"
 #include "mongo/db/s/transaction_coordinator_service.h"
 #include "mongo/db/server_options.h"
+#include "mongo/db/vector_clock_metadata_hook.h"
 #include "mongo/executor/task_executor_pool.h"
 #include "mongo/logv2/log.h"
 #include "mongo/rpc/metadata/egress_metadata_hook_list.h"
 #include "mongo/s/catalog_cache.h"
-#include "mongo/s/client/shard_connection.h"
 #include "mongo/s/client/shard_factory.h"
-#include "mongo/s/client/shard_local.h"
 #include "mongo/s/client/shard_remote.h"
 #include "mongo/s/client/sharding_connection_hook.h"
 #include "mongo/s/config_server_catalog_cache_loader.h"
 #include "mongo/s/grid.h"
-#include "mongo/s/sharding_initialization.h"
 
 namespace mongo {
 
 // Failpoint for disabling updateShardIdentityConfigString calls on signaled RS nodes.
 MONGO_FAIL_POINT_DEFINE(failUpdateShardIdentityConfigString);
+MONGO_FAIL_POINT_DEFINE(hangDuringShardingInitialization);
 
 namespace {
 
@@ -78,9 +78,8 @@ const auto getInstance = ServiceContext::declareDecoration<ShardingInitializatio
 
 auto makeEgressHooksList(ServiceContext* service) {
     auto unshardedHookList = std::make_unique<rpc::EgressMetadataHookList>();
-    unshardedHookList->addHook(std::make_unique<rpc::LogicalTimeMetadataHook>(service));
+    unshardedHookList->addHook(std::make_unique<rpc::VectorClockMetadataHook>(service));
     unshardedHookList->addHook(std::make_unique<rpc::ClientMetadataPropagationEgressHook>());
-    unshardedHookList->addHook(std::make_unique<rpc::ShardingEgressMetadataHookForMongod>(service));
 
     return unshardedHookList;
 }
@@ -106,7 +105,10 @@ public:
             LOGV2(471691,
                   "Updating the shard registry with confirmed replica set",
                   "connectionString"_attr = connStr);
-            Grid::get(_serviceContext)->shardRegistry()->updateReplSetHosts(connStr);
+            Grid::get(_serviceContext)
+                ->shardRegistry()
+                ->updateReplSetHosts(connStr,
+                                     ShardRegistry::ConnectionStringUpdateType::kConfirmed);
         } catch (const ExceptionForCat<ErrorCategory::ShutdownError>& e) {
             LOGV2(471692, "Unable to update the shard registry", "error"_attr = e);
         }
@@ -131,7 +133,10 @@ public:
 
     void onPossibleSet(const State& state) noexcept final {
         try {
-            Grid::get(_serviceContext)->shardRegistry()->updateReplSetHosts(state.connStr);
+            Grid::get(_serviceContext)
+                ->shardRegistry()
+                ->updateReplSetHosts(state.connStr,
+                                     ShardRegistry::ConnectionStringUpdateType::kPossible);
         } catch (const DBException& ex) {
             LOGV2_DEBUG(22070,
                         2,
@@ -169,7 +174,7 @@ private:
     void _updateShardIdentityConfigString(Status status,
                                           std::string setName,
                                           ConnectionString update) {
-        if (ErrorCodes::isCancelationError(status.code())) {
+        if (ErrorCodes::isCancellationError(status.code())) {
             LOGV2_DEBUG(22067,
                         2,
                         "Unable to schedule confirmed replica set update due to {error}",
@@ -253,42 +258,9 @@ private:
 
 }  // namespace
 
-void ShardingInitializationMongoD::initializeShardingEnvironmentOnShardServer(
-    OperationContext* opCtx, const ShardIdentity& shardIdentity, StringData distLockProcessId) {
-    initializeGlobalShardingStateForMongoD(
-        opCtx, shardIdentity.getConfigsvrConnectionString(), distLockProcessId);
-
-    _replicaSetChangeListener =
-        ReplicaSetMonitor::getNotifier().makeListener<ShardingReplicaSetChangeListener>(
-            opCtx->getServiceContext());
-
-    // Determine primary/secondary/standalone state in order to properly initialize sharding
-    // components.
-    const auto replCoord = repl::ReplicationCoordinator::get(opCtx);
-    bool isReplSet = replCoord->getReplicationMode() == repl::ReplicationCoordinator::modeReplSet;
-    bool isStandaloneOrPrimary =
-        !isReplSet || (replCoord->getMemberState() == repl::MemberState::RS_PRIMARY);
-
-    CatalogCacheLoader::get(opCtx).initializeReplicaSetRole(isStandaloneOrPrimary);
-    ChunkSplitter::get(opCtx).onShardingInitialization(isStandaloneOrPrimary);
-    PeriodicBalancerConfigRefresher::get(opCtx).onShardingInitialization(opCtx->getServiceContext(),
-                                                                         isStandaloneOrPrimary);
-
-    // Start the transaction coordinator service only if the node is the primary of a replica set
-    TransactionCoordinatorService::get(opCtx)->onShardingInitialization(
-        opCtx, isReplSet && isStandaloneOrPrimary);
-
-    Grid::get(opCtx)->setShardingInitialized();
-
-    LOGV2(22071,
-          "Finished initializing sharding components for {memberState} node.",
-          "Finished initializing sharding components",
-          "memberState"_attr = (isStandaloneOrPrimary ? "primary" : "secondary"));
-}
-
 ShardingInitializationMongoD::ShardingInitializationMongoD()
     : _initFunc([this](auto... args) {
-          this->initializeShardingEnvironmentOnShardServer(std::forward<decltype(args)>(args)...);
+          _initializeShardingEnvironmentOnShardServer(std::forward<decltype(args)>(args)...);
       }) {}
 
 ShardingInitializationMongoD::~ShardingInitializationMongoD() = default;
@@ -303,14 +275,14 @@ ShardingInitializationMongoD* ShardingInitializationMongoD::get(ServiceContext*
 
 void ShardingInitializationMongoD::shutDown(OperationContext* opCtx) {
     auto const shardingState = ShardingState::get(opCtx);
-    auto const grid = Grid::get(opCtx);
-
     if (!shardingState->enabled())
         return;
 
-    grid->getExecutorPool()->shutdownAndJoin();
-    grid->catalogClient()->shutDown(opCtx);
+    DistLockManager::get(opCtx)->shutDown(opCtx);
+
+    auto const grid = Grid::get(opCtx);
     grid->shardRegistry()->shutdown();
+
     _replicaSetChangeListener.reset();
 }
 
@@ -433,6 +405,8 @@ void ShardingInitializationMongoD::initializeFromShardIdentity(
     auto const shardingState = ShardingState::get(opCtx);
     auto const shardRegistry = Grid::get(opCtx)->shardRegistry();
 
+    hangDuringShardingInitialization.pauseWhileSet();
+
     stdx::unique_lock<Latch> ul(_initSynchronizationMutex);
 
     if (shardingState->enabled()) {
@@ -440,7 +414,9 @@ void ShardingInitializationMongoD::initializeFromShardIdentity(
         uassert(40372, "", shardingState->clusterId() == shardIdentity.getClusterId());
 
         auto prevConfigsvrConnStr = shardRegistry->getConfigServerConnectionString();
-        uassert(40373, "", prevConfigsvrConnStr.type() == ConnectionString::SET);
+        uassert(40373,
+                "",
+                prevConfigsvrConnStr.type() == ConnectionString::ConnectionType::kReplicaSet);
         uassert(40374, "", prevConfigsvrConnStr.getSetName() == configSvrConnStr.getSetName());
 
         return;
@@ -454,12 +430,18 @@ void ShardingInitializationMongoD::initializeFromShardIdentity(
             !initializationStatus);
 
     try {
-        _initFunc(opCtx, shardIdentity, generateDistLockProcessId(opCtx));
+        _initFunc(opCtx, shardIdentity);
         shardingState->setInitialized(shardIdentity.getShardName().toString(),
                                       shardIdentity.getClusterId());
     } catch (const DBException& ex) {
         shardingState->setInitialized(ex.toStatus());
     }
+
+    Grid::get(opCtx)->setShardingInitialized();
+
+    if (audit::initializeSynchronizeJob) {
+        audit::initializeSynchronizeJob(opCtx->getServiceContext());
+    }
 }
 
 void ShardingInitializationMongoD::updateShardIdentityConfigString(
@@ -467,15 +449,16 @@ void ShardingInitializationMongoD::updateShardIdentityConfigString(
     BSONObj updateObj(
         ShardIdentityType::createConfigServerUpdateObject(newConnectionString.toString()));
 
-    UpdateRequest updateReq(NamespaceString::kServerConfigurationNamespace);
+    auto updateReq = UpdateRequest();
+    updateReq.setNamespaceString(NamespaceString::kServerConfigurationNamespace);
     updateReq.setQuery(BSON("_id" << ShardIdentityType::IdName));
-    updateReq.setUpdateModification(updateObj);
+    updateReq.setUpdateModification(
+        write_ops::UpdateModification::parseFromClassicUpdate(updateObj));
 
     try {
-        AutoGetOrCreateDb autoDb(
-            opCtx, NamespaceString::kServerConfigurationNamespace.db(), MODE_X);
+        AutoGetDb autoDb(opCtx, NamespaceString::kServerConfigurationNamespace.db(), MODE_X);
 
-        auto result = update(opCtx, autoDb.getDb(), updateReq);
+        auto result = update(opCtx, autoDb.ensureDbExists(), updateReq);
         if (result.numMatched == 0) {
             LOGV2_WARNING(22076,
                           "Failed to update config server connection string of shard identity "
@@ -503,32 +486,28 @@ void ShardingInitializationMongoD::updateShardIdentityConfigString(
 }
 
 void initializeGlobalShardingStateForMongoD(OperationContext* opCtx,
-                                            const ConnectionString& configCS,
-                                            StringData distLockProcessId) {
+                                            const ShardId& shardId,
+                                            const ConnectionString& configCS) {
+    uassert(ErrorCodes::BadValue, "Unrecognized connection string.", configCS);
+
     auto targeterFactory = std::make_unique<RemoteCommandTargeterFactoryImpl>();
     auto targeterFactoryPtr = targeterFactory.get();
 
-    ShardFactory::BuilderCallable setBuilder = [targeterFactoryPtr](
-                                                   const ShardId& shardId,
-                                                   const ConnectionString& connStr) {
-        return std::make_unique<ShardRemote>(shardId, connStr, targeterFactoryPtr->create(connStr));
-    };
-
-    ShardFactory::BuilderCallable masterBuilder = [targeterFactoryPtr](
-                                                      const ShardId& shardId,
-                                                      const ConnectionString& connStr) {
-        return std::make_unique<ShardRemote>(shardId, connStr, targeterFactoryPtr->create(connStr));
-    };
-
-    ShardFactory::BuilderCallable localBuilder = [](const ShardId& shardId,
-                                                    const ConnectionString& connStr) {
-        return std::make_unique<ShardLocal>(shardId);
-    };
-
     ShardFactory::BuildersMap buildersMap{
-        {ConnectionString::SET, std::move(setBuilder)},
-        {ConnectionString::MASTER, std::move(masterBuilder)},
-        {ConnectionString::LOCAL, std::move(localBuilder)},
+        {ConnectionString::ConnectionType::kReplicaSet,
+         [targeterFactoryPtr](const ShardId& shardId, const ConnectionString& connStr) {
+             return std::make_unique<ShardRemote>(
+                 shardId, connStr, targeterFactoryPtr->create(connStr));
+         }},
+        {ConnectionString::ConnectionType::kLocal,
+         [](const ShardId& shardId, const ConnectionString& connStr) {
+             return std::make_unique<ShardLocal>(shardId);
+         }},
+        {ConnectionString::ConnectionType::kStandalone,
+         [targeterFactoryPtr](const ShardId& shardId, const ConnectionString& connStr) {
+             return std::make_unique<ShardRemote>(
+                 shardId, connStr, targeterFactoryPtr->create(connStr));
+         }},
     };
 
     auto shardFactory =
@@ -544,8 +523,10 @@ void initializeGlobalShardingStateForMongoD(OperationContext* opCtx,
                                     std::make_unique<ShardServerCatalogCacheLoader>(
                                         std::make_unique<ConfigServerCatalogCacheLoader>()));
         }
-    } else {
+    } else if (serverGlobalParams.clusterRole == ClusterRole::ConfigServer) {
         CatalogCacheLoader::set(service, std::make_unique<ConfigServerCatalogCacheLoader>());
+    } else {
+        MONGO_UNREACHABLE;
     }
 
     auto validator = LogicalTimeValidator::get(service);
@@ -553,10 +534,9 @@ void initializeGlobalShardingStateForMongoD(OperationContext* opCtx,
         validator->stopKeyManager();
     }
 
-    globalConnPool.addHook(new ShardingConnectionHook(false, makeEgressHooksList(service)));
-    shardConnectionPool.addHook(new ShardingConnectionHook(true, makeEgressHooksList(service)));
+    globalConnPool.addHook(new ShardingConnectionHook(makeEgressHooksList(service)));
 
-    auto catalogCache = std::make_unique<CatalogCache>(CatalogCacheLoader::get(opCtx));
+    auto catalogCache = std::make_unique<CatalogCache>(service, CatalogCacheLoader::get(opCtx));
 
     // List of hooks which will be called by the ShardRegistry when it discovers a shard has been
     // removed.
@@ -568,16 +548,11 @@ void initializeGlobalShardingStateForMongoD(OperationContext* opCtx,
             catCache->invalidateEntriesThatReferenceShard(removedShard);
         }};
 
-    uassert(ErrorCodes::BadValue,
-            "Unrecognized connection string.",
-            configCS.type() != ConnectionString::INVALID);
-
     auto shardRegistry = std::make_unique<ShardRegistry>(
         std::move(shardFactory), configCS, std::move(shardRemovalHooks));
 
     uassertStatusOK(
         initializeGlobalShardingState(opCtx,
-                                      distLockProcessId,
                                       std::move(catalogCache),
                                       std::move(shardRegistry),
                                       [service] { return makeEgressHooksList(service); },
@@ -585,6 +560,15 @@ void initializeGlobalShardingStateForMongoD(OperationContext* opCtx,
                                       // executors aren't used for user queries in mongod.
                                       1));
 
+    DistLockManager::create(
+        service,
+        std::make_unique<ReplSetDistLockManager>(service,
+                                                 shardId,
+                                                 std::make_unique<DistLockCatalogImpl>(),
+                                                 ReplSetDistLockManager::kDistLockPingInterval,
+                                                 ReplSetDistLockManager::kDistLockExpirationTime));
+    DistLockManager::get(opCtx)->startUp();
+
     auto const replCoord = repl::ReplicationCoordinator::get(service);
     if (serverGlobalParams.clusterRole == ClusterRole::ConfigServer &&
         replCoord->getMemberState().primary()) {
@@ -592,4 +576,37 @@ void initializeGlobalShardingStateForMongoD(OperationContext* opCtx,
     }
 }
 
+void ShardingInitializationMongoD::_initializeShardingEnvironmentOnShardServer(
+    OperationContext* opCtx, const ShardIdentity& shardIdentity) {
+
+    _replicaSetChangeListener =
+        ReplicaSetMonitor::getNotifier().makeListener<ShardingReplicaSetChangeListener>(
+            opCtx->getServiceContext());
+
+    initializeGlobalShardingStateForMongoD(opCtx,
+                                           shardIdentity.getShardName().toString(),
+                                           shardIdentity.getConfigsvrConnectionString());
+
+    // Determine primary/secondary/standalone state in order to properly initialize sharding
+    // components.
+    const auto replCoord = repl::ReplicationCoordinator::get(opCtx);
+    bool isReplSet = replCoord->getReplicationMode() == repl::ReplicationCoordinator::modeReplSet;
+    bool isStandaloneOrPrimary =
+        !isReplSet || (replCoord->getMemberState() == repl::MemberState::RS_PRIMARY);
+
+    CatalogCacheLoader::get(opCtx).initializeReplicaSetRole(isStandaloneOrPrimary);
+    ChunkSplitter::get(opCtx).onShardingInitialization(isStandaloneOrPrimary);
+    PeriodicBalancerConfigRefresher::get(opCtx).onShardingInitialization(opCtx->getServiceContext(),
+                                                                         isStandaloneOrPrimary);
+
+    // Start the transaction coordinator service only if the node is the primary of a replica set
+    TransactionCoordinatorService::get(opCtx)->onShardingInitialization(
+        opCtx, isReplSet && isStandaloneOrPrimary);
+
+    LOGV2(22071,
+          "Finished initializing sharding components for {memberState} node.",
+          "Finished initializing sharding components",
+          "memberState"_attr = (isStandaloneOrPrimary ? "primary" : "secondary"));
+}
+
 }  // namespace mongo
