@@ -92,7 +92,7 @@ public:
             }
 
             for (auto doneFuture : futuresToWait) {
-                doneFuture.get(opCtx);
+                doneFuture.wait(opCtx);
             }
 
             // If abort actually went through, the resharding documents should be cleaned up.
@@ -143,6 +143,11 @@ public:
         }
     };
 
+    bool skipApiVersionCheck() const override {
+        // Internal command (server to server).
+        return true;
+    }
+
     std::string help() const override {
         return "Internal command, which is exported by the shard server. Do not call directly. "
                "Aborts any in-progress resharding operations.";
