@@ -40,7 +40,7 @@ class TransactionCoordinatorMetricsObserver;
 
 /**
  * State machine, which implements the two-phase commit protocol for a specific transaction,
- * identified by lsid + txnNumber.
+ * identified by lsid + txnNumber + txnRetryCounter.
  *
  * The lifetime of a coordinator starts with a construction and ends with the `onCompletion()`
  * future getting signaled. It is illegal to destroy a coordinator without waiting for
@@ -64,8 +64,8 @@ public:
     };
 
     /**
-     * Instantiates a new TransactioncCoordinator for the specified lsid + txnNumber pair and gives
-     * it a 'scheduler' to use for any asynchronous tasks it spawns.
+     * Instantiates a new TransactionCoordinator for the specified lsid + txnNumber +
+     * txnRetryCounter and gives it a 'scheduler' to use for any asynchronous tasks it spawns.
      *
      * If the 'coordinateCommitDeadline' parameter is specified, a timed task will be scheduled to
      * cause the coordinator to be put in a cancelled state, if runCommit is not eventually
@@ -73,7 +73,7 @@ public:
      */
     TransactionCoordinator(OperationContext* operationContext,
                            const LogicalSessionId& lsid,
-                           TxnNumber txnNumber,
+                           const TxnNumberAndRetryCounter& txnNumberAndRetryCounter,
                            std::unique_ptr<txn::AsyncWorkScheduler> scheduler,
                            Date_t deadline);
 
@@ -95,8 +95,6 @@ public:
     /**
      * Gets a Future that will contain the decision that the coordinator reaches. Note that this
      * will never be signaled unless runCommit has been called.
-     *
-     * TODO (SERVER-37364): Remove this when it is no longer needed by the coordinator service.
      */
     SharedSemiFuture<txn::CommitDecision> getDecision() const;
 
@@ -117,6 +115,10 @@ public:
      */
     void cancelIfCommitNotYetStarted();
 
+    TxnRetryCounter getTxnRetryCounterForTest() const {
+        return *_txnNumberAndRetryCounter.getTxnRetryCounter();
+    }
+
     /**
      * Returns the TransactionCoordinatorMetricsObserver for this TransactionCoordinator.
      */
@@ -153,9 +155,9 @@ private:
     // Shortcut to the service context under which this coordinator runs
     ServiceContext* const _serviceContext;
 
-    // The lsid + transaction number that this coordinator is coordinating
+    // The lsid + txnNumber + txnRetryCounter that this coordinator is coordinating.
     const LogicalSessionId _lsid;
-    const TxnNumber _txnNumber;
+    const TxnNumberAndRetryCounter _txnNumberAndRetryCounter;
 
     // Scheduler and context wrapping all asynchronous work dispatched by this coordinator
     std::unique_ptr<txn::AsyncWorkScheduler> _scheduler;
