@@ -536,7 +536,6 @@ public:
                 str::stream() << "Unexpected empty file: " << this->_file->path().string(),
                 ranges.empty() || boost::filesystem::file_size(this->_file->path()) != 0);
 
-        this->_numSpills += ranges.size();
         this->_iters.reserve(ranges.size());
         std::transform(ranges.begin(),
                        ranges.end(),
@@ -613,7 +612,6 @@ private:
     }
 
     void spill() {
-        this->_numSpills++;
         if (_data.empty())
             return;
 
@@ -896,7 +894,6 @@ private:
     void spill() {
         invariant(!_done);
 
-        this->_numSpills += 1;
         if (_data.empty())
             return;
 
@@ -953,15 +950,14 @@ private:
 template <typename Key, typename Value>
 Sorter<Key, Value>::Sorter(const SortOptions& opts)
     : _opts(opts),
-      _file(opts.extSortAllowed ? std::make_shared<Sorter<Key, Value>::File>(
-                                      opts.tempDir + "/" + nextFileName(), opts.sorterFileStats)
-                                : nullptr) {}
+      _file(opts.extSortAllowed
+                ? std::make_shared<Sorter<Key, Value>::File>(opts.tempDir + "/" + nextFileName())
+                : nullptr) {}
 
 template <typename Key, typename Value>
 Sorter<Key, Value>::Sorter(const SortOptions& opts, const std::string& fileName)
     : _opts(opts),
-      _file(std::make_shared<Sorter<Key, Value>::File>(opts.tempDir + "/" + fileName,
-                                                       opts.sorterFileStats)) {
+      _file(std::make_shared<Sorter<Key, Value>::File>(opts.tempDir + "/" + fileName)) {
     invariant(opts.extSortAllowed);
     invariant(!opts.tempDir.empty());
     invariant(!fileName.empty());
@@ -983,10 +979,6 @@ typename Sorter<Key, Value>::PersistedState Sorter<Key, Value>::persistDataForSh
 
 template <typename Key, typename Value>
 Sorter<Key, Value>::File::~File() {
-    if (_stats && _file.is_open()) {
-        _stats->closed.addAndFetch(1);
-    }
-
     if (_keep) {
         return;
     }
@@ -1077,10 +1069,6 @@ void Sorter<Key, Value>::File::_open() {
             str::stream() << "Error opening file " << _path.string() << ": "
                           << sorter::myErrnoWithDescription(),
             _file.good());
-
-    if (_stats) {
-        _stats->opened.addAndFetch(1);
-    }
 }
 
 template <typename Key, typename Value>
