@@ -44,6 +44,9 @@ namespace {
 // expect to see in an indexed field.
 const size_t kMaxKeyPatternPathLength = 2048;
 
+const std::string kTimeseriesBucketsMayHaveMixedSchemaDataFieldName =
+    "timeseriesBucketsMayHaveMixedSchemaData";
+
 /**
  * Encodes 'multikeyPaths' as binary data and appends it to 'bob'.
  *
@@ -137,6 +140,13 @@ void BSONCollectionCatalogEntry::IndexMetaData::updateHiddenSetting(bool hidden)
 }
 
 
+void BSONCollectionCatalogEntry::IndexMetaData::updateUniqueSetting() {
+    BSONObjBuilder b(spec);
+    b.appendBool("unique", true);
+    spec = b.obj();
+}
+
+
 // --------------------------
 
 int BSONCollectionCatalogEntry::MetaData::getTotalIndexCount() const {
@@ -218,6 +228,12 @@ BSONObj BSONCollectionCatalogEntry::MetaData::toBSON(bool hasExclusiveAccess) co
         }
         arr.doneFast();
     }
+
+    if (timeseriesBucketsMayHaveMixedSchemaData) {
+        b.append(kTimeseriesBucketsMayHaveMixedSchemaDataFieldName,
+                 *timeseriesBucketsMayHaveMixedSchemaData);
+    }
+
     return b.obj();
 }
 
@@ -261,5 +277,10 @@ void BSONCollectionCatalogEntry::MetaData::parse(const BSONObj& obj) {
             indexes.push_back(imd);
         }
     }
+
+    BSONElement timeseriesMixedSchemaElem = obj[kTimeseriesBucketsMayHaveMixedSchemaDataFieldName];
+    if (!timeseriesMixedSchemaElem.eoo() && timeseriesMixedSchemaElem.isBoolean()) {
+        timeseriesBucketsMayHaveMixedSchemaData = timeseriesMixedSchemaElem.Bool();
+    }
 }
 }  // namespace mongo
