@@ -97,7 +97,7 @@ private:
 // Helper to interpret index data buffer for unique index
 class UniqueIndexData {
 public:
-    UniqueIndexData() : _size(0), _begin(nullptr), _end(nullptr) {}
+    UniqueIndexData() : _size(0), _begin(nullptr), _end(nullptr), _keyFormat(KeyFormat::Long) {}
     UniqueIndexData(const std::string& indexData, KeyFormat keyFormat) {
         std::memcpy(&_size, indexData.data(), sizeof(uint64_t));
         _begin = reinterpret_cast<const uint8_t*>(indexData.data() + sizeof(uint64_t));
@@ -508,7 +508,7 @@ public:
                std::string _prefix,
                // This is a string immediately after the ident and before other idents.
                std::string _identEnd,
-               StringStore* workingCopy,
+               std::shared_ptr<StringStore> workingCopy,
                Ordering order,
                KeyFormat keyFormat,
                std::string prefixBSON,
@@ -518,14 +518,13 @@ public:
                                                 RequestedInfo parts = kKeyAndLoc) override;
     virtual boost::optional<KeyStringEntry> seekForKeyString(
         const KeyString::Value& keyStringValue) override;
-    virtual boost::optional<KeyStringEntry> seekExactForKeyString(
-        const KeyString::Value& keyStringValue) override;
-    virtual boost::optional<IndexKeyEntry> seekExact(const KeyString::Value& keyStringValue,
-                                                     RequestedInfo) override;
     virtual void save() override;
     virtual void restore() override;
     virtual void detachFromOperationContext() override;
     virtual void reattachToOperationContext(OperationContext* opCtx) override;
+    void setSaveStorageCursorOnDetachFromOperationContext(bool) override {
+        // Noop for EFT, since we always keep the cursor's contents valid across save/restore.
+    }
 
 private:
     // CRTP Interface
@@ -555,6 +554,11 @@ private:
     }
 
 protected:
+    // Helper function which changes the cursor to point to data in the latest snapshot from the
+    // recovery unit.  If no transaction was committed or aborted since the last call, this is a
+    // noop.
+    void advanceSnapshotIfChanged();
+
     bool advanceNext();
     // This is a helper function to check if the cursor was explicitly set by the user or not.
     bool endPosSet();
@@ -563,7 +567,9 @@ protected:
     boost::optional<KeyStringEntry> seekAfterProcessing(const KeyString::Value& keyString);
     OperationContext* _opCtx;
     // This is the "working copy" of the master "branch" in the git analogy.
-    StringStore* _workingCopy;
+    // Its ownership is split across the RecoveryUnit and associated cursors. It is not
+    // shared _between_ recovery units.
+    std::shared_ptr<StringStore> _workingCopy;
     // These store the end positions.
     boost::optional<StringStore::const_iterator> _endPos;
     boost::optional<StringStore::const_reverse_iterator> _endPosReverse;
@@ -601,13 +607,13 @@ CursorBase<CursorImpl>::CursorBase(OperationContext* opCtx,
                                    bool isForward,
                                    std::string _prefix,
                                    std::string _identEnd,
-                                   StringStore* workingCopy,
+                                   std::shared_ptr<StringStore> workingCopy,
                                    Ordering order,
                                    KeyFormat keyFormat,
                                    std::string _KSForIdentStart,
                                    std::string identEndBSON)
     : _opCtx(opCtx),
-      _workingCopy(workingCopy),
+      _workingCopy(std::move(workingCopy)),
       _endPos(boost::none),
       _endPosReverse(boost::none),
       _forward(isForward),
@@ -615,14 +621,22 @@ CursorBase<CursorImpl>::CursorBase(OperationContext* opCtx,
       _lastMoveWasRestore(false),
       _prefix(_prefix),
       _identEnd(_identEnd),
-      _forwardIt(workingCopy->begin()),
-      _reverseIt(workingCopy->rbegin()),
+      _forwardIt(_workingCopy->begin()),
+      _reverseIt(_workingCopy->rbegin()),
       _order(order),
       _keyFormat(keyFormat),
       _endPosIncl(false),
       _KSForIdentStart(_KSForIdentStart),
       _KSForIdentEnd(identEndBSON) {}
 
+template <class CursorImpl>
+void CursorBase<CursorImpl>::advanceSnapshotIfChanged() {
+    if (_workingCopy.get() != RecoveryUnit::get(_opCtx)->getHead()) {
+        save();
+        restore();
+    }
+}
+
 template <class CursorImpl>
 bool CursorBase<CursorImpl>::advanceNext() {
     if (!_atEOF) {
@@ -770,77 +784,45 @@ boost::optional<IndexKeyEntry> CursorBase<CursorImpl>::seek(const KeyString::Val
 template <class CursorImpl>
 boost::optional<KeyStringEntry> CursorBase<CursorImpl>::seekForKeyString(
     const KeyString::Value& keyStringValue) {
+    advanceSnapshotIfChanged();
+
     _lastMoveWasRestore = false;
     _atEOF = false;
     return seekAfterProcessing(keyStringValue);
 }
 
-template <class CursorImpl>
-boost::optional<KeyStringEntry> CursorBase<CursorImpl>::seekExactForKeyString(
-    const KeyString::Value& keyStringValue) {
-    dassert(KeyString::decodeDiscriminator(keyStringValue.getBuffer(),
-                                           keyStringValue.getSize(),
-                                           _order,
-                                           keyStringValue.getTypeBits()) ==
-            KeyString::Discriminator::kInclusive);
-    auto ksEntry = seekForKeyString(keyStringValue);
-    if (!ksEntry) {
-        return {};
-    }
-    auto sizeWithoutRecordId = KeyFormat::Long == _keyFormat
-        ? KeyString::sizeWithoutRecordIdLongAtEnd(ksEntry->keyString.getBuffer(),
-                                                  ksEntry->keyString.getSize())
-        : KeyString::sizeWithoutRecordIdStrAtEnd(ksEntry->keyString.getBuffer(),
-                                                 ksEntry->keyString.getSize());
-    if (KeyString::compare(ksEntry->keyString.getBuffer(),
-                           keyStringValue.getBuffer(),
-                           sizeWithoutRecordId,
-                           keyStringValue.getSize()) == 0) {
-        return KeyStringEntry(ksEntry->keyString, ksEntry->loc);
-    }
-    return {};
-}
-
-template <class CursorImpl>
-boost::optional<IndexKeyEntry> CursorBase<CursorImpl>::seekExact(
-    const KeyString::Value& keyStringValue, RequestedInfo parts) {
-    auto ksEntry = seekExactForKeyString(keyStringValue);
-    if (!ksEntry) {
-        return {};
-    }
-
-    BSONObj bson;
-    if (parts & SortedDataInterface::Cursor::kWantKey) {
-        bson = KeyString::toBson(ksEntry->keyString.getBuffer(),
-                                 ksEntry->keyString.getSize(),
-                                 _order,
-                                 ksEntry->keyString.getTypeBits());
-    }
-    return IndexKeyEntry(std::move(bson), ksEntry->loc);
-}
-
 template <class CursorImpl>
 void CursorBase<CursorImpl>::save() {
     _atEOF = false;
-    if (_lastMoveWasRestore) {
+    if (_lastMoveWasRestore || _workingCopy == nullptr) {
+        // If we just restored, or have no working copy (in which case we're already in a saved
+        // state), do nothing.
         return;
-    } else if (_forward && checkCursorValid()) {
-        _saveKey = _forwardIt->first;
+    }
+
+    // Any dereference of the _forwardIt and _reverseIt may result in them getting repositioned if
+    // the key they were pointing at was removed in our snapshot. Before accessing them
+    // (checking for validity, dereferencing etc) we save the current key they are pointing at.
+    if (_forward && _forwardIt.currentRaw()) {
+        _saveKey = _forwardIt.currentRaw()->first;
+    } else if (_reverseIt.currentRaw()) {
+        _saveKey = _reverseIt.currentRaw()->first;
+    }
+
+    if (_forward && checkCursorValid()) {
         saveForward();
     } else if (!_forward && checkCursorValid()) {  // reverse
-        _saveKey = _reverseIt->first;
         saveReverse();
     } else {
         _saveKey = "";
         _saveLoc = RecordId();
     }
+    _workingCopy = nullptr;
 }
 
 template <class CursorImpl>
 void CursorBase<CursorImpl>::restore() {
-    StringStore* workingCopy(RecoveryUnit::get(_opCtx)->getHead());
-
-    this->_workingCopy = workingCopy;
+    _workingCopy = RecoveryUnit::get(_opCtx)->getHeadShared();
 
     // Here, we have to reset the end position if one was set earlier.
     if (endPosSet()) {
@@ -852,17 +834,17 @@ void CursorBase<CursorImpl>::restore() {
     // next().
     if (_forward) {
         if (_saveKey.length() == 0) {
-            _forwardIt = workingCopy->end();
+            _forwardIt = _workingCopy->end();
         } else {
-            _forwardIt = workingCopy->lower_bound(_saveKey);
+            _forwardIt = _workingCopy->lower_bound(_saveKey);
         }
         restoreForward();
     } else {
         // Now we are dealing with reverse cursors, and use similar logic.
         if (_saveKey.length() == 0) {
-            _reverseIt = workingCopy->rend();
+            _reverseIt = _workingCopy->rend();
         } else {
-            _reverseIt = StringStore::const_reverse_iterator(workingCopy->upper_bound(_saveKey));
+            _reverseIt = StringStore::const_reverse_iterator(_workingCopy->upper_bound(_saveKey));
         }
         restoreReverse();
     }
@@ -911,7 +893,7 @@ private:
     UniqueIndexData _indexData;
     UniqueIndexData::const_iterator _indexDataIt;
     UniqueIndexData::const_iterator _indexDataEnd;
-    size_t _reversePos;
+    size_t _reversePos = 0;
 };
 
 bool CursorUnique::advanceNextInternal() {
@@ -1010,6 +992,8 @@ void CursorUnique::initReverseDataIterators() {
 }
 
 boost::optional<IndexKeyEntry> CursorUnique::next(RequestedInfo parts) {
+    advanceSnapshotIfChanged();
+
     if (!advanceNext()) {
         return {};
     }
@@ -1023,6 +1007,8 @@ boost::optional<IndexKeyEntry> CursorUnique::next(RequestedInfo parts) {
 }
 
 boost::optional<KeyStringEntry> CursorUnique::nextKeyString() {
+    advanceSnapshotIfChanged();
+
     if (!advanceNext()) {
         return {};
     }
@@ -1150,6 +1136,7 @@ private:
 // if it was set, and 3) whether the cursor is still in the ident.
 bool CursorStandard::checkCursorValid() {
     if (_forward) {
+        invariant(_workingCopy);
         if (_forwardIt == _workingCopy->end()) {
             return false;
         }
@@ -1173,6 +1160,8 @@ bool CursorStandard::checkCursorValid() {
 
 
 boost::optional<IndexKeyEntry> CursorStandard::next(RequestedInfo parts) {
+    advanceSnapshotIfChanged();
+
     if (!advanceNext()) {
         return {};
     }
@@ -1186,6 +1175,8 @@ boost::optional<IndexKeyEntry> CursorStandard::next(RequestedInfo parts) {
 }
 
 boost::optional<KeyStringEntry> CursorStandard::nextKeyString() {
+    advanceSnapshotIfChanged();
+
     if (!advanceNext()) {
         return {};
     }
@@ -1487,15 +1478,40 @@ bool SortedDataInterfaceBase::isEmpty(OperationContext* opCtx) {
                                  workingCopy->upper_bound(_KSForIdentEnd)) == 0;
 }
 
+boost::optional<RecordId> SortedDataInterfaceBase::findLoc(
+    OperationContext* opCtx, const KeyString::Value& keyStringValue) const {
+    dassert(KeyString::decodeDiscriminator(keyStringValue.getBuffer(),
+                                           keyStringValue.getSize(),
+                                           _ordering,
+                                           keyStringValue.getTypeBits()) ==
+            KeyString::Discriminator::kInclusive);
+    auto cursor = newCursor(opCtx);
+    auto ksEntry = cursor->seekForKeyString(keyStringValue);
+    if (!ksEntry) {
+        return boost::none;
+    }
+
+    auto sizeWithoutRecordId = KeyFormat::Long == _rsKeyFormat
+        ? KeyString::sizeWithoutRecordIdLongAtEnd(ksEntry->keyString.getBuffer(),
+                                                  ksEntry->keyString.getSize())
+        : KeyString::sizeWithoutRecordIdStrAtEnd(ksEntry->keyString.getBuffer(),
+                                                 ksEntry->keyString.getSize());
+    if (KeyString::compare(ksEntry->keyString.getBuffer(),
+                           keyStringValue.getBuffer(),
+                           sizeWithoutRecordId,
+                           keyStringValue.getSize()) == 0) {
+        return ksEntry->loc;
+    }
+    return boost::none;
+}
+
 std::unique_ptr<mongo::SortedDataInterface::Cursor> SortedDataInterfaceUnique::newCursor(
     OperationContext* opCtx, bool isForward) const {
-    StringStore* workingCopy(RecoveryUnit::get(opCtx)->getHead());
-
     return std::make_unique<CursorUnique>(opCtx,
                                           isForward,
                                           _prefix,
                                           _identEnd,
-                                          workingCopy,
+                                          RecoveryUnit::get(opCtx)->getHeadShared(),
                                           _ordering,
                                           _rsKeyFormat,
                                           _KSForIdentStart,
@@ -1604,13 +1620,11 @@ void SortedDataInterfaceStandard::fullValidate(OperationContext* opCtx,
 
 std::unique_ptr<mongo::SortedDataInterface::Cursor> SortedDataInterfaceStandard::newCursor(
     OperationContext* opCtx, bool isForward) const {
-    StringStore* workingCopy(RecoveryUnit::get(opCtx)->getHead());
-
     return std::make_unique<CursorStandard>(opCtx,
                                             isForward,
                                             _prefix,
                                             _identEnd,
-                                            workingCopy,
+                                            RecoveryUnit::get(opCtx)->getHeadShared(),
                                             _ordering,
                                             _rsKeyFormat,
                                             _KSForIdentStart,
