@@ -39,7 +39,6 @@
 #include "mongo/bson/timestamp.h"
 #include "mongo/db/namespace_string.h"
 #include "mongo/db/storage/durable_catalog.h"
-#include "mongo/db/storage/durable_catalog_feature_tracker.h"
 #include "mongo/db/storage/journal_listener.h"
 #include "mongo/db/storage/kv/kv_drop_pending_ident_reaper.h"
 #include "mongo/db/storage/record_store.h"
@@ -69,8 +68,6 @@ public:
 
     virtual ~StorageEngineImpl();
 
-    virtual void finishInit() override;
-
     virtual void notifyStartupComplete() override;
 
     virtual RecoveryUnit* newRecoveryUnit() override;
@@ -98,7 +95,7 @@ public:
 
     virtual void endNonBlockingBackup(OperationContext* opCtx) override;
 
-    virtual StatusWith<std::deque<std::string>> extendBackupCursor(
+    virtual StatusWith<std::vector<std::string>> extendBackupCursor(
         OperationContext* opCtx) override;
 
     virtual bool isDurable() const override;
@@ -110,10 +107,10 @@ public:
                                      const NamespaceString& nss) override;
 
     virtual std::unique_ptr<TemporaryRecordStore> makeTemporaryRecordStore(
-        OperationContext* opCtx) override;
+        OperationContext* opCtx, KeyFormat keyFormat) override;
 
     virtual std::unique_ptr<TemporaryRecordStore> makeTemporaryRecordStoreForResumableIndexBuild(
-        OperationContext* opCtx) override;
+        OperationContext* opCtx, KeyFormat keyFormat) override;
 
     virtual std::unique_ptr<TemporaryRecordStore> makeTemporaryRecordStoreFromExistingIdent(
         OperationContext* opCtx, StringData ident) override;
@@ -250,44 +247,43 @@ public:
             Callback _callback;
         };
 
-        TimestampMonitor(KVEngine* engine, PeriodicRunner* runner);
+        /**
+         * Starts monitoring timestamp changes in the background with an initial listener.
+         */
+        TimestampMonitor(KVEngine* engine, TimestampListener* listener, PeriodicRunner* runner);
+
         ~TimestampMonitor();
 
         /**
-         * Monitor changes in timestamps and to notify the listeners on change. Notifies all
-         * listeners on Timestamp::min() in order to support standalone mode that is untimestamped.
+         * Removes registered listeners from the monitor.
          */
-        void startup();
+        void clearListeners();
 
         /**
          * Adds a new listener to the monitor if it isn't already registered. A listener can only be
          * bound to one type of timestamp at a time.
          */
-        void addListener(TimestampListener* listener);
+        void addListener_forTestOnly(TimestampListener* listener);
 
         /**
-         * Removes an existing listener from the monitor if it was registered.
+         * Remove a listener.
          */
-        void removeListener(TimestampListener* listener);
+        void removeListener_forTestOnly(TimestampListener* listener);
 
         bool isRunning_forTestOnly() const {
             return _running;
         }
 
     private:
-        struct MonitoredTimestamps {
-            Timestamp checkpoint;
-            Timestamp oldest;
-            Timestamp stable;
-            Timestamp minOfCheckpointAndOldest;
-        };
+        /**
+         * Monitor changes in timestamps and to notify the listeners on change. Notifies all
+         * listeners on Timestamp::min() in order to support standalone mode that is untimestamped.
+         */
+        void _startup();
 
         KVEngine* _engine;
         bool _running;
 
-        // The set of timestamps that were last reported to the listeners by the monitor.
-        MonitoredTimestamps _currentTimestamps;
-
         // Periodic runner that the timestamp monitor schedules its job on.
         PeriodicRunner* _periodicRunner;
 
@@ -318,21 +314,19 @@ public:
                              std::shared_ptr<Ident> ident,
                              DropIdentCallback&& onDrop) override;
 
+    void startDropPendingIdentReaper() override;
+
     void checkpoint() override;
 
-    DurableCatalog* getCatalog() override {
-        return _catalog.get();
-    }
-
-    const DurableCatalog* getCatalog() const override {
-        return _catalog.get();
-    }
-
     StatusWith<ReconcileResult> reconcileCatalogAndIdents(
         OperationContext* opCtx, LastShutdownState lastShutdownState) override;
 
     std::string getFilesystemPathForDb(const std::string& dbName) const override;
 
+    DurableCatalog* getCatalog() override;
+
+    const DurableCatalog* getCatalog() const override;
+
     /**
      * When loading after an unclean shutdown, this performs cleanup on the DurableCatalogImpl.
      */
@@ -348,11 +342,6 @@ public:
         return _dropPendingIdentReaper.getAllIdentNames();
     }
 
-    Status currentFilesCompatible(OperationContext* opCtx) const override {
-        // Delegate to the FeatureTracker as to whether the data files are compatible or not.
-        return _catalog->getFeatureTracker()->isCompatibleWithCurrentCode(opCtx);
-    }
-
     int64_t sizeOnDiskForDb(OperationContext* opCtx, StringData dbName) override;
 
     bool isUsingDirectoryPerDb() const override {
@@ -368,10 +357,11 @@ public:
 
     void setPinnedOplogTimestamp(const Timestamp& pinnedTimestamp) override;
 
-    StatusWith<BSONObj> getSanitizedStorageOptionsForSecondaryReplication(
-        const BSONObj& options) const override;
+    void dump() const override;
 
 private:
+    using CollIter = std::list<std::string>::iterator;
+
     void _initCollection(OperationContext* opCtx,
                          RecordId catalogId,
                          const NamespaceString& nss,
