@@ -38,7 +38,6 @@ namespace mongo {
  */
 class StorageEngineMock : public StorageEngine {
 public:
-    void finishInit() final {}
     RecoveryUnit* newRecoveryUnit() final {
         return nullptr;
     }
@@ -78,7 +77,7 @@ public:
                       "The current storage engine doesn't support backup mode");
     }
     void endNonBlockingBackup(OperationContext* opCtx) final {}
-    StatusWith<std::deque<std::string>> extendBackupCursor(OperationContext* opCtx) final {
+    StatusWith<std::vector<std::string>> extendBackupCursor(OperationContext* opCtx) final {
         return Status(ErrorCodes::CommandNotSupported,
                       "The current storage engine doesn't support backup mode");
     }
@@ -87,11 +86,12 @@ public:
                              const NamespaceString& ns) final {
         return Status::OK();
     }
-    std::unique_ptr<TemporaryRecordStore> makeTemporaryRecordStore(OperationContext* opCtx) final {
+    std::unique_ptr<TemporaryRecordStore> makeTemporaryRecordStore(OperationContext* opCtx,
+                                                                   KeyFormat keyFormat) final {
         return {};
     }
     std::unique_ptr<TemporaryRecordStore> makeTemporaryRecordStoreForResumableIndexBuild(
-        OperationContext* opCtx) final {
+        OperationContext* opCtx, KeyFormat keyFormat) final {
         return {};
     }
     std::unique_ptr<TemporaryRecordStore> makeTemporaryRecordStoreFromExistingIdent(
@@ -169,10 +169,10 @@ public:
     void addDropPendingIdent(const Timestamp& dropTimestamp,
                              std::shared_ptr<Ident> ident,
                              DropIdentCallback&& onDrop) final {}
+    void startDropPendingIdentReaper() final {}
+
     void checkpoint() final {}
-    Status currentFilesCompatible(OperationContext* opCtx) const final {
-        return Status::OK();
-    }
+
     int64_t sizeOnDiskForDb(OperationContext* opCtx, StringData dbName) final {
         return 0;
     }
@@ -203,10 +203,7 @@ public:
 
     void setPinnedOplogTimestamp(const Timestamp& pinnedTimestamp) final {}
 
-    StatusWith<BSONObj> getSanitizedStorageOptionsForSecondaryReplication(
-        const BSONObj& options) const final {
-        return options;
-    }
+    void dump() const final {}
 };
 
 }  // namespace mongo
