@@ -64,8 +64,12 @@ struct WiredTigerFileVersion {
     // (IS_44_FCV_42). MongoDB 4.2.x always writes out IS_42.
     enum class StartupVersion { IS_42, IS_44_FCV_42, IS_44_FCV_44 };
 
+    inline static const std::string kLastLTSWTRelease = "compatibility=(release=10.0)";
+    inline static const std::string kLastContinuousWTRelease = "compatibility=(release=10.0)";
+    inline static const std::string kLatestWTRelease = "compatibility=(release=10.0)";
+
     StartupVersion _startupVersion;
-    bool shouldDowngrade(bool readOnly, bool repairMode, bool hasRecoveryTimestamp);
+    bool shouldDowngrade(bool readOnly, bool hasRecoveryTimestamp);
     std::string getDowngradeString();
 };
 
@@ -129,7 +133,8 @@ public:
     Status createRecordStore(OperationContext* opCtx,
                              StringData ns,
                              StringData ident,
-                             const CollectionOptions& options) override;
+                             const CollectionOptions& options,
+                             KeyFormat keyFormat = KeyFormat::Long) override;
 
     std::unique_ptr<RecordStore> getRecordStore(OperationContext* opCtx,
                                                 StringData ns,
@@ -137,7 +142,8 @@ public:
                                                 const CollectionOptions& options) override;
 
     std::unique_ptr<RecordStore> makeTemporaryRecordStore(OperationContext* opCtx,
-                                                          StringData ident) override;
+                                                          StringData ident,
+                                                          KeyFormat keyFormat) override;
 
     Status createSortedDataInterface(OperationContext* opCtx,
                                      const CollectionOptions& collOptions,
@@ -169,6 +175,10 @@ public:
 
     void dropIdentForImport(OperationContext* opCtx, StringData ident) override;
 
+    void alterIdentMetadata(OperationContext* opCtx,
+                            StringData ident,
+                            const IndexDescriptor* desc) override;
+
     void flushAllFiles(OperationContext* opCtx, bool callerHoldsReadLock) override;
 
     Status beginBackup(OperationContext* opCtx) override;
@@ -182,7 +192,7 @@ public:
 
     void endNonBlockingBackup(OperationContext* opCtx) override;
 
-    virtual StatusWith<std::deque<std::string>> extendBackupCursor(
+    virtual StatusWith<std::vector<std::string>> extendBackupCursor(
         OperationContext* opCtx) override;
 
     int64_t getIdentSize(OperationContext* opCtx, StringData ident) override;
@@ -354,8 +364,7 @@ public:
 
     void setPinnedOplogTimestamp(const Timestamp& pinnedTimestamp) override;
 
-    StatusWith<BSONObj> getSanitizedStorageOptionsForSecondaryReplication(
-        const BSONObj& options) const override;
+    void dump() const override;
 
 private:
     class WiredTigerSessionSweeper;
