@@ -35,6 +35,7 @@
 #include "mongo/bson/bsonobjbuilder.h"
 #include "mongo/db/catalog/collection_mock.h"
 #include "mongo/db/catalog/index_catalog_entry.h"
+#include "mongo/db/concurrency/locker_noop_client_observer.h"
 #include "mongo/db/index/index_descriptor.h"
 #include "mongo/db/json.h"
 #include "mongo/db/operation_context_noop.h"
@@ -56,17 +57,24 @@ using std::string;
 class WiredTigerIndexHarnessHelper final : public SortedDataInterfaceHarnessHelper {
 public:
     WiredTigerIndexHarnessHelper() : _dbpath("wt_test"), _conn(nullptr) {
+        auto service = getServiceContext();
+        service->registerClientObserver(std::make_unique<LockerNoopClientObserver>());
+
         const char* config = "create,cache_size=1G,";
         int ret = wiredtiger_open(_dbpath.path().c_str(), nullptr, config, &_conn);
         invariantWTOK(ret);
 
         _fastClockSource = std::make_unique<SystemClockSource>();
         _sessionCache = new WiredTigerSessionCache(_conn, _fastClockSource.get());
+
+        WiredTigerUtil::notifyStartupComplete();
     }
 
     ~WiredTigerIndexHarnessHelper() final {
         delete _sessionCache;
         _conn->close(_conn, nullptr);
+
+        WiredTigerUtil::resetTableLoggingInfo();
     }
 
     std::unique_ptr<SortedDataInterface> newIdIndexSortedDataInterface() final {
@@ -122,8 +130,8 @@ public:
         invariantWTOK(WiredTigerIndex::Create(&opCtx, uri, result.getValue()));
 
         if (unique) {
-            invariant(keyFormat == KeyFormat::Long);
-            return std::make_unique<WiredTigerIndexUnique>(&opCtx, uri, "" /* ident */, &desc);
+            return std::make_unique<WiredTigerIndexUnique>(
+                &opCtx, uri, "" /* ident */, keyFormat, &desc);
         }
         return std::make_unique<WiredTigerIndexStandard>(
             &opCtx, uri, "" /* ident */, keyFormat, &desc);
@@ -243,53 +251,5 @@ TEST(WiredTigerStandardIndexText, CursorInActiveTxnAfterSeek) {
     }
 }
 
-TEST(WiredTigerStandardIndexText, CursorInActiveTxnAfterSeekExact) {
-    auto harnessHelper = makeWTIndexHarnessHelper();
-    bool unique = false;
-    bool partial = false;
-    auto sdi = harnessHelper->newSortedDataInterface(unique, partial);
-
-    // Populate data.
-    {
-        ServiceContext::UniqueOperationContext opCtx(harnessHelper->newOperationContext());
-
-        WriteUnitOfWork uow(opCtx.get());
-        auto ks = makeKeyString(sdi.get(), BSON("" << 1), RecordId(1));
-        auto res = sdi->insert(opCtx.get(), ks, true);
-        ASSERT_OK(res);
-
-        ks = makeKeyString(sdi.get(), BSON("" << 2), RecordId(2));
-        res = sdi->insert(opCtx.get(), ks, true);
-        ASSERT_OK(res);
-
-        uow.commit();
-    }
-
-    // Cursors should always ensure they are in an active transaction when seekExact() is
-    // called.
-    {
-        ServiceContext::UniqueOperationContext opCtx(harnessHelper->newOperationContext());
-        auto ru = WiredTigerRecoveryUnit::get(opCtx.get());
-
-        auto cursor = sdi->newCursor(opCtx.get());
-
-        auto seekKs = makeKeyString(sdi.get(), BSON("" << 1));
-
-        ASSERT(cursor->seekExact(seekKs));
-        ASSERT_TRUE(ru->isActive());
-
-        // Committing a WriteUnitOfWork will end the current transaction.
-        WriteUnitOfWork wuow(opCtx.get());
-        ASSERT_TRUE(ru->isActive());
-        wuow.commit();
-        ASSERT_FALSE(ru->isActive());
-
-        // If a cursor is used after a WUOW commits, it should implicitly start a new
-        // transaction.
-        ASSERT(cursor->seekExact(seekKs));
-        ASSERT_TRUE(ru->isActive());
-    }
-}
-
 }  // namespace
 }  // namespace mongo
