@@ -37,11 +37,10 @@
 
 #include <boost/filesystem.hpp>
 #include <boost/filesystem/path.hpp>
-#include <pcrecpp.h>
 
 #include "mongo/base/simple_string_data_comparator.h"
 #include "mongo/bson/bsonobjbuilder.h"
-#include "mongo/db/concurrency/exception_util_gen.h"
+#include "mongo/db/concurrency/write_conflict_exception.h"
 #include "mongo/db/snapshot_window_options_gen.h"
 #include "mongo/db/storage/storage_file_util.h"
 #include "mongo/db/storage/wiredtiger/wiredtiger_kv_engine.h"
@@ -50,17 +49,55 @@
 #include "mongo/db/storage/wiredtiger/wiredtiger_session_cache.h"
 #include "mongo/logv2/log.h"
 #include "mongo/util/assert_util.h"
+#include "mongo/util/fail_point.h"
 #include "mongo/util/processinfo.h"
 #include "mongo/util/scopeguard.h"
-#include "mongo/util/static_immortal.h"
 #include "mongo/util/str.h"
+#include "mongo/util/testing_proctor.h"
 
 namespace mongo {
 
+MONGO_FAIL_POINT_DEFINE(crashAfterUpdatingFirstTableLoggingSettings);
+
 namespace {
 
 const std::string kTableChecksFileName = "_wt_table_checks";
 
+/**
+ * Returns true if the 'kTableChecksFileName' file exists in the dbpath.
+ *
+ * Must be called before createTableChecksFile() or removeTableChecksFile() to get accurate results.
+ */
+bool hasPreviouslyIncompleteTableChecks() {
+    auto path = boost::filesystem::path(storageGlobalParams.dbpath) /
+        boost::filesystem::path(kTableChecksFileName);
+
+    return boost::filesystem::exists(path);
+}
+
+/**
+ * Creates the 'kTableChecksFileName' file in the dbpath.
+ */
+void createTableChecksFile() {
+    auto path = boost::filesystem::path(storageGlobalParams.dbpath) /
+        boost::filesystem::path(kTableChecksFileName);
+
+    boost::filesystem::ofstream fileStream(path);
+    fileStream << "This file indicates that a WiredTiger table check operation is in progress or "
+                  "incomplete."
+               << std::endl;
+    if (fileStream.fail()) {
+        LOGV2_FATAL_NOTRACE(4366400,
+                            "Failed to write to file",
+                            "file"_attr = path.generic_string(),
+                            "error"_attr = errnoWithDescription());
+    }
+    fileStream.close();
+
+    fassertNoTrace(4366401, fsyncFile(path));
+    fassertNoTrace(4366402, fsyncParentDirectory(path));
+}
+
 /**
  * Removes the 'kTableChecksFileName' file in the dbpath, if it exists.
  */
@@ -83,23 +120,47 @@ void removeTableChecksFile() {
     }
 }
 
+void setTableWriteTimestampAssertion(WT_SESSION* session, const std::string& uri, bool on) {
+    const std::string setting = on ? "assert=(write_timestamp=on)" : "assert=(write_timestamp=off)";
+    LOGV2_DEBUG(6003700,
+                1,
+                "Changing table write timestamp assertion settings",
+                "uri"_attr = uri,
+                "writeTimestampAssertionOn"_attr = on);
+    int ret = session->alter(session, uri.c_str(), setting.c_str());
+    if (ret) {
+        LOGV2_FATAL(6003701,
+                    "Failed to update write timestamp assertion setting",
+                    "uri"_attr = uri,
+                    "writeTimestampAssertionOn"_attr = on,
+                    "error"_attr = ret,
+                    "metadata"_attr =
+                        redact(WiredTigerUtil::getMetadataCreate(session, uri).getValue()),
+                    "message"_attr = session->strerror(session, ret));
+    }
+}
+
 }  // namespace
 
 using std::string;
 
-Status wtRCToStatus_slow(int retCode, WT_SESSION* session, const char* prefix) {
+Mutex WiredTigerUtil::_tableLoggingInfoMutex =
+    MONGO_MAKE_LATCH("WiredTigerUtil::_tableLoggingInfoMutex");
+WiredTigerUtil::TableLoggingInfo WiredTigerUtil::_tableLoggingInfo;
+
+Status wtRCToStatus_slow(int retCode, StringData prefix) {
     if (retCode == 0)
         return Status::OK();
 
     if (retCode == WT_ROLLBACK) {
-        throw WriteConflictException();
+        throw WriteConflictException(prefix);
     }
 
     // Don't abort on WT_PANIC when repairing, as the error will be handled at a higher layer.
     fassert(28559, retCode != WT_PANIC || storageGlobalParams.repair);
 
     str::stream s;
-    if (prefix)
+    if (!prefix.empty())
         s << prefix << " ";
     s << retCode << ": " << wiredtiger_strerror(retCode);
 
@@ -151,12 +212,12 @@ StatusWith<std::string> _getMetadata(WT_CURSOR* cursor, StringData uri) {
         return StatusWith<std::string>(ErrorCodes::NoSuchKey,
                                        str::stream() << "Unable to find metadata for " << uri);
     } else if (ret != 0) {
-        return StatusWith<std::string>(wtRCToStatus(ret, cursor->session));
+        return StatusWith<std::string>(wtRCToStatus(ret));
     }
     const char* metadata = nullptr;
     ret = cursor->get_value(cursor, &metadata);
     if (ret != 0) {
-        return StatusWith<std::string>(wtRCToStatus(ret, cursor->session));
+        return StatusWith<std::string>(wtRCToStatus(ret));
     }
     invariant(metadata);
     return StatusWith<std::string>(metadata);
@@ -165,9 +226,9 @@ StatusWith<std::string> _getMetadata(WT_CURSOR* cursor, StringData uri) {
 
 StatusWith<std::string> WiredTigerUtil::getMetadataCreate(WT_SESSION* session, StringData uri) {
     WT_CURSOR* cursor;
-    invariantWTOK(session->open_cursor(session, "metadata:create", nullptr, "", &cursor), session);
+    invariantWTOK(session->open_cursor(session, "metadata:create", nullptr, "", &cursor));
     invariant(cursor);
-    ON_BLOCK_EXIT([cursor, session] { invariantWTOK(cursor->close(cursor), session); });
+    ON_BLOCK_EXIT([cursor] { invariantWTOK(cursor->close(cursor)); });
 
     return _getMetadata(cursor, uri);
 }
@@ -188,17 +249,18 @@ StatusWith<std::string> WiredTigerUtil::getMetadataCreate(OperationContext* opCt
         LOGV2_FATAL_NOTRACE(51257, "Cursor not found", "error"_attr = ex);
     }
     invariant(cursor);
-    auto releaser = makeGuard(
-        [&] { session->releaseCursor(WiredTigerSession::kMetadataCreateTableId, cursor, ""); });
+    ScopeGuard releaser = [&] {
+        session->releaseCursor(WiredTigerSession::kMetadataCreateTableId, cursor, "");
+    };
 
     return _getMetadata(cursor, uri);
 }
 
 StatusWith<std::string> WiredTigerUtil::getMetadata(WT_SESSION* session, StringData uri) {
     WT_CURSOR* cursor;
-    invariantWTOK(session->open_cursor(session, "metadata:", nullptr, "", &cursor), session);
+    invariantWTOK(session->open_cursor(session, "metadata:", nullptr, "", &cursor));
     invariant(cursor);
-    ON_BLOCK_EXIT([cursor, session] { invariantWTOK(cursor->close(cursor), session); });
+    ON_BLOCK_EXIT([cursor] { invariantWTOK(cursor->close(cursor)); });
 
     return _getMetadata(cursor, uri);
 }
@@ -218,8 +280,9 @@ StatusWith<std::string> WiredTigerUtil::getMetadata(OperationContext* opCtx, Str
         LOGV2_FATAL_NOTRACE(31293, "Cursor not found", "error"_attr = ex);
     }
     invariant(cursor);
-    auto releaser =
-        makeGuard([&] { session->releaseCursor(WiredTigerSession::kMetadataTableId, cursor, ""); });
+    ScopeGuard releaser = [&] {
+        session->releaseCursor(WiredTigerSession::kMetadataTableId, cursor, "");
+    };
 
     return _getMetadata(cursor, uri);
 }
@@ -272,7 +335,7 @@ Status WiredTigerUtil::getApplicationMetadata(OperationContext* opCtx,
         }
     }
     if (ret != WT_NOTFOUND) {
-        return wtRCToStatus(ret, nullptr);
+        return wtRCToStatus(ret);
     }
 
     return Status::OK();
@@ -348,7 +411,7 @@ StatusWith<int64_t> WiredTigerUtil::checkApplicationMetadataFormatVersion(Operat
 
 // static
 Status WiredTigerUtil::checkTableCreationOptions(const BSONElement& configElem) {
-    invariant(configElem.fieldNameStringData() == WiredTigerUtil::kConfigStringField);
+    invariant(configElem.fieldNameStringData() == "configString");
 
     if (configElem.type() != String) {
         return {ErrorCodes::TypeMismatch, "'configString' must be a string."};
@@ -364,8 +427,7 @@ Status WiredTigerUtil::checkTableCreationOptions(const BSONElement& configElem)
     }
 
     Status status = wtRCToStatus(
-        wiredtiger_config_validate(nullptr, &eventHandler, "WT_SESSION.create", config.rawData()),
-        nullptr);
+        wiredtiger_config_validate(nullptr, &eventHandler, "WT_SESSION.create", config.rawData()));
     if (!status.isOK()) {
         StringBuilder errorMsg;
         errorMsg << status.reason();
@@ -388,8 +450,11 @@ StatusWith<int64_t> WiredTigerUtil::getStatisticsValue(WT_SESSION* session,
     const char* cursorConfig = config.empty() ? nullptr : config.c_str();
     int ret = session->open_cursor(session, uri.c_str(), nullptr, cursorConfig, &cursor);
     if (ret != 0) {
+        // The numerical 'statisticsKey' can be located in the WT_STATS_* preprocessor macros in
+        // wiredtiger.h.
         return StatusWith<int64_t>(ErrorCodes::CursorNotFound,
                                    str::stream() << "unable to open cursor at URI " << uri
+                                                 << " for statistic: " << statisticsKey
                                                  << ". reason: " << wiredtiger_strerror(ret));
     }
     invariant(cursor);
@@ -612,7 +677,7 @@ int WiredTigerUtil::verifyTable(OperationContext* opCtx,
     // Open a new session with custom error handlers.
     WT_CONNECTION* conn = WiredTigerRecoveryUnit::get(opCtx)->getSessionCache()->conn();
     WT_SESSION* session;
-    invariantWTOK(conn->open_session(conn, &eventHandler, nullptr, &session), nullptr);
+    invariantWTOK(conn->open_session(conn, &eventHandler, nullptr, &session));
     ON_BLOCK_EXIT([&] { session->close(session, ""); });
 
     // Do the verify. Weird parens prevent treating "verify" as a macro.
@@ -620,7 +685,20 @@ int WiredTigerUtil::verifyTable(OperationContext* opCtx,
 }
 
 void WiredTigerUtil::notifyStartupComplete() {
-    removeTableChecksFile();
+    {
+        stdx::lock_guard<Latch> lk(_tableLoggingInfoMutex);
+        invariant(_tableLoggingInfo.isInitializing);
+        _tableLoggingInfo.isInitializing = false;
+    }
+
+    if (!storageGlobalParams.readOnly) {
+        removeTableChecksFile();
+    }
+}
+
+void WiredTigerUtil::resetTableLoggingInfo() {
+    stdx::lock_guard<Latch> lk(_tableLoggingInfoMutex);
+    _tableLoggingInfo = TableLoggingInfo();
 }
 
 bool WiredTigerUtil::useTableLogging(NamespaceString ns, bool replEnabled) {
@@ -641,7 +719,8 @@ bool WiredTigerUtil::useTableLogging(NamespaceString ns, bool replEnabled) {
         return false;
     }
 
-    // The remainder of local gets logged. In particular, the oplog and user created collections.
+    // The remainder of local gets logged. In particular, the oplog and user created
+    // collections.
     return true;
 }
 
@@ -658,11 +737,112 @@ Status WiredTigerUtil::setTableLogging(OperationContext* opCtx, const std::strin
 
 Status WiredTigerUtil::setTableLogging(WT_SESSION* session, const std::string& uri, bool on) {
     invariant(!storageGlobalParams.readOnly);
+    stdx::lock_guard<Latch> lk(_tableLoggingInfoMutex);
+
+    // Update the table logging settings regardless if we're no longer starting up the process.
+    if (!_tableLoggingInfo.isInitializing) {
+        return _setTableLogging(session, uri, on);
+    }
+
+    // During the start up process, the table logging settings are checked for each table to verify
+    // that they are set appropriately. We can speed this process up by assuming that the logging
+    // setting is identical for each table.
+    // We cross reference the logging settings for the first table and if it isn't correctly set, we
+    // change the logging settings for all tables during start up.
+    // In the event that the server wasn't shutdown cleanly, the logging settings will be modified
+    // for all tables as a safety precaution, or if repair mode is running.
+    if (_tableLoggingInfo.isFirstTable && hasPreviouslyIncompleteTableChecks()) {
+        _tableLoggingInfo.hasPreviouslyIncompleteTableChecks = true;
+    }
 
     if (gWiredTigerSkipTableLoggingChecksOnStartup) {
+        if (_tableLoggingInfo.hasPreviouslyIncompleteTableChecks) {
+            LOGV2_FATAL_NOTRACE(
+                5548300,
+                "Cannot use the 'wiredTigerSkipTableLoggingChecksOnStartup' startup parameter when "
+                "there are previously incomplete table checks");
+        }
+
+        // Only log this warning once.
+        if (_tableLoggingInfo.isFirstTable) {
+            _tableLoggingInfo.isFirstTable = false;
+            LOGV2_WARNING_OPTIONS(
+                5548301,
+                {logv2::LogTag::kStartupWarnings},
+                "Skipping table logging checks for all existing WiredTiger tables on startup",
+                "wiredTigerSkipTableLoggingChecksOnStartup"_attr =
+                    gWiredTigerSkipTableLoggingChecksOnStartup);
+        }
+
         LOGV2_DEBUG(5548302, 1, "Skipping table logging check", "uri"_attr = uri);
         return Status::OK();
     }
+
+    if (storageGlobalParams.repair || _tableLoggingInfo.hasPreviouslyIncompleteTableChecks) {
+        if (_tableLoggingInfo.isFirstTable) {
+            _tableLoggingInfo.isFirstTable = false;
+            if (!_tableLoggingInfo.hasPreviouslyIncompleteTableChecks) {
+                createTableChecksFile();
+            }
+
+            LOGV2(4366405,
+                  "Modifying the table logging settings for all existing WiredTiger tables",
+                  "loggingEnabled"_attr = on,
+                  "repair"_attr = storageGlobalParams.repair,
+                  "hasPreviouslyIncompleteTableChecks"_attr =
+                      _tableLoggingInfo.hasPreviouslyIncompleteTableChecks);
+        }
+
+        return _setTableLogging(session, uri, on);
+    }
+
+    if (!_tableLoggingInfo.isFirstTable) {
+        if (_tableLoggingInfo.changeTableLogging) {
+            return _setTableLogging(session, uri, on);
+        }
+
+        // The table logging settings do not need to be modified.
+        return Status::OK();
+    }
+
+    invariant(_tableLoggingInfo.isFirstTable);
+    invariant(!_tableLoggingInfo.hasPreviouslyIncompleteTableChecks);
+
+    // When repair or a forced modification to the table logging settings isn't running, check that
+    // the first table is the catalog.
+    invariant(uri == "table:_mdb_catalog", str::stream() << "First table checked was: " << uri);
+    _tableLoggingInfo.isFirstTable = false;
+
+    // Check if the first tables logging settings need to be modified.
+    const std::string setting = on ? "log=(enabled=true)" : "log=(enabled=false)";
+    const std::string existingMetadata = getMetadataCreate(session, uri).getValue();
+    if (existingMetadata.find(setting) != std::string::npos) {
+        // The table is running with the expected logging settings.
+        LOGV2(4366408,
+              "No table logging settings modifications are required for existing WiredTiger tables",
+              "loggingEnabled"_attr = on);
+        return Status::OK();
+    }
+
+    // The first table is running with the incorrect logging settings. All tables will need to have
+    // their logging settings modified.
+    _tableLoggingInfo.changeTableLogging = true;
+    createTableChecksFile();
+
+    LOGV2(4366406,
+          "Modifying the table logging settings for all existing WiredTiger tables",
+          "loggingEnabled"_attr = on);
+
+    Status status = _setTableLogging(session, uri, on);
+
+    if (MONGO_unlikely(crashAfterUpdatingFirstTableLoggingSettings.shouldFail())) {
+        LOGV2_FATAL_NOTRACE(
+            4366407, "Crashing due to 'crashAfterUpdatingFirstTableLoggingSettings' fail point");
+    }
+    return status;
+}
+
+Status WiredTigerUtil::_setTableLogging(WT_SESSION* session, const std::string& uri, bool on) {
     const std::string setting = on ? "log=(enabled=true)" : "log=(enabled=false)";
 
     // This method does some "weak" parsing to see if the table is in the expected logging
@@ -699,6 +879,15 @@ Status WiredTigerUtil::setTableLogging(WT_SESSION* session, const std::string& u
                     "message"_attr = session->strerror(session, ret));
     }
 
+    // The write timestamp assertion setting only needs to be changed at startup. It will be turned
+    // on when logging is disabled, and off when logging is enabled.
+    if (TestingProctor::instance().isEnabled()) {
+        setTableWriteTimestampAssertion(session, uri, !on);
+    } else {
+        // Disables the assertion when the testing proctor is off.
+        setTableWriteTimestampAssertion(session, uri, false /* on */);
+    }
+
     return Status::OK();
 }
 
@@ -860,9 +1049,4 @@ void WiredTigerUtil::appendSnapshotWindowSettings(WiredTigerKVEngine* engine,
     settings.append("min pinned timestamp", minPinned);
 }
 
-void WiredTigerUtil::removeEncryptionFromConfigString(std::string* configString) {
-    static const StaticImmortal<pcrecpp::RE> encryptionOptsRegex(R"re(encryption=\([^\)]*\),?)re");
-    encryptionOptsRegex->GlobalReplace("", configString);
-}
-
 }  // namespace mongo
