@@ -46,49 +46,45 @@ class WiredTigerConfigParser;
 class WiredTigerKVEngine;
 class WiredTigerSession;
 
-Status wtRCToStatus_slow(int retCode, WT_SESSION* session, const char* prefix);
+Status wtRCToStatus_slow(int retCode, StringData prefix);
 
 /**
  * converts wiredtiger return codes to mongodb statuses.
  */
-inline Status wtRCToStatus(int retCode, WT_SESSION* session, const char* prefix = nullptr) {
+inline Status wtRCToStatus(int retCode, const char* prefix = nullptr) {
     if (MONGO_likely(retCode == 0))
         return Status::OK();
 
-    return wtRCToStatus_slow(retCode, session, prefix);
+    return wtRCToStatus_slow(retCode, prefix);
 }
 
 template <typename ContextExpr>
-Status wtRCToStatus(int retCode, WT_SESSION* session, ContextExpr&& contextExpr) {
+Status wtRCToStatus(int retCode, ContextExpr&& contextExpr) {
     if (MONGO_likely(retCode == 0))
         return Status::OK();
 
-    return wtRCToStatus_slow(retCode, session, std::forward<ContextExpr>(contextExpr)());
+    return wtRCToStatus_slow(retCode, std::forward<ContextExpr>(contextExpr)());
 }
 
-inline void uassertWTOK(int ret, WT_SESSION* session) {
-    uassertStatusOK(wtRCToStatus(ret, session));
-}
-
-#define MONGO_invariantWTOK_2(expression, session)                                               \
-    do {                                                                                         \
-        int _invariantWTOK_retCode = expression;                                                 \
-        if (MONGO_unlikely(_invariantWTOK_retCode != 0)) {                                       \
-            invariantOKFailed(                                                                   \
-                #expression, wtRCToStatus(_invariantWTOK_retCode, session), __FILE__, __LINE__); \
-        }                                                                                        \
+#define MONGO_invariantWTOK_1(expression)                                               \
+    do {                                                                                \
+        int _invariantWTOK_retCode = expression;                                        \
+        if (MONGO_unlikely(_invariantWTOK_retCode != 0)) {                              \
+            invariantOKFailed(                                                          \
+                #expression, wtRCToStatus(_invariantWTOK_retCode), __FILE__, __LINE__); \
+        }                                                                               \
     } while (false)
 
-#define MONGO_invariantWTOK_3(expression, session, contextExpr)                     \
-    do {                                                                            \
-        int _invariantWTOK_retCode = expression;                                    \
-        if (MONGO_unlikely(_invariantWTOK_retCode != 0)) {                          \
-            invariantOKFailedWithMsg(#expression,                                   \
-                                     wtRCToStatus(_invariantWTOK_retCode, session), \
-                                     contextExpr,                                   \
-                                     __FILE__,                                      \
-                                     __LINE__);                                     \
-        }                                                                           \
+#define MONGO_invariantWTOK_2(expression, contextExpr)                     \
+    do {                                                                   \
+        int _invariantWTOK_retCode = expression;                           \
+        if (MONGO_unlikely(_invariantWTOK_retCode != 0)) {                 \
+            invariantOKFailedWithMsg(#expression,                          \
+                                     wtRCToStatus(_invariantWTOK_retCode), \
+                                     contextExpr,                          \
+                                     __FILE__,                             \
+                                     __LINE__);                            \
+        }                                                                  \
     } while (false)
 
 #define invariantWTOK(...) \
@@ -156,8 +152,6 @@ private:
     WiredTigerUtil();
 
 public:
-    static constexpr StringData kConfigStringField = "configString"_sd;
-
     /**
      * Fetch the type and source fields out of the colgroup metadata.  'tableUri' must be a
      * valid table: uri.
@@ -299,6 +293,8 @@ public:
 
     static void notifyStartupComplete();
 
+    static void resetTableLoggingInfo();
+
     static bool useTableLogging(NamespaceString ns, bool replEnabled);
 
     static Status setTableLogging(OperationContext* opCtx, const std::string& uri, bool on);
@@ -312,14 +308,6 @@ public:
     template <typename T>
     static T castStatisticsValue(uint64_t statisticsValue);
 
-    /**
-     * Removes encryption configuration from a config string. Should only be applied on custom
-     * config strings on secondaries. Fixes an issue where encryption configuration might be
-     * replicated to non-encrypted nodes, or nodes with different encryption options, causing
-     * initial sync or replication to fail. See SERVER-68122.
-     */
-    static void removeEncryptionFromConfigString(std::string* configString);
-
 private:
     /**
      * Casts unsigned 64-bit statistics value to T.
@@ -327,6 +315,18 @@ private:
      */
     template <typename T>
     static T _castStatisticsValue(uint64_t statisticsValue, T maximumResultType);
+
+    static Status _setTableLogging(WT_SESSION* session, const std::string& uri, bool on);
+
+    // Used to keep track of the table logging setting modifications during start up. The mutex must
+    // be held prior to accessing any of the member variables in the struct.
+    static Mutex _tableLoggingInfoMutex;
+    static struct TableLoggingInfo {
+        bool isInitializing = true;
+        bool isFirstTable = true;
+        bool changeTableLogging = false;
+        bool hasPreviouslyIncompleteTableChecks = false;
+    } _tableLoggingInfo;
 };
 
 class WiredTigerConfigParser {
@@ -336,18 +336,16 @@ class WiredTigerConfigParser {
 public:
     WiredTigerConfigParser(StringData config) {
         invariantWTOK(
-            wiredtiger_config_parser_open(nullptr, config.rawData(), config.size(), &_parser),
-            nullptr);
+            wiredtiger_config_parser_open(nullptr, config.rawData(), config.size(), &_parser));
     }
 
     WiredTigerConfigParser(const WT_CONFIG_ITEM& nested) {
         invariant(nested.type == WT_CONFIG_ITEM::WT_CONFIG_ITEM_STRUCT);
-        invariantWTOK(wiredtiger_config_parser_open(nullptr, nested.str, nested.len, &_parser),
-                      nullptr);
+        invariantWTOK(wiredtiger_config_parser_open(nullptr, nested.str, nested.len, &_parser));
     }
 
     ~WiredTigerConfigParser() {
-        invariantWTOK(_parser->close(_parser), nullptr);
+        invariantWTOK(_parser->close(_parser));
     }
 
     int next(WT_CONFIG_ITEM* key, WT_CONFIG_ITEM* value) {
