@@ -32,7 +32,6 @@
 #include "mongo/platform/basic.h"
 
 #include <fstream>
-#include <signal.h>
 #include <sstream>
 #include <string>
 #include <vector>
@@ -41,6 +40,7 @@
 #include "mongo/bson/bsonobjbuilder.h"
 #include "mongo/bson/json.h"
 #include "mongo/bson/oid.h"
+#include "mongo/db/auth/security_token.h"
 #include "mongo/logv2/bson_formatter.h"
 #include "mongo/logv2/component_settings_filter.h"
 #include "mongo/logv2/composite_backend.h"
@@ -59,10 +59,8 @@
 #include "mongo/logv2/uassert_sink.h"
 #include "mongo/platform/decimal128.h"
 #include "mongo/stdx/thread.h"
-#include "mongo/unittest/death_test.h"
 #include "mongo/unittest/temp_dir.h"
 #include "mongo/unittest/unittest.h"
-#include "mongo/util/str_escape.h"
 #include "mongo/util/string_map.h"
 #include "mongo/util/uuid.h"
 
@@ -192,7 +190,7 @@ void applyDefaultFilterToSink(SinkPtr&& sink) {
 class LogDuringInitShutdownTester {
 public:
     LogDuringInitShutdownTester() {
-        auto sink = LogCaptureBackend::create(lines);
+        auto sink = LogCaptureBackend::create(lines, true);
         applyDefaultFilterToSink(sink);
         // We have to leave this sink installed as it is not allowed to install sinks during
         // shutdown. Add a filter so it is only used during this test.
@@ -200,7 +198,7 @@ public:
         sink->set_formatter(PlainFormatter());
         boost::log::core::get()->add_sink(sink);
 
-        auto enabledGuard = makeGuard([this] { enabled = false; });
+        ScopeGuard enabledGuard([this] { enabled = false; });
         LOGV2(20001, "log during init");
         ASSERT_EQUALS(lines.back(), "log during init");
     }
@@ -220,9 +218,10 @@ class LogV2Test : public unittest::Test {
 public:
     class LineCapture {
     public:
-        LineCapture()
+        LineCapture() = delete;
+        LineCapture(bool stripEol)
             : _lines{std::make_unique<std::vector<std::string>>()},
-              _sink{LogCaptureBackend::create(*_lines)} {}
+              _sink{LogCaptureBackend::create(*_lines, stripEol)} {}
         auto& lines() {
             return *_lines;
         }
@@ -283,8 +282,8 @@ public:
     }
 
     template <typename Fmt>
-    LineCapture makeLineCapture(Fmt&& formatter) {
-        LineCapture ret;
+    LineCapture makeLineCapture(Fmt&& formatter, bool stripEol = true) {
+        LineCapture ret(stripEol);
         auto& s = ret.sink();
         applyDefaultFilterToSink(s);
         s->set_formatter(std::forward<Fmt>(formatter));
@@ -358,46 +357,15 @@ TEST_F(LogV2Test, Basic) {
     }
 }
 
-namespace bl_sinks = boost::log::sinks;
-// Sink backend which will grab a mutex, then immediately segfault.
-class ConsumeSegfaultsBackend
-    : public bl_sinks::basic_formatted_sink_backend<char, bl_sinks::synchronized_feeding> {
-public:
-    static auto create() {
-        return boost::make_shared<bl_sinks::synchronous_sink<ConsumeSegfaultsBackend>>(
-            boost::make_shared<ConsumeSegfaultsBackend>());
-    }
-
-    void consume(boost::log::record_view const& rec, string_type const& formattedString) {
-        if (firstRun) {
-            firstRun = false;
-            raise(SIGSEGV);
-        } else {
-            // Reentrance of consume(), which could cause deadlock. Exit normally, causing the death
-            // test to fail.
-            exit(0);
-        }
-    }
-
-private:
-    bool firstRun = true;
-};
-
-// Test that signals thrown during logging will not hang process death. Uses the
-// ConsumeSegfaultsBackend so that upon the initial log call, ConsumeSegfaultsBackend::consume will
-// be called, sending SIGSEGV. If the signal handler incorrectly invokes the logging subsystem, the
-// ConsumeSegfaultsBackend::consume function will be again invoked, failing the test since this
-// could result in deadlock.
-DEATH_TEST_F(LogV2Test, SIGSEGVDoesNotHang, "Got signal: ") {
-    auto sink = ConsumeSegfaultsBackend::create();
-    attachSink(sink);
-    LOGV2(6384304, "will SIGSEGV {str}", "str"_attr = "sigsegv");
-    // If we get here, we didn't segfault, and the test will fail.
-}
-
 class LogV2TypesTest : public LogV2Test {
 public:
     using LogV2Test::LogV2Test;
+    LogV2TypesTest() : LogV2Test() {
+        detail::setGetTenantIDCallback([this]() -> boost::optional<OID> { return this->tenant; });
+    }
+    ~LogV2TypesTest() {
+        detail::setGetTenantIDCallback(nullptr);
+    }
 
     // The JSON formatter should make the types round-trippable without data loss
     template <typename T>
@@ -406,13 +374,16 @@ public:
         std::istringstream json_stream(json.back());
         pt::ptree ptree;
         pt::json_parser::read_json(json_stream, ptree);
+        ASSERT_EQUALS(OID(ptree.get<std::string>("tenant")), OID(tenant));
         ASSERT_EQUALS(ptree.get<T>(std::string(kAttributesFieldName) + ".name"), expected);
     }
 
     auto lastBSONElement() {
+        ASSERT_EQUALS(BSONObj(bson.back().data()).getField("tenant").OID(), tenant);
         return BSONObj(bson.back().data()).getField(kAttributesFieldName).Obj().getField("name"_sd);
     }
 
+    OID tenant = OID::gen();
     LineCapture text = makeLineCapture(PlainFormatter());
     LineCapture json = makeLineCapture(JSONFormatter());
     LineCapture bson = makeLineCapture(BSONFormatter());
@@ -1514,54 +1485,6 @@ TEST_F(LogV2Test, JsonTruncation) {
     validateArrayTruncation(mongo::fromjson(lines.back()));
 }
 
-TEST_F(LogV2Test, StringTruncation) {
-    const AtomicWord<int32_t> maxAttributeSizeKB(1);
-    auto lines = makeLineCapture(JSONFormatter(&maxAttributeSizeKB));
-
-    std::size_t maxLength = maxAttributeSizeKB.load() << 10;
-    std::string prefix(maxLength - 3, 'a');
-
-    struct TestCase {
-        std::string input;
-        std::string suffix;
-        std::string note;
-    };
-
-    TestCase tests[] = {
-        {prefix + "LMNOPQ", "LMN", "unescaped 1-byte octet"},
-        // "\n\"NOPQ" expands to "\\n\\\"NOPQ" after escape, and the limit
-        // is reached at the 2nd '\\' octet, but since it splits the "\\\""
-        // sequence, the actual truncation happens after the 'n' octet.
-        {prefix + "\n\"NOPQ", "\n", "2-byte escape sequence"},
-        // "L\vNOPQ" expands to "L\\u000bNOPQ" after escape, and the limit
-        // is reached at the 'u' octet, so the entire sequence is truncated.
-        {prefix + "L\vNOPQ", "L", "multi-byte escape sequence"},
-        {prefix + "LM\xC3\xB1PQ", "LM", "2-byte UTF-8 sequence"},
-        {prefix + "L\xE1\x9B\x8FPQ", "L", "3-byte UTF-8 sequence"},
-        {prefix + "L\xF0\x90\x8C\xBCQ", "L", "4-byte UTF-8 sequence"},
-        {prefix + "\xE1\x9B\x8E\xE1\x9B\x8F", "\xE1\x9B\x8E", "UTF-8 codepoint boundary"},
-        // The invalid UTF-8 codepoint 0xC3 is replaced with "\\ufffd", and truncated entirely
-        {prefix + "L\xC3NOPQ", "L", "escaped invalid codepoint"},
-        {std::string(maxLength, '\\'), "\\", "escaped backslash"},
-    };
-
-    for (const auto& [input, suffix, note] : tests) {
-        LOGV2(6694001, "name", "name"_attr = input);
-        BSONObj obj = fromjson(lines.back());
-
-        auto str = obj[constants::kAttributesFieldName]["name"].checkAndGetStringData();
-        std::string context = "Failed test: " + note;
-
-        ASSERT_LTE(str.size(), maxLength) << context;
-        ASSERT(str.endsWith(suffix))
-            << context << " - string " << str << " does not end with " << suffix;
-
-        auto trunc = obj[constants::kTruncatedFieldName]["name"];
-        ASSERT_EQUALS(trunc["type"].String(), typeName(BSONType::String)) << context;
-        ASSERT_EQUALS(trunc["size"].numberLong(), str::escapeForJSON(input).size()) << context;
-    }
-}
-
 TEST_F(LogV2Test, Threads) {
     auto linesPlain = makeLineCapture(PlainFormatter());
     auto linesText = makeLineCapture(TextFormatter());
@@ -1606,13 +1529,18 @@ TEST_F(LogV2Test, Ramlog) {
     sink->set_formatter(PlainFormatter());
     attachSink(sink);
 
-    auto lines = makeLineCapture(PlainFormatter());
+    auto lines = makeLineCapture(PlainFormatter(), false);
 
     auto verifyRamLog = [&] {
         RamLog::LineIterator iter(ramlog);
-        for (const auto& s : lines.lines())
-            if (s != iter.next())
+        for (const auto& s : lines.lines()) {
+            const auto next = iter.next();
+            if (s != next) {
+                std::cout << "\n\n\n********************** s='" << s << "', next='" << next
+                          << "'\n";
                 return false;
+            }
+        }
         return true;
     };
 
@@ -1729,7 +1657,7 @@ TEST_F(LogV2Test, MultipleDomains) {
     };
     LogDomain other_domain(std::make_unique<OtherDomain>());
     std::vector<std::string> other_lines;
-    auto other_sink = LogCaptureBackend::create(other_lines);
+    auto other_sink = LogCaptureBackend::create(other_lines, true);
     other_sink->set_filter(ComponentSettingsFilter(other_domain, mgr().getGlobalSettings()));
     other_sink->set_formatter(PlainFormatter());
     attachSink(other_sink);
@@ -1796,7 +1724,7 @@ TEST_F(LogV2Test, FileLogging) {
 TEST_F(LogV2Test, UserAssert) {
     std::vector<std::string> lines;
     auto sink = wrapInSynchronousSink(wrapInCompositeBackend(
-        boost::make_shared<LogCaptureBackend>(lines), boost::make_shared<UserAssertSink>()));
+        boost::make_shared<LogCaptureBackend>(lines, true), boost::make_shared<UserAssertSink>()));
     applyDefaultFilterToSink(sink);
     sink->set_formatter(PlainFormatter());
     attachSink(sink);
