@@ -37,8 +37,6 @@
 
 #include "mongo/s/catalog_cache.h"
 
-#include <fmt/format.h>
-
 #include "mongo/bson/bsonobjbuilder.h"
 #include "mongo/db/query/collation/collator_factory_interface.h"
 #include "mongo/db/repl/optime_with.h"
@@ -46,7 +44,6 @@
 #include "mongo/s/catalog/type_collection.h"
 #include "mongo/s/catalog/type_database.h"
 #include "mongo/s/client/shard_registry.h"
-#include "mongo/s/database_version.h"
 #include "mongo/s/grid.h"
 #include "mongo/s/is_mongos.h"
 #include "mongo/s/mongod_and_mongos_server_parameters_gen.h"
@@ -76,6 +73,9 @@ const OperationContext::Decoration<bool> operationBlockedBehindCatalogCacheRefre
 
 }  // namespace
 
+AtomicWord<uint64_t> ComparableDatabaseVersion::_disambiguatingSequenceNumSource{1ULL};
+AtomicWord<uint64_t> ComparableDatabaseVersion::_forcedRefreshSequenceNumSource{1ULL};
+
 CachedDatabaseInfo::CachedDatabaseInfo(DatabaseTypeValueHandle&& dbt) : _dbt(std::move(dbt)){};
 
 DatabaseType CachedDatabaseInfo::getDatabaseType() const {
@@ -94,6 +94,70 @@ DatabaseVersion CachedDatabaseInfo::databaseVersion() const {
     return _dbt->getVersion();
 }
 
+ComparableDatabaseVersion ComparableDatabaseVersion::makeComparableDatabaseVersion(
+    const boost::optional<DatabaseVersion>& version) {
+    return ComparableDatabaseVersion(version,
+                                     _disambiguatingSequenceNumSource.fetchAndAdd(1),
+                                     _forcedRefreshSequenceNumSource.load());
+}
+
+ComparableDatabaseVersion
+ComparableDatabaseVersion::makeComparableDatabaseVersionForForcedRefresh() {
+    return ComparableDatabaseVersion(boost::none /* version */,
+                                     _disambiguatingSequenceNumSource.fetchAndAdd(1),
+                                     _forcedRefreshSequenceNumSource.addAndFetch(2) - 1);
+}
+
+void ComparableDatabaseVersion::setDatabaseVersion(const DatabaseVersion& version) {
+    _dbVersion = version;
+}
+
+BSONObj ComparableDatabaseVersion::toBSONForLogging() const {
+    BSONObjBuilder builder;
+    if (_dbVersion)
+        builder.append("dbVersion"_sd, _dbVersion->toBSON());
+    else
+        builder.append("dbVersion"_sd, "None");
+
+    builder.append("disambiguatingSequenceNum"_sd,
+                   static_cast<int64_t>(_disambiguatingSequenceNum));
+
+    builder.append("forcedRefreshSequenceNum"_sd, static_cast<int64_t>(_forcedRefreshSequenceNum));
+
+    return builder.obj();
+}
+
+bool ComparableDatabaseVersion::operator==(const ComparableDatabaseVersion& other) const {
+    if (_forcedRefreshSequenceNum != other._forcedRefreshSequenceNum)
+        return false;  // Values created on two sides of a forced refresh sequence number are always
+                       // considered different
+    if (_forcedRefreshSequenceNum == 0)
+        return true;  // Only default constructed values have _forcedRefreshSequenceNum == 0 and
+                      // they are always equal
+
+    // Relying on the boost::optional<DatabaseVersion>::operator== comparison
+    return _dbVersion == other._dbVersion;
+}
+
+bool ComparableDatabaseVersion::operator<(const ComparableDatabaseVersion& other) const {
+    if (_forcedRefreshSequenceNum < other._forcedRefreshSequenceNum)
+        return true;  // Values created on two sides of a forced refresh sequence number are always
+                      // considered different
+    if (_forcedRefreshSequenceNum > other._forcedRefreshSequenceNum)
+        return false;  // Values created on two sides of a forced refresh sequence number are always
+                       // considered different
+    if (_forcedRefreshSequenceNum == 0)
+        return false;  // Only default constructed values have _forcedRefreshSequenceNum == 0 and
+                       // they are always equal
+
+    // If both versions are valid we rely on the underlying DatabaseVersion comparison
+    if (_dbVersion && other._dbVersion)
+        return _dbVersion < other._dbVersion;
+
+    // Finally, we do a disambiguating sequence number comparison
+    return _disambiguatingSequenceNum < other._disambiguatingSequenceNum;
+}
+
 CatalogCache::CatalogCache(ServiceContext* const service, CatalogCacheLoader& cacheLoader)
     : _cacheLoader(cacheLoader),
       _executor(std::make_shared<ThreadPool>([] {
@@ -119,11 +183,13 @@ CatalogCache::~CatalogCache() {
 StatusWith<CachedDatabaseInfo> CatalogCache::getDatabase(OperationContext* opCtx,
                                                          StringData dbName,
                                                          bool allowLocks) {
-    tassert(7032313,
+    if (!allowLocks) {
+        invariant(
+            !opCtx->lockState() || !opCtx->lockState()->isLocked(),
             "Do not hold a lock while refreshing the catalog cache. Doing so would potentially "
             "hold the lock during a network call, and can lead to a deadlock as described in "
-            "SERVER-37398.",
-            allowLocks || !opCtx->lockState() || !opCtx->lockState()->isLocked());
+            "SERVER-37398.");
+    }
 
     try {
         auto dbEntry = _databaseCache.acquire(opCtx, dbName, CacheCausalConsistency::kLatestKnown);
@@ -142,11 +208,13 @@ StatusWith<ChunkManager> CatalogCache::_getCollectionRoutingInfoAt(
     const NamespaceString& nss,
     boost::optional<Timestamp> atClusterTime,
     bool allowLocks) {
-    tassert(7032314,
-            "Do not hold a lock while refreshing the catalog cache. Doing so would potentially "
-            "hold the lock during a network call, and can lead to a deadlock as described in "
-            "SERVER-37398.",
-            allowLocks || !opCtx->lockState() || !opCtx->lockState()->isLocked());
+    if (!allowLocks) {
+        invariant(!opCtx->lockState() || !opCtx->lockState()->isLocked(),
+                  "Do not hold a lock while refreshing the catalog cache. Doing so would "
+                  "potentially hold "
+                  "the lock during a network call, and can lead to a deadlock as described in "
+                  "SERVER-37398.");
+    }
 
     try {
         const auto swDbInfo = getDatabase(opCtx, nss.db(), allowLocks);
@@ -207,18 +275,19 @@ StatusWith<ChunkManager> CatalogCache::_getCollectionRoutingInfoAt(
                                     std::move(collEntry),
                                     atClusterTime);
             } catch (const DBException& ex) {
+                _stats.totalRefreshWaitTimeMicros.addAndFetch(t.micros());
                 bool isCatalogCacheRetriableError = ex.isA<ErrorCategory::SnapshotError>() ||
                     ex.code() == ErrorCodes::ConflictingOperationInProgress ||
                     ex.code() == ErrorCodes::QueryPlanKilled;
                 if (!isCatalogCacheRetriableError) {
                     return ex.toStatus();
                 }
+
                 LOGV2_FOR_CATALOG_REFRESH(4086500,
                                           0,
                                           "Collection refresh failed",
                                           "namespace"_attr = nss,
                                           "exception"_attr = redact(ex));
-                _stats.totalRefreshWaitTimeMicros.addAndFetch(t.micros());
                 acquireTries++;
                 if (acquireTries == kMaxInconsistentRoutingInfoRefreshAttempts) {
                     return ex.toStatus();
@@ -585,23 +654,66 @@ CatalogCache::CollectionCache::LookupResult CatalogCache::CollectionCache::_look
 
         auto collectionAndChunks = _catalogCacheLoader.getChunksSince(nss, lookupVersion).get();
 
+        // If a refresh doesn't find new information -> re-use the existing RoutingTableHistory
+        if (isIncremental && collectionAndChunks.changedChunks.size() == 1 &&
+            collectionAndChunks.changedChunks[0].getVersion() ==
+                existingHistory->optRt->getVersion()) {
+
+            invariant(
+                collectionAndChunks.allowMigrations == existingHistory->optRt->allowMigrations(),
+                str::stream() << "allowMigrations field of " << nss
+                              << " collection changed without changing the collection version "
+                              << existingHistory->optRt->getVersion().toString()
+                              << ". Old value: " << existingHistory->optRt->allowMigrations()
+                              << ", new value: " << collectionAndChunks.allowMigrations);
+
+
+            const auto& oldReshardingFields = existingHistory->optRt->getReshardingFields();
+            const auto& newReshardingFields = collectionAndChunks.reshardingFields;
+
+            invariant(
+                [&] {
+                    if (oldReshardingFields && newReshardingFields)
+                        return oldReshardingFields->toBSON().woCompare(
+                                   newReshardingFields->toBSON()) == 0;
+                    else
+                        return !oldReshardingFields && !newReshardingFields;
+                }(),
+                str::stream() << "reshardingFields field of " << nss
+                              << " collection changed without changing the collection version "
+                              << existingHistory->optRt->getVersion().toString()
+                              << ". Old value: " << oldReshardingFields->toBSON()
+                              << ", new value: " << newReshardingFields->toBSON());
+
+            // Despite we didn't find new info, we must update the time of this entry on the cache
+            newComparableVersion.setChunkVersion(collectionAndChunks.changedChunks[0].getVersion());
+            return LookupResult(OptionalRoutingTableHistory(existingHistory->optRt),
+                                std::move(newComparableVersion));
+        }
+
+        const auto maxChunkSize = [&]() -> boost::optional<uint64_t> {
+            if (!collectionAndChunks.allowAutoSplit) {
+                // maxChunkSize = 0 is an invalid chunkSize so we use it to detect noAutoSplit
+                // on the steady-state path in incrementChunkOnInsertOrUpdate(...)
+                return 0;
+            }
+            if (collectionAndChunks.maxChunkSizeBytes) {
+                invariant(collectionAndChunks.maxChunkSizeBytes.get() > 0);
+                return uint64_t(*collectionAndChunks.maxChunkSizeBytes);
+            }
+            return boost::none;
+        }();
+
         auto newRoutingHistory = [&] {
             // If we have routing info already and it's for the same collection epoch, we're
             // updating. Otherwise, we're making a whole new routing table.
             if (isIncremental &&
                 existingHistory->optRt->getVersion().epoch() == collectionAndChunks.epoch) {
-                if (existingHistory->optRt->getVersion().getTimestamp().is_initialized() !=
-                    collectionAndChunks.creationTime.is_initialized()) {
-                    return existingHistory->optRt
-                        ->makeUpdatedReplacingTimestamp(collectionAndChunks.creationTime)
-                        .makeUpdated(collectionAndChunks.reshardingFields,
-                                     collectionAndChunks.allowMigrations,
-                                     collectionAndChunks.changedChunks);
-                } else {
-                    return existingHistory->optRt->makeUpdated(collectionAndChunks.reshardingFields,
-                                                               collectionAndChunks.allowMigrations,
-                                                               collectionAndChunks.changedChunks);
-                }
+                return existingHistory->optRt->makeUpdated(collectionAndChunks.timeseriesFields,
+                                                           collectionAndChunks.reshardingFields,
+                                                           maxChunkSize,
+                                                           collectionAndChunks.allowMigrations,
+                                                           collectionAndChunks.changedChunks);
             }
 
             auto defaultCollator = [&]() -> std::unique_ptr<CollatorInterface> {
@@ -615,7 +727,7 @@ CatalogCache::CollectionCache::LookupResult CatalogCache::CollectionCache::_look
             }();
 
             return RoutingTableHistory::makeNew(nss,
-                                                collectionAndChunks.uuid,
+                                                *collectionAndChunks.uuid,
                                                 KeyPattern(collectionAndChunks.shardKeyPattern),
                                                 std::move(defaultCollator),
                                                 collectionAndChunks.shardKeyIsUnique,
@@ -623,6 +735,7 @@ CatalogCache::CollectionCache::LookupResult CatalogCache::CollectionCache::_look
                                                 collectionAndChunks.creationTime,
                                                 collectionAndChunks.timeseriesFields,
                                                 std::move(collectionAndChunks.reshardingFields),
+                                                maxChunkSize,
                                                 collectionAndChunks.allowMigrations,
                                                 collectionAndChunks.changedChunks);
         }();
@@ -639,31 +752,8 @@ CatalogCache::CollectionCache::LookupResult CatalogCache::CollectionCache::_look
         }
 
         const ChunkVersion newVersion = newRoutingHistory.getVersion();
-
         newComparableVersion.setChunkVersion(newVersion);
 
-        if (isIncremental && existingHistory->optRt->getVersion() == newVersion) {
-            tassert(
-                7032310,
-                fmt::format("allowMigrations field of collection '{}' changed without changing the "
-                            "collection version {}. Old value: {}, new value: {}",
-                            nss.toString(),
-                            newVersion.toString(),
-                            existingHistory->optRt->allowMigrations(),
-                            newRoutingHistory.allowMigrations()),
-                newRoutingHistory.sameAllowMigrations(*existingHistory->optRt));
-
-            tassert(
-                7032311,
-                fmt::format("reshardingFields field of collection '{}' changed without changing "
-                            "the collection version {}. Old value: {}, new value: {}",
-                            nss.toString(),
-                            newVersion.toString(),
-                            existingHistory->optRt->getReshardingFields()->toBSON().toString(),
-                            newRoutingHistory.getReshardingFields()->toBSON().toString()),
-                newRoutingHistory.sameReshardingFields(*existingHistory->optRt));
-        }
-
         LOGV2_FOR_CATALOG_REFRESH(4619901,
                                   isIncremental || newComparableVersion != previousVersion ? 0 : 1,
                                   "Refreshed cached collection",
@@ -674,7 +764,8 @@ CatalogCache::CollectionCache::LookupResult CatalogCache::CollectionCache::_look
                                   "duration"_attr = Milliseconds(t.millis()));
         _updateRefreshesStats(isIncremental, false);
 
-        return LookupResult(OptionalRoutingTableHistory(std::move(newRoutingHistory)),
+        return LookupResult(OptionalRoutingTableHistory(std::make_shared<RoutingTableHistory>(
+                                std::move(newRoutingHistory))),
                             std::move(newComparableVersion));
     } catch (const DBException& ex) {
         _stats.countFailedRefreshes.addAndFetch(1);
