@@ -55,7 +55,6 @@
 #include "mongo/db/ftdc/ftdc_mongos.h"
 #include "mongo/db/initialize_server_global_state.h"
 #include "mongo/db/kill_sessions.h"
-#include "mongo/db/lasterror.h"
 #include "mongo/db/log_process_details.h"
 #include "mongo/db/logical_session_cache_impl.h"
 #include "mongo/db/logical_time_validator.h"
@@ -80,6 +79,7 @@
 #include "mongo/s/client/sharding_connection_hook.h"
 #include "mongo/s/commands/kill_sessions_remote.h"
 #include "mongo/s/committed_optime_metadata_hook.h"
+#include "mongo/s/concurrency/locker_mongos_client_observer.h"
 #include "mongo/s/config_server_catalog_cache_loader.h"
 #include "mongo/s/grid.h"
 #include "mongo/s/is_mongos.h"
@@ -93,7 +93,6 @@
 #include "mongo/s/service_entry_point_mongos.h"
 #include "mongo/s/session_catalog_router.h"
 #include "mongo/s/sessions_collection_sharded.h"
-#include "mongo/s/sharding_egress_metadata_hook_for_mongos.h"
 #include "mongo/s/sharding_initialization.h"
 #include "mongo/s/sharding_uptime_reporter.h"
 #include "mongo/s/transaction_router.h"
@@ -284,15 +283,7 @@ void cleanupTask(const ShutdownTaskArgs& shutdownArgs) {
             quiesceTime = Milliseconds(mongosShutdownTimeoutMillisForSignaledShutdown.load());
         }
 
-        // Enter quiesce mode so that existing and new short operations are allowed to finish.
-        // At this point, we will start responding to any hello request with ShutdownInProgress
-        // so that clients can re-route their operations.
-        //
-        // TODO SERVER-49138: Remove this FCV check when 5.0 becomes last-lts.
-        if (serverGlobalParams.featureCompatibility.isVersionInitialized() &&
-                serverGlobalParams.featureCompatibility.isGreaterThanOrEqualTo(
-                    ServerGlobalParams::FeatureCompatibility::Version::kVersion47);
-            auto mongosTopCoord = MongosTopologyCoordinator::get(opCtx)) {
+        if (auto mongosTopCoord = MongosTopologyCoordinator::get(opCtx)) {
             mongosTopCoord->enterQuiesceModeAndWait(opCtx, quiesceTime);
         }
 
@@ -329,12 +320,6 @@ void cleanupTask(const ShutdownTaskArgs& shutdownArgs) {
                 LOGV2(4701800, "pauseWhileKillingOperationsAtShutdown failpoint enabled");
                 sleepsecs(1);
             }
-            FailPoint* hangBeforeInterruptfailPoint =
-                globalFailPointRegistry().find("hangBeforeCheckingMongosShutdownInterrupt");
-            if (hangBeforeInterruptfailPoint) {
-                hangBeforeInterruptfailPoint->setMode(FailPoint::Mode::off);
-                sleepsecs(3);
-            }
         }
 
         // Perform all shutdown operations after setKillAllOperations is called in order to ensure
@@ -442,8 +427,6 @@ Status initializeSharding(OperationContext* opCtx) {
             hookList->addHook(
                 std::make_unique<rpc::CommittedOpTimeMetadataHook>(opCtx->getServiceContext()));
             hookList->addHook(std::make_unique<rpc::ClientMetadataPropagationEgressHook>());
-            hookList->addHook(std::make_unique<rpc::ShardingEgressMetadataHookForMongos>(
-                opCtx->getServiceContext()));
             return hookList;
         },
         boost::none);
@@ -693,20 +676,11 @@ ExitCode runMongosServer(ServiceContext* serviceContext) {
     auto unshardedHookList = std::make_unique<rpc::EgressMetadataHookList>();
     unshardedHookList->addHook(std::make_unique<rpc::VectorClockMetadataHook>(serviceContext));
     unshardedHookList->addHook(std::make_unique<rpc::ClientMetadataPropagationEgressHook>());
-    unshardedHookList->addHook(
-        std::make_unique<rpc::ShardingEgressMetadataHookForMongos>(serviceContext));
     unshardedHookList->addHook(std::make_unique<rpc::CommittedOpTimeMetadataHook>(serviceContext));
 
     // Add sharding hooks to both connection pools - ShardingConnectionHook includes auth hooks
     globalConnPool.addHook(new ShardingConnectionHook(std::move(unshardedHookList)));
 
-    auto shardedHookList = std::make_unique<rpc::EgressMetadataHookList>();
-    shardedHookList->addHook(std::make_unique<rpc::VectorClockMetadataHook>(serviceContext));
-    shardedHookList->addHook(std::make_unique<rpc::ClientMetadataPropagationEgressHook>());
-    shardedHookList->addHook(
-        std::make_unique<rpc::ShardingEgressMetadataHookForMongos>(serviceContext));
-    shardedHookList->addHook(std::make_unique<rpc::CommittedOpTimeMetadataHook>(serviceContext));
-
     // Hook up a Listener for changes from the ReplicaSetMonitor
     // This will last for the scope of this function. i.e. until shutdown finishes
     auto shardingRSCL =
@@ -863,20 +837,13 @@ std::unique_ptr<AuthzManagerExternalState> createAuthzManagerExternalStateMongos
 ExitCode main(ServiceContext* serviceContext) {
     serviceContext->setFastClockSource(FastClockSourceFactory::create(Milliseconds{10}));
 
-    auto const shardingContext = Grid::get(serviceContext);
-
     // We either have a setting where all processes are in localhost or none are
-    std::vector<HostAndPort> configServers = mongosGlobalParams.configdbs.getServers();
-    for (std::vector<HostAndPort>::const_iterator it = configServers.begin();
-         it != configServers.end();
-         ++it) {
-        const HostAndPort& configAddr = *it;
+    const auto& configServers = mongosGlobalParams.configdbs.getServers();
+    invariant(!configServers.empty());
+    const auto allowLocalHost = configServers.front().isLocalHost();
 
-        if (it == configServers.begin()) {
-            shardingContext->setAllowLocalHost(configAddr.isLocalHost());
-        }
-
-        if (configAddr.isLocalHost() != shardingContext->allowLocalHost()) {
+    for (const auto& configServer : configServers) {
+        if (configServer.isLocalHost() != allowLocalHost) {
             LOGV2_OPTIONS(22852,
                           {LogComponent::kDefault},
                           "cannot mix localhost and ip addresses in configdbs");
@@ -909,8 +876,7 @@ MONGO_INITIALIZER_WITH_PREREQUISITES(SetFeatureCompatibilityVersionLatest,
                                      ("EndStartupOptionStorage"))
 // (Generic FCV reference): This FCV reference should exist across LTS binary versions.
 (InitializerContext* context) {
-    serverGlobalParams.mutableFeatureCompatibility.setVersion(
-        ServerGlobalParams::FeatureCompatibility::kLatest);
+    serverGlobalParams.mutableFeatureCompatibility.setVersion(multiversion::GenericFCV::kLatest);
 }
 
 #ifdef MONGO_CONFIG_SSL
@@ -943,7 +909,10 @@ ExitCode mongos_main(int argc, char* argv[]) {
     }
 
     try {
-        setGlobalServiceContext(ServiceContext::make());
+        auto serviceContextHolder = ServiceContext::make();
+        serviceContextHolder->registerClientObserver(
+            std::make_unique<LockerMongosClientObserver>());
+        setGlobalServiceContext(std::move(serviceContextHolder));
     } catch (...) {
         auto cause = exceptionToStatus();
         LOGV2_FATAL_OPTIONS(
@@ -955,6 +924,7 @@ ExitCode mongos_main(int argc, char* argv[]) {
         return EXIT_ABRUPT;
     }
 
+    audit::rotateAuditLog();
     registerShutdownTask(cleanupTask);
 
     const auto service = getGlobalServiceContext();
