@@ -1,4 +1,3 @@
-
 /**
  *    Copyright (C) 2018-present MongoDB, Inc.
  *
@@ -51,26 +50,43 @@ class BalanceChunkRequest {
 public:
     /**
      * Parses the provided BSON content and if it is correct construct a request object with the
-     * request parameters.
+     * request parameters. If the '_id' field is missing in obj, ignore it.
      */
-    static StatusWith<BalanceChunkRequest> parseFromConfigCommand(const BSONObj& obj);
+    // TODO (SERVER-60792): Get rid of "requireUUID" once v6.0 branches out. Starting from v5.1, the
+    // collection UUID will always be present in the chunk.
+    static StatusWith<BalanceChunkRequest> parseFromConfigCommand(const BSONObj& obj,
+                                                                  bool requireUUID = true);
 
     /**
      * Produces a BSON object for the variant of the command, which requests the balancer to move a
      * chunk to a user-specified shard.
      */
     static BSONObj serializeToMoveCommandForConfig(
+        const NamespaceString& nss,
         const ChunkType& chunk,
         const ShardId& newShardId,
         int64_t maxChunkSizeBytes,
         const MigrationSecondaryThrottleOptions& secondaryThrottle,
-        bool waitForDelete);
+        bool waitForDelete,
+        bool forceJumbo);
 
     /**
      * Produces a BSON object for the variant of the command, which requests the balancer to pick a
      * better location for a chunk.
      */
-    static BSONObj serializeToRebalanceCommandForConfig(const ChunkType& chunk);
+    static BSONObj serializeToRebalanceCommandForConfig(const NamespaceString& nss,
+                                                        const ChunkType& chunk);
+
+
+    const NamespaceString& getNss() const {
+        return _nss;
+    }
+
+    // TODO (SERVER-60792): Get rid of setCollectionUUID() once v6.0 branches out. Starting from
+    // v5.1, the collection UUID will always be present in the chunk.
+    void setCollectionUUID(UUID const& uuid) {
+        _chunk.setCollectionUUID(uuid);
+    }
 
     const ChunkType& getChunk() const {
         return _chunk;
@@ -96,9 +112,15 @@ public:
         return _waitForDelete;
     }
 
+    bool getForceJumbo() const {
+        return _forceJumbo;
+    }
+
 private:
     BalanceChunkRequest(ChunkType chunk, MigrationSecondaryThrottleOptions secondaryThrottle);
 
+    NamespaceString _nss;
+
     // Complete description of the chunk to be manipulated
     ChunkType _chunk;
 
@@ -115,6 +137,8 @@ private:
     // Whether to block and wait for the range deleter to cleanup the orphaned documents at the end
     // of move.
     bool _waitForDelete;
+
+    bool _forceJumbo;
 };
 
 }  // namespace mongo
