@@ -1,6 +1,6 @@
 // mongo.js
 
-// Defined in mongo.cpp
+// Defined in mongojs.cpp
 
 if (!Mongo.prototype) {
     throw Error("Mongo.prototype not defined");
@@ -39,9 +39,7 @@ Mongo.prototype.getSlaveOk = function() {
     return this.getSecondaryOk();
 };
 
-Mongo.prototype.setSecondaryOk = function(value) {
-    if (value == undefined)
-        value = true;
+Mongo.prototype.setSecondaryOk = function(value = true) {
     this.secondaryOk = value;
 };
 
@@ -161,6 +159,18 @@ Mongo.prototype.adminCommand = function(cmd) {
     return this.getDB("admin").runCommand(cmd);
 };
 
+Mongo.prototype._setSecurityToken = function(token) {
+    this._securityToken = token;
+};
+
+Mongo.prototype.runCommand = function(dbname, cmd, options) {
+    return this._runCommandImpl(dbname, cmd, options, this._securityToken);
+};
+
+Mongo.prototype.runCommandWithMetadata = function(dbname, metadata, cmd) {
+    return this._runCommandWithMetadataImpl(dbname, metadata, cmd, this._securityToken);
+};
+
 /**
  * Returns all log components and current verbosity values
  */
@@ -320,7 +330,7 @@ Mongo.prototype.getReadConcern = function() {
     return this._readConcernLevel;
 };
 
-connect = function(url, user, pass) {
+connect = function(url, user, pass, apiParameters) {
     if (url instanceof MongoURI) {
         user = url.user;
         pass = url.password;
@@ -371,7 +381,7 @@ connect = function(url, user, pass) {
     }
     chatty("connecting to: " + safeURL);
     try {
-        var m = new Mongo(url);
+        var m = new Mongo(url, undefined /* encryptedDBClientCallback */, apiParameters);
     } catch (e) {
         var dest;
         if (url.indexOf(".query.mongodb.net") != -1) {
@@ -382,7 +392,7 @@ connect = function(url, user, pass) {
 
         if (dest) {
             print(`\n\n*** You have failed to connect to a ${dest}. Please ensure` +
-                  " that your IP whitelist allows connections from your network.\n\n");
+                  " that your IP allowlist allows connections from your network.\n\n");
         }
 
         throw e;
@@ -420,126 +430,6 @@ connect = function(url, user, pass) {
     return db;
 };
 
-/**
- * deprecated, use writeMode below
- *
- */
-Mongo.prototype.useWriteCommands = function() {
-    return (this.writeMode() != "legacy");
-};
-
-Mongo.prototype.forceWriteMode = function(mode) {
-    this._writeMode = mode;
-};
-
-Mongo.prototype.hasWriteCommands = function() {
-    var hasWriteCommands = (this.getMinWireVersion() <= 2 && 2 <= this.getMaxWireVersion());
-    return hasWriteCommands;
-};
-
-Mongo.prototype.hasExplainCommand = function() {
-    var hasExplain = (this.getMinWireVersion() <= 3 && 3 <= this.getMaxWireVersion());
-    return hasExplain;
-};
-
-/**
- * {String} Returns the current mode set. Will be commands/legacy/compatibility
- *
- * Sends isMaster to determine if the connection is capable of using bulk write operations, and
- * caches the result.
- */
-
-Mongo.prototype.writeMode = function() {
-    if ('_writeMode' in this) {
-        return this._writeMode;
-    }
-
-    // get default from shell params
-    if (_writeMode)
-        this._writeMode = _writeMode();
-
-    // can't use "commands" mode unless server version is good.
-    if (this.hasWriteCommands()) {
-        // good with whatever is already set
-    } else if (this._writeMode == "commands") {
-        this._writeMode = "compatibility";
-    }
-
-    return this._writeMode;
-};
-
-/**
- * Returns true if the shell is configured to use find/getMore commands rather than the C++ client.
- *
- * Currently, the C++ client will always use OP_QUERY find and OP_GET_MORE.
- */
-Mongo.prototype.useReadCommands = function() {
-    return (this.readMode() === "commands");
-};
-
-/**
- * For testing, forces the shell to use the readMode specified in 'mode'. Must be either "commands"
- * (use the find/getMore commands), "legacy" (use legacy OP_QUERY/OP_GET_MORE wire protocol reads),
- * or "compatibility" (auto-detect mode based on wire version).
- */
-Mongo.prototype.forceReadMode = function(mode) {
-    if (mode !== "commands" && mode !== "compatibility" && mode !== "legacy") {
-        throw new Error("Mode must be one of {commands, compatibility, legacy}, but got: " + mode);
-    }
-
-    this._readMode = mode;
-};
-
-/**
- * Get the readMode string (either "commands" for find/getMore commands, "legacy" for OP_QUERY find
- * and OP_GET_MORE, or "compatibility" for detecting based on wire version).
- */
-Mongo.prototype.readMode = function() {
-    // Get the readMode from the shell params if we don't have one yet.
-    if (typeof _readMode === "function" && !this.hasOwnProperty("_readMode")) {
-        this._readMode = _readMode();
-    }
-
-    if (this.hasOwnProperty("_readMode") && this._readMode !== "compatibility") {
-        // We already have determined our read mode. Just return it.
-        return this._readMode;
-    } else {
-        // We're in compatibility mode. Determine whether the server supports the find/getMore
-        // commands. If it does, use commands mode. If not, degrade to legacy mode.
-        try {
-            var hasReadCommands = (this.getMinWireVersion() <= 4 && 4 <= this.getMaxWireVersion());
-            if (hasReadCommands) {
-                this._readMode = "commands";
-            } else {
-                this._readMode = "legacy";
-            }
-        } catch (e) {
-            // We failed trying to determine whether the remote node supports the find/getMore
-            // commands. In this case, we keep _readMode as "compatibility" and the shell should
-            // issue legacy reads. Next time around we will issue another isMaster to try to
-            // determine the readMode decisively.
-        }
-    }
-
-    return this._readMode;
-};
-
-/**
- * Run a function while forcing a certain readMode, and then return the readMode to its original
- * setting afterwards. Passes this connection to the given function, and returns the function's
- * result.
- */
-Mongo.prototype._runWithForcedReadMode = function(forcedReadMode, fn) {
-    let origReadMode = this.readMode();
-    this.forceReadMode(forcedReadMode);
-    try {
-        var res = fn(this);
-    } finally {
-        this.forceReadMode(origReadMode);
-    }
-    return res;
-};
-
 //
 // Write Concern can be set at the connection level, and is used for all write operations unless
 // overridden at the collection level.
@@ -593,9 +483,9 @@ Mongo.prototype.startSession = function startSession(options = {}) {
 
     // Only log this message if we are running a test
     if (typeof TestData === "object" && TestData.testName) {
-        jsTest.log("New session started with sessionID: " +
-                   tojsononeline(newDriverSession.getSessionId()) +
-                   " and options: " + tojsononeline(options));
+        print("New session started with sessionID: " +
+              tojsononeline(newDriverSession.getSessionId()) +
+              " and options: " + tojsononeline(options));
     }
 
     return newDriverSession;
@@ -699,6 +589,16 @@ Mongo.prototype._extractChangeStreamOptions = function(options) {
         delete options.allChangesForCluster;
     }
 
+    if (options.hasOwnProperty("allowToRunOnConfigDB")) {
+        changeStreamOptions.allowToRunOnConfigDB = options.allowToRunOnConfigDB;
+        delete options.allowToRunOnConfigDB;
+    }
+
+    if (options.hasOwnProperty("allowToRunOnSystemNS")) {
+        changeStreamOptions.allowToRunOnSystemNS = options.allowToRunOnSystemNS;
+        delete options.allowToRunOnSystemNS;
+    }
+
     return [{$changeStream: changeStreamOptions}, options];
 };
 
