@@ -53,8 +53,6 @@
  *       rs: same as above
  *       chunkSize: same as above
  *       keyFile {string}: the location of the keyFile
- *       shardAsReplicaSet {boolean}: if true, start shards as 2 node replica sets. default
- *          is true.
  *
  *       shardOptions {Object}: same as the shards property above.
  *          Can be used to specify options that are common all shards.
@@ -95,11 +93,10 @@
  * s {Mongo} - connection to the first mongos
  * s0, s1, ... {Mongo} - connection to different mongos
  * rs0, rs1, ... {ReplSetTest} - test objects to replica sets
- * shard0, shard1, ... {Mongo} - connection to shards (not available for replica sets)
- * d0, d1, ... {Mongo} - same as shard0, shard1, ...
+ * shard0, shard1, ... {Mongo} - connection to shards
  * config0, config1, ... {Mongo} - connection to config servers
  * c0, c1, ... {Mongo} - same as config0, config1, ...
- * configRS - If the config servers are a replset, this will contain the config ReplSetTest object
+ * configRS - the config ReplSetTest object
  */
 var ShardingTest = function(params) {
     if (!(this instanceof ShardingTest)) {
@@ -112,14 +109,13 @@ var ShardingTest = function(params) {
     // Used for counting the test duration
     var _startTime = new Date();
 
-    // Populated with the paths of all shard hosts (config servers + hosts) and is used for
-    // cleaning up the data files on shutdown
-    var _alldbpaths = [];
-
     // Timeout to be used for operations scheduled by the sharding test, which must wait for write
     // concern (5 minutes)
     var kDefaultWTimeoutMs = 5 * 60 * 1000;
 
+    // Ensure we don't mutate the passed-in parameters.
+    params = Object.extend({}, params, true);
+
     // Publicly exposed variables
 
     /**
@@ -393,25 +389,14 @@ var ShardingTest = function(params) {
         }
     };
 
-    this.stopAllConfigServers = function(opts) {
-        if (this.configRS) {
-            this.configRS.stopSet(undefined, undefined, opts);
-        } else {
-            // Old style config triplet
-            for (var i = 0; i < this._configServers.length; i++) {
-                this.stopConfigServer(i, opts);
-            }
-        }
+    this.stopAllConfigServers = function(opts, forRestart = undefined) {
+        this.configRS.stopSet(undefined, forRestart, opts);
     };
 
-    this.stopAllShards = function(opts) {
-        for (var i = 0; i < this._connections.length; i++) {
-            if (this._rs[i]) {
-                this._rs[i].test.stopSet(15, undefined, opts);
-            } else {
-                this.stopMongod(i, opts);
-            }
-        }
+    this.stopAllShards = function(opts, forRestart = undefined) {
+        this._rs.forEach((rs) => {
+            rs.test.stopSet(15, forRestart, opts);
+        });
     };
 
     this.stopAllMongos = function(opts) {
@@ -441,12 +426,6 @@ var ShardingTest = function(params) {
               this._connections.length + " shards.");
 
         this.stopAllConfigServers(opts);
-        if (!opts.noCleanData) {
-            print("ShardingTest stop deleting all dbpaths");
-            for (var i = 0; i < _alldbpaths.length; i++) {
-                resetDbpath(MongoRunner.dataPath + _alldbpaths[i]);
-            }
-        }
 
         var timeMillis = new Date().getTime() - _startTime.getTime();
 
@@ -454,12 +433,6 @@ var ShardingTest = function(params) {
               (timeMillis / 1000) + " seconds ***");
     };
 
-    this.getDBPaths = function() {
-        return _alldbpaths.map((path) => {
-            return MongoRunner.dataPath + path;
-        });
-    };
-
     this.adminCommand = function(cmd) {
         var res = this.admin.runCommand(cmd);
         if (res && res.ok == 1)
@@ -468,6 +441,22 @@ var ShardingTest = function(params) {
         throw _getErrorWithCode(res, "command " + tojson(cmd) + " failed: " + tojson(res));
     };
 
+    this.restartAllConfigServers = function(opts) {
+        this.configRS.startSet(opts, true);
+    };
+
+    this.restartAllShards = function(opts) {
+        this._rs.forEach((rs) => {
+            rs.test.startSet(opts, true);
+        });
+    };
+
+    this.restartAllMongos = function(opts) {
+        for (var i = 0; i < this._mongos.length; i++) {
+            this.restartMongos(i, opts);
+        }
+    };
+
     this.forEachConnection = function(fn) {
         this._connections.forEach(function(conn) {
             fn(conn);
@@ -481,7 +470,7 @@ var ShardingTest = function(params) {
     };
 
     this.forEachConfigServer = function(fn) {
-        this._configServers.forEach(function(conn) {
+        this.configRS.nodes.forEach(function(conn) {
             fn(conn);
         });
     };
@@ -836,28 +825,11 @@ var ShardingTest = function(params) {
         }
     };
 
-    /**
-     * Kills the shard mongod with index n.
-     */
-    this.stopMongod = function(n, opts) {
-        if (otherParams.useBridge) {
-            MongoRunner.stopMongod(unbridgedConnections[n], undefined, opts);
-            this["d" + n].stop();
-        } else {
-            MongoRunner.stopMongod(this["d" + n], undefined, opts);
-        }
-    };
-
     /**
      * Kills the config server mongod with index n.
      */
     this.stopConfigServer = function(n, opts) {
-        if (otherParams.useBridge) {
-            MongoRunner.stopMongod(unbridgedConfigServers[n], undefined, opts);
-            this._configServers[n].stop();
-        } else {
-            MongoRunner.stopMongod(this._configServers[n], undefined, opts);
-        }
+        this.configRS.stop(n, undefined, opts);
     };
 
     /**
@@ -929,71 +901,6 @@ var ShardingTest = function(params) {
         }
     };
 
-    /**
-     * Stops and restarts a shard mongod process.
-     *
-     * If opts is specified, the new mongod is started using those options. Otherwise, it is started
-     * with its previous parameters. The 'beforeRestartCallback' parameter is an optional function
-     * that will be run after the MongoD is stopped, but before it is restarted. The intended uses
-     * of the callback are modifications to the dbpath of the mongod that must be made while it is
-     * stopped.
-     *
-     * Warning: Overwrites the old dn/shardn member variables.
-     */
-    this.restartMongod = function(n, opts, beforeRestartCallback) {
-        var mongod;
-        if (otherParams.useBridge) {
-            mongod = unbridgedConnections[n];
-        } else {
-            mongod = this["d" + n];
-        }
-
-        opts = opts || mongod;
-        opts.port = opts.port || mongod.port;
-
-        this.stopMongod(n);
-
-        if (otherParams.useBridge) {
-            var bridgeOptions =
-                (opts !== mongod) ? opts.bridgeOptions : mongod.fullOptions.bridgeOptions;
-            bridgeOptions = Object.merge(otherParams.bridgeOptions, bridgeOptions || {});
-            bridgeOptions = Object.merge(bridgeOptions, {
-                hostName: otherParams.useHostname ? hostName : "localhost",
-                port: this._connections[n].port,
-                // The mongod processes identify themselves to mongobridge as host:port, where the
-                // host is the actual hostname of the machine and not localhost.
-                dest: hostName + ":" + opts.port,
-            });
-
-            this._connections[n] = new MongoBridge(bridgeOptions);
-        }
-
-        if (arguments.length >= 3) {
-            if (typeof (beforeRestartCallback) !== "function") {
-                throw new Error("beforeRestartCallback must be a function but was of type " +
-                                typeof (beforeRestartCallback));
-            }
-            beforeRestartCallback();
-        }
-
-        opts.restart = true;
-
-        var newConn = MongoRunner.runMongod(opts);
-        if (!newConn) {
-            throw new Error("Failed to restart shard " + n);
-        }
-
-        if (otherParams.useBridge) {
-            this._connections[n].connectToBridge();
-            unbridgedConnections[n] = newConn;
-        } else {
-            this._connections[n] = newConn;
-        }
-
-        this["shard" + n] = this._connections[n];
-        this["d" + n] = this._connections[n];
-    };
-
     /**
      * Restarts each node in a particular shard replica set using the shard's original startup
      * options by default.
@@ -1024,46 +931,10 @@ var ShardingTest = function(params) {
      *
      * Warning: Overwrites the old cn/confign member variables.
      */
-    this.restartConfigServer = function(n) {
-        var mongod;
-
-        if (otherParams.useBridge) {
-            mongod = unbridgedConfigServers[n];
-        } else {
-            mongod = this["c" + n];
-        }
-
-        this.stopConfigServer(n);
-
-        if (otherParams.useBridge) {
-            var bridgeOptions =
-                Object.merge(otherParams.bridgeOptions, mongod.fullOptions.bridgeOptions || {});
-            bridgeOptions = Object.merge(bridgeOptions, {
-                hostName: otherParams.useHostname ? hostName : "localhost",
-                port: this._configServers[n].port,
-                // The mongod processes identify themselves to mongobridge as host:port, where the
-                // host is the actual hostname of the machine and not localhost.
-                dest: hostName + ":" + mongod.port,
-            });
-
-            this._configServers[n] = new MongoBridge(bridgeOptions);
-        }
-
-        mongod.restart = true;
-        var newConn = MongoRunner.runMongod(mongod);
-        if (!newConn) {
-            throw new Error("Failed to restart config server " + n);
-        }
-
-        if (otherParams.useBridge) {
-            this._configServers[n].connectToBridge();
-            unbridgedConfigServers[n] = newConn;
-        } else {
-            this._configServers[n] = newConn;
-        }
-
-        this["config" + n] = this._configServers[n];
-        this["c" + n] = this._configServers[n];
+    this.restartConfigServer = function(n, options, signal, wait) {
+        this.configRS.restart(n, options, signal, wait);
+        this["config" + n] = this.configRS.nodes[n];
+        this["c" + n] = this.configRS.nodes[n];
     };
 
     /**
@@ -1103,12 +974,12 @@ var ShardingTest = function(params) {
             // Must check shardMixedBinVersion because it causes shardOptions.binVersion to be an
             // object (versionIterator) rather than a version string. Must check mongosBinVersion,
             // as well, because it does not update mongosOptions.binVersion.
-            // TODO SERVER-50389: Differentiate between 'last-lts' and 'last-continuous' when
-            // last-continuous is supported with shardMixedBinVersions.
-            if ((MongoRunner.areBinVersionsTheSame(binVersion, "last-lts") &&
-                 jsTestOptions().shardMixedBinVersions) ||
-                (jsTestOptions().mongosBinVersion &&
-                 MongoRunner.areBinVersionsTheSame(binVersion, jsTestOptions().mongosBinVersion))) {
+            const isMixedVersionShard = jsTestOptions().shardMixedBinVersions &&
+                MongoRunner.areBinVersionsTheSame(binVersion,
+                                                  jsTestOptions().shardMixedBinVersions);
+            const isMixedVersionMongos = jsTestOptions().mongosBinVersion &&
+                MongoRunner.areBinVersionsTheSame(binVersion, jsTestOptions().mongosBinVersion);
+            if (isMixedVersionShard || isMixedVersionMongos) {
                 return true;
             }
 
@@ -1185,13 +1056,7 @@ var ShardingTest = function(params) {
      * Returns the total number of mongod nodes across all shards, excluding config server nodes.
      * Used only for diagnostic logging.
      */
-    function totalNumShardNodes(shardsAsReplSets) {
-        // Standalone mongod shards.
-        if (!shardsAsReplSets) {
-            return self._connections.length;
-        }
-
-        // Replica set shards.
+    function totalNumShardNodes() {
         const numNodesPerReplSet = self._rs.map(r => r.test.nodes.length);
         return numNodesPerReplSet.reduce((a, b) => a + b, 0);
     }
@@ -1207,8 +1072,10 @@ var ShardingTest = function(params) {
     var mongosVerboseLevel = otherParams.hasOwnProperty('verbose') ? otherParams.verbose : 1;
     var numMongos = otherParams.hasOwnProperty('mongos') ? otherParams.mongos : 1;
     var numConfigs = otherParams.hasOwnProperty('config') ? otherParams.config : 3;
-    var startShardsAsRS =
-        otherParams.hasOwnProperty('shardAsReplicaSet') ? otherParams.shardAsReplicaSet : true;
+
+    if ("shardAsReplicaSet" in otherParams) {
+        throw new Error("Use of deprecated option 'shardAsReplicaSet'");
+    }
 
     // Default enableBalancer to false.
     otherParams.enableBalancer =
@@ -1294,8 +1161,6 @@ var ShardingTest = function(params) {
     this._rs = [];
     this._rsObjects = [];
 
-    let unbridgedConnections;
-    let unbridgedConfigServers;
     let unbridgedMongos;
     let _makeAllocatePortFn;
     let _allocatePortForMongos;
@@ -1304,8 +1169,6 @@ var ShardingTest = function(params) {
     let _allocatePortForBridgeForShard;
 
     if (otherParams.useBridge) {
-        unbridgedConnections = [];
-        unbridgedConfigServers = [];
         unbridgedMongos = [];
 
         _makeAllocatePortFn = (preallocatedPorts, errorMessage) => {
@@ -1329,8 +1192,7 @@ var ShardingTest = function(params) {
         _allocatePortForMongos =
             _makeAllocatePortFn(allocatePorts(MongoBridge.kBridgeOffset), errorMessage);
 
-        errorMessage = (length) =>
-            "Cannot use more than " + length + " stand-alone shards when useBridge=true";
+        errorMessage = (length) => "Cannot use more than " + length + " shards when useBridge=true";
         _allocatePortForBridgeForShard =
             _makeAllocatePortFn(allocatePorts(MongoBridge.kBridgeOffset), errorMessage);
         _allocatePortForShard =
@@ -1355,127 +1217,43 @@ var ShardingTest = function(params) {
         randomSeedAlreadySet = true;
     }
 
-    // Should we start up shards as replica sets.
-    const shardsAsReplSets = (otherParams.rs || otherParams["rs" + i] || startShardsAsRS);
-
     //
-    // Start each shard replica set or standalone mongod.
+    // Start each shard replica set.
     //
     let startTime = new Date();  // Measure the execution time of startup and initiate.
     for (var i = 0; i < numShards; i++) {
-        if (shardsAsReplSets) {
-            var setName = testName + "-rs" + i;
+        var setName = testName + "-rs" + i;
 
-            var rsDefaults = {
-                useHostname: otherParams.useHostname,
-                oplogSize: 16,
-                shardsvr: '',
-                pathOpts: Object.merge(pathOpts, {shard: i}),
-            };
+        var rsDefaults = {
+            useHostname: otherParams.useHostname,
+            oplogSize: 16,
+            shardsvr: '',
+            pathOpts: Object.merge(pathOpts, {shard: i}),
+        };
 
-            if (otherParams.rs || otherParams["rs" + i]) {
-                if (otherParams.rs) {
-                    rsDefaults = Object.merge(rsDefaults, otherParams.rs);
-                }
-                if (otherParams["rs" + i]) {
-                    rsDefaults = Object.merge(rsDefaults, otherParams["rs" + i]);
-                }
-                rsDefaults = Object.merge(rsDefaults, otherParams.rsOptions);
-                rsDefaults.nodes = rsDefaults.nodes || otherParams.numReplicas;
+        if (otherParams.rs || otherParams["rs" + i]) {
+            if (otherParams.rs) {
+                rsDefaults = Object.merge(rsDefaults, otherParams.rs);
             }
-
-            if (startShardsAsRS && !(otherParams.rs || otherParams["rs" + i])) {
-                if (jsTestOptions().shardMixedBinVersions) {
-                    if (!otherParams.shardOptions) {
-                        otherParams.shardOptions = {};
-                    }
-                    // If the test doesn't depend on specific shard binVersions, create a mixed
-                    // version
-                    // shard cluster that randomly assigns shard binVersions, half "latest" and half
-                    // "last-lts".
-                    // TODO SERVER-50389: Support last-continuous binary version with
-                    // shardMixedBinVersions.
-                    if (!otherParams.shardOptions.binVersion) {
-                        Random.setRandomSeed();
-                        otherParams.shardOptions.binVersion =
-                            MongoRunner.versionIterator(["latest", "last-lts"], true);
-                    }
-                }
-
-                if (otherParams.shardOptions && otherParams.shardOptions.binVersion) {
-                    otherParams.shardOptions.binVersion =
-                        MongoRunner.versionIterator(otherParams.shardOptions.binVersion);
-                }
-
-                rsDefaults = Object.merge(rsDefaults, otherParams["d" + i]);
-                rsDefaults = Object.merge(rsDefaults, otherParams.shardOptions);
+            if (otherParams["rs" + i]) {
+                rsDefaults = Object.merge(rsDefaults, otherParams["rs" + i]);
             }
-
-            rsDefaults.setParameter = rsDefaults.setParameter || {};
-            rsDefaults.setParameter.migrationLockAcquisitionMaxWaitMS =
-                otherParams.migrationLockAcquisitionMaxWaitMS;
-
-            var rsSettings = rsDefaults.settings;
-            delete rsDefaults.settings;
-
-            // If both rs and startShardsAsRS are specfied, the number of nodes
-            // in the rs field should take priority.
-            if (otherParams.rs || otherParams["rs" + i]) {
-                var numReplicas = rsDefaults.nodes || 3;
-            } else if (startShardsAsRS) {
-                var numReplicas = 1;
-            }
-            delete rsDefaults.nodes;
-
-            var protocolVersion = rsDefaults.protocolVersion;
-            delete rsDefaults.protocolVersion;
-
-            var rs = new ReplSetTest({
-                name: setName,
-                nodes: numReplicas,
-                host: hostName,
-                useHostName: otherParams.useHostname,
-                useBridge: otherParams.useBridge,
-                bridgeOptions: otherParams.bridgeOptions,
-                keyFile: this.keyFile,
-                protocolVersion: protocolVersion,
-                waitForKeys: false,
-                settings: rsSettings,
-                seedRandomNumberGenerator: !randomSeedAlreadySet,
-            });
-
-            print("ShardingTest starting replica set for shard: " + setName);
-
-            // Start up the replica set but don't wait for it to complete. This allows the startup
-            // of each shard to proceed in parallel.
-            this._rs[i] =
-                {setName: setName, test: rs, nodes: rs.startSetAsync(rsDefaults), url: rs.getURL()};
+            rsDefaults = Object.merge(rsDefaults, otherParams.rsOptions);
+            rsDefaults.nodes = rsDefaults.nodes || otherParams.numReplicas;
         } else {
-            var options = {
-                useHostname: otherParams.useHostname,
-                pathOpts: Object.merge(pathOpts, {shard: i}),
-                dbpath: "$testName$shard",
-                shardsvr: '',
-                keyFile: this.keyFile
-            };
-
-            options.setParameter = options.setParameter || {};
-            options.setParameter.migrationLockAcquisitionMaxWaitMS =
-                otherParams.migrationLockAcquisitionMaxWaitMS;
-
             if (jsTestOptions().shardMixedBinVersions) {
                 if (!otherParams.shardOptions) {
                     otherParams.shardOptions = {};
                 }
-                // If the test doesn't depend on specific shard binVersions, create a mixed version
+                // If the test doesn't depend on specific shard binVersions, create a mixed
+                // version
                 // shard cluster that randomly assigns shard binVersions, half "latest" and half
-                // "last-lts".
-                // TODO SERVER-50389: Support last-continuous binary version with
+                // "last-continuous" or "last-lts".
                 // shardMixedBinVersions.
                 if (!otherParams.shardOptions.binVersion) {
                     Random.setRandomSeed();
-                    otherParams.shardOptions.binVersion =
-                        MongoRunner.versionIterator(["latest", "last-lts"], true);
+                    otherParams.shardOptions.binVersion = MongoRunner.versionIterator(
+                        ["latest", jsTestOptions().shardMixedBinVersions], true);
                 }
             }
 
@@ -1484,53 +1262,54 @@ var ShardingTest = function(params) {
                     MongoRunner.versionIterator(otherParams.shardOptions.binVersion);
             }
 
-            options = Object.merge(options, otherParams.shardOptions);
-            options = Object.merge(options, otherParams["d" + i]);
-
-            options.port = options.port || _allocatePortForShard();
-
-            if (otherParams.useBridge) {
-                var bridgeOptions =
-                    Object.merge(otherParams.bridgeOptions, options.bridgeOptions || {});
-                bridgeOptions = Object.merge(bridgeOptions, {
-                    hostName: otherParams.useHostname ? hostName : "localhost",
-                    port: _allocatePortForBridgeForShard(),
-                    // The mongod processes identify themselves to mongobridge as host:port, where
-                    // the host is the actual hostname of the machine and not localhost.
-                    dest: hostName + ":" + options.port,
-                });
-
-                var bridge = new MongoBridge(bridgeOptions);
-            }
-
-            var conn = MongoRunner.runMongod(options);
-            if (!conn) {
-                throw new Error("Failed to start shard " + i);
-            }
-
-            if (otherParams.useBridge) {
-                bridge.connectToBridge();
-                this._connections.push(bridge);
-                unbridgedConnections.push(conn);
-            } else {
-                this._connections.push(conn);
-            }
-
-            _alldbpaths.push(testName + i);
-            this["shard" + i] = this._connections[i];
-            this["d" + i] = this._connections[i];
-
-            this._rs[i] = null;
-            this._rsObjects[i] = null;
+            rsDefaults = Object.merge(rsDefaults, otherParams["d" + i]);
+            rsDefaults = Object.merge(rsDefaults, otherParams.shardOptions);
         }
+
+        rsDefaults.setParameter = rsDefaults.setParameter || {};
+        rsDefaults.setParameter.migrationLockAcquisitionMaxWaitMS =
+            otherParams.migrationLockAcquisitionMaxWaitMS;
+
+        var rsSettings = rsDefaults.settings;
+        delete rsDefaults.settings;
+
+        // The number of nodes in the rs field will take priority.
+        if (otherParams.rs || otherParams["rs" + i]) {
+            var numReplicas = rsDefaults.nodes || 3;
+        } else {
+            var numReplicas = 1;
+        }
+        delete rsDefaults.nodes;
+
+        var protocolVersion = rsDefaults.protocolVersion;
+        delete rsDefaults.protocolVersion;
+
+        var rs = new ReplSetTest({
+            name: setName,
+            nodes: numReplicas,
+            host: hostName,
+            useHostName: otherParams.useHostname,
+            useBridge: otherParams.useBridge,
+            bridgeOptions: otherParams.bridgeOptions,
+            keyFile: this.keyFile,
+            protocolVersion: protocolVersion,
+            waitForKeys: false,
+            settings: rsSettings,
+            seedRandomNumberGenerator: !randomSeedAlreadySet,
+        });
+
+        print("ShardingTest starting replica set for shard: " + setName);
+
+        // Start up the replica set but don't wait for it to complete. This allows the startup
+        // of each shard to proceed in parallel.
+        this._rs[i] =
+            {setName: setName, test: rs, nodes: rs.startSetAsync(rsDefaults), url: rs.getURL()};
     }
 
     //
     // Start up the config server replica set.
     //
-    this._configServers = [];
 
-    // Using replica set for config servers
     var rstOptions = {
         useHostName: otherParams.useHostname,
         host: hostName,
@@ -1543,7 +1322,7 @@ var ShardingTest = function(params) {
         isConfigServer: true,
     };
 
-    // when using CSRS, always use wiredTiger as the storage engine
+    // always use wiredTiger as the storage engine for CSRS
     var startOptions = {
         pathOpts: pathOpts,
         // Ensure that journaling is always enabled for config servers.
@@ -1575,11 +1354,9 @@ var ShardingTest = function(params) {
     //
     // Wait for each shard replica set to finish starting up.
     //
-    if (shardsAsReplSets) {
-        for (let i = 0; i < numShards; i++) {
-            print("Waiting for shard " + this._rs[i].setName + " to finish starting up.");
-            this._rs[i].test.startSetAwait();
-        }
+    for (let i = 0; i < numShards; i++) {
+        print("Waiting for shard " + this._rs[i].setName + " to finish starting up.");
+        this._rs[i].test.startSetAwait();
     }
 
     //
@@ -1592,14 +1369,14 @@ var ShardingTest = function(params) {
     config.settings = config.settings || {};
 
     print("ShardingTest startup for all nodes took " + (new Date() - startTime) + "ms with " +
-          this.configRS.nodeList().length + " config server nodes and " +
-          totalNumShardNodes(shardsAsReplSets) + " total shard nodes.");
+          this.configRS.nodeList().length + " config server nodes and " + totalNumShardNodes() +
+          " total shard nodes.");
 
     //
     // Initiate each shard replica set and wait for replication. Also initiate the config replica
     // set. Whenever possible, in parallel.
     //
-    const shardsRS = shardsAsReplSets ? this._rs.map(obj => obj.test) : [];
+    const shardsRS = this._rs.map(obj => obj.test);
     const replicaSetsToInitiate = [...shardsRS, this.configRS].map(rst => {
         const rstConfig = rst.getReplSetConfig();
 
@@ -1715,26 +1492,20 @@ var ShardingTest = function(params) {
         }
     }
 
-    if (shardsAsReplSets) {
-        for (let i = 0; i < numShards; i++) {
-            let rs = this._rs[i].test;
+    for (let i = 0; i < numShards; i++) {
+        let rs = this._rs[i].test;
 
-            this["rs" + i] = rs;
-            this._rsObjects[i] = rs;
+        this["rs" + i] = rs;
+        this._rsObjects[i] = rs;
 
-            _alldbpaths.push(null);
-            this._connections.push(null);
+        this._connections.push(null);
 
-            if (otherParams.useBridge) {
-                unbridgedConnections.push(null);
-            }
-            let rsConn = new Mongo(rs.getURL());
-            rsConn.name = rs.getURL();
+        let rsConn = new Mongo(rs.getURL());
+        rsConn.name = rs.getURL();
 
-            this._connections[i] = rsConn;
-            this["shard" + i] = rsConn;
-            rsConn.rs = rs;
-        }
+        this._connections[i] = rsConn;
+        this["shard" + i] = rsConn;
+        rsConn.rs = rs;
     }
 
     // Wait for master to be elected before starting mongos
@@ -1743,7 +1514,7 @@ var ShardingTest = function(params) {
 
     print("ShardingTest startup and initiation for all nodes took " + (new Date() - startTime) +
           "ms with " + this.configRS.nodeList().length + " config server nodes and " +
-          totalNumShardNodes(shardsAsReplSets) + " total shard nodes.");
+          totalNumShardNodes() + " total shard nodes.");
 
     // If 'otherParams.mongosOptions.binVersion' is an array value, then we'll end up constructing a
     // version iterator.
@@ -1780,7 +1551,8 @@ var ShardingTest = function(params) {
     if (_hasNewFeatureCompatibilityVersion() && clusterVersionInfo.isMixedVersion) {
         const fcv = binVersionToFCV(clusterVersionInfo.oldestBinVersion);
         function setFeatureCompatibilityVersion() {
-            assert.commandWorked(csrsPrimary.adminCommand({setFeatureCompatibilityVersion: fcv}));
+            assert.commandWorked(csrsPrimary.adminCommand(
+                {setFeatureCompatibilityVersion: fcv, fromConfigServer: true}));
 
             // Wait for the new featureCompatibilityVersion to propagate to all nodes in the CSRS
             // to ensure that older versions of mongos can successfully connect.
@@ -1813,17 +1585,14 @@ var ShardingTest = function(params) {
     }
 
     this._configDB = this.configRS.getURL();
-    this._configServers = this.configRS.nodes;
     for (var i = 0; i < numConfigs; ++i) {
-        var conn = this._configServers[i];
+        var conn = this.configRS.nodes[i];
         this["config" + i] = conn;
         this["c" + i] = conn;
     }
 
     printjson('Config servers: ' + this._configDB);
 
-    var configConnection = _connectWithRetry(this._configDB);
-
     print("ShardingTest " + this._testName + " :\n" +
           tojson({config: this._configDB, shards: this._connections}));
 
@@ -1913,6 +1682,11 @@ var ShardingTest = function(params) {
         throw e;
     }
 
+    // Ensure that the sessions collection exists so jstests can run things with
+    // logical sessions and test them. We do this by forcing an immediate cache refresh
+    // on the config server, which auto-shards the collection for the cluster.
+    this.configRS.getPrimary().getDB("admin").runCommand({refreshLogicalSessionCacheNow: 1});
+
     // Ensure that all CSRS nodes are up to date. This is strictly needed for tests that use
     // multiple mongoses. In those cases, the first mongos initializes the contents of the 'config'
     // database, but without waiting for those writes to replicate to all the config servers then
@@ -1921,16 +1695,9 @@ var ShardingTest = function(params) {
     this.configRS.awaitLastOpCommitted();
 
     if (jsTestOptions().keyFile) {
-        jsTest.authenticate(configConnection);
-        jsTest.authenticateNodes(this._configServers);
         jsTest.authenticateNodes(this._mongos);
     }
 
-    // Ensure that the sessions collection exists so jstests can run things with
-    // logical sessions and test them. We do this by forcing an immediate cache refresh
-    // on the config server, which auto-shards the collection for the cluster.
-    this.configRS.getPrimary().getDB("admin").runCommand({refreshLogicalSessionCacheNow: 1});
-
     // Flushes the routing table cache on connection 'conn'. If 'keyFileLocal' is defined,
     // authenticates the keyfile user.
     const flushRT = function flushRoutingTableAndHandleAuth(conn, keyFileLocal) {
@@ -1975,13 +1742,8 @@ var ShardingTest = function(params) {
                 ? otherParams.shards[i].keyFile
                 : this.keyFile;
 
-            if (otherParams.rs || otherParams["rs" + i] || startShardsAsRS) {
-                const rs = this._rs[i].test;
-                flushRT(rs.getPrimary(), keyFileLocal);
-            } else {
-                // If specified, use the keyFile for the standalone shard.
-                flushRT(this["shard" + i], keyFileLocal);
-            }
+            const rs = this._rs[i].test;
+            flushRT(rs.getPrimary(), keyFileLocal);
         }
 
         self.waitForShardingInitialized();
