@@ -29,14 +29,16 @@
 
 #pragma once
 
+#include <list>
 
 #include "mongo/platform/atomic_word.h"
 #include "mongo/platform/mutex.h"
 #include "mongo/stdx/condition_variable.h"
-#include "mongo/stdx/list.h"
 #include "mongo/stdx/variant.h"
 #include "mongo/transport/service_entry_point.h"
+#include "mongo/transport/service_executor_fixed.h"
 #include "mongo/transport/service_executor_reserved.h"
+#include "mongo/transport/service_executor_synchronous.h"
 #include "mongo/transport/service_state_machine.h"
 #include "mongo/util/hierarchical_acquisition.h"
 #include "mongo/util/net/cidr.h"
@@ -65,9 +67,12 @@ public:
     void startSession(transport::SessionHandle session) override;
 
     void endAllSessions(transport::Session::TagMask tags) final;
+    void endAllSessionsNoTagMask();
 
     Status start() final;
     bool shutdown(Milliseconds timeout) final;
+    bool shutdownAndWait(Milliseconds timeout);
+    bool waitForNoSessions(Milliseconds timeout);
 
     void appendStats(BSONObjBuilder* bob) const final;
 
@@ -75,8 +80,15 @@ public:
         return _currentConnections.load();
     }
 
+    size_t maxOpenSessions() const final {
+        return _maxNumConnections;
+    }
+
 private:
-    using SSMList = stdx::list<std::shared_ptr<ServiceStateMachine>>;
+    void _terminateAll(WithLock);
+    bool _waitForNoSessions(stdx::unique_lock<Mutex>& lk, Date_t deadline);
+
+    using SSMList = std::list<transport::ServiceStateMachine>;
     using SSMListIterator = SSMList::iterator;
 
     ServiceContext* const _svcCtx;
@@ -84,14 +96,12 @@ private:
 
     mutable Mutex _sessionsMutex =
         MONGO_MAKE_LATCH(HierarchicalAcquisitionLevel(0), "ServiceEntryPointImpl::_sessionsMutex");
-    stdx::condition_variable _shutdownCondition;
+    stdx::condition_variable _sessionsCV;
     SSMList _sessions;
 
-    size_t _maxNumConnections{DEFAULT_MAX_CONN};
+    const size_t _maxNumConnections{DEFAULT_MAX_CONN};
     AtomicWord<size_t> _currentConnections{0};
     AtomicWord<size_t> _createdConnections{0};
-
-    std::unique_ptr<transport::ServiceExecutorReserved> _adminInternalPool;
 };
 
 /*
