@@ -31,6 +31,7 @@
 
 #include <memory>
 
+#include "mongo/config.h"
 #include "mongo/db/baton.h"
 #include "mongo/platform/atomic_word.h"
 #include "mongo/rpc/message.h"
@@ -40,8 +41,14 @@
 #include "mongo/util/net/hostandport.h"
 #include "mongo/util/net/sockaddr.h"
 #include "mongo/util/time_support.h"
+#ifdef MONGO_CONFIG_SSL
+#include "mongo/util/net/ssl_types.h"
+#endif
 
 namespace mongo {
+
+class SSLManagerInterface;
+
 namespace transport {
 
 class TransportLayer;
@@ -105,16 +112,23 @@ public:
     /**
      * Source (receive) a new Message from the remote host for this Session.
      */
-    virtual StatusWith<Message> sourceMessage() = 0;
-    virtual Future<Message> asyncSourceMessage(const BatonHandle& handle = nullptr) = 0;
+    virtual StatusWith<Message> sourceMessage() noexcept = 0;
+    virtual Future<Message> asyncSourceMessage(const BatonHandle& handle = nullptr) noexcept = 0;
+
+    /**
+     * Waits for the availability of incoming data.
+     */
+    virtual Status waitForData() noexcept = 0;
+    virtual Future<void> asyncWaitForData() noexcept = 0;
 
     /**
      * Sink (send) a Message to the remote host for this Session.
      *
      * Async version will keep the buffer alive until the operation completes.
      */
-    virtual Status sinkMessage(Message message) = 0;
-    virtual Future<void> asyncSinkMessage(Message message, const BatonHandle& handle = nullptr) = 0;
+    virtual Status sinkMessage(Message message) noexcept = 0;
+    virtual Future<void> asyncSinkMessage(Message message,
+                                          const BatonHandle& handle = nullptr) noexcept = 0;
 
     /**
      * Cancel any outstanding async operations. There is no way to cancel synchronous calls.
@@ -143,6 +157,11 @@ public:
      */
     virtual bool isConnected() = 0;
 
+    /**
+     * Returns true if this session was connected through an L4 load balancer.
+     */
+    virtual bool isFromLoadBalancer() const = 0;
+
     virtual const HostAndPort& remote() const = 0;
     virtual const HostAndPort& local() const = 0;
 
@@ -180,10 +199,22 @@ public:
      * of the 'mutateFunc' call. The 'kPending' tag is only for new sessions; callers should never
      * try to set it.
      */
-    void mutateTags(const stdx::function<TagMask(TagMask)>& mutateFunc);
+    void mutateTags(const std::function<TagMask(TagMask)>& mutateFunc);
 
     TagMask getTags() const;
 
+#ifdef MONGO_CONFIG_SSL
+    /**
+     * Get the configuration from the SSL manager.
+     */
+    virtual const SSLConfiguration* getSSLConfiguration() const = 0;
+
+    /**
+     * Get the SSL manager associated with this session.
+     */
+    virtual const std::shared_ptr<SSLManagerInterface> getSSLManager() const = 0;
+#endif
+
 protected:
     Session();
 
