@@ -33,9 +33,11 @@
 
 #include "mongo/util/producer_consumer_queue.h"
 
+#include "mongo/db/concurrency/locker_noop_client_observer.h"
+#include "mongo/db/operation_context.h"
 #include "mongo/db/service_context.h"
+#include "mongo/platform/mutex.h"
 #include "mongo/stdx/condition_variable.h"
-#include "mongo/stdx/mutex.h"
 #include "mongo/stdx/thread.h"
 #include "mongo/util/assert_util.h"
 
@@ -134,6 +136,10 @@ std::enable_if_t<requiresMultiProducer && requiresMultiConsumer> runCallbackWith
 
 class ProducerConsumerQueueTest : public unittest::Test {
 public:
+    ProducerConsumerQueueTest() {
+        _serviceCtx->registerClientObserver(std::make_unique<LockerNoopClientObserver>());
+    }
+
     template <bool requiresMultiProducer, bool requiresMultiConsumer, typename Callback>
     void runPermutations(Callback&& callback) {
         runCallbackWithPerms<requiresMultiProducer, requiresMultiConsumer>([&](auto x_, auto y_) {
@@ -318,7 +324,7 @@ PRODUCER_CONSUMER_QUEUE_TEST(pushesWithTimeout, runTimeoutPermutations<false, fa
     {
         MoveOnly mo(1);
         pcq.push(std::move(mo));
-        ASSERT(mo.movedFrom());
+        ASSERT(mo.movedFrom());  // NOLINT(bugprone-use-after-move)
     }
 
     helper
@@ -622,7 +628,7 @@ PRODUCER_CONSUMER_QUEUE_TEST(popManyUpToPopWithBlockingWithSpecialCost,
 PRODUCER_CONSUMER_QUEUE_TEST(singleProducerMultiConsumer, runPermutations<false, true>) {
     typename Helper::template ProducerConsumerQueue<MoveOnly> pcq{};
 
-    stdx::mutex mutex;
+    auto mutex = MONGO_MAKE_LATCH();
     size_t successes = 0;
     size_t failures = 0;
 
@@ -632,10 +638,10 @@ PRODUCER_CONSUMER_QUEUE_TEST(singleProducerMultiConsumer, runPermutations<false,
             {
                 try {
                     pcq.pop(opCtx);
-                    stdx::lock_guard<stdx::mutex> lk(mutex);
+                    stdx::lock_guard<Latch> lk(mutex);
                     successes++;
                 } catch (const ExceptionFor<ErrorCodes::ProducerConsumerQueueConsumed>&) {
-                    stdx::lock_guard<stdx::mutex> lk(mutex);
+                    stdx::lock_guard<Latch> lk(mutex);
                     failures++;
                 }
             }
@@ -665,7 +671,7 @@ PRODUCER_CONSUMER_QUEUE_TEST(multiProducerSingleConsumer, runPermutations<true,
 
     pcq.push(MoveOnly(1));
 
-    stdx::mutex mutex;
+    auto mutex = MONGO_MAKE_LATCH();
     size_t success = 0;
     size_t failure = 0;
 
@@ -675,10 +681,10 @@ PRODUCER_CONSUMER_QUEUE_TEST(multiProducerSingleConsumer, runPermutations<true,
             {
                 try {
                     pcq.push(MoveOnly(1), opCtx);
-                    stdx::lock_guard<stdx::mutex> lk(mutex);
+                    stdx::lock_guard<Latch> lk(mutex);
                     success++;
                 } catch (const ExceptionFor<ErrorCodes::ProducerConsumerQueueEndClosed>&) {
-                    stdx::lock_guard<stdx::mutex> lk(mutex);
+                    stdx::lock_guard<Latch> lk(mutex);
                     failure++;
                 }
             }
@@ -688,7 +694,7 @@ PRODUCER_CONSUMER_QUEUE_TEST(multiProducerSingleConsumer, runPermutations<true,
     pcq.pop();
 
     while (true) {
-        stdx::lock_guard<stdx::mutex> lk(mutex);
+        stdx::lock_guard<Latch> lk(mutex);
         if (success == 1)
             break;
         stdx::this_thread::yield();
@@ -744,7 +750,7 @@ PRODUCER_CONSUMER_QUEUE_TEST(multiProducerMiddleWaiterBreaks, runPermutations<tr
 
     pcq.push(MoveOnly(1));
 
-    stdx::mutex mutex;
+    auto mutex = MONGO_MAKE_LATCH();
     bool failed = false;
     OperationContext* threadBopCtx = nullptr;
 
@@ -757,7 +763,7 @@ PRODUCER_CONSUMER_QUEUE_TEST(multiProducerMiddleWaiterBreaks, runPermutations<tr
 
     auto threadB = helper.runThread("ProducerB", [&](OperationContext* opCtx) {
         {
-            stdx::lock_guard<stdx::mutex> lk(mutex);
+            stdx::lock_guard<Latch> lk(mutex);
             threadBopCtx = opCtx;
         }
 
@@ -773,7 +779,7 @@ PRODUCER_CONSUMER_QUEUE_TEST(multiProducerMiddleWaiterBreaks, runPermutations<tr
     };
 
     {
-        stdx::lock_guard<stdx::mutex> lk(mutex);
+        stdx::lock_guard<Latch> lk(mutex);
         ASSERT(threadBopCtx != nullptr);
     }
 
